@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,6 +25,11 @@
 #ifndef ACTIONS_H
 #define ACTIONS_H
 
-int actions_function(char **args);
-int run_action(char *action, char **args);
+__BEGIN_DECLS
+
+int actions_function(char **);
+int run_action(char *, char **);
+
+__END_DECLS
+
 #endif /* ACTIONS_H */
