@@ -1,8 +1,8 @@
 /* archives.h
  *
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -24,7 +24,11 @@
 #ifndef ARCHIVES_H
 #define ARCHIVES_H
 
-int is_compressed(char *file, int test_iso);
-int archiver(char **args, char mode);
+__BEGIN_DECLS
+
+int is_compressed(char *, const int);
+int archiver(char **, char);
+
+__END_DECLS
 
 #endif /* ARCHIVES_H */
