@@ -47,23 +47,32 @@
 #include "strings.h"
 
 #if defined(_NO_PROFILES) || defined(_NO_FZF) || defined(_NO_ICONS) \
-|| defined(_NO_TRASH)
+|| defined(_NO_TRASH) || defined(_BE_POSIX)
 # include "messages.h"
 #endif /* _NO_PROFILES || _NO_FZF || _NO_ICONS || _NO_TRASH */
 
-#define PREVIEW_FILE 1
-#define OPEN_FILE    2
+#ifndef _NO_LIRA
+# define PREVIEW_FILE 1
+# define OPEN_FILE    2
+#endif /* _NO_LIRA */
 
 /* Macros to be able to consult the value of a macro string */
-#define STRINGIZE_(x) #x
-#define STRINGIZE(x) STRINGIZE_(x)
+#ifdef CLIFM_DATADIR
+# define STRINGIZE_(x) #x
+# define STRINGIZE(x) STRINGIZE_(x)
+#endif /* CLIFM_DATADIR */
 
-#ifdef RUN_CMD
-# define OPTSTRING "+:aAb:c:C:D:eEfFgGhHiIk:lLmoOp:P:rsStvw:xyz:"
+#ifdef _BE_POSIX
+# define OPTSTRING ":aAb:B:c:CdDeEfFgGhHij:J:k:lLmMnNo:O:p:P:qQrRsSt:TuvV:w:WxXyYz:Z"
 #else
-# define OPTSTRING "+:aAb:c:D:eEfFgGhHiIk:lLmoOp:P:rsStvw:xyz:"
-#endif /* RUN_CMD */
+# ifdef RUN_CMD
+#  define OPTSTRING "+:aAb:c:C:D:eEfFgGhHiIk:lLmoOp:P:rsStvw:xyz:"
+# else
+#  define OPTSTRING "+:aAb:c:D:eEfFgGhHiIk:lLmoOp:P:rsStvw:xyz:"
+# endif /* RUN_CMD */
+#endif /* _BE_POSIX */
 
+#ifndef _BE_POSIX
 /* Macros for long options. Let's use values >= 200 to avoid conflicts
  * with short options (a-Z == 65-122). */
 #define LOPT_NO_CD_AUTO             200
@@ -137,6 +146,10 @@
 #define LOPT_NO_BOLD                269
 #define LOPT_FNFTAB                 270
 
+#define LOPT_STAT                   271
+#define LOPT_STAT_FULL              272
+#define LOPT_READONLY               273
+
 /* Link long (--option) and short options (-o) for the getopt_long function. */
 static const struct option longopts[] = {
 	{"no-hidden", no_argument, 0, 'a'},
@@ -233,6 +246,7 @@ static const struct option longopts[] = {
 	{"opener", required_argument, 0, LOPT_OPENER},
 	{"preview", required_argument, 0, LOPT_PREVIEW},
 	{"print-sel", no_argument, 0, LOPT_PRINT_SEL},
+	{"readonly", no_argument, 0, LOPT_READONLY},
 	{"rl-vi-mode", no_argument, 0, LOPT_RL_VI_MODE},
 	{"share-selbox", no_argument, 0, LOPT_SHARE_SELBOX},
 	{"sort-reverse", no_argument, 0, LOPT_SORT_REVERSE},
@@ -244,22 +258,33 @@ static const struct option longopts[] = {
 	{"shotgun-file", required_argument, 0, LOPT_SHOTGUN_FILE},
 	{"si", no_argument, 0, LOPT_SI},
 	{"smenutab", no_argument, 0, LOPT_SMENUTAB},
+	{"stat", required_argument, 0, LOPT_STAT},
+	{"stat-full", required_argument, 0, LOPT_STAT_FULL},
 	{"stdtab", no_argument, 0, LOPT_STDTAB},
 	{"virtual-dir", required_argument, 0, LOPT_VIRTUAL_DIR},
 	{"virtual-dir-full-paths", no_argument, 0, LOPT_VIRTUAL_DIR_FULL_PATHS},
 	{"vt100", no_argument, 0, LOPT_VT100},
     {0, 0, 0, 0}
 };
+#endif /* !_BE_POSIX */
 
+__attribute__ ((noreturn))
 static void
 err_arg_required(const char *arg)
 {
+#ifdef _BE_POSIX
+	char *optname = "-h";
+#else
+	char *optname = "--help";
+#endif /* _BE_POSIX */
 	fprintf(stderr, _("%s: '%s': Option requires an argument\n"
-		"Try '%s --help' for more information.\n"), PROGRAM_NAME,
-		arg, PROGRAM_NAME);
+		"Try '%s %s' for more information.\n"), PROGRAM_NAME,
+		arg, PROGRAM_NAME, optname);
 	exit(EXIT_FAILURE);
 }
 
+#ifndef _BE_POSIX
+__attribute__ ((noreturn))
 static void
 err_invalid_opt(const char *arg)
 {
@@ -268,6 +293,7 @@ err_invalid_opt(const char *arg)
 		PROGRAM_NAME, arg, PROGRAM_NAME);
 	exit(EXIT_FAILURE);
 }
+#endif
 
 /* If path was not set (neither in the config file nor via command line nor
  * via the RestoreLastPath option), set the default (CWD), and if CWD is not
@@ -314,7 +340,7 @@ set_start_path(void)
 
 	if (cur_ws > MAX_WS - 1) {
 		cur_ws = DEF_CUR_WS;
-		_err('w', PRINT_PROMPT, _("%s: %zu: Invalid workspace."
+		err('w', PRINT_PROMPT, _("%s: %zu: Invalid workspace."
 			"\nFalling back to workspace %zu\n"),
 			PROGRAM_NAME, cur_ws, cur_ws + 1);
 	}
@@ -331,11 +357,11 @@ set_start_path(void)
 	/* If chdir() fails, set path to PWD, list files and print the
 	 * error message. If no access to PWD either, exit. */
 	if (ret == -1) {
-		_err('e', PRINT_PROMPT, "%s: chdir: '%s': %s\n", PROGRAM_NAME,
+		err('e', PRINT_PROMPT, "%s: chdir: '%s': %s\n", PROGRAM_NAME,
 		    workspaces[cur_ws].path, strerror(errno));
 
 		if (!pwd || !*pwd) {
-			_err(0, NOPRINT_PROMPT, _("%s: Fatal error! Failure "
+			err(0, NOPRINT_PROMPT, _("%s: Fatal error! Failure "
 				"retrieving current working directory\n"), PROGRAM_NAME);
 			exit(EXIT_FAILURE);
 		}
@@ -537,7 +563,7 @@ get_data_dir(void)
 	if (get_data_dir_from_path(argv_bk[0]) == EXIT_SUCCESS)
 		return;
 
-	_err('w', PRINT_PROMPT, _("%s: No data directory found. Data files, "
+	err('w', PRINT_PROMPT, _("%s: No data directory found. Data files, "
 		"such as plugins and color schemes, might not be available.\n"),
 		PROGRAM_NAME);
 }
@@ -550,7 +576,7 @@ get_home_sec_env(void)
 	uid_t u = geteuid();
 	pw = getpwuid(u);
 	if (!pw) {
-		_err('e', PRINT_PROMPT, "%s: getpwuid: %s\n",
+		err('e', PRINT_PROMPT, "%s: getpwuid: %s\n",
 			PROGRAM_NAME, strerror(errno));
 		exit(EXIT_FAILURE);
 	}
@@ -559,6 +585,7 @@ get_home_sec_env(void)
 }
 
 /* Opener function: open FILENAME and exit */
+__attribute__ ((noreturn))
 static void
 open_reg_exit(char *filename, const int url, const int preview)
 {
@@ -603,19 +630,20 @@ open_reg_exit(char *filename, const int url, const int preview)
 	exit(ret);
 }
 
-static inline int
+static int
 set_sort_by_name(const char *name)
 {
 	size_t i;
 	for (i = 0; i <= SORT_TYPES; i++) {
-		if (*name == *_sorts[i].name && strcmp(name, _sorts[i].name) == 0)
-			return _sorts[i].num;
+		if (*name == *sort_methods[i].name
+		&& strcmp(name, sort_methods[i].name) == 0)
+			return sort_methods[i].num;
 	}
 
 	return SNAME;
 }
 
-static inline void
+static void
 set_sort(const char *arg)
 {
 	int n = 0;
@@ -625,21 +653,24 @@ set_sort(const char *arg)
 	else
 		n = atoi(arg);
 
-	if (n < 0 || n > SORT_TYPES)
-		conf.sort = SNAME;
-	else
-		conf.sort = n;
+	if (n < 0 || n > SORT_TYPES) {
+		fprintf(stderr, _("%s: -z: %s: Valid values are 0-%d\n"),
+			PROGRAM_NAME, arg, SORT_TYPES);
+		exit(EXIT_FAILURE);
+	}
 
-	xargs.sort = conf.sort;
+	xargs.sort = conf.sort = n;
 }
 
+#ifndef _NO_LIRA
 /* Open/preview FILE according to MODE: either PREVIEW_FILE or OPEN_FILE */
+__attribute__ ((noreturn))
 static void
 open_preview_file(char *file, const int mode)
 {
 	if (xargs.stealth_mode == 1) {
-		fprintf(stderr, "%s: Running in stealth mode. Access to "
-			"configuration files is not allowed\n", PROGRAM_NAME);
+		fprintf(stderr, _("%s: Running in stealth mode. Access to "
+			"configuration files is not allowed\n"), PROGRAM_NAME);
 		exit(EXIT_FAILURE);
 	}
 
@@ -673,7 +704,161 @@ RUN:
 
 	open_reg_exit(fpath, url, preview);
 }
+#endif /* !_NO_LIRA */
 
+static void
+set_alt_bm_file(char *file)
+{
+	char *p = (char *)NULL;
+
+	if (*file == '~') {
+		p = tilde_expand(file);
+		file = p;
+	}
+
+	if (access(file, R_OK) == -1) {
+		err('e', PRINT_PROMPT, _("%s: %s: %s\n"
+			"Falling back to the default bookmarks file\n"),
+		    PROGRAM_NAME, file, strerror(errno));
+		free(p);
+		return;
+	}
+
+	alt_bm_file = savestring(file, strlen(file));
+	err(ERR_NO_LOG, PRINT_PROMPT, _("%s: Loaded alternative "
+		"bookmarks file\n"), PROGRAM_NAME);
+
+	free(p);
+}
+
+#ifndef _BE_POSIX
+static void
+set_vt100(void)
+{
+	xargs.vt100 = 1;
+	xargs.clear_screen = conf.clear_screen = 0;
+	fzftab = 0; tabmode = STD_TAB;
+}
+
+__attribute__ ((noreturn))
+static void
+set_fzytab(void)
+{
+	fprintf(stderr, _("%s: --fzytab: We have migrated to 'fnf'.\n"
+		"Install 'fnf' (https://github.com/leo-arch/fnf) and then "
+		"use --fnftab instead\n"), PROGRAM_NAME);
+	exit(EXIT_FAILURE);
+}
+
+static void
+set_fzfpreview(const int optc)
+{
+#if !defined(_NO_FZF) && !defined(_NO_LIRA)
+	xargs.fzf_preview = 1;
+	conf.fzf_preview = optc == LOPT_FZFPREVIEW ? 1 : 2;
+	xargs.fzftab = fzftab = 1; tabmode = FZF_TAB;
+#else
+	UNUSED(optc);
+	fprintf(stderr, "%s: --fzf-preview: %s\n",
+		PROGRAM_NAME, _(NOT_AVAILABLE));
+	exit(EXIT_FAILURE);
+#endif /* !_NO_FZF && !_NO_LIRA */
+}
+
+static void
+set_datadir(char *opt)
+{
+	if (!opt || !*opt || *opt == '-')
+		err_arg_required("--data-dir"); /* noreturn */
+
+	get_data_dir_from_path(opt);
+}
+
+static void
+set_fuzzy_algo(const char *opt)
+{
+	int a = opt ? atoi(opt) : -1;
+
+	if (a < 1 || a > FUZZY_ALGO_MAX) {
+		fprintf(stderr, _("%s: %s: Invalid fuzzy algorithm. Valid "
+			"values are either 1 or 2.\n"), PROGRAM_NAME, opt ? opt : "NULL");
+		exit(EXIT_FAILURE);
+	}
+
+	xargs.fuzzy_match_algo = conf.fuzzy_match_algo = a;
+}
+
+static void
+set_bell_style(const char *opt)
+{
+	int a = atoi(opt);
+
+	if (!is_number(opt) || a < 0 || a > 3) {
+		fprintf(stderr, _("%s: %s: Invalid bell style. Valid values "
+			"are 0:none, 1:audible, 2:visible (requires readline >= 8.1), "
+			"3:flash. Defaults to 'visible', and, if not possible, 'none'.\n"),
+			PROGRAM_NAME, opt);
+		exit(EXIT_FAILURE);
+	}
+
+	xargs.bell_style = a;
+}
+
+static void
+set_alt_config_dir(char *dir)
+{
+	char *dir_exp = (char *)NULL;
+
+	if (*dir == '~') {
+		dir_exp = tilde_expand(dir);
+		dir = dir_exp;
+	}
+
+	int dir_ok = 1;
+	struct stat attr;
+
+	if (stat(dir, &attr) == -1) {
+		char *tmp_cmd[] = {"mkdir", "-p", dir, NULL};
+		int ret = launch_execv(tmp_cmd, FOREGROUND, E_NOSTDERR);
+		if (ret != EXIT_SUCCESS) {
+			err('e', PRINT_PROMPT, _("%s: %s: Cannot create directory "
+				"(error %d)\nFalling back to default configuration "
+				"directory\n"), PROGRAM_NAME, dir, ret);
+			dir_ok = 0;
+		}
+	}
+
+	if (access(dir, W_OK) == -1) {
+		if (dir_ok == 1) {
+			err('e', PRINT_PROMPT, _("%s: %s: %s\n"
+				"Falling back to default configuration directory\n"),
+				PROGRAM_NAME, dir, strerror(errno));
+		}
+	} else {
+		alt_config_dir = savestring(dir, strlen(dir));
+		err(ERR_NO_LOG, PRINT_PROMPT, _("%s: %s: Using alternative "
+			"configuration directory\n"), PROGRAM_NAME, alt_config_dir);
+	}
+
+	free(dir_exp);
+}
+
+static void
+set_custom_selfile(char *file)
+{
+	if (!file || !*file || *file == '-')
+		err_arg_required("--sel-file"); /* noreturn */
+
+	if ( (sel_file = normalize_path(file, strlen(file))) ) {
+		xargs.sel_file = 1;
+		return;
+	}
+
+	err('e', PRINT_PROMPT, _("%s: %s: Error setting custom "
+		"selections file\n"), PROGRAM_NAME, file);
+}
+
+#ifndef _NO_LIRA
 static char *
 stat_file(char *file)
 {
@@ -702,83 +887,15 @@ stat_file(char *file)
 }
 
 static void
-set_custom_selfile(char *file)
+set_shotgun_file(char *opt)
 {
-	if (!file || !*file || *file == '-')
-		err_arg_required("--sel-file");
+	if (!opt || !*opt || *opt == '-')
+		err_arg_required("--shotgun-file"); /* noreturn */
 
-	if ( (sel_file = normalize_path(file, strlen(file))) ) {
-		xargs.sel_file = 1;
-		return;
-	}
-
-	_err('e', PRINT_PROMPT, _("%s: %s: Error setting custom "
-		"selections file\n"), PROGRAM_NAME, file);
-}
-
-static void
-set_alt_bm_file(char *file)
-{
-	char *p = (char *)NULL;
-
-	if (*file == '~') {
-		p = tilde_expand(file);
-		file = p;
-	}
-
-	if (access(file, R_OK) == -1) {
-		_err('e', PRINT_PROMPT, _("%s: %s: %s\n"
-			"Falling back to the default bookmarks file\n"),
-		    PROGRAM_NAME, file, strerror(errno));
-		free(p);
-		return;
-	}
-
-	alt_bm_file = savestring(file, strlen(file));
-	_err(ERR_NO_LOG, PRINT_PROMPT, _("%s: Loaded alternative "
-		"bookmarks file\n"), PROGRAM_NAME);
-
-	free(p);
-}
-
-static void
-set_alt_config_dir(char *dir)
-{
-	char *dir_exp = (char *)NULL;
-
-	if (*dir == '~') {
-		dir_exp = tilde_expand(dir);
-		dir = dir_exp;
-	}
-
-	int dir_ok = 1;
-	struct stat attr;
-
-	if (stat(dir, &attr) == -1) {
-		char *tmp_cmd[] = {"mkdir", "-p", dir, NULL};
-		int ret = launch_execv(tmp_cmd, FOREGROUND, E_NOSTDERR);
-		if (ret != EXIT_SUCCESS) {
-			_err('e', PRINT_PROMPT, _("%s: %s: Cannot create directory "
-				"(error %d)\nFalling back to default configuration "
-				"directory\n"), PROGRAM_NAME, dir, ret);
-			dir_ok = 0;
-		}
-	}
-
-	if (access(dir, W_OK) == -1) {
-		if (dir_ok == 1) {
-			_err('e', PRINT_PROMPT, _("%s: %s: %s\n"
-				"Falling back to default configuration directory\n"),
-				PROGRAM_NAME, dir, strerror(errno));
-		}
-	} else {
-		alt_config_dir = savestring(dir, strlen(dir));
-		_err(ERR_NO_LOG, PRINT_PROMPT, _("%s: %s: Using alternative "
-			"configuration directory\n"), PROGRAM_NAME, alt_config_dir);
-	}
-
-	free(dir_exp);
+	alt_preview_file = stat_file(opt);
 }
+#endif /* !_NO_LIRA */
+#endif /* !_BE_POSIX */
 
 static void
 set_alt_kbinds_file(char *file)
@@ -790,12 +907,12 @@ set_alt_kbinds_file(char *file)
 	}
 
 	if (access(file, R_OK) == -1) {
-		_err('e', PRINT_PROMPT, _("%s: %s: %s\n"
+		err('e', PRINT_PROMPT, _("%s: %s: %s\n"
 			"Falling back to the default keybindings file\n"),
 		    PROGRAM_NAME, file, strerror(errno));
 	} else {
 		alt_kbinds_file = savestring(file, strlen(file));
-		_err(ERR_NO_LOG, PRINT_PROMPT, _("%s: Loaded alternative "
+		err(ERR_NO_LOG, PRINT_PROMPT, _("%s: Loaded alternative "
 			"keybindings file\n"), PROGRAM_NAME);
 	}
 
@@ -813,12 +930,12 @@ set_alt_config_file(char *file)
 	}
 
 	if (access(file, R_OK) == -1) {
-		_err('e', PRINT_PROMPT, _("%s: %s: %s\nFalling back to default\n"),
+		err('e', PRINT_PROMPT, _("%s: %s: %s\nFalling back to default\n"),
 			PROGRAM_NAME, file, strerror(errno));
 		xargs.config = -1;
 	} else {
 		alt_config_file = savestring(file, strlen(file));
-		_err(ERR_NO_LOG, PRINT_PROMPT, _("%s: Loaded alternative "
+		err(ERR_NO_LOG, PRINT_PROMPT, _("%s: Loaded alternative "
 			"configuration file\n"), PROGRAM_NAME);
 	}
 
@@ -840,7 +957,7 @@ resolve_path(char *file)
 	} else if (*file == '~') {
 		_path = tilde_expand(file);
 		if (!_path) {
-			fprintf(stderr, "%s: %s: Error expanding tilde\n",
+			fprintf(stderr, _("%s: %s: Error expanding tilde\n"),
 				PROGRAM_NAME, file);
 			exit(EXIT_FAILURE);
 		}
@@ -872,11 +989,8 @@ resolve_starting_path(char *file)
 
 	if (IS_FILE_URI(file)) {
 		_path = savestring(file + 7, strlen(file + 7));
-
 	} else if (is_url(file) == EXIT_SUCCESS) {
-		open_reg_exit(file, 1, 0);
-		exit(EXIT_SUCCESS); /* Never reached */
-
+		open_reg_exit(file, 1, 0); /* noreturn */
 	} else {
 		_path = resolve_path(file);
 	}
@@ -899,7 +1013,7 @@ resolve_starting_path(char *file)
 }
 
 static void
-_set_starting_path(char *_path)
+set_starting_path(char *_path)
 {
 	if (xchdir(_path, SET_TITLE) == 0) {
 		if (cur_ws == UNSET)
@@ -914,7 +1028,7 @@ _set_starting_path(char *_path)
 			exit(EXIT_FAILURE);
 		}
 
-		_err('w', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME,
+		err('w', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME,
 			_path, strerror(errno));
 	}
 }
@@ -935,7 +1049,7 @@ set_opener(const char *str)
 		conf.opener = savestring(ep, strlen(ep));
 		free(ep);
 	} else {
-		_err('w', PRINT_PROMPT, _("%s: Error expanding tilde. "
+		err('w', PRINT_PROMPT, _("%s: Error expanding tilde. "
 			"Using default opener.\n"), PROGRAM_NAME);
 	}
 }
@@ -962,11 +1076,11 @@ set_alt_profile(const char *name)
 }
 
 static void
-set_virtual_dir(const char *str)
+set_virtual_dir(const char *str, const char *optname)
 {
 	if (!str || *str != '/') {
-		fprintf(stderr, "%s: '--virtual-dir': Absolute path "
-			"is required as argument\n", PROGRAM_NAME);
+		fprintf(stderr, _("%s: '%s': Absolute path is required "
+			"as argument\n"), PROGRAM_NAME, optname);
 		exit(EXIT_FAILURE);
 	}
 
@@ -998,108 +1112,20 @@ set_workspace(const char *opt)
 	}
 
 ERROR:
-	fprintf(stderr, "%s: %s: Invalid workspace. Valid "
-		"values are 1-8.\n", PROGRAM_NAME, opt);
+	fprintf(stderr, _("%s: %s: Invalid workspace. Valid "
+		"values are 1-8.\n"), PROGRAM_NAME, opt);
 	exit(EXIT_FAILURE);
 }
 
 static void
-set_bell_style(const char *opt)
-{
-	int a = atoi(opt);
-
-	if (!is_number(opt) || a < 0 || a > 3) {
-		fprintf(stderr, "%s: %s: Invalid bell style. Valid values "
-			"are 0:none, 1:audible, 2:visible (requires readline >= 8.1), "
-			"3:flash. Defaults to 'visible', and, if not possible, 'none'.\n",
-			PROGRAM_NAME, opt);
-		exit(EXIT_FAILURE);
-	}
-
-	xargs.bell_style = a;
-}
-
-static void
-set_fuzzy_algo(const char *opt)
-{
-	int a = opt ? atoi(opt) : -1;
-
-	if (a < 1 || a > FUZZY_ALGO_MAX) {
-		fprintf(stderr, "%s: %s: Invalid fuzzy algorithm. Valid "
-			"values are either 1 or 2.\n", PROGRAM_NAME, opt ? opt : "NULL");
-		exit(EXIT_FAILURE);
-	}
-
-	xargs.fuzzy_match_algo = conf.fuzzy_match_algo = a;
-}
-
-static void
-set_color_scheme(const char *opt)
+set_color_scheme(const char *opt, const char *optname)
 {
 	if (!opt || !*opt || *opt == '-')
-		err_arg_required("--color-scheme");
-		/* err_arg_required calls exit() */
+		err_arg_required(optname); /* noreturn */
 
 	conf.usr_cscheme = savestring(opt, strlen(opt));
 }
 
-static void
-set_datadir(char *opt)
-{
-	if (!opt || !*opt || *opt == '-')
-		err_arg_required("--data-dir");
-		/* err_arg_required calls exit() */
-
-	get_data_dir_from_path(opt);
-}
-
-static void
-set_fnftab(void)
-{
-#ifndef _NO_FZF
-	xargs.fnftab = 1; fzftab = 1; tabmode = FNF_TAB;
-#else
-	fprintf(stderr, _("%s: fnf-tab: %s\n"), PROGRAM_NAME, _(NOT_AVAILABLE));
-	exit(EXIT_FAILURE);
-#endif /* _NO_FZF */
-}
-
-static void
-set_fzfpreview(const int optc)
-{
-#ifndef _NO_FZF
-	xargs.fzf_preview = 1;
-	conf.fzf_preview = optc == LOPT_FZFPREVIEW ? 1 : 2;
-	xargs.fzftab = fzftab = 1; tabmode = FZF_TAB;
-#else
-	UNUSED(optc);
-	fprintf(stderr, _("%s: fzf-preview: %s\n"),
-		PROGRAM_NAME, _(NOT_AVAILABLE));
-	exit(EXIT_FAILURE);
-#endif /* !_NO_FZF */
-}
-
-static void
-set_fzftab(void)
-{
-#ifndef _NO_FZF
-	xargs.fzftab = fzftab = 1; tabmode = FZF_TAB;
-#else
-	fprintf(stderr, _("%s: fzf-tab: %s\n"),
-		PROGRAM_NAME, _(NOT_AVAILABLE));
-	exit(EXIT_FAILURE);
-#endif /* !_NO_FZF */
-}
-
-static void
-set_fzytab(void)
-{
-	fprintf(stderr, "%s: --fzytab: We have migrated to 'fnf'.\n"
-		"Install 'fnf' (https://github.com/leo-arch/fnf) and then "
-		"use --fnftab instead\n", PROGRAM_NAME);
-	exit(EXIT_FAILURE);
-}
-
 static void
 set_no_colors(void)
 {
@@ -1110,13 +1136,26 @@ set_no_colors(void)
 }
 
 static void
-set_shotgun_file(char *opt)
+set_fnftab(void)
 {
-	if (!opt || !*opt || *opt == '-')
-		err_arg_required("--shotgun-file");
-		/* err_arg_required calls exit() */
+#ifndef _NO_FZF
+	xargs.fnftab = 1; fzftab = 1; tabmode = FNF_TAB;
+#else
+	fprintf(stderr, "%s: fnf-tab: %s\n", PROGRAM_NAME, _(NOT_AVAILABLE));
+	exit(EXIT_FAILURE);
+#endif /* _NO_FZF */
+}
 
-	alt_preview_file = stat_file(opt);
+static void
+set_fzftab(void)
+{
+#ifndef _NO_FZF
+	xargs.fzftab = fzftab = 1; tabmode = FZF_TAB;
+#else
+	fprintf(stderr, "%s: fzf-tab: %s\n",
+		PROGRAM_NAME, _(NOT_AVAILABLE));
+	exit(EXIT_FAILURE);
+#endif /* !_NO_FZF */
 }
 
 static void
@@ -1125,7 +1164,7 @@ set_smenutab(void)
 #ifndef _NO_FZF
 	xargs.smenutab = 1; fzftab = 1; tabmode = SMENU_TAB;
 #else
-	fprintf(stderr, _("%s: smenu-tab: %s\n"),
+	fprintf(stderr, "%s: smenu-tab: %s\n",
 		PROGRAM_NAME, _(NOT_AVAILABLE));
 	exit(EXIT_FAILURE);
 #endif /* !_NO_FZF */
@@ -1155,21 +1194,205 @@ set_trash_as_rm(void)
 #ifndef _NO_TRASH
 	xargs.trasrm = conf.tr_as_rm = 1;
 #else
-	fprintf(stderr, _("%s: trash: %s\n"), PROGRAM_NAME, _(NOT_AVAILABLE));
+	fprintf(stderr, "%s: trash: %s\n", PROGRAM_NAME, _(NOT_AVAILABLE));
 	exit(EXIT_FAILURE);
 #endif /* !_NO_TRASH */
 }
 
 static void
-set_vt100(void)
+set_stat(const int optc, const char *optval)
 {
-	xargs.vt100 = 1;
-	xargs.clear_screen = conf.clear_screen = 0;
-	fzftab = 0; tabmode = STD_TAB;
+	if (!optval || !*optval || *optval == '-')
+#ifndef _BE_POSIX
+		err_arg_required(optc == LOPT_STAT ? "--stat" : "--stat-full");
+
+	xargs.stat = (optc == LOPT_STAT ? SIMPLE_STAT : FULL_STAT);
+#else
+		err_arg_required(optc == 'j' ? "-j" : "-J");
+
+	xargs.stat = (optc == 'j' ? SIMPLE_STAT : FULL_STAT);
+#endif /* !_BE_POSIX */
+
+	xargs.restore_last_path = conf.restore_last_path = 0;
+	stat_filename = savestring(optval, strlen(optval));
 }
 
+__attribute__ ((noreturn))
+static void
+print_version(void)
+{
+#ifdef _BE_POSIX
+	char *posix = "-POSIX";
+#else
+	char *posix = "";
+#endif /* _BE_POSIX */
+	printf("%s%s\n", VERSION, posix);
+	exit(EXIT_SUCCESS);
+}
+
+#ifdef _BE_POSIX
+static void
+set_tab_mode(const char *opt)
+{
+	if (!opt || !*opt || opt[1])
+		return;
+
+	int n = *opt - '0';
+
+	switch (n) {
+	case 0:	set_stdtab(); break;
+	case 1:	set_fzftab(); break;
+	case 2: set_smenutab(); break;
+	case 3: set_fnftab(); break;
+	default:
+		fprintf(stderr, _("%s: %s: Valid values are 0-3\n"), PROGRAM_NAME, opt);
+		exit(EXIT_FAILURE);
+	}
+}
+#endif /* _BE_POSIX */
+
 /* Evaluate command line arguments, if any, and change initial variables to
  * their corresponding values. */
+#ifdef _BE_POSIX
+/* POSIX version: getopt(3) is used (instead of getopt_long(3)) and only
+ * short options are provided. */
+void
+parse_cmdline_args(const int argc, char **argv)
+{
+	opterr = optind = 0;
+	int optc;
+#ifndef _NO_LIRA
+	int open_prev_mode = 0;
+	char *open_prev_file = (char *)NULL;
+#endif /* !_NO_LIRA */
+
+	while ((optc = getopt(argc, argv, OPTSTRING)) != EOF) {
+		switch (optc) {
+		case 'a': conf.show_hidden = xargs.hidden = 0; break;
+		case 'A': conf.show_hidden = xargs.hidden = 1; break;
+		case 'b': xargs.bm_file = 1; set_alt_bm_file(optarg); break;
+		case 'B': set_tab_mode(optarg); break;
+		case 'c': xargs.config = 1; set_alt_config_file(optarg); break;
+		case 'C': xargs.clear_screen = conf.clear_screen = 0; break;
+		case 'd': xargs.disk_usage = conf.disk_usage = 1; break;
+		case 'D': xargs.only_dirs = conf.only_dirs = 1; break;
+		case 'e': xargs.auto_open = conf.auto_open = 0; break;
+		case 'E': xargs.autocd = conf.autocd = 0; break;
+		case 'f': xargs.full_dir_size = conf.full_dir_size = 1; break;
+		case 'F': xargs.files_counter = conf.files_counter = 0; break;
+		case 'g': xargs.si = 1; break;
+		case 'G': xargs.apparent_size = conf.apparent_size = 0; break;
+		case 'h': help_function(); break; /* noreturn */
+		case 'H':
+#ifndef _NO_HIGHLIGHT
+			xargs.highlight = conf.highlight = 0;
+#endif /* !_NO_HIGHLIGHT */
+			break;
+		case 'i':
+#ifndef _NO_ICONS
+			xargs.icons = conf.icons = 1; break;
+#else
+			fprintf(stderr, "%s: icons: %s\n", PROGRAM_NAME, _(NOT_AVAILABLE));
+			exit(EXIT_FAILURE);
+#endif /* !_NO_ICONS */
+		case 'j': set_stat(optc, optarg); break;
+		case 'J': set_stat(optc, optarg); break;
+		case 'k': set_alt_kbinds_file(optarg); break;
+		case 'l': conf.long_view = xargs.longview = 1; break;
+		case 'L': xargs.follow_symlinks = follow_symlinks = 0; break;
+		case 'm': xargs.fuzzy_match = conf.fuzzy_match = 1; break;
+		case 'M': set_no_colors(); break;
+		case 'n': xargs.history = 0; break;
+		case 'N': xargs.no_bold = 1; break;
+		case 'o': set_opener(optarg); break;
+		case 'O':
+#ifdef _NO_LIRA
+			fprintf(stderr, "%s: open: %s\n", PROGRAM_NAME, _(NOT_AVAILABLE));
+			exit(EXIT_FAILURE);
+#else
+			open_prev_file = optarg;
+			open_prev_mode = OPEN_FILE;
+			break;
+#endif /* _NO_LIRA */
+		case 'p': set_alt_profile(optarg); break;
+		case 'P':
+#ifdef _NO_LIRA
+			fprintf(stderr, "%s: preview: %s\n", PROGRAM_NAME, _(NOT_AVAILABLE));
+			exit(EXIT_FAILURE);
+#else
+			open_prev_file = optarg;
+			open_prev_mode = PREVIEW_FILE;
+			break;
+#endif /* _NO_LIRA */
+		case 'q': xargs.list_and_quit = 1; break;
+		case 'Q': xargs.cd_on_quit = conf.cd_on_quit = 1; break;
+		case 'r': set_trash_as_rm(); break;
+		case 'R': xargs.classify = conf.classify = 0; break;
+		case 's': xargs.stealth_mode = 1; break;
+		case 'S': set_no_suggestions(); break;
+		case 't': set_color_scheme(optarg, "-t"); break;
+		case 'T': xargs.trim_names = conf.trim_names = 0; break;
+		case 'u': xargs.disk_usage_analyzer = 1; break;
+		case 'v': print_version(); /* noreturn */
+		case 'V': set_virtual_dir(optarg, "-V"); break;
+		case 'w': set_workspace(optarg); break;
+		case 'W': xargs.printsel = conf.print_selfiles = 1; break;
+		case 'x':
+			xargs.secure_env = 1;
+			xsecure_env(SECURE_ENV_IMPORT);
+			break;
+		case 'X':
+			xargs.secure_env_full = 1;
+			xsecure_env(SECURE_ENV_FULL);
+			break;
+		case 'y': conf.light_mode = xargs.light = 1; break;
+		case 'Y':
+			xargs.secure_cmds = xargs.secure_env = 1;
+			xsecure_env(SECURE_ENV_IMPORT);
+			break;
+		case 'z': set_sort(optarg); break;
+		case 'Z':
+			set_max_value(optarg, &xargs.max_files, &max_files);
+			break;
+
+		/* Handle error */
+		case ':':
+			fprintf(stderr, _("%s: Option '-%c' requires an argument.\n"
+				"Try '%s -h' for more information.\n"),
+				PROGRAM_NAME, optopt, PROGRAM_NAME);
+			exit(EXIT_FAILURE);
+		case '?':
+			fprintf(stderr, _("%s: Unrecognized option: '-%c'\n"
+				"Try '%s -h' for more information.\n"),
+				PROGRAM_NAME, optopt, PROGRAM_NAME);
+			exit(EXIT_FAILURE);
+
+		default: break;
+		}
+	}
+
+#ifndef _NO_LIRA
+	if (open_prev_mode != 0)
+		open_preview_file(open_prev_file, open_prev_mode); /* noreturn */
+#endif /* !_NO_LIRA */
+
+	if (argv[optind]) { /* Starting path passed as positional parameter */
+		char *spath = resolve_starting_path(argv[optind]);
+		if (spath) {
+			set_starting_path(spath);
+			free(spath);
+		}
+	} else {
+		if (xargs.list_and_quit == 1) {
+			conf.restore_last_path = 0;
+			set_start_path();
+		}
+	}
+}
+
+#else
+/* Non-POSIX version: getopt_long(3) is used and long options are
+* supported. */
 void
 parse_cmdline_args(const int argc, char **argv)
 {
@@ -1178,9 +1401,11 @@ parse_cmdline_args(const int argc, char **argv)
 	opterr = optind = 0;
 
 	int optc;
-	int open_prev_mode = 0;
 	char *path_value = (char *)NULL;
+#ifndef _NO_LIRA
+	int open_prev_mode = 0;
 	char *open_prev_file = (char *)NULL;
+#endif /* _NO_LIRA */
 
 	while ((optc = getopt_long(argc, argv, OPTSTRING,
 		longopts, (int *)0)) != EOF) {
@@ -1194,10 +1419,9 @@ parse_cmdline_args(const int argc, char **argv)
 
 #ifdef RUN_CMD
 		case 'C':
-			if (!optarg || !*optarg || *optarg == '-') {
-				err_arg_required("--cmd");
-				exit(EXIT_FAILURE);
-			}
+			if (!optarg || !*optarg || *optarg == '-')
+				err_arg_required("--cmd"); /* noreturn */
+
 			cmd_line_cmd = savestring(optarg, strlen(optarg));
 			break;
 #endif /* RUN_CMD */
@@ -1209,7 +1433,7 @@ parse_cmdline_args(const int argc, char **argv)
 		case 'F': conf.list_dirs_first = xargs.dirs_first = 1; break;
 		case 'g': conf.pager = xargs.pager = 1; break;
 		case 'G': conf.pager = xargs.pager = 0; break;
-		case 'h': help_function(); exit(EXIT_SUCCESS); break;
+		case 'h': help_function(); break; /* noreturn */
 		case 'H': xargs.horizontal_list = 1; conf.listing_mode = HORLIST; break;
 		case 'i': conf.case_sens_list = xargs.case_sens_list = 0; break;
 		case 'I': conf.case_sens_list = xargs.case_sens_list = 1; break;
@@ -1225,7 +1449,7 @@ parse_cmdline_args(const int argc, char **argv)
 		case 's': conf.splash_screen = xargs.splash = 1; break;
 		case 'S': xargs.stealth_mode = 1; break;
 		case 't': xargs.disk_usage_analyzer = 1; break;
-		case 'v': printf("%s\n", VERSION); exit(EXIT_SUCCESS); break;
+		case 'v': print_version(); /* noreturn */
 		case 'w': set_workspace(optarg); break;
 		case 'x': conf.ext_cmd_ok = xargs.ext = 0; break;
 		case 'y': conf.light_mode = xargs.light = 1; break;
@@ -1241,7 +1465,7 @@ parse_cmdline_args(const int argc, char **argv)
 		case LOPT_CD_ON_QUIT:
 			xargs.cd_on_quit = conf.cd_on_quit = 1; break;
 		case LOPT_COLOR_SCHEME:
-			set_color_scheme(optarg); break;
+			set_color_scheme(optarg, "--color-scheme"); break;
 		case LOPT_CWD_IN_TITLE:
 			xargs.cwd_in_title = 1; break;
 		case LOPT_DATA_DIR:
@@ -1265,7 +1489,7 @@ parse_cmdline_args(const int argc, char **argv)
 		case LOPT_FZFTAB:
 			set_fzftab(); break;
 		case LOPT_FZYTAB:
-			set_fzytab(); break;
+			set_fzytab(); /* noreturn */
 
 #ifndef _NO_ICONS
 		case LOPT_ICONS:
@@ -1275,7 +1499,7 @@ parse_cmdline_args(const int argc, char **argv)
 #else
 		case LOPT_ICONS: /* fallthrough */
 		case LOPT_ICONS_USE_FILE_COLOR:
-			fprintf(stderr, _("%s: icons: %s\n"), PROGRAM_NAME, _(NOT_AVAILABLE));
+			fprintf(stderr, "%s: icons: %s\n", PROGRAM_NAME, _(NOT_AVAILABLE));
 			exit(EXIT_FAILURE);
 			break;
 #endif /* !_NO_ICONS */
@@ -1283,7 +1507,10 @@ parse_cmdline_args(const int argc, char **argv)
 		case LOPT_INT_VARS:
 			xargs.int_vars = int_vars = 1; break;
 		case LOPT_LIST_AND_QUIT:
-			xargs.list_and_quit = 1; break;
+			xargs.list_and_quit = 1;
+			xargs.no_dirjump = 1;
+			xargs.restore_last_path = conf.restore_last_path = 0;
+			break;
 		case LOPT_MAX_DIRHIST:
 			set_max_value(optarg, &xargs.max_dirhist, &conf.max_dirhist);
 			break;
@@ -1348,7 +1575,13 @@ parse_cmdline_args(const int argc, char **argv)
 		case LOPT_ONLY_DIRS:
 			xargs.only_dirs = conf.only_dirs = 1; break;
 
-		case LOPT_OPEN: { /* --open or --preview */
+		case LOPT_OPEN: /* --open or --preview */
+#ifdef _NO_LIRA
+			fprintf(stderr, "%s: --open/--preview: %s\n",
+				PROGRAM_NAME, _(NOT_AVAILABLE));
+			exit(EXIT_FAILURE);
+#else
+		{
 			open_prev_file = optarg;
 			int n = *argv[optind - 1] == '-' ? 1 : 2;
 			if (*(argv[optind - n] + 2) == 'p')
@@ -1356,11 +1589,13 @@ parse_cmdline_args(const int argc, char **argv)
 			else
 				open_prev_mode = OPEN_FILE; /* --open */
 		} break;
+#endif /* _NO_LIRA */
 
 		case LOPT_OPENER:
 			set_opener(optarg); break;
 		case LOPT_PRINT_SEL:
 			xargs.printsel = conf.print_selfiles = 1; break;
+		case LOPT_READONLY: xargs.readonly = conf.readonly = 1; break;
 		case LOPT_RL_VI_MODE:
 			xargs.rl_vi_mode = 1; break;
 		case LOPT_SECURE_CMDS:
@@ -1383,19 +1618,28 @@ parse_cmdline_args(const int argc, char **argv)
 		case LOPT_SHARE_SELBOX:
 			xargs.share_selbox = conf.share_selbox = 1; break;
 		case LOPT_SHOTGUN_FILE:
+#ifdef _NO_LIRA
+			fprintf(stderr, "%s: --shotgun-file: %s\n",
+				PROGRAM_NAME, _(NOT_AVAILABLE));
+			exit(EXIT_FAILURE);
+#else
 			set_shotgun_file(optarg); break;
+#endif /* !_NO_LIRA */
 		case LOPT_SI:
 			xargs.si = 1; break;
 		case LOPT_SMENUTAB:
 			set_smenutab(); break;
 		case LOPT_SORT_REVERSE:
 			xargs.sort_reverse = conf.sort_reverse = 1; break;
+		case LOPT_STAT: /* fallthrough */
+		case LOPT_STAT_FULL:
+			set_stat(optc, optarg); break;
 		case LOPT_STDTAB:
 			set_stdtab(); break;
 		case LOPT_TRASH_AS_RM:
 			set_trash_as_rm(); break;
 		case LOPT_VIRTUAL_DIR:
-			set_virtual_dir(optarg); break;
+			set_virtual_dir(optarg, "--virtual-dir"); break;
 		case LOPT_VIRTUAL_DIR_FULL_PATHS:
 			xargs.virtual_dir_full_paths = 1; break;
 		case LOPT_VT100:
@@ -1403,18 +1647,18 @@ parse_cmdline_args(const int argc, char **argv)
 
 		/* Handle error */
 		case ':':
-			err_arg_required(argv[optind - 1]); exit(EXIT_FAILURE); break;
+			err_arg_required(argv[optind - 1]); /* noreturn */
 		case '?':
-			err_invalid_opt(argv[optind - 1]); exit(EXIT_FAILURE); break;
+			err_invalid_opt(argv[optind - 1]); /* noreturn */
 
 		default: break;
 		}
 	}
 
-	if (open_prev_mode != 0) {
-		open_preview_file(open_prev_file, open_prev_mode);
-		exit(EXIT_SUCCESS); /* Never reached */
-	}
+#ifndef _NO_LIRA
+	if (open_prev_mode != 0)
+		open_preview_file(open_prev_file, open_prev_mode); /* noreturn */
+#endif /* !_NO_LIRA */
 
 	char *spath = (char *)NULL;
 	if (argv[optind]) { /* Starting path passed as positional parameter */
@@ -1425,7 +1669,15 @@ parse_cmdline_args(const int argc, char **argv)
 	}
 
 	if (spath) {
-		_set_starting_path(spath);
+		set_starting_path(spath);
 		free(spath);
+	} else {
+		if (xargs.list_and_quit == 1) {
+			/* Starting path not specified in the command line, let's use
+			 * the current directory. */
+			conf.restore_last_path = 0;
+			set_start_path();
+		}
 	}
 }
+#endif /* _BE_POSIX */
