@@ -30,17 +30,11 @@
 
 #include "helpers.h"
 
-// Unrequired on __linux__, __CYGWIN__, __TERMUX__, __sun, __FreeBSD__,
-// __NetBSD__, __HAIKU__, __DragonFly__, and __OpenBSD__ at least
-//#include <ctype.h>
-
-#include <dirent.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <termios.h>
 #include <unistd.h>
-#include <fcntl.h>
 #include <time.h>
 #include <limits.h>
 #include <readline/readline.h>
@@ -53,6 +47,25 @@
 # include "highlight.h"
 #endif /* !_NO_HIGHLIGHT */
 
+/* Print the regex error ERRCODE (returned by either regcomp(3) or regexec(3)),
+ * whose compiled regular expression is REGEXP and original pattern PATTERN,
+ * in the next prompt if PROMPT_ERR == 1, or immediately otherwise. */
+void
+xregerror(const char *cmd_name, const char *pattern, const int errcode,
+	const regex_t regexp, const int prompt_err)
+{
+	const size_t err_len = regerror(errcode, &regexp, NULL, 0);
+	char *buf = (char *)xnmalloc(err_len + 1, sizeof(char));
+	regerror(errcode, &regexp, buf, err_len);
+
+	if (prompt_err == 1)
+		err('w', PRINT_PROMPT, "%s: %s: %s\n", cmd_name, pattern, buf);
+	else
+		xerror("%s: %s: %s\n", cmd_name, pattern, buf);
+
+	free(buf);
+}
+
 #ifndef _NO_ICONS
 /* Generate a hash of the string STR (case sensitively if CASE_SENTITIVE is
  * set to 1).
@@ -100,6 +113,10 @@ gen_time_str(char *buf, const size_t size, const time_t _time)
 void
 set_fzf_preview_border_type(void)
 {
+#ifdef _NO_LIRA
+	return;
+#endif /* _NO_LIRA */
+
 	fzf_preview_border_type = FZF_BORDER_ROUNDED; /* fzf default */
 
 	char *p = (conf.fzftab_options && *conf.fzftab_options)
@@ -206,14 +223,19 @@ find_digit(char *str)
 /* Check whether a given command needs ELN's to be expanded/completed/suggested.
  * Returns 1 if yes or 0 if not. */
 int
-_expand_eln(const char *text)
+should_expand_eln(const char *text)
 {
 	char *l = rl_line_buffer;
 	if (!l || !*l || !is_number(text))
 		return 0;
 
-	int a = atoi(text); /* Only expand numbers matching ELN's */
-	if (a <= 0 || a > (int)files)
+	/* Exclude 'ws' and 'st/sort' commands. */
+	if ((*l == 'w' && l[1] == 's' && !l[2])
+	|| (*l == 's' && ((l[1] == 't' && !l[2]) || strcmp(l + 1, "ort") == 0)))
+		return 0;
+
+	const filesn_t a = xatof(text);
+	if (a <= 0 || a > files) /* Only expand numbers matching ELN's */
 		return 0;
 
 	if (words_num == 1) { /* First word */
@@ -238,16 +260,18 @@ _expand_eln(const char *text)
 		flags &= ~STATE_COMPLETING;
 		return 0;
 	}
+
 	flags &= ~STATE_COMPLETING;
 	*p = t;
 
 	return 1;
 }
 
+#if !defined(_POSIX_C_SOURCE) || _POSIX_C_SOURCE >= 199309L
 /* Sleep for MSEC milliseconds. */
 /* Taken from https://stackoverflow.com/questions/1157209/is-there-an-alternative-sleep-function-in-c-to-milliseconds */
 static int
-msleep(long msec)
+msleep(const long msec)
 {
 	struct timespec ts;
 	int res;
@@ -266,6 +290,7 @@ msleep(long msec)
 
 	return res;
 }
+#endif /* _POSIX_C_SOURCE >= 199309L */
 
 /* Convert the file named STR (as absolute path) into a more friendly format.
  * Change absolute paths into:
@@ -287,15 +312,15 @@ abbreviate_file_name(char *str)
 		return (char *)NULL;
 
 	char *name = (char *)NULL;
-	size_t len = strlen(str);
-	size_t wlen = (workspaces && workspaces[cur_ws].path)
+	const size_t len = strlen(str);
+	const size_t wlen = (workspaces && workspaces[cur_ws].path)
 		? strlen(workspaces[cur_ws].path) : 0;
 
 	/* If STR is in CWD -> ./STR */
 	if (workspaces && workspaces[cur_ws].path && wlen > 1 && len > wlen
 	&& strncmp(str, workspaces[cur_ws].path, wlen) == 0
 	&& *(str + wlen) == '/') {
-		size_t name_len = strlen(str + wlen + 1) + 3;
+		const size_t name_len = strlen(str + wlen + 1) + 3;
 		name = (char *)xnmalloc(name_len, sizeof(char));
 		snprintf(name, name_len, "./%s", str + wlen + 1);
 		return name;
@@ -337,6 +362,24 @@ get_cwd(char *buf, const size_t buflen, const int check_workspace)
 	return tmp;
 }
 
+/* OpenBSD implementation of memrchr(), licensed MIT.
+ * Modified code is licensed GPL2+. */
+static const void *
+xmemrchr(const void *s, const int c, size_t n)
+{
+	const unsigned char *cp;
+
+	if (n != 0) {
+		cp = (unsigned char *)s + n;
+		do {
+			if (*(--cp) == (unsigned char)c)
+				return((void *)cp);
+		} while (--n != 0);
+	}
+
+	return (NULL);
+}
+
 /* Canonicalize/normalize the path SRC without resolving symlinks.
  * SRC is deescaped if necessary.
  * ~/./.. are resolved.
@@ -349,7 +392,7 @@ normalize_path(char *src, const size_t src_len)
 
 	/* Deescape SRC */
 	char *tmp = (char *)NULL;
-	int is_escaped = *src == '\\';
+	const int is_escaped = *src == '\\';
 
 	if (strchr(src, '\\')) {
 		tmp = dequote_str(src, 0);
@@ -358,7 +401,7 @@ normalize_path(char *src, const size_t src_len)
 			return (char *)NULL;
 		}
 
-		size_t tlen = strlen(tmp);
+		const size_t tlen = strlen(tmp);
 		if (tlen > 0 && tmp[tlen - 1] == '/')
 			tmp[tlen - 1] = '\0';
 
@@ -374,7 +417,7 @@ normalize_path(char *src, const size_t src_len)
 			xerror(_("%s: %s: Error expanding tilde\n"), PROGRAM_NAME, src);
 			return (char *)NULL;
 		}
-		size_t tlen = strlen(tmp);
+		const size_t tlen = strlen(tmp);
 		if (tlen > 0 && tmp[tlen - 1] == '/')
 			tmp[tlen - 1] = '\0';
 
@@ -383,11 +426,11 @@ normalize_path(char *src, const size_t src_len)
 	}
 
 	char *s = tmp ? tmp : src;
-	size_t l = tmp ? strlen(tmp) : src_len;
+	const size_t l = tmp ? strlen(tmp) : src_len;
 
 	/* Resolve references to . and .. */
-	char *res;
-	size_t res_len;
+	char *res = (char *)NULL;
+	size_t res_len = 0;
 
 	if (l == 0 || *s != '/') {
 		/* Relative path */
@@ -399,7 +442,7 @@ normalize_path(char *src, const size_t src_len)
 			return (char *)NULL;
 		}
 
-		size_t pwd_len = strlen(cwd);
+		const size_t pwd_len = strlen(cwd);
 		if (pwd_len == 1 && *cwd == '/') {
 			/* If CWD is root (/) do not copy anything. Just create a buffer
 			 * big enough to hold "/dir", which will be appended next */
@@ -436,11 +479,7 @@ normalize_path(char *src, const size_t src_len)
 
 		case 2:
 			if (ptr[0] == '.' && ptr[1] == '.') {
-#if !defined(__HAIKU__) && !defined(_BE_POSIX) && !defined(__APPLE__)
-				const char *slash = memrchr(res, '/', res_len);
-#else
-				const char *slash = strrchr(res, '/');
-#endif /* !__HAIKU && !_BE_POSIX && !__APPLE__ */
+				const char *slash = xmemrchr(res, '/', res_len);
 				if (slash)
 					res_len = (size_t)(slash - res);
 				continue;
@@ -479,6 +518,7 @@ rl_ring_bell(void)
 	case BELL_AUDIBLE:
 		RING_BELL; fflush(stderr); return;
 
+#if !defined(_POSIX_C_SOURCE) || _POSIX_C_SOURCE >= 199309L
 	case BELL_FLASH:
 		SET_RVIDEO;
 		fflush(stderr);
@@ -487,7 +527,7 @@ rl_ring_bell(void)
 		fflush(stderr);
 		return;
 
-#ifdef _READLINE_HAS_ACTIVATE_MARK /* Readline >= 8.1 */
+# ifdef READLINE_HAS_ACTIVATE_MARK /* Readline >= 8.1 */
 	case BELL_VISIBLE: {
 		int point = rl_point;
 
@@ -504,16 +544,17 @@ rl_ring_bell(void)
 		msleep(VISIBLE_BELL_DELAY);
 		rl_deactivate_mark();
 
-# ifndef _NO_HIGHLIGHT
+#  ifndef _NO_HIGHLIGHT
 		if (conf.highlight && !wrong_cmd) {
 			rl_point = rl_mark;
 			recolorize_line();
 		}
-# endif /* !_NO_HIGHLIGHT */
+#  endif /* !_NO_HIGHLIGHT */
 		rl_point = point;
 		return;
 	}
-#endif /* _READLINE_HAS_ACTIVATE_MARK */
+# endif /* READLINE_HAS_ACTIVATE_MARK */
+#endif /* _POSIX_C_SOURCE >= 199309L */
 
 	case BELL_NONE: /* fallthrough */
 	default: return;
@@ -572,7 +613,8 @@ open_fread(char *name, int *fd)
 	return fp;
 }
 
-/* Create a file for writing (truncating it to zero length if it already exists).
+/* Create a file for writing (truncating it to zero length if it already exists,
+ * and with permissions 600).
  * Return a file stream associated to the file named NAME and update FD to hold
  * the corresponding file descriptor.
  * NOTE: As stated here, file streams should be preferred over file descriptors:
@@ -601,16 +643,26 @@ inline mode_t
 get_dt(const mode_t mode)
 {
 	switch (mode & S_IFMT) {
+	case S_IFREG:  return DT_REG;
+	case S_IFDIR:  return DT_DIR;
+	case S_IFLNK:  return DT_LNK;
+	case S_IFIFO:  return DT_FIFO;
+	case S_IFSOCK: return DT_SOCK;
 	case S_IFBLK:  return DT_BLK;
 	case S_IFCHR:  return DT_CHR;
-	case S_IFDIR:  return DT_DIR;
-#ifdef __sun
+#ifndef _BE_POSIX
+# ifdef SOLARIS_DOORS
 	case S_IFDOOR: return DT_DOOR;
-#endif /* __sun */
-	case S_IFIFO:  return DT_FIFO;
-	case S_IFLNK:  return DT_LNK;
-	case S_IFREG:  return DT_REG;
-	case S_IFSOCK: return DT_SOCK;
+	case S_IFPORT: return DT_PORT;
+# endif /* SOLARIS_DOORS */
+# ifdef S_IFWHT
+	case S_IFWHT:  return DT_WHT;
+# endif
+# ifdef S_ARCH1
+	case S_ARCH1:  return DT_ARCH1;
+	case S_ARCH2:  return DT_ARCH2;
+# endif
+#endif /* !_BE_POSIX */
 	default:       return DT_UNKNOWN;
 	}
 }
@@ -661,13 +713,13 @@ get_rgb(char *hex, int *attr, int *r, int *g, int *b)
 	char tmp[3];
 	tmp[2] = '\0';
 
-	tmp[0] = *hex, tmp[1] = *(hex + 1);
+	tmp[0] = *hex; tmp[1] = *(hex + 1);
 	*r = hex2int(tmp);
 
-	tmp[0] = *(hex + 2), tmp[1] = *(hex + 3);
+	tmp[0] = *(hex + 2); tmp[1] = *(hex + 3);
 	*g = hex2int(tmp);
 
-	tmp[0] = *(hex + 4), tmp[1] = *(hex + 5);
+	tmp[0] = *(hex + 4); tmp[1] = *(hex + 5);
 	*b = hex2int(tmp);
 
 	*attr = 0;
@@ -715,24 +767,37 @@ hex2rgb(char *hex)
 /* Count files in the directory DIR, including self and parent. If POP is set
  * to 1, the function will just check if the directory is populated (it has at
  * least 3 files, including self and parent).
- * Returns -1 in case of error or an integer (0-INT_MAX) in case of success. */
-int
+ * Returns -1 in case of error or an integer (0-INTMAX_MAX) in case of success. */
+filesn_t
 count_dir(const char *dir, const int pop)
 {
 	if (!dir)
 		return (-1);
 
 	DIR *p;
+
 	if ((p = opendir(dir)) == NULL) {
 		if (errno == ENOMEM)
 			exit(ENOMEM);
 		return (-1);
 	}
+/* Let's set the O_NOATIME flag (Linux only) to prevent updating atime, avoiding
+ * thus unnecessary disk writes. */
+/*#ifdef O_NOATIME
+	int fd = dirfd(p);
+	if (fd != -1) {
+		int fflags = fcntl(fd, F_GETFL);
+		if (fflags != -1) {
+			fflags |= O_NOATIME;
+			fcntl(fd, F_SETFL, fflags);
+		}
+	}
+#endif // O_NOATIME */
 
-	unsigned int c = 0;
+	filesn_t c = 0;
 
 	while (readdir(p)) {
-		if (c + 1 > INT_MAX)
+		if (c > FILESN_MAX - 1)
 			break;
 
 		c++;
@@ -741,7 +806,7 @@ count_dir(const char *dir, const int pop)
 	}
 
 	closedir(p);
-	return (int)c;
+	return c;
 }
 
 /* Get the path of a the command CMD inspecting all paths in the PATH
@@ -791,33 +856,69 @@ get_cmd_path(const char *cmd)
 	return (char *)NULL;
 }
 
-/* Convert SIZE to human readeable form (at most 2 decimal places).
- * Returns a string of at most MAX_UNIT_SIZE, defined in aux.h */
+/*
 char *
-get_size_unit(const off_t size)
+construct_human_size(const off_t size)
+{
+	static char str[MAX_UNIT_SIZE];
+
+	const off_t base = xargs.si == 1 ? 1000 : 1024;
+	off_t s = size;
+//	uint16_t rem = 0; // much faster than off_t/long, but fails
+	off_t rem = 0;
+	uint8_t n = 0;
+
+	while (s >= base) {
+		rem = s; // reminder (integer form) of the last division
+		s = 0; // quotient part of the last division
+		while (rem >= base) {rem -= base; ++s;}
+		++n; // number of performed divisions
+	}
+
+	float float_rem = rem == 0 ? 0.00f : (float)rem / (float)base;
+
+	static const char *const u = "BKMGTPEZYRQ";
+	snprintf(str, MAX_UNIT_SIZE, "%.*f%c%c",
+		rem == 0 ? 0 : 2,
+		(float)s + float_rem,
+		u[n],
+		(u[n] != 'B' && xargs.si == 1) ? 'B' : 0);
+
+	return str;
+} */
+
+/* Convert SIZE to human readable form (at most 2 decimal places).
+ * Returns a pointer to a string of at most MAX_UNIT_SIZE.
+ * We follow here the du(1) notation: K, M, G... for powers of 1024
+ * and KB, MB, KB... for power of 1000. */
+char *
+construct_human_size(const off_t size)
 {
 	/* MAX_UNIT_SIZE == 10 == "1023.99YB\0" */
-	char *str = xnmalloc(MAX_UNIT_SIZE, sizeof(char));
+	static char str[MAX_UNIT_SIZE];
 
-	float base = xargs.si == 1 ? 1000 : 1024;
+	const float base = xargs.si == 1 ? 1000 : 1024;
+	static float mult_factor = 0;
+	if (mult_factor == 0)
+		mult_factor = 1.0f / base;
 
 	size_t n = 0;
 	float s = (float)size;
 
 	while (s >= base) {
-		s = s / base;
+		s = s * mult_factor; /* == (s = s / base), but faster */
 		++n;
 	}
 
 	int x = (int)s;
-	/* If (s - x || s - (float)x) == 0, then S has no reminder (zero)
-	 * We don't want to print the reminder when it is zero */
-
-	/* R: Ronnabyte, Q: Quettabyte. It's highly unlikely to have files of
-	 * these huge sizes in the near future, but anyway... */
-	const char *const u = "BKMGTPEZYRQ";
+	/* If (s == 0 || s - (float)x) == 0, then S has no reminder (zero)
+	 * We don't want to print the reminder when it is zero.
+	 *
+	 * R: Ronnabyte, Q: Quettabyte. It's highly unlikely to have files of
+	 * such huge sizes (and even less) in the near future, but anyway... */
+	static const char *const u = "BKMGTPEZYRQ";
 	snprintf(str, MAX_UNIT_SIZE, "%.*f%c%c",
-		(s == 0 || s - (float)x == 0) ? 0 : 2, /* NOLINT */
+		(s == 0.00f || s - (float)x == 0.00f) ? 0 : 2,
 		(double)s,
 		u[n],
 		(u[n] != 'B' && xargs.si == 1) ? 'B' : 0);
@@ -825,6 +926,167 @@ get_size_unit(const off_t size)
 	return str;
 }
 
+#ifdef USE_XDU
+struct hlink_t {
+	dev_t dev;
+	ino_t ino;
+};
+
+static struct hlink_t *xdu_hardlinks = {0};
+static size_t xdu_hardlink_n = 0;
+
+static inline int
+check_xdu_hardlinks(const dev_t dev, const ino_t ino)
+{
+	if (!xdu_hardlinks || xdu_hardlink_n == 0)
+		return 0;
+
+	size_t i;
+	for (i = 0; i < xdu_hardlink_n; i++) {
+		if (dev == xdu_hardlinks[i].dev && ino == xdu_hardlinks[i].ino)
+			return 1;
+	}
+
+	return 0;
+}
+
+static inline void
+add_xdu_hardlink(const dev_t dev, const ino_t ino)
+{
+	xdu_hardlinks = (struct hlink_t *)xrealloc(xdu_hardlinks,
+		(xdu_hardlink_n + 1) * sizeof(struct hlink_t));
+
+	xdu_hardlinks[xdu_hardlink_n].dev = dev;
+	xdu_hardlinks[xdu_hardlink_n].ino = ino;
+	xdu_hardlink_n++;
+}
+
+static inline void
+free_xdu_hardlinks(void)
+{
+	free(xdu_hardlinks);
+	xdu_hardlinks = (struct hlink_t *)NULL;
+	xdu_hardlink_n = 0;
+}
+
+/* The st_size member of a stat struct is meaningful only:
+ * 1. When computing disk usage (not apparent sizes).
+ * 2. If apparent sizes, only for symlinks and regular files.
+ * NOTE: Here we add shared memory object and typed memory object just to
+ * match the check made by du(1). These objects are not implmented on most
+ * systems, but this might change in the future. */
+static inline int
+usable_st_size(struct stat const *s)
+{
+	return (conf.apparent_size != 1 || S_ISLNK(s->st_mode)
+		|| S_ISREG(s->st_mode) || S_TYPEISSHM(s) || S_TYPEISTMO(s));
+}
+
+/* Read no more than this many directory entries at a time. Without this
+ * limit, processing a directory with 4,000,000 entries requires ~1GiB of
+ * memory, and handling 64M entries would require 16GiB of memory. */
+//#define MAX_READDIR_ENTRIES 100000
+
+/* Hand-made implementation of du(1) providing only those features required
+ * by clifm.
+ *
+ * Returns the full size of the directory DIR in bytes, computing apparent
+ * sizes, if conf.apparent_size is set to 1, or disk usage otherwise. In case
+ * of error, STATUS is set to the appropiate error code (errno): in this case,
+ * the return value may be zero, if DIR couldn't be read for some reason, or
+ * a positive integer representing the size of the entries that have been
+ * actually processed.
+ *
+ * FIRST_LEVEL must be always 1 when calling this function (this value will
+ * be zero whenever the function calls itself recursively).
+ *
+ * NOTE: Old versions of du (at least up to 8.30) count the size of directories
+ * themselves when computing apparent sizes. At least since 9.3, this is not
+ * the case anymore (See
+ * https://git.savannah.gnu.org/gitweb/?p=coreutils.git;a=commit;h=110bcd28386b1f47a4cd876098acb708fdcbbb25).
+ * As stated in 'info du': "Apparent sizes are meaningful only for regular
+ * files and symbolic links. Other file types do not contribute to apparent
+ * size." We follow here the last behavior. */
+off_t
+dir_size(char *dir, const int first_level, int *status)
+{
+	if (!dir || !*dir) {
+		*status = ENOENT;
+		return 0;
+	}
+
+	struct stat a;
+//	const dev_t base_dev = lstat(dir, &a) != -1 ? a.st_dev : 0;
+	off_t size = 0;
+	DIR *p;
+
+	if ((p = opendir(dir)) == NULL) {
+		*status = errno;
+		return 0;
+	}
+
+	/* Compute the size of the base directory itself */
+	if (conf.apparent_size != 1 && first_level == 1
+	&& stat(dir, &a) != -1)
+		size += (a.st_blocks * S_BLKSIZE);
+
+	struct dirent *ent;
+
+	while ((ent = readdir(p)) != NULL) {
+		if (SELFORPARENT(ent->d_name))
+			continue;
+
+		char buf[PATH_MAX + 1];
+		snprintf(buf, sizeof(buf), "%s/%s", dir, ent->d_name);
+
+		if (lstat(buf, &a) == -1) {
+			*status = errno;
+			continue;
+		}
+
+/*		if (base_dev > 0 && base_dev != a.st_dev) {
+			*status = EXDEV; // File is on a different file system.
+			continue;
+		} */
+
+		if (S_ISDIR(a.st_mode)) {
+			/* Even if a subdirectory is unreadable or we can't chdir into
+			 * it, do let its size contribute to the total (provided we're
+			 * not computing apparent sizes). */
+			if (conf.apparent_size != 1)
+				size += (a.st_blocks * S_BLKSIZE);
+
+			if (user.uid != 0
+			&& check_file_access(a.st_mode, a.st_uid, a.st_gid) == 0)
+				*status = EACCES;
+			else
+				size += dir_size(buf, 0, status);
+
+			continue;
+		}
+
+		if (usable_st_size(&a) == 0)
+			continue;
+
+		if (a.st_nlink > 1) {
+			if (check_xdu_hardlinks(a.st_dev, a.st_ino) == 1)
+				continue;
+			else
+				add_xdu_hardlink(a.st_dev, a.st_ino);
+		}
+
+		size += conf.apparent_size == 1 ? a.st_size
+			: (a.st_blocks * S_BLKSIZE);
+	}
+
+	closedir(p);
+
+	if (first_level == 1)
+		free_xdu_hardlinks();
+
+	return size;
+}
+#else
 /* Return the full size of the directory DIR using du(1).
  * The size is reported in bytes if SIZE_IN_BYTES is set to 1.
  * Otherwise, human format is used.
@@ -837,10 +1099,10 @@ dir_size(char *dir, const int size_in_bytes, int *status)
 
 	char file[PATH_MAX];
 #if !defined(__OpenBSD__)
-	snprintf(file, sizeof(file), "%s/duXXXXXX", xargs.stealth_mode == 1
+	snprintf(file, sizeof(file), "%s/.duXXXXXX", xargs.stealth_mode == 1
 		? P_tmpdir : tmp_dir); /* NOLINT */
 #else
-	snprintf(file, sizeof(file), "%s/duXXXXXXXXXX", xargs.stealth_mode == 1
+	snprintf(file, sizeof(file), "%s/.duXXXXXXXXXX", xargs.stealth_mode == 1
 		? P_tmpdir : tmp_dir); /* NOLINT */
 #endif /* !__OpenBSD__ */
 
@@ -897,13 +1159,10 @@ dir_size(char *dir, const int size_in_bytes, int *status)
 	off_t retval = -1;
 	/* We only need here the first field of the line, which is a file
 	 * size and usually takes only a few digits: since a yottabyte takes 26
-	 * digits, 32 is more than enough */
+	 * digits, 32 is more than enough. */
 	char line[32];
-	if (fgets(line, (int)sizeof(line), fp) == NULL) {
-		fclose(fp);
-		unlink(file);
-		return (-1);
-	}
+	if (fgets(line, (int)sizeof(line), fp) == NULL)
+		goto END;
 
 	char *p = strchr(line, '\t');
 	if (p && p != line) {
@@ -911,10 +1170,12 @@ dir_size(char *dir, const int size_in_bytes, int *status)
 		retval = (off_t)atoll(line);
 	}
 
+END:
 	fclose(fp);
 	unlink(file);
 	return retval;
 }
+#endif // USE_XDU
 
 /* Return the file type of the file pointed to by LINK, or -1 in case of
  * error. Possible return values:
@@ -942,16 +1203,16 @@ get_link_ref(const char *link)
 /* Transform an integer (N) into a string of chars.
  * This exists because some operating systems do not support itoa(3). */
 char *
-xitoa(int n)
+xitoa(long long n)
 {
 	if (!n)
 		return "0";
 
 	static char buf[32] = {0};
-	int i = 30;
+	long long i = 30;
 
 	while (n && i) {
-		int rem = n / 10;
+		long long rem = n / 10;
 		buf[i] = (char)('0' + (n - (rem * 10)));
 		n = rem;
 		--i;
@@ -960,13 +1221,39 @@ xitoa(int n)
 	return &buf[++i];
 }
 
+/* Convert the string S into a number in the range of valid ELN's
+ * (1 - FILESN_MAX). Returns this value if valid or -1 in case of error. */
+filesn_t
+xatof(const char *s)
+{
+	errno = 0;
+	const long long ret = strtoll(s, NULL, 10);
+
+#if FILESN_MAX < LLONG_MAX
+	/* 1 - FILESN_MAX */
+	if (ret < 1 || ret > FILESN_MAX) {
+#elif FILESN_MAX > LLONG_MAX
+	/* 1 - LLONG_MAX-1 */
+	if (ret < 1 || ret == LLONG_MAX) {
+#else
+	/* 1 - FILESN_MAX-1 */
+	if (ret < 1 || ret >= FILESN_MAX) {
+#endif /* FILESN_MAX < LLONG_MAX */
+		errno = ERANGE;
+		return (-1);
+	}
+
+	return (filesn_t)ret;
+}
+
 /* A secure atoi implementation to prevent integer under- and over-flow.
  * Returns the corresponding integer, if valid, or INT_MIN if invalid,
  * setting errno to ERANGE. */
 int
 xatoi(const char *s)
 {
-	long ret = strtol(s, NULL, 10);
+	errno = 0;
+	const long ret = strtol(s, NULL, 10);
 
 	if (ret < INT_MIN || ret > INT_MAX) {
 		errno = ERANGE;
@@ -983,7 +1270,7 @@ xrealloc(void *ptr, const size_t size)
 	void *p = realloc(ptr, size);
 
 	if (!p) {
-		_err(0, NOPRINT_PROMPT, _("%s: %s failed to allocate %zu bytes\n"),
+		err(0, NOPRINT_PROMPT, _("%s: %s failed to allocate %zu bytes\n"),
 			PROGRAM_NAME, __func__, size);
 		exit(ENOMEM);
 	}
@@ -997,7 +1284,7 @@ xcalloc(const size_t nmemb, const size_t size)
 	void *p = calloc(nmemb, size);
 
 	if (!p) {
-		_err(0, NOPRINT_PROMPT, _("%s: %s failed to allocate %zu bytes\n"),
+		err(0, NOPRINT_PROMPT, _("%s: %s failed to allocate %zu bytes\n"),
 			PROGRAM_NAME, __func__, nmemb * size);
 		exit(ENOMEM);
 	}
@@ -1011,7 +1298,7 @@ xnmalloc(const size_t nmemb, const size_t size)
 	void *p = malloc(nmemb * size);
 
 	if (!p) {
-		_err(0, NOPRINT_PROMPT, _("%s: %s failed to allocate %zu bytes\n"),
+		err(0, NOPRINT_PROMPT, _("%s: %s failed to allocate %zu bytes\n"),
 			PROGRAM_NAME, __func__, nmemb * size);
 		exit(ENOMEM);
 	}
@@ -1088,9 +1375,9 @@ url_encode(char *str)
 			/* Encode char to URL format. Example: space char to %20 */
 			*pbuf = '%';
 			pbuf++;
-			*pbuf = to_hex(*pstr >> 4); /* Right shift operation */
+			*pbuf = to_hex(*pstr >> 4);
 			pbuf++;
-			*pbuf = to_hex(*pstr & 15); /* Bitwise AND operation */
+			*pbuf = to_hex(*pstr & 15);
 			pbuf++;
 		}
 	}
@@ -1109,9 +1396,8 @@ url_decode(char *str)
 	char *buf = (char *)xnmalloc(strlen(str) + 1, sizeof(char));
 	/* The decoded string will be at most as long as the encoded string */
 
-	char *pstr, *pbuf;
-	pstr = str;
-	pbuf = buf;
+	char *pstr = str, *pbuf = buf;
+
 	for (; *pstr; pstr++) {
 		if (*pstr == '%') {
 			if (pstr[1] && pstr[2]) {
