@@ -29,7 +29,7 @@
 
 #ifdef RL_READLINE_VERSION
 # if RL_READLINE_VERSION >= 0x0801
-#  define _READLINE_HAS_ACTIVATE_MARK
+#  define READLINE_HAS_ACTIVATE_MARK
 # endif /* RL_READLINE_VERSION >= 0x0801 */
 #endif /* RL_READLINE_VERSION */
 
@@ -38,10 +38,10 @@
 
 __BEGIN_DECLS
 
-int  _expand_eln(const char *);
 char *abbreviate_file_name(char *);
+char *construct_human_size(const off_t);
 char *get_cwd(char *, const size_t, const int);
-int  count_dir(const char *, const int);
+filesn_t count_dir(const char *, const int);
 off_t dir_size(char *, const int, int *);
 char from_hex(char);
 char *gen_date_suffix(const struct tm);
@@ -51,7 +51,6 @@ int  get_rgb(char *, int *, int *, int *, int *);
 void clear_term_img(void);
 mode_t get_dt(const mode_t);
 int  get_link_ref(const char *);
-char *get_size_unit(const off_t);
 char *hex2rgb(char *);
 char *normalize_path(char *, const size_t);
 FILE *open_fread(char *, int *);
@@ -59,12 +58,15 @@ FILE *open_fwrite(char *, int *);
 int  read_octal(char *);
 void rl_ring_bell(void);
 void set_fzf_preview_border_type(void);
+int  should_expand_eln(const char *);
 char *url_encode(char *);
 char *url_decode(char *);
+filesn_t xatof(const char *s);
 int  xatoi(const char *);
-char *xitoa(int);
+char *xitoa(long long);
 char xgetchar(void);
 int  xmkdir(char *, const mode_t);
+void xregerror(const char *, const char *, const int, const regex_t, const int);
 
 #ifndef _NO_ICONS
 size_t hashme(const char *, const int);
