@@ -27,29 +27,15 @@
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
-
-#if defined(__NetBSD__)
-# include <sys/param.h>
-# if __NetBSD_Prereq__(9,99,63)
-#  include <sys/acl.h>
-#  define _ACL_OK
-# endif /* __NetBSD_Prereq__ */
-#elif !defined(__HAIKU__) && !defined(__OpenBSD__) && !defined(__sun) \
-&& !defined(__DragonFly__)
-# include <sys/acl.h>
-# define _ACL_OK
-#endif /* __NetBSD__ */
-
 #include <unistd.h>
 
-#if defined(LINUX_FILE_ATTRS)
+#if defined(HAVE_FILE_ATTRS)
 # include <linux/fs.h> /* FS_IMMUTABLE_FL */
 # include <sys/ioctl.h> /* ioctl(3) */
 # ifdef __TINYC__
 #  undef SYNC_FILE_RANGE_WRITE_AND_WAIT /* Silence redefinition error */
 # endif
-# include <fcntl.h> /* O_RDONLY */
-#endif /* LINUX_FILE_ATTRS */
+#endif /* HAVE_FILE_ATTRS */
 
 #include "aux.h"
 #include "misc.h"
@@ -57,7 +43,7 @@
 
 #define TRUE_COLOR 16777216
 
-/* Check whether parameter S is -f or --force
+/* Check whether parameter S is -f or --force.
  * Returns 1 if yes, and there is no "-f" or "--force" file in the current
  * dir, or 0 otherwise. */
 int
@@ -139,7 +125,7 @@ check_truecolor(void)
 static void
 set_term_caps(const int i)
 {
-	int true_color = check_truecolor();
+	const int true_color = check_truecolor();
 
 	if (i == -1) { /* TERM not found in our terminfo database */
 		term_caps.color = true_color == 1 ? TRUE_COLOR : 0;
@@ -169,7 +155,7 @@ set_term_caps(const int i)
 
 /* Check whether current terminal (_TERM) supports colors and requesting
  * cursor position (needed to print suggestions). If not, disable the
- * feature accordingly */
+ * feature accordingly. */
 static void
 check_term_support(const char *_term)
 {
@@ -195,7 +181,7 @@ check_term_support(const char *_term)
 	}
 
 	if (index == -1) {
-		_err('w', PRINT_PROMPT, _("%s: '%s': Terminal type not supported. "
+		err('w', PRINT_PROMPT, _("%s: '%s': Terminal type not supported. "
 			"Limited functionality is expected.\n"), PROGRAM_NAME, _term);
 	}
 
@@ -208,7 +194,7 @@ check_term(void)
 	char *t = getenv("TERM");
 	if (!t || !*t) {
 		t = "xterm";
-		_err('w', PRINT_PROMPT, _("%s: The TERM environment variable is unset.\n"
+		err('w', PRINT_PROMPT, _("%s: The TERM environment variable is unset.\n"
 			"Running in xterm compatibility mode\n"), PROGRAM_NAME);
 	}
 
@@ -219,7 +205,7 @@ static void
 set_mount_cmd(const int udisks2ok, const int udevilok)
 {
 	if (xargs.mount_cmd == MNT_UDISKS2 && !udisks2ok && udevilok) {
-		_err('w', PRINT_PROMPT, _("%s: udisks2 not found. Falling back to "
+		err('w', PRINT_PROMPT, _("%s: udisks2 not found. Falling back to "
 			"udevil\n"), PROGRAM_NAME);
 		xargs.mount_cmd = MNT_UDEVIL;
 		return;
@@ -242,19 +228,19 @@ check_completion_mode(void)
 {
 	if (fzftab == 1) { /* fzftab is zero only if running with --stdtab */
 		if (!(bin_flags & FZF_BIN_OK) && tabmode == FZF_TAB) {
-			_err('w', PRINT_PROMPT, _("%s: fzf: Command not found. Falling "
+			err('w', PRINT_PROMPT, _("%s: fzf: Command not found. Falling "
 				"back to standard TAB completion\n"), PROGRAM_NAME);
 			tabmode = STD_TAB;
 			fzftab = 0;
 		}
 
 		if (!(bin_flags & FNF_BIN_OK) && tabmode == FNF_TAB) {
-			_err('w', PRINT_PROMPT, _("%s: fnf: Command not found. Falling "
+			err('w', PRINT_PROMPT, _("%s: fnf: Command not found. Falling "
 				"back to the default value (fzf, if found, or standard)\n"),
 				PROGRAM_NAME);
 			tabmode = (bin_flags & FZF_BIN_OK) ? FZF_TAB : STD_TAB;
 		} else if (!(bin_flags & SMENU_BIN_OK) && tabmode == SMENU_TAB) {
-			_err('w', PRINT_PROMPT, _("%s: smenu: Command not found. Falling "
+			err('w', PRINT_PROMPT, _("%s: smenu: Command not found. Falling "
 				"back to the default value (fzf, if found, or standard)\n"),
 				PROGRAM_NAME);
 			tabmode = (bin_flags & FZF_BIN_OK) ? FZF_TAB : STD_TAB;
@@ -421,8 +407,9 @@ check_user_groups(const gid_t gid)
 	return 0;
 }
 
-/* Return 1 if current user has access to file with mode MODE, uid UID,
- * and gid GID. Otherwise, return zero */
+/* Return 1 if current user has access (read for files and read/exec for dirs)
+ * to the file with mode MODE, uid UID, and gid GID. Otherwise, 0 is
+ * returned. */
 int
 check_file_access(const mode_t mode, const uid_t uid, const gid_t gid)
 {
@@ -514,43 +501,8 @@ check_immutable_bit(char *file)
 #endif /* !FS_IOC_GETFLAGS || !FS_IMMUTABLE_FL */
 }
 
-/* Return 1 if FILE has some ACL property and zero if none
- * See: https://man7.org/tlpi/code/online/diff/acl/acl_view.c.html */
-int
-is_acl(char *file)
-{
-	if (!file || !*file)
-		return 0;
-
-#ifndef _ACL_OK
-	return 0;
-#else /* _ACL_OK */
-
-	acl_t acl;
-	acl = acl_get_file(file, ACL_TYPE_ACCESS);
-
-	if (!acl)
-		return 0;
-
-	acl_entry_t entry;
-	int entryid, num = 0;
-
-	for (entryid = ACL_FIRST_ENTRY;; entryid = ACL_NEXT_ENTRY) {
-		if (acl_get_entry(acl, entryid, &entry) != 1)
-			break;
-		num++;
-	}
-
-	acl_free(acl);
-
-	/* If num > 3 we have something else besides owner, group, and others,
-	 * that is, we have at least one ACL property */
-	return (num > 3 ? 1 : 0);
-#endif /* !_ACL_OK */
-}
-
 /* Check whether a given string contains only digits. Returns 1 if true
- * and 0 if false. It does not work with negative numbers */
+ * and 0 if false. It does not work with negative numbers. */
 int
 is_number(const char *restrict str)
 {
@@ -563,8 +515,8 @@ is_number(const char *restrict str)
 
 /* Check if command STR contains a digit and this digit is not the first
  * char of STR. Used by find_cmd() (called by is_internal_c()) to check
- * for fused parameters in internal commands
- * Returns the index of the digit in STR or -1 if no digit is found */
+ * for fused parameters in internal commands.
+ * Returns the index of the digit in STR or -1 if no digit is found. */
 static int
 contains_digit(const char *str)
 {
@@ -625,8 +577,8 @@ is_internal_c(char *restrict cmd)
 	return 0;
 }
 
-/* Check whether S is an action name
- * Returns 1 if true or 0 otherwise */
+/* Check whether S is an action name.
+ * Returns 1 if true or 0 otherwise. */
 int
 is_action_name(const char *s)
 {
@@ -645,11 +597,11 @@ is_action_name(const char *s)
 /* Check cmd against a list of internal commands. Used by parse_input_str()
  * to know whether it should perform additional expansions, like glob, regex,
  * tilde, and so on. Only internal commands dealing with ELN/filenames
- * should be checked here */
+ * should be checked here. */
 int
 is_internal(char *restrict cmd)
 {
-	const struct cmdslist_t int_cmds[] = {
+	static const struct cmdslist_t int_cmds[] = {
 		{"ac", 2},
 		{"ad", 2},
 		{"bb", 2},
@@ -715,20 +667,20 @@ is_internal(char *restrict cmd)
 }
 
 /* Check CMD against a list of internal commands taking ELN's or numbers
- * as parameters. Used by split_fusedcmd() */
+ * as parameters. Used by split_fusedcmd(). */
 int
 is_internal_f(const char *restrict cmd)
 {
 	/* If we are completing/suggesting, do not take 'ws', 'mf', and 'st'
 	 * commands into account: they do not take ELN/filenames as parameters,
 	 * but just numbers, in which case no ELN-filename completion should
-	 * be made */
+	 * be made. */
 	if (flags & STATE_COMPLETING
 	&& (*cmd == 'w' || (*cmd == 'm' && *(cmd + 1) == 'f')
 	|| (*cmd == 's' && (*(cmd + 1) == 't' || *(cmd + 1) == 'o')) ) )
 		return 0;
 
-	const struct cmdslist_t int_cmds[] = {
+	static const struct cmdslist_t int_cmds[] = {
 		{"ac", 2},
 		{"ad", 2},
 		{"alias", 5}, /* 'alias import' takes file names */
@@ -812,7 +764,7 @@ is_internal_f(const char *restrict cmd)
 	return 0;
 }
 
-/* Return one if STR is a command in PATH or zero if not */
+/* Return one if STR is a command in PATH or zero if not. */
 int
 is_bin_cmd(char *str)
 {
@@ -866,10 +818,8 @@ check_regex(char *str)
 	}
 
 	/* And if STR is not a file name, take it as a possible regex */
-	if (char_found == 1) {
-		if (access(str, F_OK) == -1)
-			return EXIT_SUCCESS;
-	}
+	if (char_found == 1 && access(str, F_OK) == -1)
+		return EXIT_SUCCESS;
 
 	return EXIT_FAILURE;
 }
@@ -945,11 +895,11 @@ truncate_file(char *file, const int max, const int check_dups)
 	FILE *fp = (FILE *)NULL;
 	struct stat attr;
 
-	int fd;
+	int fd = 0;
 	if (stat(file, &attr) == -1) {
 		fp = open_fwrite(file, &fd);
 		if (!fp) {
-			_err(0, NOPRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME, file,
+			err(0, NOPRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME, file,
 				strerror(errno));
 		} else {
 			fclose(fp);
@@ -961,7 +911,7 @@ truncate_file(char *file, const int max, const int check_dups)
 	/* Once we know the files exists, keep only MAX entries. */
 	fp = open_fread(file, &fd);
 	if (!fp) {
-		_err(0, NOPRINT_PROMPT, "%s: log: %s: %s\n", PROGRAM_NAME,
+		err(0, NOPRINT_PROMPT, "%s: log: %s: %s\n", PROGRAM_NAME,
 		    file, strerror(errno));
 		return;
 	}
@@ -998,16 +948,16 @@ truncate_file(char *file, const int max, const int check_dups)
 		return;
 	}
 
-#if defined(__HAIKU__) || defined(__sun)
+#ifndef HAVE_DPRINTF
 	FILE *fpp = fopen(tmp, "w");
 	if (!fpp) {
-		_err('e', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME, tmp,
+		err('e', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME, tmp,
 			strerror(errno));
 		fclose(fp);
 		free(tmp);
 		return;
 	}
-#endif /* __HAIKU__ || __sun */
+#endif /* !HAVE_DPRINTF */
 
 	int i = 1;
 	size_t line_size = 0;
@@ -1024,11 +974,11 @@ truncate_file(char *file, const int max, const int check_dups)
 
 		/* Delete old entries = copy only new ones */
 		if (i++ >= n - (max - 1))
-#if !defined(__HAIKU__) && !defined(__sun)
+#ifdef HAVE_DPRINTF
 			dprintf(fdd, "%s", line);
 #else
 			fprintf(fpp, "%s", line);
-#endif /* !__HAIKU__ && !__sun */
+#endif /* HAVE_DPRINTF */
 
 		if (check_dups == 1) {
 			free(prev_line);
@@ -1039,9 +989,9 @@ truncate_file(char *file, const int max, const int check_dups)
 
 	free(prev_line);
 
-#if defined(__HAIKU__) || defined(__sun)
+#ifndef HAVE_DPRINTF
 	fclose(fpp);
-#endif /* __HAIKU__ || __sun */
+#endif /* !HAVE_DPRINTF */
 
 	free(line);
 	unlinkat(fd, file, 0);
