@@ -28,9 +28,11 @@
 #ifdef __linux__
 # include <sys/capability.h>
 #endif /* __linux__ */
-#include <dirent.h>
 #include <errno.h>
 #include <string.h>
+#ifndef CLIFM_SUCKLESS
+# include <strings.h> /* str(n)casecmp() */
+#endif /* CLIFM_SUCKLESS */
 
 /* Only used to check the readline version */
 #ifdef __OpenBSD__
@@ -428,39 +430,46 @@ get_regfile_color(const char *filename, const struct stat *attr, int *is_ext)
 	if (!ext)
 		return color ? color : fi_c;
 
-	char *extcolor = get_ext_color(ext);
-	ext = (char *)NULL;
+	size_t color_len = 0;
+	char *extcolor = get_ext_color(ext, &color_len);
 	if (!extcolor)
 		return color ? color : fi_c;
 
-	snprintf(tmp_color, sizeof(tmp_color), "\x1b[%sm", extcolor); /* NOLINT */
+	if (color_len + 4 > sizeof(tmp_color)) {
+		*tmp_color = '\0';
+	} else {
+		*tmp_color = '\x1b'; tmp_color[1] = '[';
+		memcpy(tmp_color + 2, extcolor, color_len);
+		tmp_color[color_len + 2] = 'm';
+		tmp_color[color_len + 3] = '\0';
+	}
+
 	color = tmp_color;
-	extcolor = (char *)NULL;
 	*is_ext = 1;
 
 	return color;
 }
 
-/* Retrieve the color corresponding to dir FILENAME with mode MODE
+/* Retrieve the color corresponding to dir FILENAME with mode MODE.
  * If LINKS > 2, we know the directory is populated, so that there's no need
  * to run count_dir(). If COUNT > -1, we already know whether the
  * directory is populatoed or not: use this value for FILES_DIR (do not run
  * count_dir() either). */
 char *
 get_dir_color(const char *filename, const mode_t mode, const nlink_t links,
-	const int count)
+	const filesn_t count)
 {
 	char *color = (char *)NULL;
-	int sticky = 0;
-	int is_oth_w = 0;
+	uint8_t sticky = 0;
+	uint8_t is_oth_w = 0;
 	if (mode & S_ISVTX)
 		sticky = 1;
 
 	if (mode & S_IWOTH)
 		is_oth_w = 1;
 
-	size_t files_dir = count > -1 ? (size_t)count : (links > 2
-		? (size_t)links : (size_t)count_dir(filename, CPOP));
+	filesn_t files_dir = count > -1 ? count : (links > 2
+		? (filesn_t)links : count_dir(filename, CPOP));
 
 	color = sticky ? (is_oth_w ? tw_c : st_c) : is_oth_w ? ow_c
 		: ((files_dir == 2 || files_dir == 0) ? ed_c : di_c);
@@ -473,26 +482,26 @@ get_file_color(const char *filename, const struct stat *attr)
 {
 	char *color = (char *)NULL;
 
-#ifdef _LINUX_CAP
+#ifdef LINUX_FILE_CAPS
 	cap_t cap;
 #else
 	UNUSED(filename);
-#endif /* _LINUX_CAP */
+#endif /* LINUX_FILE_CAPS */
 	if (attr->st_mode & 04000) { /* SUID */
 		color = su_c;
 	} else if (attr->st_mode & 02000) { /* SGID */
 		color = sg_c;
 	}
-#ifdef _LINUX_CAP
+#ifdef LINUX_FILE_CAPS
 	else if (check_cap && (cap = cap_get_file(filename))) {
 		color = ca_c;
 		cap_free(cap);
 	}
-#endif /* _LINUX_CAP */
+#endif /* LINUX_FILE_CAPS */
 	else if ((attr->st_mode & 00100) /* Exec */
 	|| (attr->st_mode & 00010) || (attr->st_mode & 00001)) {
-		color = FILE_SIZE_PTR == 0 ? ee_c : ex_c;
-	} else if (FILE_SIZE_PTR == 0) {
+		color = FILE_SIZE_PTR(attr) == 0 ? ee_c : ex_c;
+	} else if (FILE_SIZE_PTR(attr) == 0) {
 		color = ef_c;
 	} else if (attr->st_nlink > 1) { /* Multi-hardlink */
 		color = mh_c;
@@ -595,7 +604,7 @@ update_warning_prompt_text_color(void)
 		snprintf(wp_c, sizeof(wp_c), "\x1b[%sm", start);
 
 		if (rl_readline_version < 0x0700) {
-			_err('w', PRINT_PROMPT, _("%s: Escape sequence detected in the "
+			err('w', PRINT_PROMPT, _("%s: Escape sequence detected in the "
 				"warning prompt string: this might cause a few glichtes in the "
 				"prompt due to some bugs in the current readline library (%s). "
 				"Please consider removing these escape sequences (via either "
@@ -623,9 +632,10 @@ check_names(const char *str)
 	}
 
 	int found = -1;
+	char up_str = TOUPPER(*str);
 	size_t i;
 	for (i = 0; color_names[i].name; i++) {
-		if (TOUPPER(*str) == TOUPPER(*color_names[i].name)
+		if (up_str == *color_names[i].name /* All names start with upper case */
 		&& strcasecmp(str + 1, color_names[i].name + 1) == 0) {
 			found = (int)i;
 			break;
@@ -657,6 +667,7 @@ check_defs(const char *str)
 		if (!defs[i].name || !defs[i].value || !*defs[i].name
 		|| !*defs[i].value)
 			continue;
+
 		if (*defs[i].name == *str && strcmp(defs[i].name, str) == 0
 		&& is_color_code(defs[i].value) == 1) {
 			val = defs[i].value;
@@ -691,7 +702,7 @@ END:
 #endif /* !CLIFM_SUCKLESS */
 
 /* Look for the hash HASH in the hash table.
- * Return a pointer to the corresponding color if found or NULL
+ * Return a pointer to the corresponding color if found or NULL.
  *
  * NOTE: We check all available hashes to avoid conflicts.
  * The hash table is built on user supplied data (color scheme file)
@@ -699,7 +710,7 @@ END:
  * However, at least with the color schemes provided by default, we do know
  * there are no conflicts. In this latter case we can return the first matching
  * entry, which is a performace improvement. But we cannot assume the user
- * is using one of the provided themes and that it has been not modified. */
+ * is using one of the provided themes and that it hasn't been modified. */
 /*
 static char *
 check_ext_hash(const size_t hash, size_t *count)
@@ -721,8 +732,11 @@ check_ext_hash(const size_t hash, size_t *count)
 	return p;
 } */
 
+
+/* Return the color code associated to the file extension EXT, updating
+ * VAL_LEN, if not NULL, to the length of this color code. */
 static char *
-check_ext_string(const char *ext)
+check_ext_string(const char *ext, size_t *val_len)
 {
 	/* Hold extension names. NAME_MAX should be enough: no file name should
 	 * go beyond NAME_MAX, so it's pretty safe to assume that no file extension
@@ -733,7 +747,7 @@ check_ext_string(const char *ext)
 	int i;
 	for (i = 0; ext[i] && i < NAME_MAX; i++) {
 		if (ext[i] >= 'A' && ext[i] <= 'Z')
-			tmp_ext[i] = ext[i] + ' '; // tolower
+			tmp_ext[i] = ext[i] + ' '; /* Tolower */
 		else
 			tmp_ext[i] = ext[i];
 	}
@@ -762,6 +776,9 @@ check_ext_string(const char *ext)
 		if (match == 0 || *q != '\0')
 			continue;
 
+		if (val_len)
+			*val_len = ext_colors[i].value_len;
+
 		return ext_colors[i].value;
 	}
 
@@ -769,11 +786,11 @@ check_ext_string(const char *ext)
 }
 
 /* Returns a pointer to the corresponding color code for the file
- * extension EXT.
+ * extension EXT (updating VAL_LEN to the length of this code).
  * The hash table is checked first, and, in case of a conflict, a regular
  * string comparison is performed to resolve it. */
 char *
-get_ext_color(const char *ext)
+get_ext_color(const char *ext, size_t *val_len)
 {
 	if (!ext || !*ext || !*(++ext) || ext_colors_n == 0)
 		return (char *)NULL;
@@ -787,8 +804,8 @@ get_ext_color(const char *ext)
 		return ret; */
 
 	/* We have a conflict: a single hash is used for two or more extensions.
-	 * Let's try to match the extension name itself */
-	return check_ext_string(ext);
+	 * Let's try to match the extension name itself. */
+	return check_ext_string(ext, val_len);
 }
 
 #ifndef CLIFM_SUCKLESS
@@ -843,6 +860,7 @@ void
 reset_filetype_colors(void)
 {
 	*bd_c = '\0';
+	*bk_c = '\0';
 	*ca_c = '\0';
 	*cd_c = '\0';
 	*di_c = '\0';
@@ -856,9 +874,9 @@ reset_filetype_colors(void)
 	*nd_c = '\0';
 	*nf_c = '\0';
 	*no_c = '\0';
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 	*oo_c = '\0';
-#endif /* __sun */
+#endif /* SOLARIS_DOORS */
 	*or_c = '\0';
 	*ow_c = '\0';
 	*pi_c = '\0';
@@ -883,6 +901,7 @@ reset_iface_colors(void)
 	*hr_c = '\0';
 	*hs_c = '\0';
 	*hv_c = '\0';
+	*hw_c = '\0';
 
 	*sb_c = '\0';
 	*sc_c = '\0';
@@ -903,6 +922,7 @@ reset_iface_colors(void)
 	*li_cb = '\0';
 	*mi_c = '\0';
 	*nm_c = '\0';
+	*ro_c = '\0';
 	*si_c = '\0';
 	*ti_c = '\0';
 	*tt_c = '\0';
@@ -935,7 +955,7 @@ reset_iface_colors(void)
 	*dz_c = '\0';
 }
 
-/* Import the color scheme NAME from DATADIR (usually /usr/local/share)
+/* Import the color scheme NAME from DATADIR (usually /usr/local/share).
  * Return zero on success or one on failure. */
 int
 import_color_scheme(const char *name)
@@ -952,7 +972,7 @@ import_color_scheme(const char *name)
 	if (stat(dfile, &attr) == -1)
 		return EXIT_FAILURE;
 
-	char *cmd[] = {"cp", dfile, colors_dir, NULL};
+	char *cmd[] = {"cp", "--", dfile, colors_dir, NULL};
 	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) == EXIT_SUCCESS)
 		return EXIT_SUCCESS;
 
@@ -980,7 +1000,7 @@ list_colorschemes(void)
 }
 
 /* Edit the current color scheme file.
- * If the file is not in the local colors dir, try to copy it from DATADIR
+ * If the file is not in the local colors dir, try to copy it from DATADIR.
  * into the local dir to avoid permission issues. */
 static int
 edit_colorscheme(char *app)
@@ -1207,10 +1227,10 @@ set_filetype_colors(char **colors, const size_t words)
 				set_color(colors[i] + 3, or_c, RL_PRINTABLE);
 			else if (colors[i][1] == 'w')
 				set_color(colors[i] + 3, ow_c, RL_PRINTABLE);
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 			else if (colors[i][1] == 'o') /* oo = Door */
 				set_color(colors[i] + 3, oo_c, RL_PRINTABLE);
-#endif /* __sun */
+#endif /* SOLARIS_DOORS */
 		}
 
 		else if (*colors[i] == 'p' && colors[i][1] == 'i')
@@ -1308,6 +1328,8 @@ set_iface_colors(char **colors, const size_t words)
 				set_color(colors[i] + 3, hs_c, RL_PRINTABLE);
 			else if (colors[i][1] == 'v' && colors[i][2] == '=')
 				set_color(colors[i] + 3, hv_c, RL_PRINTABLE);
+			else if (colors[i][1] == 'w' && colors[i][2] == '=')
+				set_color(colors[i] + 3, hw_c, RL_PRINTABLE);
 		}
 
 		else if (*colors[i] == 'l') {
@@ -1327,6 +1349,11 @@ set_iface_colors(char **colors, const size_t words)
 		&& colors[i][2] == '=')
 			set_color(colors[i] + 3, nm_c, RL_NO_PRINTABLE);
 
+		else if (*colors[i] == 'r') {
+			if (colors[i][1] == 'o' && colors[i][2] == '=')
+				set_color(colors[i] + 3, ro_c, RL_NO_PRINTABLE);
+		}
+
 		else if (*colors[i] == 's') {
 			if (colors[i][1] == 'b' && colors[i][2] == '=')
 				set_color(colors[i] + 3, sb_c, RL_PRINTABLE);
@@ -1534,6 +1561,7 @@ store_extension_line(const char *line)
 	size_t elen = strlen(code) + 3;
 	ext_colors[ext_colors_n].value = (char *)xnmalloc(elen, sizeof(char));
 	snprintf(ext_colors[ext_colors_n].value, elen, "0;%s", code);
+	ext_colors[ext_colors_n].value_len = elen - 1;
 //	ext_color[ext_colors_n].hash = hashme(line, 0);
 
 	if (xargs.no_bold == 1)
@@ -1607,9 +1635,128 @@ split_extension_colors(char *extcolors)
 			(ext_colors_n + 1) * sizeof(struct ext_t));
 		ext_colors[ext_colors_n].name = (char *)NULL;
 		ext_colors[ext_colors_n].value = (char *)NULL;
+		ext_colors[ext_colors_n].len = 0;
+		ext_colors[ext_colors_n].value_len = 0;
 	}
 }
 
+static void
+set_default_colors_256(void)
+{
+	if (!ext_colors)
+		split_extension_colors(DEF_EXT_COLORS_256);
+
+	/* If a definition for TEMP exists in the color scheme file, BK_C should
+	 * have been set to this color in store_defintions(). If not, set it here
+	 * to the default file color (fi_c). */
+	if (!*bk_c) xstrsncpy(bk_c, DEF_FI_C256, sizeof(bk_c));
+
+	if (!*hb_c) xstrsncpy(hb_c, DEF_HB_C256, sizeof(hb_c));
+	if (!*hc_c) xstrsncpy(hc_c, DEF_HC_C256, sizeof(hc_c));
+	if (!*hd_c) xstrsncpy(hd_c, DEF_HD_C256, sizeof(hd_c));
+	if (!*he_c) xstrsncpy(he_c, DEF_HE_C256, sizeof(he_c));
+	if (!*hn_c) xstrsncpy(hn_c, DEF_HN_C256, sizeof(hn_c));
+	if (!*hp_c) xstrsncpy(hp_c, DEF_HP_C256, sizeof(hp_c));
+	if (!*hq_c) xstrsncpy(hq_c, DEF_HQ_C256, sizeof(hq_c));
+	if (!*hr_c) xstrsncpy(hr_c, DEF_HR_C256, sizeof(hr_c));
+	if (!*hs_c) xstrsncpy(hs_c, DEF_HS_C256, sizeof(hs_c));
+	if (!*hv_c) xstrsncpy(hv_c, DEF_HV_C256, sizeof(hv_c));
+	if (!*hw_c) xstrsncpy(hw_c, DEF_HW_C256, sizeof(hw_c));
+
+	if (!*tt_c) xstrsncpy(tt_c, DEF_TT_C256, sizeof(tt_c));
+
+	if (!*sb_c) xstrsncpy(sb_c, DEF_SB_C256, sizeof(sb_c));
+	if (!*sc_c) xstrsncpy(sc_c, DEF_SC_C256, sizeof(sc_c));
+	if (!*sd_c) xstrsncpy(sd_c, DEF_SD_C256, sizeof(sd_c));
+	if (!*sh_c) xstrsncpy(sh_c, DEF_SH_C256, sizeof(sh_c));
+	if (!*sf_c) xstrsncpy(sf_c, DEF_SF_C256, sizeof(sf_c));
+	if (!*sx_c) xstrsncpy(sx_c, DEF_SX_C256, sizeof(sx_c));
+	if (!*sp_c) xstrsncpy(sp_c, DEF_SP_C256, sizeof(sp_c));
+	if (!*sz_c) xstrsncpy(sz_c, DEF_SZ_C256, sizeof(sz_c));
+
+	if (!*el_c) xstrsncpy(el_c, DEF_EL_C256, sizeof(el_c));
+	if (!*mi_c) xstrsncpy(mi_c, DEF_MI_C256, sizeof(mi_c));
+	/* If unset from the config file, use current workspace color */
+	if (!*dl_c && config_ok == 0)
+		xstrsncpy(dl_c, DEF_DL_C256, sizeof(dl_c));
+
+	if (!*df_c) xstrsncpy(df_c, DEF_DF_C256, sizeof(df_c));
+	if (!*fc_c) xstrsncpy(fc_c, DEF_FC_C256, sizeof(fc_c));
+	if (!*wc_c) xstrsncpy(wc_c, DEF_WC_C256, sizeof(wc_c));
+	if (!*tx_c) xstrsncpy(tx_c, DEF_TX_C256, sizeof(tx_c));
+
+	if (!*lc_c)
+		xstrsncpy(lc_c, DEF_LC_C256, sizeof(lc_c));
+
+	if (!*li_c) xstrsncpy(li_c, DEF_LI_C256, sizeof(li_c));
+	if (!*li_cb) xstrsncpy(li_cb, DEF_LI_CB256, sizeof(li_cb));
+	if (!*ti_c) xstrsncpy(ti_c, DEF_TI_C256, sizeof(ti_c));
+	if (!*em_c) xstrsncpy(em_c, DEF_EM_C256, sizeof(em_c));
+	if (!*wm_c) xstrsncpy(wm_c, DEF_WM_C256, sizeof(wm_c));
+	if (!*nm_c) xstrsncpy(nm_c, DEF_NM_C256, sizeof(nm_c));
+	if (!*ro_c) xstrsncpy(ro_c, DEF_RO_C256, sizeof(ro_c));
+	if (!*si_c) xstrsncpy(si_c, DEF_SI_C256, sizeof(si_c));
+	if (!*bm_c) xstrsncpy(bm_c, DEF_BM_C256, sizeof(bm_c));
+	if (!*ts_c) xstrsncpy(ts_c, DEF_TS_C256, sizeof(ts_c));
+	if (!*wp_c) xstrsncpy(wp_c, DEF_WP_C256, sizeof(wp_c));
+	if (!*xs_c) xstrsncpy(xs_c, DEF_XS_C256, sizeof(xs_c));
+	if (!*xf_c) xstrsncpy(xf_c, DEF_XF_C256, sizeof(xf_c));
+
+	if (!*ws1_c) xstrsncpy(ws1_c, DEF_WS1_C256, sizeof(ws1_c));
+	if (!*ws2_c) xstrsncpy(ws2_c, DEF_WS2_C256, sizeof(ws2_c));
+	if (!*ws3_c) xstrsncpy(ws3_c, DEF_WS3_C256, sizeof(ws3_c));
+	if (!*ws4_c) xstrsncpy(ws4_c, DEF_WS4_C256, sizeof(ws4_c));
+	if (!*ws5_c) xstrsncpy(ws5_c, DEF_WS5_C256, sizeof(ws5_c));
+	if (!*ws6_c) xstrsncpy(ws6_c, DEF_WS6_C256, sizeof(ws6_c));
+	if (!*ws7_c) xstrsncpy(ws7_c, DEF_WS7_C256, sizeof(ws7_c));
+	if (!*ws8_c) xstrsncpy(ws8_c, DEF_WS8_C256, sizeof(ws8_c));
+
+	if (!*di_c) xstrsncpy(di_c, DEF_DI_C256, sizeof(di_c));
+	if (!*nd_c) xstrsncpy(nd_c, DEF_ND_C256, sizeof(nd_c));
+	if (!*ed_c) xstrsncpy(ed_c, DEF_ED_C256, sizeof(ed_c));
+	if (!*fi_c) xstrsncpy(fi_c, DEF_FI_C256, sizeof(fi_c));
+	if (!*ef_c) xstrsncpy(ef_c, DEF_EF_C256, sizeof(ef_c));
+	if (!*nf_c) xstrsncpy(nf_c, DEF_NF_C256, sizeof(nf_c));
+	if (!*ln_c) xstrsncpy(ln_c, DEF_LN_C256, sizeof(ln_c));
+	if (!*or_c) xstrsncpy(or_c, DEF_OR_C256, sizeof(or_c));
+	if (!*pi_c) xstrsncpy(pi_c, DEF_PI_C256, sizeof(pi_c));
+	if (!*so_c) xstrsncpy(so_c, DEF_SO_C256, sizeof(so_c));
+	if (!*bd_c) xstrsncpy(bd_c, DEF_BD_C256, sizeof(bd_c));
+	if (!*cd_c) xstrsncpy(cd_c, DEF_CD_C256, sizeof(cd_c));
+	if (!*su_c) xstrsncpy(su_c, DEF_SU_C256, sizeof(su_c));
+	if (!*sg_c) xstrsncpy(sg_c, DEF_SG_C256, sizeof(sg_c));
+	if (!*st_c) xstrsncpy(st_c, DEF_ST_C256, sizeof(st_c));
+	if (!*tw_c) xstrsncpy(tw_c, DEF_TW_C256, sizeof(tw_c));
+	if (!*ow_c) xstrsncpy(ow_c, DEF_OW_C256, sizeof(ow_c));
+	if (!*ex_c) xstrsncpy(ex_c, DEF_EX_C256, sizeof(ex_c));
+	if (!*ee_c) xstrsncpy(ee_c, DEF_EE_C256, sizeof(ee_c));
+	if (!*ca_c) xstrsncpy(ca_c, DEF_CA_C256, sizeof(ca_c));
+	if (!*no_c) xstrsncpy(no_c, DEF_NO_C256, sizeof(no_c));
+	if (!*uf_c) xstrsncpy(uf_c, DEF_UF_C256, sizeof(uf_c));
+	if (!*mh_c) xstrsncpy(mh_c, DEF_MH_C256, sizeof(mh_c));
+#ifdef SOLARIS_DOORS
+	if (!*oo_c) xstrsncpy(oo_c, DEF_OO_C256, sizeof(oo_c));
+#endif /* SOLARIS_DOORS */
+
+#ifndef _NO_ICONS
+	if (!*dir_ico_c)
+		xstrsncpy(dir_ico_c, DEF_DIR_ICO_C256, sizeof(dir_ico_c));
+#endif /* !_NO_ICONS */
+
+	if (!*dr_c) xstrsncpy(dr_c, DEF_DR_C256, sizeof(dr_c));
+	if (!*dw_c) xstrsncpy(dw_c, DEF_DW_C256, sizeof(dw_c));
+	if (!*dxd_c) xstrsncpy(dxd_c, DEF_DXD_C256, sizeof(dxd_c));
+	if (!*dxr_c) xstrsncpy(dxr_c, DEF_DXR_C256, sizeof(dxr_c));
+	if (!*dg_c) xstrsncpy(dg_c, DEF_DG_C256, sizeof(dg_c));
+/*	if (!*dd_c) // Date color unset: let's use shades
+		xstrsncpy(dd_c, DEF_DD_C, sizeof(dd_c));
+	if (!*dz_c) // Size color unset: let's use shades
+		xstrsncpy(dz_c, DEF_DZ_C, sizeof(dz_c)); */
+	if (!*do_c) xstrsncpy(do_c, DEF_DO_C256, sizeof(do_c));
+	if (!*dp_c) xstrsncpy(dp_c, DEF_DP_C256, sizeof(dp_c));
+	if (!*dn_c) xstrsncpy(dn_c, DEF_DN_C256, sizeof(dn_c));
+}
+
 void
 set_default_colors(void)
 {
@@ -1619,9 +1766,19 @@ set_default_colors(void)
 	if (date_shades.type == SHADE_TYPE_UNSET)
 		set_default_date_shades();
 
+	if (term_caps.color >= 256) {
+		set_default_colors_256();
+		return;
+	}
+
 	if (!ext_colors)
 		split_extension_colors(DEF_EXT_COLORS);
 
+	/* If a definition for TEMP exists in the color scheme file, BK_C should
+	 * have been set to this color in store_defintions(). If not, set it here
+	 * to the default file color (fi_c). */
+	if (!*bk_c) xstrsncpy(bk_c, DEF_FI_C, sizeof(bk_c));
+
 	if (!*hb_c) xstrsncpy(hb_c, DEF_HB_C, sizeof(hb_c));
 	if (!*hc_c) xstrsncpy(hc_c, DEF_HC_C, sizeof(hc_c));
 	if (!*hd_c) xstrsncpy(hd_c, DEF_HD_C, sizeof(hd_c));
@@ -1632,6 +1789,8 @@ set_default_colors(void)
 	if (!*hr_c) xstrsncpy(hr_c, DEF_HR_C, sizeof(hr_c));
 	if (!*hs_c) xstrsncpy(hs_c, DEF_HS_C, sizeof(hs_c));
 	if (!*hv_c) xstrsncpy(hv_c, DEF_HV_C, sizeof(hv_c));
+	if (!*hw_c) xstrsncpy(hw_c, DEF_HW_C, sizeof(hw_c));
+
 	if (!*tt_c) xstrsncpy(tt_c, DEF_TT_C, sizeof(tt_c));
 
 	if (!*sb_c) xstrsncpy(sb_c, DEF_SB_C, sizeof(sb_c));
@@ -1654,9 +1813,7 @@ set_default_colors(void)
 	if (!*wc_c) xstrsncpy(wc_c, DEF_WC_C, sizeof(wc_c));
 	if (!*tx_c) xstrsncpy(tx_c, DEF_TX_C, sizeof(tx_c));
 
-	if (!*lc_c)
-		xstrsncpy(lc_c, term_caps.color >= 256
-			? DEF_LC_C_256 : DEF_LC_C, sizeof(lc_c));
+	if (!*lc_c) xstrsncpy(lc_c, DEF_LC_C, sizeof(lc_c));
 
 	if (!*li_c) xstrsncpy(li_c, DEF_LI_C, sizeof(li_c));
 	if (!*li_cb) xstrsncpy(li_cb, DEF_LI_CB, sizeof(li_cb));
@@ -1664,6 +1821,7 @@ set_default_colors(void)
 	if (!*em_c) xstrsncpy(em_c, DEF_EM_C, sizeof(em_c));
 	if (!*wm_c) xstrsncpy(wm_c, DEF_WM_C, sizeof(wm_c));
 	if (!*nm_c) xstrsncpy(nm_c, DEF_NM_C, sizeof(nm_c));
+	if (!*ro_c) xstrsncpy(ro_c, DEF_RO_C, sizeof(ro_c));
 	if (!*si_c) xstrsncpy(si_c, DEF_SI_C, sizeof(si_c));
 	if (!*bm_c) xstrsncpy(bm_c, DEF_BM_C, sizeof(bm_c));
 	if (!*ts_c) xstrsncpy(ts_c, DEF_TS_C, sizeof(ts_c));
@@ -1703,33 +1861,26 @@ set_default_colors(void)
 	if (!*no_c) xstrsncpy(no_c, DEF_NO_C, sizeof(no_c));
 	if (!*uf_c) xstrsncpy(uf_c, DEF_UF_C, sizeof(uf_c));
 	if (!*mh_c) xstrsncpy(mh_c, DEF_MH_C, sizeof(mh_c));
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 	if (!*oo_c) xstrsncpy(oo_c, DEF_OO_C, sizeof(oo_c));
-#endif /* __sun */
+#endif /* SOLARIS_DOORS */
 
 #ifndef _NO_ICONS
 	if (!*dir_ico_c)
 		xstrsncpy(dir_ico_c, DEF_DIR_ICO_C, sizeof(dir_ico_c));
 #endif /* !_NO_ICONS */
 
-	if (!*dr_c) xstrsncpy(dr_c, term_caps.color >= 256
-		? DEF_DR_C256 : DEF_DR_C, sizeof(dr_c));
-	if (!*dw_c) xstrsncpy(dw_c, term_caps.color >= 256
-		? DEF_DW_C256 : DEF_DW_C, sizeof(dw_c));
-	if (!*dxd_c) xstrsncpy(dxd_c, term_caps.color >= 256
-		? DEF_DXD_C256 : DEF_DXD_C, sizeof(dxd_c));
-	if (!*dxr_c) xstrsncpy(dxr_c, term_caps.color >= 256
-		? DEF_DXR_C256 : DEF_DXR_C, sizeof(dxr_c));
-	if (!*dg_c) xstrsncpy(dg_c, term_caps.color >= 256
-		? DEF_DG_C256 : DEF_DG_C, sizeof(dg_c));
+	if (!*dr_c) xstrsncpy(dr_c, DEF_DR_C, sizeof(dr_c));
+	if (!*dw_c) xstrsncpy(dw_c, DEF_DW_C, sizeof(dw_c));
+	if (!*dxd_c) xstrsncpy(dxd_c, DEF_DXD_C, sizeof(dxd_c));
+	if (!*dxr_c) xstrsncpy(dxr_c, DEF_DXR_C, sizeof(dxr_c));
+	if (!*dg_c) xstrsncpy(dg_c, DEF_DG_C, sizeof(dg_c));
 /*	if (!*dd_c) // Date color unset: let's use shades
 		xstrsncpy(dd_c, DEF_DD_C, sizeof(dd_c));
 	if (!*dz_c) // Size color unset: let's use shades
 		xstrsncpy(dz_c, DEF_DZ_C, sizeof(dz_c)); */
-	if (!*do_c) xstrsncpy(do_c, term_caps.color >= 256
-		? DEF_DO_C256 : DEF_DO_C, sizeof(do_c));
-	if (!*dp_c) xstrsncpy(dp_c, term_caps.color >= 256
-		? DEF_DP_C256 : DEF_DP_C, sizeof(dp_c));
+	if (!*do_c) xstrsncpy(do_c, DEF_DO_C, sizeof(do_c));
+	if (!*dp_c) xstrsncpy(dp_c, DEF_DP_C, sizeof(dp_c));
 	if (!*dn_c) xstrsncpy(dn_c, DEF_DN_C, sizeof(dn_c));
 }
 
@@ -1753,7 +1904,7 @@ get_cur_colorscheme(const char *colorscheme)
 	}
 
 	if (!cur_cscheme) {
-		_err('w', PRINT_PROMPT, _("%s: colors: %s: No such color scheme. "
+		err('w', PRINT_PROMPT, _("%s: colors: %s: No such color scheme. "
 			"Falling back to default\n"), PROGRAM_NAME, colorscheme);
 
 		if (def_cscheme)
@@ -1828,6 +1979,12 @@ store_definition(char *str)
 	}
 
 	defs[defs_n].value = savestring(value, val_len);
+
+	/* If we find a definition for TEMP, let's use this color for backup files
+	 * (bk_c color code). */
+	if (!*bk_c && *name == 'T' && strcmp(name + 1, "EMP") == 0)
+		snprintf(bk_c, sizeof(bk_c), "\x1b[0;%sm", value);
+
 	defs_n++;
 }
 
@@ -1842,50 +1999,6 @@ init_defs(void)
 	}
 }
 
-#ifndef _NO_FZF
-static void
-set_fzf_opts(char *line)
-{
-	free(conf.fzftab_options);
-
-	if (!line) {
-		char *p = conf.colorize == 1 ? DEF_FZFTAB_OPTIONS
-			: DEF_FZFTAB_OPTIONS_NO_COLOR;
-		conf.fzftab_options = savestring(p, strlen(p));
-	}
-
-	else if (*line == 'n' && strcmp(line, "none") == 0) {
-		conf.fzftab_options = (char *)xnmalloc(1, sizeof(char));
-		*conf.fzftab_options = '\0';
-	}
-
-	else if (sanitize_cmd(line, SNT_BLACKLIST) == EXIT_SUCCESS) {
-		conf.fzftab_options = savestring(line, strlen(line));
-	}
-
-	else {
-		_err('w', PRINT_PROMPT, _("%s: FzfTabOptions contains unsafe "
-			"characters (<>|;&$`). Falling back to default values.\n"),
-			PROGRAM_NAME);
-		if (conf.colorize == 1) {
-			conf.fzftab_options =
-				savestring(DEF_FZFTAB_OPTIONS, strlen(DEF_FZFTAB_OPTIONS));
-		} else {
-			conf.fzftab_options =
-				savestring(DEF_FZFTAB_OPTIONS_NO_COLOR,
-				strlen(DEF_FZFTAB_OPTIONS_NO_COLOR));
-		}
-/*		char *p = conf.colorize == 1 ? DEF_FZFTAB_OPTIONS
-			: DEF_FZFTAB_OPTIONS_NO_COLOR;
-		conf.fzftab_options = savestring(p, strlen(p)); */
-	}
-
-	fzf_height_set = 0;
-	if (strstr(conf.fzftab_options, "--height"))
-		fzf_height_set = 1;
-}
-#endif /* !_NO_FZF */
-
 static void
 set_cs_prompt(char *line)
 {
@@ -1945,6 +2058,51 @@ set_cs_warning_prompt_str(char *line)
 }
 
 #ifndef _NO_FZF
+static void
+set_fzf_opts(char *line)
+{
+	free(conf.fzftab_options);
+
+	if (!line) {
+		char *p = conf.colorize == 1 ? DEF_FZFTAB_OPTIONS
+			: DEF_FZFTAB_OPTIONS_NO_COLOR;
+		conf.fzftab_options = savestring(p, strlen(p));
+	}
+
+	else if (*line == 'n' && strcmp(line, "none") == 0) {
+		conf.fzftab_options = (char *)xnmalloc(1, sizeof(char));
+		*conf.fzftab_options = '\0';
+	}
+
+	else if (sanitize_cmd(line, SNT_BLACKLIST) == EXIT_SUCCESS) {
+		conf.fzftab_options = savestring(line, strlen(line));
+	}
+
+	else {
+		err('w', PRINT_PROMPT, _("%s: FzfTabOptions contains unsafe "
+			"characters (<>|;&$`). Falling back to default values.\n"),
+			PROGRAM_NAME);
+		if (conf.colorize == 1) {
+			conf.fzftab_options =
+				savestring(DEF_FZFTAB_OPTIONS, strlen(DEF_FZFTAB_OPTIONS));
+		} else {
+			conf.fzftab_options =
+				savestring(DEF_FZFTAB_OPTIONS_NO_COLOR,
+				strlen(DEF_FZFTAB_OPTIONS_NO_COLOR));
+		}
+/*		char *p = conf.colorize == 1 ? DEF_FZFTAB_OPTIONS
+			: DEF_FZFTAB_OPTIONS_NO_COLOR;
+		conf.fzftab_options = savestring(p, strlen(p)); */
+	}
+
+	if (strstr(conf.fzftab_options, "--preview "))
+		conf.fzf_preview = FZF_EXTERNAL_PREVIEWER;
+
+	fzf_height_set = 0;
+	if (strstr(conf.fzftab_options, "--height"))
+		fzf_height_set = 1;
+}
+
 static void
 set_cs_fzftabopts(char *line)
 {
@@ -2057,7 +2215,7 @@ read_color_scheme_file(const char *colorscheme, char **filecolors,
 				colorscheme_file, strerror(errno));
 			return EXIT_FAILURE;
 		} else {
-			_err('w', PRINT_PROMPT, _("%s: colors: %s: No such color scheme. "
+			err('w', PRINT_PROMPT, _("%s: colors: %s: No such color scheme. "
 				"Falling back to default\n"), PROGRAM_NAME, colorscheme);
 			return EXIT_SUCCESS;
 		}
@@ -2247,9 +2405,9 @@ disable_bold(void)
 	remove_bold_attr(nd_c);
 	remove_bold_attr(nf_c);
 	remove_bold_attr(no_c);
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 	remove_bold_attr(oo_c);
-#endif /* __sun */
+#endif /* SOLARIS_DOORS */
 	remove_bold_attr(or_c);
 	remove_bold_attr(ow_c);
 	remove_bold_attr(pi_c);
@@ -2298,6 +2456,7 @@ disable_bold(void)
 	remove_bold_attr(hr_c);
 	remove_bold_attr(hs_c);
 	remove_bold_attr(hv_c);
+	remove_bold_attr(hw_c);
 
 	/* File properties */
 	remove_bold_attr(dd_c);
@@ -2326,6 +2485,7 @@ disable_bold(void)
 	remove_bold_attr(li_c);
 	remove_bold_attr(li_cb);
 	remove_bold_attr(nm_c);
+	remove_bold_attr(ro_c);
 	remove_bold_attr(si_c);
 	remove_bold_attr(ti_c);
 	remove_bold_attr(tx_c);
@@ -2486,8 +2646,8 @@ colors_list(char *ent, const int eln, const int pad, const int new_line)
 
 	char *wname = (char *)NULL;
 	size_t wlen = wc_xstrlen(ent);
-	if (wlen == 0) /* Embedded control chars */
-		wname = replace_ctrl_chars(ent);
+	if (wlen == 0) /* Invalid chars found. */
+		wname = replace_invalid_chars(ent);
 
 	char *color = (char *)NULL;
 
@@ -2527,9 +2687,10 @@ colors_list(char *ent, const int eln, const int pad, const int new_line)
 		case S_IFIFO: color = pi_c; break;
 		case S_IFBLK: color = bd_c; break;
 		case S_IFCHR: color = cd_c; break;
-#ifdef __sun
+#ifdef SOLARIS_DOORS
+		case S_IFPORT: /* fallthrough */
 		case S_IFDOOR: color = oo_c; break;
-#endif /* __sun */
+#endif /* SOLARIS_DOORS */
 		case S_IFSOCK: color = so_c; break;
 		default: color = no_c; break;
 		}
@@ -2588,9 +2749,9 @@ get_colorschemes(void)
 	size_t i = 0;
 
 	if (colors_dir && stat(colors_dir, &attr) != -1
-	&& (schemes_total = count_dir(colors_dir, NO_CPOP) - 2) > 0) {
+	&& (schemes_total = (int)count_dir(colors_dir, NO_CPOP) - 2) > 0) {
 		if (!(dir_p = opendir(colors_dir))) {
-			_err('e', PRINT_PROMPT, "opendir: %s: %s\n", colors_dir,
+			err('e', PRINT_PROMPT, "opendir: %s: %s\n", colors_dir,
 				strerror(errno));
 			return 0;
 		}
@@ -2621,13 +2782,13 @@ get_colorschemes(void)
 		goto END;
 
 	int total_tmp = schemes_total;
-	schemes_total += (count_dir(sys_colors_dir, NO_CPOP) - 2);
+	schemes_total += ((int)count_dir(sys_colors_dir, NO_CPOP) - 2);
 
 	if (schemes_total <= total_tmp)
 		goto END;
 
 	if (!(dir_p = opendir(sys_colors_dir))) {
-		_err('e', PRINT_PROMPT, "opendir: %s: %s\n", sys_colors_dir,
+		err('e', PRINT_PROMPT, "opendir: %s: %s\n", sys_colors_dir,
 			strerror(errno));
 		goto END;
 	}
@@ -2702,9 +2863,9 @@ color_codes(void)
 	printf(_(" %sfile name%s: cd: Character special file\n"), cd_c, df_c);
 	printf(_(" %sfile name%s: so: Socket file\n"), so_c, df_c);
 	printf(_(" %sfile name%s: pi: Pipe or FIFO special file\n"), pi_c, df_c);
-#ifdef __sun
-	printf(_(" %sfile name%s: oo: Door file\n"), oo_c, df_c);
-#endif /* __sun */
+#ifdef SOLARIS_DOORS
+	printf(_(" %sfile name%s: oo: Door/Port file\n"), oo_c, df_c);
+#endif /* SOLARIS_DOORS */
 	printf(_(" %sfile name%s: su: SUID file\n"), su_c, df_c);
 	printf(_(" %sfile name%s: sg: SGID file\n"), sg_c, df_c);
 	printf(_(" %sfile name%s: ca: File with capabilities\n"), ca_c, df_c);
