@@ -36,8 +36,7 @@ int  edit_function(char **);
 void init_config(void);
 int  reload_config(void);
 void set_div_line(char *);
-void set_env(void);
-void set_finder_paths(void);
+void set_env(const int);
 void set_sel_file(void);
 
 __END_DECLS
