@@ -24,13 +24,7 @@
 
 #include "helpers.h"
 
-#ifdef __OpenBSD__
-# include <sys/dirent.h>
-#endif /* __OpenBSD__ */
-
-#include <dirent.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/wait.h>
@@ -46,6 +40,7 @@ typedef char *rl_cpvfunc_t;
 #ifdef __TINYC__
 # undef CHAR_MAX /* Silence redefinition error */
 #endif /* __TINYC__ */
+
 #include <limits.h>
 
 #include "actions.h"
@@ -81,7 +76,9 @@ typedef char *rl_cpvfunc_t;
 # include "trash.h"
 #endif /* !_NO_TRASH */
 #include "messages.h"
-#include "media.h"
+#ifndef NO_MEDIA_FUNC
+# include "media.h"
+#endif /* NO_MEDIA_FUNC */
 #ifndef _NO_BLEACH
 # include "name_cleaner.h"
 #endif /* !_NO_BLEACH */
@@ -365,7 +362,7 @@ export_var_function(char **args)
 	size_t i;
 	for (i = 0; args[i]; i++) {
 		/* ARG might have been escaped by parse_input_str(), in the command
-		 * and parameter substitution block. Let's deescape it */
+		 * and parameter substitution block. Let's deescape it. */
 		char *ds = dequote_str(args[i], 0);
 		if (!ds) {
 			xerror("%s\n", _("export: Error dequoting argument"));
@@ -487,7 +484,7 @@ run_shell_cmd(char **args)
 
 /* Free everything and exit the program */
 static void
-_quit(char **args, const int exit_status)
+quit_func(char **args, const int exit_status)
 {
 	if (!args || !args[0])
 		return;
@@ -1021,7 +1018,7 @@ hidden_files_function(const char *arg)
 }
 
 static int
-_toggle_exec(char **args)
+toggle_exec_func(char **args)
 {
 	if (!args[1] || IS_HELP(args[1]))
 		{ puts(_(TE_USAGE)); return EXIT_SUCCESS; }
@@ -1127,7 +1124,7 @@ export_files_function(char **args)
 		return EXIT_SUCCESS;
 	}
 
-	char *ret = _export(args, 1);
+	char *ret = export_files(args, 1);
 	if (ret) {
 		printf(_("Files exported to: %s\n"), ret);
 		free(ret);
@@ -1138,7 +1135,7 @@ export_files_function(char **args)
 }
 
 static int
-_bookmarks_function(char **args)
+bookmarks_func(char **args)
 {
 	if (args[1] && IS_HELP(args[1])) {
 		puts(_(BOOKMARKS_USAGE));
@@ -1218,6 +1215,7 @@ reload_function(void)
 	return exit_status;
 }
 
+#ifndef NO_MEDIA_FUNC
 /* MODE could be either MEDIA_LIST (mp command) or MEDIA_MOUNT (media command) */
 static int
 media_function(char *arg, const int mode)
@@ -1238,9 +1236,10 @@ media_function(char *arg, const int mode)
 
 	return exit_status;
 }
+#endif /* NO_MEDIA_FUNC */
 
 static int
-_cd_function(char *arg)
+chdir_function(char *arg)
 {
 	if (arg && IS_HELP(arg)) {
 		puts(_(CD_USAGE));
@@ -1251,7 +1250,7 @@ _cd_function(char *arg)
 }
 
 static int
-_sort_function(char **args)
+sort_func(char **args)
 {
 	if (args[1] && IS_HELP(args[1])) {
 		puts(_(SORT_USAGE));
@@ -1289,7 +1288,7 @@ check_actions(char **args)
 			// REMOVE ONCE THE DH PLUGIN'S DEPRECATION PERIOD IS OVER
 			if (*usr_actions[i].name == 'd' && usr_actions[i].name[1] == 'h'
 			&& !usr_actions[i].name[2]) {
-				_err('n', PRINT_PROMPT, _("%s: The 'dh' plugin is deprecated. "
+				err('n', PRINT_PROMPT, _("%s: The 'dh' plugin is deprecated. "
 					"Use the built-in 'dh' command instead disabling the "
 					"'dh' plugin ('actions edit'). Once done, run 'dh --help' "
 					"for more information about the new command.\n"),
@@ -1343,7 +1342,7 @@ expand_and_deescape(char **arg, char **deq_str)
 }
 
 static inline int
-_open_file(char **args, const int i)
+open_file_func(char **args, const filesn_t i)
 {
 	if (conf.autocd && (file_info[i].type == DT_DIR || file_info[i].dir == 1))
 		return cd_function(args[0], CD_PRINT_ERROR);
@@ -1362,7 +1361,7 @@ _open_file(char **args, const int i)
  * is no second argument or if second argument is "&"
  * If just 'edit', do not try to open a file named 'edit': always run
  * the 'edit' command */
-static inline int
+static int
 check_auto_first(char **args)
 {
 	if (!args || !args[0] || !*args[0])
@@ -1393,14 +1392,14 @@ check_auto_first(char **args)
 		return EXIT_SUCCESS;
 	}
 
-	int i = (int)files;
+	filesn_t i = files;
 	while (--i >= 0) {
 		if (*tmp != *file_info[i].name || strcmp(tmp, file_info[i].name) != 0)
 			continue;
 
 		free(deq_str);
 		deq_str = (char *)NULL;
-		int ret = _open_file(args, i);
+		int ret = open_file_func(args, i);
 		if (ret != -1)
 			return ret;
 		break;
@@ -1413,7 +1412,7 @@ check_auto_first(char **args)
 	return (-1);
 }
 
-static inline int
+static int
 auto_open_file(char **args, char *tmp)
 {
 	char *cmd[] = {"open", tmp, (args_n >= 1) ? args[1]
@@ -1446,7 +1445,7 @@ autocd_dir(char *tmp)
 /* If there is a second word (parameter, ARGS[1]) and this word starts
  * with a dash, do not take the first word (ARGS[0]) as a file to be
  * opened, but as a command to be executed */
-static inline int
+static int
 check_auto_second(char **args)
 {
 	if (args[1] && *args[1] == '-')
@@ -1547,7 +1546,7 @@ print_stats(void)
 	if (conf.light_mode == 1)
 		puts(_("Running in light mode: Some files statistics are not available\n"));
 
-	printf(_("Total files:                 %zu\n\
+	printf(_("Total files:                 %jd\n\
 Directories:                 %zu\n\
 Regular files:               %zu\n\
 Executable files:            %zu\n\
@@ -1568,21 +1567,31 @@ Sticky files:                %zu\n\
 Unknown file types:          %zu\n\
 Unstatable files:            %zu\n\
 "),
-	files, stats.dir, stats.reg, stats.exec, stats.hidden, stats.suid,
+	(intmax_t)files, stats.dir, stats.reg, stats.exec, stats.hidden, stats.suid,
 	stats.sgid, stats.caps, stats.fifo, stats.socket, stats.block_dev,
 	stats.char_dev, stats.link, stats.broken_link, stats.multi_link,
 	stats.extended, stats.other_writable, stats.sticky, stats.unknown,
 	stats.unstat);
 
-#ifdef __sun
-	printf("Doors:                 %zu\n", stats.door);
-#endif /* __sun */
+#ifndef _BE_POSIX
+# ifdef SOLARIS_DOORS
+	printf(_("Doors:                 %zu\n"), stats.door);
+	printf(_("Ports:                 %zu\n"), stats.port);
+# endif /* SOLARIS_DOORS */
+# ifdef S_ARCH1
+	printf(_("Archive state 1:       %zu\n"), stats.arch1);
+	printf(_("Archive state 2:       %zu\n"), stats.arch2);
+# endif /* S_ARCH1 */
+# ifdef S_IFWHT
+	printf(_("Whiteout:              %zu\n"), stats.whiteout);
+# endif /* S_IFWHT */
+#endif /* _BE_POSIX */
 
 	return EXIT_SUCCESS;
 }
 
 static int
-_trash_function(char **args, int *_cont)
+trash_func(char **args, int *_cont)
 {
 #ifndef _NO_TRASH
 	if (args[1] && IS_HELP(args[1])) {
@@ -1593,7 +1602,7 @@ _trash_function(char **args, int *_cont)
 
 	int exit_status = trash_function(args);
 
-	if (is_sel) { /* If 'tr sel', deselect everything */
+	if (is_sel && sel_n > 0) { /* If 'tr sel', deselect everything */
 		int i = (int)sel_n;
 		while (--i >= 0)
 			free(sel_elements[i].name);
@@ -1612,7 +1621,7 @@ _trash_function(char **args, int *_cont)
 }
 
 static int
-_untrash_function(char **args, int *_cont)
+untrash_func(char **args, int *_cont)
 {
 #ifndef _NO_TRASH
 	if (args[1] && IS_HELP(args[1])) {
@@ -1690,35 +1699,35 @@ set_cp_cmd(char **cmd, const int cp_force)
 
 	switch (conf.cp_cmd) {
 	case CP_ADVCP:
-		dst_len = strlen(_DEF_ADVCP_CMD);
+		dst_len = sizeof(DEFAULT_ADVCP_CMD) - 1;
 		*cmd = (char *)xrealloc(*cmd, (dst_len + 1) * sizeof(char));
-		xstrsncpy(*cmd, _DEF_ADVCP_CMD, dst_len + 1);
+		xstrsncpy(*cmd, DEFAULT_ADVCP_CMD, dst_len + 1);
 		break;
 	case CP_ADVCP_FORCE:
-		dst_len = strlen(_DEF_ADVCP_CMD_FORCE);
+		dst_len = sizeof(DEFAULT_ADVCP_CMD_FORCE) - 1;
 		*cmd = (char *)xrealloc(*cmd, (dst_len + 1) * sizeof(char));
-		xstrsncpy(*cmd, _DEF_ADVCP_CMD_FORCE, dst_len + 1);
+		xstrsncpy(*cmd, DEFAULT_ADVCP_CMD_FORCE, dst_len + 1);
 		break;
 	case CP_WCP:
-		dst_len = strlen(_DEF_WCP_CMD);
+		dst_len = sizeof(DEFAULT_WCP_CMD) - 1;
 		*cmd = (char *)xrealloc(*cmd, (dst_len + 1) * sizeof(char));
-		xstrsncpy(*cmd, _DEF_WCP_CMD, dst_len + 1);
+		xstrsncpy(*cmd, DEFAULT_WCP_CMD, dst_len + 1);
 		break;
 	case CP_RSYNC:
-		dst_len = strlen(_DEF_RSYNC_CMD);
+		dst_len = sizeof(DEFAULT_RSYNC_CMD) - 1;
 		*cmd = (char *)xrealloc(*cmd, (dst_len + 1) * sizeof(char));
-		xstrsncpy(*cmd, _DEF_RSYNC_CMD, dst_len + 1);
+		xstrsncpy(*cmd, DEFAULT_RSYNC_CMD, dst_len + 1);
 		break;
 	case CP_CP_FORCE:
-		dst_len = strlen(_DEF_CP_CMD_FORCE);
+		dst_len = sizeof(DEFAULT_CP_CMD_FORCE) - 1;
 		*cmd = (char *)xrealloc(*cmd, (dst_len + 1) * sizeof(char));
-		xstrsncpy(*cmd, _DEF_CP_CMD_FORCE, dst_len + 1);
+		xstrsncpy(*cmd, DEFAULT_CP_CMD_FORCE, dst_len + 1);
 		break;
 	case CP_CP: /* fallthrough */
 	default:
-		dst_len = strlen(_DEF_CP_CMD);
+		dst_len = sizeof(DEFAULT_CP_CMD) - 1;
 		*cmd = (char *)xrealloc(*cmd, (dst_len + 1) * sizeof(char));
-		xstrsncpy(*cmd, _DEF_CP_CMD, dst_len + 1);
+		xstrsncpy(*cmd, DEFAULT_CP_CMD, dst_len + 1);
 		break;
 	}
 
@@ -1740,25 +1749,25 @@ set_mv_cmd(char **cmd, const int mv_force)
 
 	switch (conf.mv_cmd) {
 	case MV_ADVMV:
-		dst_len = strlen(_DEF_ADVMV_CMD);
+		dst_len = sizeof(DEFAULT_ADVMV_CMD) - 1;
 		*cmd = (char *)xrealloc(*cmd, (dst_len + 1) * sizeof(char));
-		xstrsncpy(*cmd, _DEF_ADVMV_CMD, dst_len + 1);
+		xstrsncpy(*cmd, DEFAULT_ADVMV_CMD, dst_len + 1);
 		break;
 	case MV_ADVMV_FORCE:
-		dst_len = strlen(_DEF_ADVMV_CMD);
+		dst_len = sizeof(DEFAULT_ADVMV_CMD) - 1;
 		*cmd = (char *)xrealloc(*cmd, (dst_len + 1) * sizeof(char));
-		xstrsncpy(*cmd, _DEF_ADVMV_CMD_FORCE, dst_len + 1);
+		xstrsncpy(*cmd, DEFAULT_ADVMV_CMD_FORCE, dst_len + 1);
 		break;
 	case MV_MV_FORCE:
-		dst_len = strlen(_DEF_MV_CMD_FORCE);
+		dst_len = sizeof(DEFAULT_MV_CMD_FORCE) - 1;
 		*cmd = (char *)xrealloc(*cmd, (dst_len + 1) * sizeof(char));
-		xstrsncpy(*cmd, _DEF_MV_CMD_FORCE, dst_len + 1);
+		xstrsncpy(*cmd, DEFAULT_MV_CMD_FORCE, dst_len + 1);
 		break;
 	case MV_MV: /* fallthrough */
 	default:
-		dst_len = strlen(_DEF_MV_CMD);
+		dst_len = sizeof(DEFAULT_MV_CMD) - 1;
 		*cmd = (char *)xrealloc(*cmd, (dst_len + 1) * sizeof(char));
-		xstrsncpy(*cmd, _DEF_MV_CMD, dst_len + 1);
+		xstrsncpy(*cmd, DEFAULT_MV_CMD, dst_len + 1);
 		break;
 	}
 
@@ -1816,6 +1825,7 @@ remove_backslash(char **s)
 	*s = p;
 }
 
+#ifndef _NO_LIRA
 static int
 preview_edit(char *app)
 {
@@ -1863,7 +1873,7 @@ preview_function(char **args)
 	if (tabmode != FZF_TAB) {
 		char *p = get_cmd_path("fzf");
 		if (!p) {
-			_err(0, NOPRINT_PROMPT, "%s: fzf: %s\n", PROGRAM_NAME, NOTFOUND_MSG);
+			err(0, NOPRINT_PROMPT, "%s: fzf: %s\n", PROGRAM_NAME, NOTFOUND_MSG);
 			return EXEC_NOTFOUND; /* 127, as required by exit(1) */
 		}
 		free(p);
@@ -1892,11 +1902,11 @@ preview_function(char **args)
 		putchar('\n');
 		reload_dirlist();
 	}
-#if RL_READLINE_VERSION >= 0x0700
+#if defined(RL_READLINE_VERSION) && RL_READLINE_VERSION >= 0x0700
 	else { /* Only available since readline 7.0 */
 		rl_clear_visible_line();
 	}
-#endif /* READLINE < 7.0 */
+#endif /* READLINE >= 7.0 */
 
 	if (sel_n > seln_bk) {
 		print_reload_msg(_("%zu file(s) selected\n"), sel_n - seln_bk);
@@ -1905,6 +1915,7 @@ preview_function(char **args)
 
 	return EXIT_SUCCESS;
 }
+#endif /* !_NO_LIRA */
 
 static int
 dirhist_function(char *dir)
@@ -1927,7 +1938,7 @@ dirhist_function(char *dir)
 		}
 
 		n--;
-		if (!old_pwd[n] || *old_pwd[n] == _ESC) {
+		if (!old_pwd[n] || *old_pwd[n] == KEY_ESC) {
 			xerror("%s\n", _("dh: Invalid history entry"));
 			return EXIT_FAILURE;
 		}
@@ -2081,16 +2092,101 @@ check_fs_changes(void)
 	|| !workspaces[cur_ws].path)
 		return;
 
-	int cur_files = count_dir(workspaces[cur_ws].path, 0) - 2;
+	filesn_t cur_files = count_dir(workspaces[cur_ws].path, 0) - 2;
 	struct stat a;
 
 	if (curdir_mtime != 0 && stat(workspaces[cur_ws].path, &a) != -1
 	&& curdir_mtime != a.st_mtime
-	&& cur_files >= 0 && files != (size_t)cur_files)
+	&& cur_files >= 0 && files != cur_files)
 		reload_dirlist();
 }
 #endif /* GENERIC_FS_MONITOR */
 
+/* Check the command CMD against a list of commands able to modify the
+ * filesystem. Return 1 if CMD is found in the list or zero otherwise. */
+static int
+is_write_cmd(const char *cmd)
+{
+	static struct cmdslist_t wcmds[] = {
+		/* Internal commands */
+		{"ac", 2},
+		{"ad", 2},
+		{"bl", 2},
+		{"bb", 2},
+		{"bleach", 6},
+		{"br", 2},
+		{"bulk", 4},
+		{"c", 1},
+		{"dup", 3},
+		{"l", 1},
+		{"le", 2},
+		{"m", 1},
+		{"md", 2},
+		{"n", 1},
+		{"new", 3},
+		{"oc", 2},
+		{"paste", 5},
+		{"pc", 2},
+		{"r", 1},
+		{"rr", 2},
+		{"t", 1},
+		{"ta", 2},
+		{"td", 2},
+		{"tm", 2},
+		{"tn", 2},
+		{"tu", 2},
+		{"ty", 2},
+		{"tag", 3},
+		{"te", 2},
+		{"tr", 2},
+		{"trash", 5},
+		{"u", 1},
+		{"undel", 5},
+		{"untrash", 7},
+		/* Shell commands */
+		{"cp", 2},
+		{"rm", 2},
+		{"mv", 2},
+		{"mkdir", 5},
+		{"rmdir", 5},
+		{"ln", 2},
+		{"link", 4},
+		{"unlink", 6},
+#ifdef CHECK_COREUTILS /* Mostly BSD */
+		{"gcp", 3},
+		{"grm", 3},
+		{"gmv", 3},
+		{"gmkdir", 6},
+		{"grmdir", 6},
+		{"gln", 3},
+		{"glink", 5},
+		{"gunlink", 7},
+#endif /* CHECK_COREUTILS */
+		{NULL, 0}
+	};
+
+	if (!cmd || !*cmd)
+		return 0;
+
+	static size_t wcmds_n = 0;
+	if (wcmds_n == 0)
+		wcmds_n = (sizeof(wcmds) / sizeof(wcmds[0])) - 1;
+
+	size_t clen = strlen(cmd);
+	int i = (int)wcmds_n;
+
+	while (--i >= 0) {
+		if (clen == wcmds[i].len && *cmd == *wcmds[i].name
+		&& strcmp(cmd + 1, wcmds[i].name + 1) == 0) {
+			xerror(_("%s: %s: Command not allowed in read-only mode\n"),
+				PROGRAM_NAME, cmd);
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
 /* Take the command entered by the user, already splitted into substrings
  * by parse_input_str(), and call the corresponding function. Return zero
  * in case of success and one in case of error
@@ -2107,6 +2203,12 @@ exec_cmd(char **comm)
 		check_zombies();
 	fputs(df_c, stdout);
 
+	if (conf.readonly == 1 && is_write_cmd(
+	((*comm[0] == 's' && strcmp(comm[0] + 1, "udo") == 0)
+	|| (*comm[0] == 'd' && strcmp(comm[0] + 1, "oas") == 0))
+	? comm[1] : comm[0]) == 1)
+		return EXIT_FAILURE;
+
 	int old_exit_code = exit_code;
 	exit_code = EXIT_SUCCESS;
 
@@ -2122,7 +2224,7 @@ exec_cmd(char **comm)
 	if (check_pinned_file(comm) == EXIT_FAILURE)
 		return (exit_code = EXIT_FAILURE);
 
-	/* User defined actions */
+	/* User defined actions (plugins) */
 	if ((exit_code = check_actions(comm)) != -1)
 		return exit_code;
 
@@ -2132,14 +2234,14 @@ exec_cmd(char **comm)
 		return (exit_code = create_usr_var(comm[0]));
 	}
 
-	/* Check if we need to send input directly to the system shell */
+	/* Check if we need to send input directly to the system shell. */
 	if (*comm[0] == ';' || *comm[0] == ':')
 		if ((exit_code = launch_shell(comm)) != -1)
 			return exit_code;
 
 	/* # AUTOCD & AUTO-OPEN (1) # */
 	/* rl_dispatching is set to 1 if coming from a keybind: we have a
-	 * command, not a file name. So, skip this check */
+	 * command, not a file name. So, skip this check. */
 	if (rl_dispatching == 0 && (exit_code = check_auto_first(comm)) != -1)
 		return exit_code;
 
@@ -2154,7 +2256,7 @@ exec_cmd(char **comm)
 
 	/*          ############### CD ##################     */
 	if (*comm[0] == 'c' && comm[0][1] == 'd' && !comm[0][2])
-		return (exit_code = _cd_function(comm[1]));
+		return (exit_code = chdir_function(comm[1]));
 
 	/*         ############### OPEN ##################     */
 	else if (*comm[0] == 'o' && (!comm[0][1] || strcmp(comm[0], "open") == 0))
@@ -2183,7 +2285,7 @@ exec_cmd(char **comm)
 	/*     ################# BOOKMARKS ##################     */
 	else if (*comm[0] == 'b' && ((comm[0][1] == 'm' && !comm[0][2])
 	|| strcmp(comm[0], "bookmarks") == 0))
-		return (exit_code = _bookmarks_function(comm));
+		return (exit_code = bookmarks_func(comm));
 
 	/*  ############# BACK, FORTH, and DH (dirhist) ##################     */
 	else if (*comm[0] == 'b' && (!comm[0][1] || strcmp(comm[0], "back") == 0))
@@ -2274,7 +2376,7 @@ exec_cmd(char **comm)
 	else if (*comm[0] == 't' && (!comm[0][1] || strcmp(comm[0], "tr") == 0
 	|| strcmp(comm[0], "trash") == 0)) {
 		int _cont = 1;
-		exit_code = _trash_function(comm, &_cont);
+		exit_code = trash_func(comm, &_cont);
 		if (_cont == 0)
 			return exit_code;
 	}
@@ -2282,7 +2384,7 @@ exec_cmd(char **comm)
 	else if (*comm[0] == 'u' && (!comm[0][1] || strcmp(comm[0], "undel") == 0
 	|| strcmp(comm[0], "untrash") == 0)) {
 		int _cont = 1; /* Tells whether to continue or return right here */
-		exit_code = _untrash_function(comm, &_cont);
+		exit_code = untrash_func(comm, &_cont);
 		if (_cont == 0)
 			return exit_code;
 	}
@@ -2337,12 +2439,18 @@ exec_cmd(char **comm)
 		return (exit_code = long_view_function(comm[1]));
 
 	/*    ############# PREVIEW FILES ##################     */
-	else if (*comm[0] == 'v' && strcmp(comm[0], "view") == 0)
+	else if (*comm[0] == 'v' && strcmp(comm[0], "view") == 0) {
+#ifndef _NO_LIRA
 		return (exit_code = preview_function(comm + 1));
+#else
+		fprintf(stderr, "view: %s\n", _(NOT_AVAILABLE));
+		return EXIT_FAILURE;
+#endif /* !_NO_LIRA */
+	}
 
 	/*    ############## TOGGLE EXEC ##################     */
 	else if (*comm[0] == 't' && comm[0][1] == 'e' && !comm[0][2])
-		return (exit_code = _toggle_exec(comm));
+		return (exit_code = toggle_exec_func(comm));
 
 	/*    ########### OWNERSHIP CHANGER ###############     */
 	else if (*comm[0] == 'o' && comm[0][1] == 'c' && !comm[0][2])
@@ -2374,16 +2482,6 @@ exec_cmd(char **comm)
 	&& IS_HELP(comm[1])) )
 		return (exit_code = search_function(comm));
 
-	/*      ############## HISTORY ##################     */
-	else if (*comm[0] == '!' && comm[0][1] != ' ' && comm[0][1] != '\t'
-	&& comm[0][1] != '\n' && comm[0][1] != '=' && comm[0][1] != '(') {
-		if (comm[1] && IS_HELP(comm[1])) {
-			printf("%s\n", HISTEXEC_USAGE);
-			return EXIT_SUCCESS;
-		}
-		exit_code = run_history_cmd(comm[0] + 1);
-	}
-
 	/*    ############### BATCH LINK ##################     */
 	else if (*comm[0] == 'b' && comm[0][1] == 'l' && !comm[0][2])
 		exit_code = batch_link(comm);
@@ -2396,7 +2494,7 @@ exec_cmd(char **comm)
 	/*      ################ SORT ##################     */
 	else if (*comm[0] == 's' && ((comm[0][1] == 't' && !comm[0][2])
 	|| strcmp(comm[0], "sort") == 0))
-		return (exit_code = _sort_function(comm));
+		return (exit_code = sort_func(comm));
 
 	/*    ############ FILE NAMES CLEANER ############## */
 	else if (*comm[0] == 'b' && ((comm[0][1] == 'b' && !comm[0][2])
@@ -2505,12 +2603,24 @@ exec_cmd(char **comm)
 
 	/* #### MOUNTPOINTS #### */
 	else if (*comm[0] == 'm' && ((comm[0][1] == 'p' && !comm[0][2])
-	|| strcmp(comm[0], "mountpoints") == 0))
+	|| strcmp(comm[0], "mountpoints") == 0)) {
+#ifndef NO_MEDIA_FUNC
 		return (exit_code = media_function(comm[1], MEDIA_LIST));
+#else
+		fputs(_("mountpoints: Function not available\n"), stderr);
+		return EXIT_FAILURE;
+#endif /* NO_MEDIA_FUNC */
+	}
 
 	/* #### MEDIA #### */
-	else if (*comm[0] == 'm' && strcmp(comm[0], "media") == 0)
+	else if (*comm[0] == 'm' && strcmp(comm[0], "media") == 0) {
+#ifndef NO_MEDIA_FUNC
 		return (exit_code = media_function(comm[1], MEDIA_MOUNT));
+#else
+		fputs(_("media: Function not available\n"), stderr);
+		return EXIT_FAILURE;
+#endif /* NO_MEDIA_FUNC */
+	}
 
 	/* #### MAX FILES #### */
 	else if (*comm[0] == 'm' && comm[0][1] == 'f' && !comm[0][2])
@@ -2632,17 +2742,17 @@ exec_cmd(char **comm)
 	else if ((*comm[0] == 'q' && (!comm[0][1] || strcmp(comm[0], "quit") == 0))
 	|| (*comm[0] == 'e' && strcmp(comm[0], "exit") == 0)
 	|| (*comm[0] == 'Q' && !comm[0][1]))
-		_quit(comm, old_exit_code);
+		quit_func(comm, old_exit_code);
 
 	else {
 		/* #  AUTOCD & AUTO-OPEN (2) # */
 		if ((exit_code = check_auto_second(comm)) != -1)
 			return exit_code;
 
-		if (bin_name && *bin_name && *comm[0] == *bin_name
+		if (xargs.stealth_mode == 1	&& bin_name && *bin_name && *comm[0] == *bin_name
 		&& strcmp(comm[0], bin_name) == 0) {
-			fprintf(stderr, _("%s: To launch a new instance use the 'x' "
-				"command\n"), PROGRAM_NAME);
+			fprintf(stderr, _("%s: Nested instances are not allowed in "
+				"stealth mode\n"), PROGRAM_NAME);
 			return (exit_code = EXIT_FAILURE);
 		}
 
