@@ -33,8 +33,6 @@
 # undef CHAR_MAX /* Silence redefinition error */
 #endif /* __TINYC__ */
 #include <limits.h>
-#include <fcntl.h>
-#include <dirent.h>
 
 #include "aux.h"
 #include "checks.h"
@@ -42,6 +40,7 @@
 #include "exec.h"
 #include "file_operations.h"
 #include "history.h"
+#include "init.h" /* get_sel_files() */
 #include "listing.h"
 #include "mime.h"
 #include "misc.h"
@@ -69,8 +68,21 @@
 #define UNSAFE_SYS_KEY   5
 #define UNSAFE_CONTROL   6
 #define UNSAFE_META      7
-#define UNSAFE_NOT_PORTABLE 8
-#define UNSAFE_TOO_LONG  9
+#define UNSAFE_TOO_LONG  8
+#ifdef _BE_POSIX
+# define UNSAFE_NOT_PORTABLE 9
+#endif
+
+/* Struct to store information about files to be removed via the 'r' command. */
+struct rm_info {
+	char   *name;
+	nlink_t links;
+	int     dir;
+	int     exists;
+#if defined(__sun) || defined(__OpenBSD__) || defined(__DragonFly__)
+	int     pad0;
+#endif /* __sun || __OpenBSD__ || __DragonFly__ */
+};
 
 static char *const unsafe_name_msgs[] = {
 	"Starts with a dash (-): command option flags collision",
@@ -81,8 +93,8 @@ static char *const unsafe_name_msgs[] = {
 	"Reserved shell/system keyword",
 	"Contains control characters",
 	"Contains shell meta-characters",
-	"Contains characters not in the Portable Filename Character Set",
-	"Too long"
+	"Too long",
+	"Contains characters not in the Portable Filename Character Set"
 };
 
 #ifdef _BE_POSIX
@@ -178,7 +190,7 @@ static int
 create_tmp_file(char **file, int *fd)
 {
 	size_t tmp_len = strlen(xargs.stealth_mode == 1 ? P_tmpdir : tmp_dir);
-	size_t file_len = tmp_len + strlen(TMP_FILENAME) + 2;
+	size_t file_len = tmp_len + (sizeof(TMP_FILENAME) - 1) + 2;
 
 	*file = (char *)xnmalloc(file_len, sizeof(char));
 	snprintf(*file, file_len, "%s/%s", xargs.stealth_mode == 1
@@ -204,9 +216,13 @@ get_file_suffix(const mode_t type)
 	case DT_LNK: return LINK_CHR;
 	case DT_SOCK: return SOCK_CHR;
 	case DT_FIFO: return FIFO_CHR;
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 	case DT_DOOR: return DOOR_CHR;
-#endif /* __sun */
+//	case DT_PORT: return 0;
+#endif /* SOLARIS_DOORS */
+#ifdef S_IFWHT
+	case DT_WHT: return WHT_CHR;
+#endif /* S_IFWHT */
 	case DT_UNKNOWN: return UNKNOWN_CHR;
 	default: return 0;
 	}
@@ -232,21 +248,21 @@ print_file(FILE *fp, const char *name, const mode_t type)
 }
 
 static int
-write_files_to_tmp(struct dirent ***a, int *n, const char *target,
+write_files_to_tmp(struct dirent ***a, filesn_t *n, const char *target,
 	char *tmp_file)
 {
 	int fd = 0;
 	FILE *fp = open_fwrite(tmp_file, &fd);
 	if (!fp) {
-		_err('e', PRINT_PROMPT, "%s: rr: fopen: %s: %s\n", PROGRAM_NAME,
+		err('e', PRINT_PROMPT, "%s: rr: fopen: %s: %s\n", PROGRAM_NAME,
 			tmp_file, strerror(errno));
 		return errno;
 	}
 
 	fprintf(fp, "%s", _(BULK_RM_TMP_FILE_HEADER));
 
-	size_t i;
 	if (target == workspaces[cur_ws].path) {
+		filesn_t i;
 		for (i = 0; i < files; i++)
 			print_file(fp, file_info[i].name, file_info[i].type);
 	} else {
@@ -265,7 +281,8 @@ write_files_to_tmp(struct dirent ***a, int *n, const char *target,
 			return tmp_err;
 		}
 
-		for (i = 0; i < (size_t)*n; i++) {
+		filesn_t i;
+		for (i = 0; i < *n; i++) {
 			if (SELFORPARENT((*a)[i]->d_name))
 				continue;
 #ifndef _DIRENT_HAVE_D_TYPE
@@ -284,10 +301,10 @@ write_files_to_tmp(struct dirent ***a, int *n, const char *target,
 }
 
 static int
-open_tmp_file(struct dirent ***a, const int n, char *tmp_file, char *app)
+open_tmp_file(struct dirent ***a, const filesn_t n, char *tmp_file, char *app)
 {
 	int exit_status = EXIT_SUCCESS;
-	size_t i;
+	filesn_t i;
 
 	if (!app || !*app) {
 		open_in_foreground = 1;
@@ -308,7 +325,7 @@ open_tmp_file(struct dirent ***a, const int n, char *tmp_file, char *app)
 		return EXIT_SUCCESS;
 
 END:
-	for (i = 0; i < (size_t)n && *a && (*a)[i]; i++)
+	for (i = 0; i < n && *a && (*a)[i]; i++)
 		free((*a)[i]);
 	free(*a);
 
@@ -316,9 +333,9 @@ END:
 }
 
 static char **
-get_files_from_tmp_file(const char *tmp_file, const char *target, const int n)
+get_files_from_tmp_file(const char *tmp_file, const char *target, const filesn_t n)
 {
-	size_t nfiles = (target == workspaces[cur_ws].path) ? files : (size_t)n;
+	size_t nfiles = (target == workspaces[cur_ws].path) ? (size_t)files : (size_t)n;
 	char **tmp_files = (char **)xnmalloc(nfiles + 2, sizeof(char *));
 
 	FILE *fp = fopen(tmp_file, "r");
@@ -358,7 +375,7 @@ get_files_from_tmp_file(const char *tmp_file, const char *target, const int n)
 	return tmp_files;
 }
 
-/* If FILE is not found in LIST, returns one; zero otherwise */
+/* If FILE is not found in LIST, returns one; zero otherwise. */
 static int
 remove_this_file(char *file, char **list)
 {
@@ -376,14 +393,14 @@ remove_this_file(char *file, char **list)
 
 static char **
 get_remove_files(const char *target, char **tmp_files,
-	struct dirent ***a, const int n)
+	struct dirent ***a, const filesn_t n)
 {
 	size_t i, j = 0;
-	size_t l = (target == workspaces[cur_ws].path) ? files : (size_t)n;
+	size_t l = (target == workspaces[cur_ws].path) ? (size_t)files : (size_t)n;
 	char **rem_files = (char **)xnmalloc(l + 2, sizeof(char *));
 
 	if (target == workspaces[cur_ws].path) {
-		for (i = 0; i < files; i++) {
+		for (i = 0; i < (size_t)files; i++) {
 			if (remove_this_file(file_info[i].name, tmp_files) == 1) {
 				rem_files[j] = savestring(file_info[i].name,
 					strlen(file_info[i].name));
@@ -495,7 +512,7 @@ bulk_remove_files(char ***rfiles)
 		return EXIT_FAILURE;
 
 	int i = n;
-	if (rl_get_y_or_n("Continue? [y/n] ") == 0) {
+	if (rl_get_y_or_n(_("Continue? [y/n] ")) == 0) {
 		while (--i >= 0)
 			free((*rfiles)[i]);
 		free(*rfiles);
@@ -520,13 +537,18 @@ bulk_remove_files(char ***rfiles)
 }
 
 static int
-diff_files(char *tmp_file, const int n)
+diff_files(char *tmp_file, const filesn_t n)
 {
 	FILE *fp = fopen(tmp_file, "r");
+	if (!fp) {
+		xerror("br: %s: %s\n", tmp_file, strerror(errno));
+		return 0;
+	}
+
 	char line[PATH_MAX + 6];
 	memset(line, '\0', sizeof(line));
 
-	int c = 0;
+	filesn_t c = 0;
 	while (fgets(line, (int)sizeof(line), fp)) {
 		if (*line != '#' && *line != '\n')
 			c++;
@@ -538,14 +560,15 @@ diff_files(char *tmp_file, const int n)
 }
 
 static int
-nothing_to_do(char **tmp_file, struct dirent ***a, const int n, const int fd)
+nothing_to_do(char **tmp_file, struct dirent ***a, const filesn_t n, const int fd)
 {
 	puts(_("rr: Nothing to do"));
-	unlinkat(fd, *tmp_file, 0);
+	if (unlinkat(fd, *tmp_file, 0) == 1)
+		xerror("rr: unlink: %s: %s\n", *tmp_file, strerror(errno));
 	close(fd);
 	free(*tmp_file);
 
-	int i = n;
+	filesn_t i = n;
 	while (--i >= 0)
 		free((*a)[i]);
 	free(*a);
@@ -562,7 +585,8 @@ bulk_remove(char *s1, char *s2)
 	}
 
 	char *app = (char *)NULL, *target = (char *)NULL;
-	int fd = 0, n = 0, ret = 0, i = 0;
+	int fd = 0, ret = 0, i = 0;
+	filesn_t n = 0;
 
 	if ((ret = parse_bulk_remove_params(s1, s2, &app, &target)) != EXIT_SUCCESS)
 		return ret;
@@ -583,7 +607,7 @@ bulk_remove(char *s1, char *s2)
 		goto END;
 
 	stat(tmp_file, &attr);
-	int num = (target == workspaces[cur_ws].path) ? (int)files : n - 2;
+	filesn_t num = (target == workspaces[cur_ws].path) ? files : n - 2;
 	if (old_t == attr.st_mtime || diff_files(tmp_file, num) == 0)
 		return nothing_to_do(&tmp_file, &a, n, fd);
 
@@ -600,7 +624,10 @@ bulk_remove(char *s1, char *s2)
 	free(rfiles);
 
 END:
-	unlinkat(fd, tmp_file, 0);
+	if (unlinkat(fd, tmp_file, 0) == -1) {
+		err('w', PRINT_PROMPT, "rr: unlink: %s: %s\n",
+			tmp_file, strerror(errno));
+	}
 	close(fd);
 	free(tmp_file);
 	return ret;
@@ -620,8 +647,8 @@ run_mime(char *file)
 
 	/* Convert ELN into file name (rl_line_buffer) */
 	if (p && *p >= '1' && *p <= '9') {
-		int a = atoi(p);
-		if (a > 0 && (size_t)a <= files && file_info[a - 1].name)
+		const filesn_t a = xatof(p);
+		if (a > 0 && a <= files && file_info[a - 1].name)
 			p = file_info[a - 1].name;
 	}
 
@@ -684,20 +711,20 @@ int
 xchmod(const char *file, const char *mode_str, const int flag)
 {
 	if (!file || !*file) {
-		_err(flag == 1 ? 'e' : 0, flag == 1 ? PRINT_PROMPT : NOPRINT_PROMPT,
+		err(flag == 1 ? 'e' : 0, flag == 1 ? PRINT_PROMPT : NOPRINT_PROMPT,
 			"xchmod: Empty buffer for file name\n");
 		return EXIT_FAILURE;
 	}
 
 	if (!mode_str || !*mode_str) {
-		_err(flag == 1 ? 'e' : 0, flag == 1 ? PRINT_PROMPT : NOPRINT_PROMPT,
+		err(flag == 1 ? 'e' : 0, flag == 1 ? PRINT_PROMPT : NOPRINT_PROMPT,
 			"xchmod: Empty buffer for mode\n");
 		return EXIT_FAILURE;
 	}
 
 	int fd = open(file, O_RDONLY);
 	if (fd == -1) {
-		_err(flag == 1 ? 'e' : 0, flag == 1 ? PRINT_PROMPT : NOPRINT_PROMPT,
+		err(flag == 1 ? 'e' : 0, flag == 1 ? PRINT_PROMPT : NOPRINT_PROMPT,
 			"xchmod: %s: %s\n", file, strerror(errno));
 		return errno;
 	}
@@ -705,7 +732,7 @@ xchmod(const char *file, const char *mode_str, const int flag)
 	mode_t mode = (mode_t)strtol(mode_str, 0, 8);
 	if (fchmod(fd, mode) == -1) {
 		close(fd);
-		_err(flag == 1 ? 'e' : 0, flag == 1 ? PRINT_PROMPT : NOPRINT_PROMPT,
+		err(flag == 1 ? 'e' : 0, flag == 1 ? PRINT_PROMPT : NOPRINT_PROMPT,
 			"xchmod: %s: %s\n", file, strerror(errno));
 		return errno;
 	}
@@ -714,14 +741,12 @@ xchmod(const char *file, const char *mode_str, const int flag)
 	return EXIT_SUCCESS;
 }
 
-/* Toggle executable bits on the file named FILE */
+/* Toggle executable bits on the file named FILE. */
 int
 toggle_exec(const char *file, mode_t mode)
 {
-	/* Set or unset S_IXUSR, S_IXGRP, and S_IXOTH */
-	(0100 & mode) ? (mode &= (mode_t)~0111) : (mode |= 0111);
-	// Set it only for owner, unset it for every one
-//	(0100 & mode) ? (mode &= (mode_t)~0111) : (mode |= 0100);
+	/* Set it only for owner, unset it for every one else. */
+	(0100 & mode) ? (mode &= (mode_t)~0111) : (mode |= 0100);
 
 	if (fchmodat(XAT_FDCWD, file, mode, 0) == -1) {
 		xerror("te: Changing permissions of '%s': %s\n",
@@ -752,7 +777,7 @@ get_dup_file_dest_dir(void)
 		/* Expand ELN */
 		if (IS_DIGIT(*dir) && is_number(dir)) {
 			int n = atoi(dir);
-			if (n > 0 && (size_t)n <= files) {
+			if (n > 0 && (filesn_t)n <= files) {
 				free(dir);
 				char *name = file_info[n - 1].name;
 				dir = savestring(name, strlen(name));
@@ -865,18 +890,21 @@ dup_file(char **cmd)
 
 		/* 2. Run command. */
 		if (rsync_path) {
-			char *_cmd[] = {"rsync", "-aczvAXHS", "--progress", source, dest, NULL};
+			char *_cmd[] = {"rsync", "-aczvAXHS", "--progress", "--",
+				source, dest, NULL};
 			if (launch_execv(_cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 				exit_status = EXIT_FAILURE;
 		} else {
 #ifdef _BE_POSIX
-			char *_cmd[] = {"cp", source, dest, NULL};
+			char *_cmd[] = {"cp", "--", source, dest, NULL};
 #elif defined(__sun)
-			char *name = (bin_flags & BSD_HAVE_COREUTILS) ? "gcp" : "cp";
-			char *opt = (bin_flags & BSD_HAVE_COREUTILS) ? "-a" : "--";
-			char *_cmd[] = {name, opt, source, dest, NULL};
+			int g = (bin_flags & BSD_HAVE_COREUTILS);
+			char *name = g ? "gcp" : "cp";
+			char *opt = g ? "-a" : "--";
+			char *_cmd[] = {name, opt, g ? "--" : source,
+				g ? source : dest, g ? dest : NULL, NULL};
 #else
-			char *_cmd[] = {"cp", "-a", source, dest, NULL};
+			char *_cmd[] = {"cp", "-a", "--", source, dest, NULL};
 #endif /* _BE_POSIX */
 			if (launch_execv(_cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 				exit_status = EXIT_FAILURE;
@@ -902,9 +930,10 @@ create_file(char *name)
 	char *n = name;
 	int status = EXIT_SUCCESS;
 
-	/* Dir creation mode (777). mkdir(3) will modify this according to the
-	 * current umask value. */
-	mode_t mode = S_IRWXU | S_IRWXG | S_IRWXO;
+	/* Dir creation mode (777, or 700 if running in secure-mode). mkdir(3) will
+	 * modify this according to the current umask value. */
+	mode_t mode = (xargs.secure_env == 1 || xargs.secure_env_full == 1)
+		? S_IRWXU : S_IRWXU | S_IRWXG | S_IRWXO;
 
 	if (*n == '/') /* Skip root dir. */
 		n++;
@@ -928,9 +957,13 @@ CONT:
 	}
 
 	if (*n && status != EXIT_FAILURE) { /* Regular file */
-		/* Regular file creation mode (666). open(3) will modify this according
-		 * to the current umask value. */
-		mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
+		/* Regular file creation mode (666, or 600 in secure-mode). open(2)
+		 * will modify this according to the current umask value. */
+		if (xargs.secure_env == 1 || xargs.secure_env_full == 1)
+			mode = S_IRUSR | S_IWUSR;
+		else
+			mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
+
 		int fd = open(name, O_WRONLY | O_CREAT | O_EXCL, mode);
 		if (fd == -1) {
 			xerror("new: %s: %s\n", name, strerror(errno));
@@ -944,14 +977,14 @@ CONT:
 }
 
 static void
-list_created_files(char **nfiles, const size_t nfiles_n)
+list_created_files(char **nfiles, const filesn_t nfiles_n)
 {
 	size_t i;
 	int file_in_cwd = 0;
-	int n = workspaces[cur_ws].path
+	filesn_t n = workspaces[cur_ws].path
 		? count_dir(workspaces[cur_ws].path, NO_CPOP) - 2 : 0;
 
-	if (n > 0 && (size_t)n > files)
+	if (n > 0 && n > files)
 		file_in_cwd = 1;
 
 	if (conf.autols == 1 && file_in_cwd == 1)
@@ -965,7 +998,7 @@ list_created_files(char **nfiles, const size_t nfiles_n)
 			free(f);
 	}
 
-	print_reload_msg("%zu file(s) created\n", nfiles_n);
+	print_reload_msg("%zu file(s) created\n", (size_t)nfiles_n);
 }
 
 static int
@@ -1263,7 +1296,7 @@ create_files(char **args)
 	/* Store pointers to actually created files into a pointers array.
 	 * We'll use this later to print the names of actually created files. */
 	char **new_files = (char **)xnmalloc(args_n + 1, sizeof(char *));
-	size_t new_files_n = 0;
+	filesn_t new_files_n = 0;
 
 	for (i = 0; args[i]; i++) {
 		if (validate_filename(&args[i]) == 0) {
@@ -1469,10 +1502,11 @@ print_current_target(const char *link, char **target)
 		return;
 	}
 
-	char tmp[PATH_MAX] = "";
-	ssize_t ret = readlinkat(XAT_FDCWD, link, tmp, sizeof(tmp));
+	char tmp[PATH_MAX + 1];
+	ssize_t len = readlinkat(XAT_FDCWD, link, tmp, sizeof(tmp) - 1);
 
-	if (ret != -1 && *tmp) {
+	if (len != -1) {
+		tmp[len] = '\0';
 		printf(_("%s%s%s (broken link)\n"), uf_c, tmp, df_c);
 		free(*target);
 		*target = savestring(tmp, strlen(tmp));
@@ -1585,6 +1619,14 @@ symlink_file(char **args)
 		}
 	}
 
+	if (args[1] && strchr(args[1], '\\')) {
+		char *deq = dequote_str(args[1], 0);
+		if (deq) {
+			free(args[1]);
+			args[1] = deq;
+		}
+	}
+
 	char *target = args[0];
 	char *link_name = args[1];
 	char tmp[PATH_MAX];
@@ -1731,6 +1773,20 @@ get_new_filename(char *cur_name)
 	return n;
 }
 
+/* Return 1 if at least one file is selected in the current directory.
+ * Otherwise, 0 is returned. */
+static int
+have_sel_files(void)
+{
+	filesn_t i = files;
+	while (--i >= 0) {
+		if (file_info[i].sel == 1)
+			return 1;
+	}
+
+	return 0;
+}
+
 /* Run CMD (either cp(1) or mv(1)) via execv().
  * skip_force is true (1) when the -f,--force parameter has been provided to
  * either 'c' or 'm' commands: it intructs cp/mv to run non-interactivelly
@@ -1751,7 +1807,7 @@ run_cp_mv_cmd(char **cmd, const int skip_force)
 		 * named as a number. Let's check if we have such file name in the
 		 * files list. */
 		if (is_number(cmd[1])) {
-			int i = (int)files;
+			filesn_t i = files;
 			while (--i >= 0) {
 				if (*cmd[1] != *file_info[i].name)
 					continue;
@@ -1794,7 +1850,7 @@ run_cp_mv_cmd(char **cmd, const int skip_force)
 		n++;
 	}
 
-	/* wcp does not support end of options (--) */
+	/* wcp(1) does not support end of options (--). */
 	if (strcmp(cmd[0], "wcp") != 0) {
 		tcmd[n] = savestring("--", 2);
 		n++;
@@ -1803,7 +1859,7 @@ run_cp_mv_cmd(char **cmd, const int skip_force)
 	size_t i;
 	for (i = 1; cmd[i]; i++) {
 		/* The -f,--force parameter is internal. Skip it.
-		 * It instructs cp/mv to run non-interactively (no -i param) */
+		 * It instructs cp/mv to run non-interactively (no -i param). */
 		if (skip_force == 1 && i == 1 && is_force_param(cmd[i]) == 1)
 			continue;
 		p = dequote_str(cmd[i], 0);
@@ -1842,9 +1898,9 @@ run_cp_mv_cmd(char **cmd, const int skip_force)
 	if (ret != EXIT_SUCCESS)
 		return ret;
 
-	/* Error messages are printed by launch_execv() itself */
+	/* Error messages are printed by launch_execv() itself. */
 
-	/* If 'rm sel' and command is successful, deselect everything */
+	/* If 'rm sel' and command is successful, deselect everything. */
 	if (is_sel && *cmd[0] == 'r' && cmd[0][1] == 'm' && (!cmd[0][2]
 	|| cmd[0][2] == ' ')) {
 		int j = (int)sel_n;
@@ -1854,6 +1910,10 @@ run_cp_mv_cmd(char **cmd, const int skip_force)
 		save_sel();
 	}
 
+	if (sel_n > 0 && *cmd[0] == 'm' && have_sel_files())
+		/* Just in case a selected file in the current dir was renamed. */
+		get_sel_files();
+
 #ifdef GENERIC_FS_MONITOR
 	if (*cmd[0] == 'm')
 		reload_dirlist();
@@ -1895,7 +1955,7 @@ cp_mv_file(char **args, const int copy_and_rename, const int force)
 		n++;
 	}
 
-	/* wcp does not support end of options (--) */
+	/* wcp(1) does not support end of options (--) */
 	if (strcmp(tcmd[0], "wcp") != 0) {
 		tcmd[n] = savestring("--" , 2);
 		n++;
@@ -1939,89 +1999,69 @@ cp_mv_file(char **args, const int copy_and_rename, const int force)
 	return EXIT_SUCCESS;
 }
 
+static void
+print_removed_file_info(const struct rm_info info)
+{
+	char *p = abbreviate_file_name(info.name);
+	fputs(p ? p : info.name, stdout);
+	puts(info.dir >= 1 ? "/" : "");
+
+	/* Name removed, but file is still linked to another name (hardlink) */
+	if (info.dir == 0 && info.links > 1) {
+		const nlink_t l = info.links - 1;
+		xerror(_("r: %s: File might still exist (%jd more "
+			"%s linked to this file before this operation)\n"), info.name,
+			(intmax_t)l, l > 1 ? _("names were") : _("name was"));
+	}
+
+	if (p && p != info.name)
+		free(p);
+}
+
 /* Print the list of files removed via the most recent call to the 'r' command */
 static void
-list_removed_files(char **cmd, const size_t *dirs, const size_t start,
+list_removed_files(struct rm_info *info, const size_t start,
 	const int cwd)
 {
 	size_t i, c = 0;
-	for (i = start; cmd[i]; i++);
-	char **removed_files = (char **)xnmalloc(i + 1, sizeof(char *));
-	size_t *_dirs = (size_t *)xnmalloc(i + 1, sizeof(size_t));
 
 	struct stat a;
-	for (i = start; cmd[i]; i++) {
-		_dirs[c] = 0;
-		if (lstat(cmd[i], &a) == -1 && errno == ENOENT) {
-			removed_files[c] = cmd[i];
-			_dirs[c] = dirs[i];
+	for (i = start; info[i].name; i++) {
+		if (lstat(info[i].name, &a) == -1 && errno == ENOENT) {
+			info[i].exists = 0;
 			c++;
 		}
 	}
-	removed_files[c] = (char *)NULL;
 
-	if (c == 0) { /* No file was removed */
-		free(removed_files);
-		free(_dirs);
+	if (c == 0) /* No file was removed */
 		return;
-	}
 
 	if (conf.autols == 1 && cwd == 1)
 		reload_dirlist();
 
-	for (i = 0; i < c; i++) {
-		if (!removed_files[i] || !*removed_files[i])
+	for (i = start; info[i].name; i++) {
+		if (!info[i].name || !*info[i].name || info[i].exists == 1)
 			continue;
 
-		char *p = abbreviate_file_name(removed_files[i]);
-		fputs(p ? p : removed_files[i], stdout);
-		puts(_dirs[i] == 1 ? "/" : "");
-
-		if (p && p != removed_files[i])
-			free(p);
+		print_removed_file_info(info[i]);
 	}
 
 	print_reload_msg(_("%zu file(s) removed\n"), c);
-
-	free(_dirs);
-	free(removed_files);
 }
 
-/* Return the appropriate parameters for (g)rm(1), depending on:
- * 1. The installed version of rm
- * 2. The list of files to be removed contains at least 1 dir (DIRS)
- * 3. We should run interactively (RM_FORCE) */
-static char *
-set_rm_params(const int dirs, const int rm_force)
+/* Print files to be removed and ask the user for confirmation.
+ * Returns 0 if no or 1 if yes. */
+static int
+rm_confirm(struct rm_info *info, const size_t start, const int have_dirs)
 {
-	if (dirs == 1) {
-#if defined(_BE_POSIX)
-		return (rm_force == 1 ? "-rf" : "-r");
-#elif defined(CHECK_COREUTILS)
-		if (bin_flags & BSD_HAVE_COREUTILS)
-			return (rm_force == 1 ? "-drf" : "-dIr");
-		else
-# if defined(__sun)
-			return (rm_force == 1 ? "-rf" : "-r");
-# else
-			return (rm_force == 1 ? "-drf" : "-dr");
-# endif /* __sun */
-#else
-		return (rm_force == 1 ? "-drf" : "-dIr");
-#endif /* _BE_POSIX */
-	}
+	printf(_("r: File(s) to be removed%s:\n"),
+		have_dirs == 1 ? _(" (recursively)") : "");
 
-/* No directories */
-#if defined(_BE_POSIX)
-	return "-f";
-#elif defined(CHECK_COREUTILS)
-	if (bin_flags & BSD_HAVE_COREUTILS)
-		return (rm_force == 1 ? "-f" : "-I");
-	else
-		return "-f";
-#else
-	return (rm_force == 1 ? "-f" : "-I");
-#endif /* _BE_POSIX */
+	size_t i;
+	for (i = start; info[i].name; i++)
+		printf("%s%c\n", info[i].name, info[i].dir >= 1 ? '/' : 0);
+
+	return rl_get_y_or_n(_("Continue? [y/n] "));
 }
 
 int
@@ -2031,10 +2071,9 @@ remove_file(char **args)
 
 	struct stat a;
 	char **rm_cmd = (char **)xnmalloc(args_n + 4, sizeof(char *));
-	/* Let's remember which removed files were directories. DIRS wil be later
-	 * passed to list_removed_files() to append a slash to directories when
-	 * reporting removed files. A bit convoluted, but it works. */
-	size_t *dirs = (size_t *)xnmalloc(args_n + 4, sizeof(size_t));
+	/* Let's keep information about files to be removed. */
+	struct rm_info *info =
+		(struct rm_info *)xnmalloc(args_n + 4, sizeof(struct rm_info));
 
 	int i, j, have_dirs = 0;
 	int rm_force = conf.rm_force == 1 ? 1 : 0;
@@ -2047,10 +2086,10 @@ remove_file(char **args)
 		/* Let's start storing file names in 3: 0 is for 'rm', and 1
 		 * and 2 for parameters, including end of parameters (--). */
 
-		/* If we have a symlink to dir ending with a slash, stat(3) takes it
+		/* If we have a symlink to dir ending with a slash, stat(2) takes it
 		 * as a directory, and then rm(1) complains that cannot remove it,
 		 * because "Is a directory". So, let's remove the ending slash:
-		 * stat(3) will take it as the symlink it is and rm(1) will remove
+		 * stat(2) will take it as the symlink it is and rm(1) will remove
 		 * the symlink (not the target), without complains. */
 		size_t len = strlen(args[i]);
 		if (len > 0 && args[i][len - 1] == '/')
@@ -2069,12 +2108,12 @@ remove_file(char **args)
 
 		if (lstat(tmp, &a) != -1) {
 			rm_cmd[j] = savestring(tmp, strlen(tmp));
-			if (S_ISDIR(a.st_mode)) {
-				dirs[j] = 1;
-				have_dirs = 1;
-			} else {
-				dirs[j] = 0;
-			}
+			info[j].name = rm_cmd[j];
+			info[j].dir = (S_ISDIR(a.st_mode));
+			info[j].links = a.st_nlink;
+			info[j].exists = 1;
+			if (info[j].dir == 1)
+				have_dirs++;
 			j++;
 		} else {
 			xerror("r: %s: %s\n", tmp, strerror(errno));
@@ -2084,25 +2123,25 @@ remove_file(char **args)
 		free(tmp);
 	}
 
-	rm_cmd[j] = (char *)NULL;
+	rm_cmd[j] = info[j].name = (char *)NULL;
 
-	if (errs > 0 && j > 3) { /* If errors but at least one file was deleted */
-		fputs(_("Press any key to continue... "), stdout);
-		xgetchar();
+	if (errs > 0 && j > 3) { /* If at least one error, fail anyway. */
+		fputs(_("r: No files were removed\n"), stderr);
+		goto END;
 	}
 
 	if (j == 3) { /* No file to be deleted */
 		free(rm_cmd);
-		free(dirs);
+		free(info);
 		return EXIT_FAILURE;
 	}
 
-#if defined(CHECK_COREUTILS)
-	rm_cmd[0] = (bin_flags & BSD_HAVE_COREUTILS) ? "grm" : "rm";
-#else
+	if (rm_force == 0 && ((flags & REMOVE_ELN) || have_dirs >= 1 || j > 5)
+	&& rm_confirm(info, 3, have_dirs) == 0)
+		goto END;
+
 	rm_cmd[0] = "rm";
-#endif /* CHECK_COREUTILS */
-	rm_cmd[1] = set_rm_params(have_dirs, rm_force);
+	rm_cmd[1] = have_dirs >= 1 ? "-rf" : "-f";
 	rm_cmd[2] = "--";
 
 	if (launch_execv(rm_cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
@@ -2112,14 +2151,14 @@ remove_file(char **args)
 		deselect_all();
 
 	if (print_removed_files == 1)
-		list_removed_files(rm_cmd, dirs, 3, cwd);
+		list_removed_files(info, 3, cwd);
 
+END:
 	for (i = 3; rm_cmd[i]; i++)
 		free(rm_cmd[i]);
 	free(rm_cmd);
 
-	free(dirs);
-
+	free(info);
 	return exit_status;
 }
 
@@ -2156,20 +2195,22 @@ bulk_rename(char **args)
 	size_t i, arg_total = 0;
 	FILE *fp = (FILE *)NULL;
 
-#if defined(__HAIKU__) || defined(__sun)
+#ifndef HAVE_DPRINTF
 	int tmp_fd = 0;
 	fp = open_fwrite(bulk_file, &tmp_fd);
 	if (!fp) {
+		if (unlink(bulk_file) == -1)
+			xerror("br: unlink: %s: %s\n", bulk_file, strerror(errno));
 		xerror("br: fopen: %s: %s\n", bulk_file, strerror(errno));
 		return EXIT_FAILURE;
 	}
-#endif /* __HAIKU__ || __sun */
+#endif /* !HAVE_DPRINTF */
 
-#if !defined(__HAIKU__) && !defined(__sun)
+#ifdef HAVE_DPRINTF
 	dprintf(fd, BULK_RENAME_TMP_FILE_HEADER);
 #else
 	fprintf(fp, BULK_RENAME_TMP_FILE_HEADER);
-#endif /* !__HAIKU__ && !__sun */
+#endif /* HAVE_DPRINTF */
 
 	struct stat attr;
 	size_t counter = 0;
@@ -2206,26 +2247,28 @@ bulk_rename(char **args)
 
 		counter++;
 
-#if !defined(__HAIKU__) && !defined(__sun)
+#ifdef HAVE_DPRINTF
 		dprintf(fd, "%s\n", args[i]);
 #else
 		fprintf(fp, "%s\n", args[i]);
-#endif /* !__HAIKU__ && !__sun */
+#endif /* HAVE_DPRINTF */
 	}
-#if defined(__HAIKU__) || defined(__sun)
+#ifndef HAVE_DPRINTF
 	fclose(fp);
-#endif /* __HAIKU__ || __sun */
+#endif /* !HAVE_DPRINTF */
 	arg_total = i;
 	close(fd);
 
 	if (counter == 0) { /* No valid file name */
-		if (unlinkat(fd, bulk_file, 0) == -1)
-			xerror("br: unlinkat: %s: %s\n", bulk_file, strerror(errno));
+		if (unlink(bulk_file) == -1)
+			xerror("br: unlink: %s: %s\n", bulk_file, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
 	fp = open_fread(bulk_file, &fd);
 	if (!fp) {
+		if (unlink(bulk_file) == -1)
+			xerror("br: unlink: %s: %s\n", bulk_file, strerror(errno));
 		xerror("br: %s: %s\n", bulk_file, strerror(errno));
 		return EXIT_FAILURE;
 	}
@@ -2250,6 +2293,8 @@ bulk_rename(char **args)
 	fclose(fp);
 	fp = open_fread(bulk_file, &fd);
 	if (!fp) {
+		if (unlink(bulk_file) == -1)
+			xerror("br: unlink: %s: %s\n", bulk_file, strerror(errno));
 		xerror("br: %s: %s\n", bulk_file, strerror(errno));
 		return errno;
 	}
@@ -2347,6 +2392,10 @@ bulk_rename(char **args)
 	}
 	fclose(fp);
 
+	if (sel_n > 0 && have_sel_files())
+		/* Just in case a selected file in the current dir was renamed. */
+		get_sel_files();
+
 #ifdef GENERIC_FS_MONITOR
 	if (exit_status == EXIT_SUCCESS)
 		reload_dirlist();
@@ -2359,6 +2408,7 @@ ERROR:
 		xerror("br: unlinkat: %s: %s\n", bulk_file, strerror(errno));
 		exit_status = errno;
 	}
+
 	fclose(fp);
 	return exit_status;
 }
@@ -2367,7 +2417,7 @@ ERROR:
  * into a temporary file. Return the address of this empt file if
  * success (it must be freed) or NULL in case of error */
 char *
-_export(char **filenames, const int open)
+export_files(char **filenames, const int open)
 {
 	size_t len = strlen(tmp_dir) + 14;
 	char *tmp_file = (char *)xnmalloc(len, sizeof(char));
@@ -2381,34 +2431,36 @@ _export(char **filenames, const int open)
 	}
 	
 	size_t i;
-#if defined(__HAIKU__) || defined(__sun)
+#ifndef HAVE_DPRINTF
 	int tmp_fd = 0;
 	FILE *fp = open_fwrite(tmp_file, &tmp_fd);
 	if (!fp) {
+		if (unlink(tmp_file) == -1)
+			xerror("exp: unlink: %s: %s\n", tmp_file, strerror(errno));
 		xerror("exp: %s: %s\n", tmp_file, strerror(errno));
 		free(tmp_file);
 		return (char *)NULL;
 	}
-#endif /* __HAIKU__ || __sun */
+#endif /* !HAVE_DPRINTF */
 
 	/* If no argument, export files in CWD. */
 	if (!filenames[1]) {
 		for (i = 0; file_info[i].name; i++)
-#if !defined(__HAIKU__) && !defined(__sun)
+#ifdef HAVE_DPRINTF
 			dprintf(fd, "%s\n", file_info[i].name);
 #else
 			fprintf(fp, "%s\n", file_info[i].name);
-#endif /* !__HAIKU__ && !__sun */
+#endif /* HAVE_DPRINTF */
 	} else {
 		for (i = 1; filenames[i]; i++) {
 			if (*filenames[i] == '.' && (!filenames[i][1]
 			|| (filenames[i][1] == '.' && !filenames[i][2])))
 				continue;
-#if !defined(__HAIKU__) && !defined(__sun)
+#ifdef HAVE_DPRINTF
 			dprintf(fd, "%s\n", filenames[i]);
 #else
 			fprintf(fp, "%s\n", filenames[i]);
-#endif /* !__HAIKU__ && !__sun */
+#endif /* HAVE_DPRINTF */
 		}
 	}
 #if defined(__HAIKU__) || defined(__sun)
@@ -2423,6 +2475,8 @@ _export(char **filenames, const int open)
 	if (ret == EXIT_SUCCESS)
 		return tmp_file;
 
+	if (unlink(tmp_file) == -1)
+		xerror("exp: unlink: %s: %s\n", tmp_file, strerror(errno));
 	free(tmp_file);
 	return (char *)NULL;
 }
