@@ -270,7 +270,7 @@ fuzzy_match_v1(char *s1, char *s2, const size_t s1_len)
 	int included = 0;
 	char *p = (char *)NULL;
 
-	if (cs == 1 ? (p = strstr(s2, s1)) : (p = strcasestr(s2, s1))) {
+	if (cs == 1 ? (p = strstr(s2, s1)) : (p = xstrcasestr(s2, s1))) {
 		if (p == s2) {
 			if (!*(s2 + s1_len))
 				return EXACT_MATCH_BONUS;
@@ -319,14 +319,6 @@ fuzzy_match_v1(char *s1, char *s2, const size_t s1_len)
 	score += (consecutive_chars * CONSECUTIVE_CHAR_BONUS);
 	score += ((int)l * SINGLE_CHAR_MATCH_BONUS);
 
-/*
-#include <stdio.h>
-	printf("'%s:%d=%d+%d+%d+%d+%d'", s2, score,
-		word_beginning * WORD_BEGINNING_BONUS,
-		first_char * FIRST_CHAR_BONUS,
-		included * INCLUDED_BONUS,
-		consecutive_chars * CONSECUTIVE_CHAR_BONUS,
-		(int)l * SINGLE_CHAR_MATCH_BONUS); */
 	return score;
 }
 
@@ -400,7 +392,7 @@ fuzzy_match(char *s1, char *s2, const size_t s1_len, const int type)
 		int a = utf8nextcodepoint(s1);
 		int b = utf8nextcodepoint(m);
 
-		cp1 = 0, cp2 = 0;
+		cp1 = 0; cp2 = 0;
 		utf8codepoint(s1 + a, &cp1);
 		utf8codepoint(m + b, &cp2);
 		if (*(s1 + a) && *(m + b) && (cs == 1 ? (cp1 == cp2)
@@ -430,14 +422,6 @@ fuzzy_match(char *s1, char *s2, const size_t s1_len, const int type)
 	score += (consecutive_chars * CONSECUTIVE_CHAR_BONUS);
 	score += ((int)l * SINGLE_CHAR_MATCH_BONUS);
 
-/*
-#include <stdio.h>
-	printf("'%s:%d=%d+%d+%d+%d+%d'", s2, score,
-		word_beginning * WORD_BEGINNING_BONUS,
-		first_char * FIRST_CHAR_BONUS,
-		included * INCLUDED_BONUS,
-		consecutive_chars * CONSECUTIVE_CHAR_BONUS,
-		(int)l * SINGLE_CHAR_MATCH_BONUS); */
 	return score;
 }
 
@@ -473,7 +457,7 @@ fuzzy_match2(char *s1, char *s2, const size_t s1_len, const int type)
 
 	int included = 0;
 	char *p = (char *)NULL;
-	if (conf.case_sens_path_comp == 1 ? (p = strstr(s2, s1)) : (p = strcasestr(s2, s1))) {
+	if (conf.case_sens_path_comp == 1 ? (p = strstr(s2, s1)) : (p = xstrcasestr(s2, s1))) {
 		if (p == s2) {
 			if (!*(s2 + s1_len))
 				return EXACT_MATCH_BONUS;
