@@ -25,106 +25,51 @@
 #ifndef HELPERS_H
 #define HELPERS_H
 
-#if (defined(__linux__) || defined(__CYGWIN__)) && !defined(_BE_POSIX)
-# define _GNU_SOURCE
-#else
-# define _POSIX_C_SOURCE 200809L
-# define _DEFAULT_SOURCE
-# if defined(__linux__) || defined(__CYGWIN__)
-#  define _XOPEN_SOURCE /* wcwidth() */
-# elif defined(__FreeBSD__) || defined(__DragonFly__)
-#  define _XOPEN_SOURCE
-#  define __XSI_VISIBLE 700
-#  define __BSD_VISIBLE 1
-# elif defined(__NetBSD__)
-#  define _XOPEN_SOURCE
-#  define _NETBSD_SOURCE
-#  define __BSD_VISIBLE 1
-# elif defined(__OpenBSD__)
-#  define _BSD_SOURCE
-# elif defined(__APPLE__)
-#  define _DARWIN_C_SOURCE
+#define PROGRAM_NAME_UPPERCASE "CliFM"
+#define PROGRAM_NAME "clifm"
+#define PROGRAM_DESC "The command line file manager"
+#define VERSION "1.15"
+#define DATE "Oct 25, 2023"
+#define AUTHOR "L. Abramovich"
+#define CONTACT "https://github.com/leo-arch/clifm"
+#define LICENSE "GPL2+"
+#define COLORS_REPO "https://github.com/leo-arch/clifm-colors"
+
+/* POSIX_STRICT: A more descriptive name for _BE_POSIX */
+#if defined(POSIX_STRICT) || defined(_POSIX_C_SOURCE) || defined(_XOPEN_SOURCE)
+# define _BE_POSIX
+#endif /* POSIX_STRICT || _POSIX_C_SOURCE || _XOPEN_SOURCE */
+
+#ifdef _BE_POSIX
+# ifndef _POSIX_C_SOURCE
+#  define _POSIX_C_SOURCE 200809L
+# endif /* !_POSIX_C_SOURCE */
+# ifndef _XOPEN_SOURCE
+#  define _XOPEN_SOURCE 700
+# endif /* !_XOPEN_SOURCE */
+
+# if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) \
+|| defined(__DragonFly__) || defined(__APPLE__)
+   typedef unsigned char  u_char;
+   typedef unsigned short u_short;
+   typedef unsigned int   u_int;
+   typedef unsigned long  u_long;
+#  ifdef __NetBSD__
+/* scandir, alphasort, and dirfd are POSIX (not even XSI extensions). Why
+ * hidden behind _NETBSD_SOURCE? */
+#   define _NETBSD_SOURCE
+#  endif /* __NetBSD__ */
 # elif defined(__sun)
 #  define __EXTENSIONS__
-#  define BSD_COMP
+# endif /* BSD */
+
+#else /* !POSIX */
+# if defined(__linux__) || defined(__CYGWIN__)
+#  define _GNU_SOURCE
+# elif defined(__APPLE__)
+#  define _DARWIN_C_SOURCE
 # endif /* __linux__ || __CYGWIN__ */
-#endif /* (__linux__ || __CYGWIN__) && !_BE_POSIX */
-
-#ifdef __TINYC__
-# define __STDC_NO_VLA__ 1
-/* Compilation fails on TinyC since 2021 due to regex.h.
- * See https://www.mail-archive.com/bug-gnulib@gnu.org/msg41564.html */
-//void *__dso_handle;
-#endif /* __TINYC__ */
-
-/* Setting GLOB_BRACE to ZERO disables support for GLOB_BRACE if not
- * available on current platform */
-#if !defined(__TINYC__) && !defined(GLOB_BRACE)
-# define GLOB_BRACE 0
-#endif /* !__TINYC__ && !GLOB_BRACE */
-
-#if defined(__CYGWIN__) && defined(_BE_POSIX) && !defined(GLOB_TILDE)
-# define GLOB_TILDE 0
-#endif /* __CYGWIN__ && _BE_POSIX && !GLOB_TILDE */
-
-/* Support large files on ARM and 32-bit machines */
-#if defined(__arm__) || defined(__i386__)
-# define _FILE_OFFSET_BITS 64
-# define _TIME_BITS 64 /* Address Y2038 problem for 32 bits machines */
-#endif /* __arm__ || __i386__ */
-
-/* _NO_LIRA implies _NO_MAGIC */
-#if defined(_NO_LIRA) && !defined(_NO_MAGIC)
-# define _NO_MAGIC
-#endif /* _NO_LIRA && !_NO_MAGIC */
-
-#if (defined(__linux__) || defined(__CYGWIN__) || defined(__HAIKU__)) \
-&& !defined(_BE_POSIX)
-/* du(1) can report sizes in bytes, apparent sizes, and take custom block sizes */
-# define HAVE_GNU_DU
-#endif /* (__linux__ || __CYGWIN__ || __HAIKU__) && !_BE_POSIX */
-
-#ifndef _NO_GETTEXT
-# include <libintl.h>
-#endif /* !_NO_GETTEXT*/
-
-#include <regex.h>
-#include <stdlib.h>
-#include <sys/stat.h> /* S_BLKSIZE */
-
-#if defined(__linux__)
-# include <linux/version.h>
-# include <linux/limits.h>
-# include <sys/types.h>
-# ifndef USE_GENERIC_FS_MONITOR
-#  include <sys/inotify.h>
-#  define LINUX_INOTIFY
-# else
-#  include <stdint.h> /* uint8_t */
-# endif /* !USE_GENERIC_FS_MONITOR */
-#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) \
-|| defined(__DragonFly__)
-# include <sys/types.h>
-# include <sys/time.h>
-# include <sys/param.h>
-# include <sys/syslimits.h>
-# ifndef USE_GENERIC_FS_MONITOR
-#  include <sys/event.h>
-#  define BSD_KQUEUE
-# endif /* !USE_GENERIC_FS_MONITOR */
-#elif defined(__APPLE__)
-# include <sys/types.h>
-# include <sys/time.h>
-# ifndef USE_GENERIC_FS_MONITOR
-#  include <sys/event.h>
-#  define BSD_KQUEUE
-# endif /* !USE_GENERIC_FS_MONITOR */
-#elif defined(__sun) || defined(__CYGWIN__)
-# include <sys/types.h>
-# include <sys/time.h>
-#elif defined(__HAIKU__)
-# include <stdint.h> /* uint8_t */
-#endif /* __linux__ */
+#endif /* _BE_POSIX */
 
 #ifndef __BEGIN_DECLS
 # ifdef __cpluplus
@@ -142,19 +87,219 @@
 # endif /* __cplusplus */
 #endif /* !__END_DECLS */
 
+#ifdef __TINYC__
+# define __STDC_NO_VLA__ 1
+#endif /* __TINYC__ */
+
+#define _FILE_OFFSET_BITS 64 /* Support large files */
+#define _TIME_BITS 64 /* Address Y2038 problem in 32-bit machines */
+
+#define xstrcasestr strcasestr
+
+#ifdef _BE_POSIX
+# undef  xstrcasestr
+# define xstrcasestr x_strcasestr
+# define _NO_GETTEXT
+# ifndef ALLOW_MEDIA
+#  define NO_MEDIA_FUNC
+# endif /* !ALLOW_MEDIA */
+# ifndef ALLOW_LIRA
+#  define _NO_LIRA
+# endif /* !ALLOW_LIRA */
+# ifndef ALLOW_ARCHIVING
+#  define _NO_ARCHIVING
+# endif /* ALLOW_ARCHIVING */
+#endif /* _BE_POSIX */
+
+/* _NO_LIRA implies _NO_MAGIC */
+#if defined(_NO_LIRA) && !defined(_NO_MAGIC)
+# define _NO_MAGIC
+#endif /* _NO_LIRA && !_NO_MAGIC */
+
+#if (defined(__linux__) || defined(__CYGWIN__) || defined(__HAIKU__)) \
+&& !defined(_BE_POSIX)
+/* du(1) can report sizes in bytes, apparent sizes, and take custom block sizes */
+# define HAVE_GNU_DU
+#endif /* (__linux__ || __CYGWIN__ || __HAIKU__) && !_BE_POSIX */
+
+#ifndef _NO_GETTEXT
+# include <libintl.h>
+#endif /* !_NO_GETTEXT*/
+
+#include <glob.h>
+#include <regex.h>
+#include <stdlib.h>
+#include <sys/stat.h> /* S_BLKSIZE */
+#include <fcntl.h> /* AT_* constants (like AT_FDCWD) */
+/* Included here to test _DIRENT_HAVE_D_TYPE and DT macros. */
+#include <dirent.h>
+
+#ifdef CLIFM_LEGACY
+/* Replace functions not available before POSIX-1.2008. More precisely,
+ * let's try to be POSIX-1.2001 compliant. This is still experimental:
+ * a crash was reported due to x_scandir(), the scandir(3) replacement. */
+# include "compat.h"
+#endif /* CLIFM_LEGACY */
+
+#ifdef __sun
+/* Solaris/Illumos defines AT_FDCWD as 0xffd19553 (-3041965); without the int
+ * cast, the value gets interpreted as uint (4291925331), which throws compiler
+ * warnings. See https://github.com/python/cpython/issues/60169 */
+# define XAT_FDCWD (int)AT_FDCWD
+# ifndef _BE_POSIX
+#  define SOLARIS_DOORS
+# endif /* !_BE_POSIX */
+#else
+# define XAT_FDCWD AT_FDCWD /* defined in fcntl.h */
+#endif /* __sun */
+
+#if !defined(__HAIKU__) && !defined(__sun) && !defined(CLIFM_LEGACY)
+# define HAVE_DPRINTF
+#endif /* !__HAIKU__ && !__SUN && !CLIFM_LEGACY */
+
+#if !defined(_DIRENT_HAVE_D_TYPE) || !defined(DT_DIR)
+/* Systems not providing a d_type member for the stat struct do not provide
+ * these macros either. We use them to convert an st_mode value to the
+ * appropriate d_type value (via get_dt()). */
+# define DT_UNKNOWN 0
+# define DT_FIFO    1
+# define DT_CHR     2
+# define DT_DIR     4
+# define DT_BLK     6
+# define DT_REG     8
+# define DT_LNK     10
+# define DT_SOCK    12
+# define DT_WHT     14
+# ifdef __sun
+#  define DT_DOOR   16
+#  define DT_PORT   18 /* Event port */
+# endif /* __sun */
+#endif /* !_DIRENT_HAVE_D_TYPE || !DT_DIR */
+
+/* Some extra file types */
+#define DT_SHM      100 /* Shared memory object file */
+#define DT_SEM      102 /* Semaphore file */
+#define DT_MQ       104 /* Message queue file */
+#define DT_TPO      106 /* Typed memory object file */
+#ifdef S_ARCH1
+# define DT_ARCH1   108 /* Archive state 1 (NetBSD) */
+# define DT_ARCH2   110 /* Archive state 2 (NetBSD) */
+#endif /* S_ARCH1 */
+
+#ifndef DT_WHT
+# define DT_WHT     14 /* Whiteout (FreeBSD/NetBSD/DragonFly/MacOS)*/
+#endif /* DT_WHT */
+
+/* If any of these file type checks isn't available, fake it */
+#ifndef S_TYPEISMQ
+# define S_TYPEISMQ(s)  (0)
+#endif /* !S_TYPEISMQ */
+#ifndef S_TYPEISSEM
+# define S_TYPEISSEM(s) (0)
+#endif /* !S_TYPEISSEM */
+#ifndef S_TYPEISSHM
+# define S_TYPEISSHM(s) (0)
+#endif /* !S_TYPEISSHM */
+#ifndef S_TYPEISTMO
+# define S_TYPEISTMO(s) (0)
+#endif /* !S_TYPEISTMO */
+/* About these extra file types see
+ * https://pubs.opengroup.org/onlinepubs/007904875/basedefs/sys/stat.h.html
+ * NOTE: On Linux/Solaris, all these macros always evaluate to zero (false).
+ * They don't seem to be present on BSD systems either.
+ * However, the following coreutils programs perform checks for these
+ * file types (mostly S_TYPEISSHM): split(1), sort(1), shred(1), dd(1),
+ * du(1), head(1), tail(1), truncate(1), od(1), wc(1), and shuf(1). */
+/* ASCII representation of these file types:
+ * 'F': semaphore
+ * 'Q': message queue
+ * 'S': Shared memory object
+ * 'T': Typed memory object */
+
+/* Other non-standard file types:
+if (S_ISCTG(mode)) return 'C'; high performance (contiguous data) file
+if (S_ISMPB(mode) || S_ISMPC (bits) || S_ISMPX (bits)) return 'm'; // V7
+if (S_ISNWK(mode)) return 'n'; // HP/UX: network special file
+* S_IFNAM // Solaris/XENIX: special named file
+* S_INSEM // Solaris/XENIX: semaphore subtype of IFNAM
+* S_INSHD // Solaris/XENIX: shared data subtype of IFNAM
+* S_IFDB  // DragonFly: Database record file (DT_DBF). Recognized by neither ls nor find
+*
+* For a quite comphensive list of non-standard file types see:
+* https://github.com/python/cpython/issues/55225#issuecomment-1093532804 */
+
+/* Setting GLOB_BRACE to ZERO disables support for GLOB_BRACE if not
+ * available on current platform. Same for GLOB_TILDE. */
+#ifndef GLOB_BRACE
+# define GLOB_BRACE 0
+#endif /* GLOB_BRACE */
+#ifndef GLOB_TILDE
+# define GLOB_TILDE 0
+#endif /* GLOB_TILDE */
+
+#if defined(__linux__)
+# include <linux/version.h>
+# include <linux/limits.h>
+# include <sys/types.h>
+# if !defined(__GLIBC__) || (__GLIBC__ > 2 \
+|| (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 9)) \
+&& LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
+#  define HAVE_INOTIFY
+# endif /* GLIBC >= 2.9 && linux >= 2.6.27 */
+#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) \
+|| defined(__DragonFly__) || defined(__APPLE__)
+# include <sys/types.h>
+# include <sys/time.h>
+# include <sys/param.h>
+# include <sys/syslimits.h>
+# if defined(__FreeBSD__) && __FreeBSD_version >= 410000
+#  define HAVE_KQUEUE
+# elif defined(__NetBSD__)
+#  if __NetBSD_Prereq__(2,0,0)
+#   define HAVE_KQUEUE
+#  endif /* NetBSD >= 2.0 */
+# elif defined(__OpenBSD__) && OpenBSD >= 200106 /* version 2.9 */
+#  define HAVE_KQUEUE
+# elif defined(__DragonFly__) && __DragonFly_version >= 200800 /* At least 2.8*/
+#  define HAVE_KQUEUE
+# elif defined(__APPLE__)
+#  define HAVE_KQUEUE
+# endif /* FreeBSD >= 4.1 */
+#elif defined(__sun) || defined(__CYGWIN__)
+# include <sys/types.h>
+# include <sys/time.h>
+#elif defined(__HAIKU__)
+# include <stdint.h> /* uint8_t */
+#endif /* __linux__ */
+
+#if defined(__OpenBSD__) || defined(__NetBSD__) \
+|| defined(__FreeBSD__) || defined(__APPLE__)
+# include <inttypes.h> /* uintmax_t, intmax_t */
+#endif /* BSD */
+
+/* File system event monitors (inotify and kqueue) are OS-specific.
+ * Let's fallback to our own generic monitor. */
+#if defined(_BE_POSIX) && !defined(USE_GENERIC_FS_MONITOR)
+# define USE_GENERIC_FS_MONITOR
+#endif /* _BE_POSIX && !USE_GENERIC_FS_MONITOR */
+
+#if defined(__linux__)
+# if defined(HAVE_INOTIFY) && !defined(USE_GENERIC_FS_MONITOR)
+#  include <sys/inotify.h>
+#  define LINUX_INOTIFY
+# else
+#  include <stdint.h> /* uint8_t */
+# endif /* HAVE_INOTIFY && !USE_GENERIC_FS_MONITOR */
+#endif /* __linux__ */
+
+#if defined(HAVE_KQUEUE) && !defined(USE_GENERIC_FS_MONITOR)
+# include <sys/event.h>
+# define BSD_KQUEUE
+#endif /* HAVE_KQUEUE && !USE_GENERIC_FS_MONITOR */
+
 #include "strings.h"
 #include "settings.h"
 
-#define PROGRAM_NAME_UPPERCASE "CliFM"
-#define PROGRAM_NAME "clifm"
-#define PROGRAM_DESC "The command line file manager"
-#define VERSION "1.13"
-#define AUTHOR "L. Abramovich"
-#define CONTACT "https://github.com/leo-arch/clifm"
-#define DATE "Jul 18, 2023"
-#define LICENSE "GPL2+"
-#define COLORS_REPO "https://github.com/leo-arch/clifm-colors"
-
 #define EXIT_SUCCESS 0
 #define EXIT_FAILURE 1
 
@@ -162,7 +307,11 @@
 # ifdef __linux__
 #  define PATH_MAX 4096
 # else
-#  define PATH_MAX 1024
+#  ifdef MAXPATHLEN
+#   define PATH_MAX MAXPATHLEN
+#  else
+#   define PATH_MAX 1024
+#  endif /* MAXPATHLEN */
 # endif /* __linux__ */
 #endif /* !PATH_MAX */
 
@@ -179,9 +328,18 @@
 #endif /* !NAME_MAX */
 
 /* Used by FILE_SIZE and FILE_SIZE_PTR macros to calculate file sizes */
-#ifndef S_BLKSIZE
-# define S_BLKSIZE 512 /* Not defined in Termux */
+#ifndef S_BLKSIZE /* Not defined in Termux/Solaris */
+# include <sys/param.h>
+# ifdef DEV_BSIZE
+#  define S_BLKSIZE DEV_BSIZE
+# else
+#  define S_BLKSIZE 512
+# endif /* DEV_BSIZE */
 #endif /* !S_BLKSIZE */
+/* Linux-glibc/FreeBSD/NetBSD/OpenBSD: S_BLKSIZE/DEV_BSIZE = 512.
+ * CYGWIN: S_BLKSIZE/DEV_BSIZE = 1024.
+ * Solaris/Termux/Linux-musl: S_BLKSIZE is unset, DEV_BSIZE = 512.
+ * DragonFly/MacOS/Haiku: S_BLKSIZE = 512 (DEV_BSIZE is unset). */
 
 #ifndef ARG_MAX
 # ifdef __linux__
@@ -192,44 +350,48 @@
 #endif /* !ARG_MAX */
 
 #if defined(__linux__) && !defined(_BE_POSIX)
-# if (__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 28))
-#  if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 11, 0)
-#   define _STATX
-#  endif /* LINUX_VERSION (4.11) */
-# endif /* __GLIBC__ >= 2.28 */
-# if (__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 3))
-#  if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
-#   define _LINUX_XATTR
+# if defined(STATX_TYPE) && (!defined(__ANDROID__) || __ANDROID_API__ >= 30)
+#  define LINUX_STATX
+# endif /* STATX_TYPE */
+# if !defined(__GLIBC__) || (__GLIBC__ > 2 \
+|| (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 3))
+#  if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
+#   define LINUX_FILE_XATTRS
 #  endif /* LINUX_VERSION (2.4) */
-# endif /* __GLIBC__ */
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-#  define _LINUX_CAP
+# endif /* !__GLIBC__ || __GLIBC__ >= 2.3 */
+# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+#  define LINUX_FILE_CAPS
 # endif /* LINUX_VERSION (2.6.24)*/
 # ifndef __TERMUX__
 #  define LINUX_FILE_ATTRS
 # endif /* !__TERMUX__ */
-#endif /* _GNU_SOURCE */
+#endif /* __linux__ && !_BE_POSIX */
 
 /* Do we have files birth time? If yes, define ST_BTIME. */
 /* ST_BTIME is the timespec struct for files creation time. Valid fields are
  * tv_sec and tv_nsec. */
-#ifdef _STATX
-# define ST_BTIME stx_btime
+#ifndef _BE_POSIX
+# ifdef LINUX_STATX
+#  define ST_BTIME stx_btime
 /* OpenBSD defines the interface (see sys/stat.h), but the file system doesn't
  * actually store creation times: the value of __st_birthtim is always zero.
 #elif defined(__OpenBSD__)
 # define ST_BTIME __st_birthtim */
-#elif defined(__NetBSD__) || defined(__APPLE__)
-# define ST_BTIME st_birthtimespec
-#elif defined(__FreeBSD__) || defined(__CYGWIN__)
-# define ST_BTIME st_birthtim
-#elif defined(__sun) && !defined(_NO_SUN_BIRTHTIME)
+# elif defined(__NetBSD__) || (defined(__APPLE__) && !defined(__ppc__) \
+&& !defined(__i386__))
+#  define ST_BTIME st_birthtimespec
+# elif defined(__FreeBSD__) || defined(__CYGWIN__)
+#  define ST_BTIME st_birthtim
+# elif defined(__HAIKU__)
+#  define ST_BTIME st_crtim
+# elif defined(__sun) && !defined(_NO_SUN_BIRTHTIME)
 /* In the case of Solaris, ST_BTIME is just a flag telling we should run
  * get_birthtime() to get files creation time. */
-# define ST_BTIME
-#endif /* _STATX */
+#  define ST_BTIME
+# endif /* LINUX_STATX */
+#endif /* !_BE_POSIX */
 
-/* Event handling */
+/* File system events handling */
 #if defined(LINUX_INOTIFY)
 # define NUM_EVENT_SLOTS 32 /* Make room for 32 events */
 # define EVENT_SIZE (sizeof(struct inotify_event))
@@ -237,6 +399,7 @@
 extern int inotify_fd, inotify_wd;
 extern unsigned int INOTIFY_MASK;
 extern int watch;
+
 #elif defined(BSD_KQUEUE)
 # define NUM_EVENT_SLOTS 10
 # define NUM_EVENT_FDS   10
@@ -245,19 +408,52 @@ extern struct kevent events_to_monitor[];
 extern unsigned int KQUEUE_FFLAGS;
 extern struct timespec timeout;
 extern int watch;
+
 #else
 # define GENERIC_FS_MONITOR
 extern time_t curdir_mtime;
 #endif /* LINUX_INOTIFY */
 
-#ifdef __sun
-/* Solaris/Illumos defines AT_FDCWD as 0xffd19553 (-3041965); without the int
- * cast, the value gets interpreted as uint (4291925331), which throws compiler
- * warnings. See https://github.com/python/cpython/issues/60169 */
-# define XAT_FDCWD (int)AT_FDCWD
-#else
-# define XAT_FDCWD AT_FDCWD
-#endif /* __sun */
+/* Do we have arc4random_uniform(3). If not, fallback to random(3). */
+#if !defined(_NO_ARC4RANDOM) && !defined(_BE_POSIX) && !defined(__HAIKU__)
+# if defined(__FreeBSD__) && __FreeBSD_version >= 800041
+#  define HAVE_ARC4RANDOM
+# elif defined(__NetBSD__)
+#  if __NetBSD_Prereq__(6,0,0)
+#   define HAVE_ARC4RANDOM
+#  endif /* NetBSD >= 6.0 */
+# elif defined(__OpenBSD__) && OpenBSD >= 200806 /* version 4.4 */
+#  define HAVE_ARC4RANDOM
+# elif defined(__DragonFly__) && __DragonFly_version >= 400600
+/* At least since 4.6 (Sep 2016). See https://gitweb.dragonflybsd.org/dragonfly.git/commitdiff/a2cdfb90273ff84696f4103580173cce21c12e2b
+ * It might be less though. */
+#  define HAVE_ARC4RANDOM
+# elif defined(__sun) && defined(SUN_VERSION) && SUN_VERSION >= 511
+/* 5.11 at least: it might be less. */
+#  define HAVE_ARC4RANDOM
+# elif defined(__linux__) && defined(__GLIBC__) && (__GLIBC__ > 2 \
+|| (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 36))
+#  define HAVE_ARC4RANDOM
+# elif defined(__CYGWIN__) || (defined(__linux__) && defined(__ANDROID__))
+#  define HAVE_ARC4RANDOM
+# endif /* __FreeBSD__ */
+#endif /* !_NO_ARC4RANDOM && !_BE_POSIX && !__HAIKU__ */
+
+#if !defined(_BE_POSIX) && (defined(__FreeBSD__) || defined(__OpenBSD__) \
+|| defined(__DragonFly__) || defined(__APPLE__))
+# define HAVE_STATFS
+#endif
+
+#if !defined(_BE_POSIX) && defined(__linux__)
+# define LINUX_FSINFO
+#endif
+
+#define DEV_NO_NAME "-" /* String used when no file system name/type is found */
+
+/* This is a more or less arbitrary value, but better than some huge value
+ * like INT_MAX (which most likely will cause problems long before reaching
+ * this value). */
+#define MAX_SHELL_LEVEL 1000 /* This is the value used by bash 5.2 */
 
 /* The following flags are used via an integer (FLAGS). If an integer has
  * 4 bytes, then we can use a total of 32 flags (0-31)
@@ -288,6 +484,7 @@ extern time_t curdir_mtime;
 #define KITTY_TERM          (1 << 12)
 #define NO_FIX_RL_POINT     (1 << 13)
 #define FAILED_ALIAS        (1 << 14)
+#define REMOVE_ELN          (1 << 15) // non-expanded ELN removed via 'r'
 
 /* Flags for third party binaries */
 #define FZF_BIN_OK     (1 << 0)
@@ -333,9 +530,9 @@ extern time_t curdir_mtime;
 /* Terminal columns taken by the last line of the default prompt */
 #define FALLBACK_PROMPT_OFFSET 6
 
-/* A few macros for the _err function */
-#define ERR_NO_LOG   -1 /* _err prints but doesn't log */
-#define ERR_NO_STORE -2 /* _err prints and logs, but doesn't store the msg into the messages array */
+/* A few macros for the err function */
+#define ERR_NO_LOG   -1 /* err prints but doesn't log */
+#define ERR_NO_STORE -2 /* err prints and logs, but doesn't store the msg into the messages array */
 
 /* Macros for xchdir (for setting term title or not) */
 #define SET_TITLE 1
@@ -354,15 +551,12 @@ extern time_t curdir_mtime;
 #define FOREGROUND 0
 
 /* A few fixed colors */
-#define _RED    (conf.colorize == 1 ? "\x1b[1;31m" : "")
-#define _BGREEN (conf.colorize == 1 ? "\x1b[1;32m" : "")
-#define D_CYAN  (conf.colorize == 1 ? "\x1b[0;36m" : "")
-#define BOLD    ((xargs.no_bold != 1 && conf.colorize == 1) ? "\x1b[1m" : "")
+#define BOLD_RED   (conf.colorize == 1 ? "\x1b[1;31m" : "")
+#define BOLD_GREEN (conf.colorize == 1 ? "\x1b[1;32m" : "")
+#define REG_CYAN   (conf.colorize == 1 ? "\x1b[0;36m" : "")
+#define BOLD       ((xargs.no_bold != 1 && conf.colorize == 1) ? "\x1b[1m" : "")
 /* NC: Reset color attributes to terminal defaults */
-#define NC      (conf.colorize == 1 ? "\x1b[0m" : "")
-
-/* Format to use for suggestions when running colorless */
-#define SUG_NO_COLOR "\x1b[0m" /* No color */
+#define NC         (conf.colorize == 1 ? "\x1b[0m" : "")
 
 /* Colors for the prompt: */
 /* \001 and \002 tell readline that color codes between them are
@@ -415,11 +609,12 @@ extern time_t curdir_mtime;
 #define PRINT_NEWLINE 1
 
 /* A few key macros used by the auto-suggestions system */
-#define _ESC   27
-#define _TAB   9
-#define BS     8
-#define DELETE 127
-#define ENTER  13
+#define KEY_ESC       27
+#define KEY_TAB       9
+#define KEY_BACKSPACE 8
+#define KEY_DELETE    127
+#define KEY_ENTER     13
+#define KEY_KILL      21 /* Ctrl-u */
 
 /* Macros to specify suggestions type */
 #define NO_SUG         0
@@ -469,6 +664,12 @@ extern time_t curdir_mtime;
 #define E_NOSTDERR (1 << 3)
 #define E_MUTE     (E_NOSTDOUT | E_NOSTDERR)
 
+
+#define FZF_INTERNAL_PREVIEWER 1 /* clifm itself */
+/* --preview is set from either from FzfOpts in the color scheme file or from
+ * FZF_DEFAULT_OPTS environment variable. */
+#define FZF_EXTERNAL_PREVIEWER 2
+
 /* Macros for the backdir (bd) function */
 #define BD_TAB    1
 #define BD_NO_TAB 0
@@ -497,8 +698,6 @@ extern time_t curdir_mtime;
 #define TRIM_NO_EXT 1
 #define TRIM_EXT    2
 
-#define __MB_LEN_MAX 16
-
 /* OpenBSD recommends the use of 10 trailing X's. See mkstemp(3) */
 #if defined(__OpenBSD__)
 # define TMP_FILENAME ".tempXXXXXXXXXX"
@@ -510,22 +709,6 @@ extern time_t curdir_mtime;
 # define P_tmpdir "/tmp"
 #endif /* P_tmpdir */
 
-#if defined(__HAIKU__) || defined(__sun)
-# define DT_UNKNOWN 0
-# define DT_FIFO    1
-# define DT_CHR     2
-# define DT_DIR     4
-# define DT_BLK     6
-# define DT_REG     8
-# define DT_LNK     10
-# define DT_SOCK    12
-# ifdef __sun
-#  define DT_DOOR   13
-# endif /* __sun */
-#endif /* __HAIKU__ || __sun */
-
-#define DT_NONE     14
-
 /* Macros for the get_sys_shell function */
 #define SHELL_NONE 0
 #define SHELL_BASH 1
@@ -565,11 +748,11 @@ extern time_t curdir_mtime;
 #define FILTER_MIME_TYPE 3 /* @query */
 
 /* Macros for properties string fields in long view */
-#if defined(_LINUX_XATTR)
+#if defined(LINUX_FILE_XATTRS)
 # define PROP_FIELDS_SIZE 7 /* Seven available fields */
 #else
 # define PROP_FIELDS_SIZE 6 /* Six available fields */
-#endif /* _LINUX_XATTR */
+#endif /* LINUX_FILE_XATTRS */
 
 #define PERM_SYMBOLIC 1
 #define PERM_NUMERIC  2
@@ -610,6 +793,14 @@ extern time_t curdir_mtime;
 #define SHADE_TYPE_TRUECOLOR 3
 #define NUM_SHADES 6
 
+#define QUOTING_STYLE_BACKSLASH     0
+#define QUOTING_STYLE_SINGLE_QUOTES 1
+#define QUOTING_STYLE_DOUBLE_QUOTES 2
+
+/* Macros used for the --stat and --stat-full command line options. */
+#define SIMPLE_STAT 1
+#define FULL_STAT   2
+
 /* Function macros */
 #define itoa xitoa /* itoa does not exist in some OS's */
 #define atoi xatoi /* xatoi is just a secure atoi */
@@ -622,21 +813,33 @@ extern time_t curdir_mtime;
 
 #define strlen(s) xstrnlen(s)
 
-#if (defined(__linux__) || defined(__CYGWIN__)) && defined(_BE_POSIX)
-# define strcasestr xstrcasestr
-#endif /* (__linux || __CYGWIN__) && _BE_POSIX */
-
 /* Log the message and print it to STDERR, but do not store it into the
  * messages array. */
-#define xerror(...) _err(ERR_NO_STORE, NOPRINT_PROMPT, __VA_ARGS__)
-
-#define ENTRY_N 64
+#define xerror(...) err(ERR_NO_STORE, NOPRINT_PROMPT, __VA_ARGS__)
 
 /* Macros to calculate file sizes */
-#define FILE_SIZE_PTR (conf.apparent_size == 1 ? attr->st_size \
-		: attr->st_blocks * S_BLKSIZE)
-#define FILE_SIZE (conf.apparent_size == 1 ? attr.st_size \
-		: attr.st_blocks * S_BLKSIZE)
+/* Directories, regular files, and symbolic links report meaningful sizes
+ * (when the st_size or st_blocks field of the corresponding stat struct is
+ * queried). Other file types, like sockets, block/character devices, etc,
+ * should always report a size of zero. However, this is not guarranteed:
+ * in Solaris, for example, the value of the st_size field for block devices
+ * is unspecified (it might be a big number).
+ * See https://docs.oracle.com/cd/E36784_01/html/E36872/stat-2.html
+ *
+ * Let's use this macro to make sure we report file sizes only for the
+ * appropriate file types.
+ * NOTE: In case of directories themselves (excluding their content), their
+ * apparent size is always zero as well (see 'info du'). */
+#define FILE_TYPE_NON_ZERO_SIZE(m) (S_ISDIR((m)) || S_ISREG((m)) \
+		|| S_ISLNK((m)))
+/* du(1) also checks for
+ * S_TYPEISSHM(struct stat *) // shared memory objects
+ * S_TYPEISTMO(struct stat *) // typed memory objects */
+
+#define FILE_SIZE_PTR(s) (conf.apparent_size == 1 ? (s)->st_size \
+		: (s)->st_blocks * S_BLKSIZE)
+#define FILE_SIZE(s) (conf.apparent_size == 1 ? (s).st_size \
+		: (s).st_blocks * S_BLKSIZE)
 
 #define UNUSED(x) (void)(x) /* Just silence the compiler's warning */
 #define TOUPPER(c) (((c) >= 'a' && (c) <= 'z') ? ((c) - 'a' + 'A') : (c))
@@ -652,7 +855,30 @@ extern time_t curdir_mtime;
 		: ((n) < 100000000)  ? 8   \
 		: ((n) < 1000000000) ? 9   \
 				             : 10)
+
+/* SIZE_MAX == 18446744073709551615 == 20 digits */
+#define DIGINUM_BIG(n) (((n) < 10) ? 1 \
+		: ((n) < 100)        ? 2   \
+		: ((n) < 1000)       ? 3   \
+		: ((n) < 10000)      ? 4   \
+		: ((n) < 100000)     ? 5   \
+		: ((n) < 1000000)    ? 6   \
+		: ((n) < 10000000)   ? 7   \
+		: ((n) < 100000000)  ? 8   \
+		: ((n) < 1000000000) ? 9   \
+		: ((off_t)(n) < 10000000000) ? 10 \
+		: ((off_t)(n) < 100000000000) ? 11 \
+		: ((off_t)(n) < 1000000000000) ? 12 \
+		: ((off_t)(n) < 10000000000000) ? 13 \
+		: ((off_t)(n) < 100000000000000) ? 14 \
+		: ((off_t)(n) < 1000000000000000) ? 15 \
+		: ((off_t)(n) < 10000000000000000) ? 16 \
+		: ((off_t)(n) < 100000000000000000) ? 17 \
+		: ((off_t)(n) < 1000000000000000000) ? 18 \
+                                             : 19)
+
 #define IS_DIGIT(c)    ((unsigned int)(c) >= '0' && (unsigned int)(c) <= '9')
+//#define IS_DIGIT(c)    ((unsigned int)(c) - '0' <= '9')
 #define IS_ALPHA(c)    ((unsigned int)(c) >= 'a' && (unsigned int)(c) <= 'z')
 #define IS_COMMENT(c)  ((unsigned int)(c) == '#' || (unsigned int)(c) == ';')
 #define IS_CTRL_CHR(c) ((unsigned int)(c) < ' ')
@@ -722,7 +948,7 @@ extern time_t curdir_mtime;
 
 // Safely cast the number N to the type T. The type T is never overflowed:
 // if N is bigger than what T can hold, the maximum value T can hold is
-// returned.
+// returned. What about underflow?
 #define SAFECAST(n, t) ((t)(n) < 0 ? TYPE_MAXIMUM(t) : (t)(n)) */
 
 /*
@@ -746,18 +972,36 @@ extern time_t curdir_mtime;
 				 *  #    GLOBAL VARIABLES   #
 				 *  ######################### */
 
+/* filesn_t: Let's use this to count files */
+/* ssize_t is a quite convenient data type:
+ * 1. Unlike intmax_t, it's never bigger than size_t (in 32-bit/ARM machines,
+ * INTMAX_MAX is bigger than SIZE_MAX).
+ * 2. Can be safely casted to both intmax_t and size_t (something
+ * we do sometimes).
+ * 3. It's signed; at least one negative number (-1) (something we need for
+ * decrementing loops).
+ *
+ * 32-bit/ARM: INT_MAX == LONG_MAX == SSIZE_MAX < SIZE_MAX < INTMAX_MAX == LLONG_MAX
+ * 64-bit:     INT_MAX < LONG_MAX == LLONG_MAX == INTMAX_MAX == SSIZE_MAX < SIZE_MAX */
+#define FILESN_MAX SSIZE_MAX
+typedef ssize_t filesn_t;
+extern filesn_t files;
+
 /* User settings (mostly from the config file) */
 struct config_t {
 	char *opener;
+	char *dirhistignore_regex;
+	char *histignore_regex;
 	char *encoded_prompt;
 	char *term;
 	char *time_str;
+	char *ptime_str;
 	char *welcome_message_str;
 	char *wprompt_str;
 #ifndef _NO_SUGGESTIONS
 	char *suggestion_strategy;
 #else
-	char *pad1; /* Keep the struct alignment */
+	char *pad0; /* Keep the struct alignment */
 #endif /* !_NO_SUGGESTIONS */
 	char *usr_cscheme;
 	char *fzftab_options;
@@ -769,7 +1013,7 @@ struct config_t {
 	int case_sens_dirjump;
 	int case_sens_path_comp;
 	int case_sens_search;
-	int case_sens_list; // files list
+	int case_sens_list; /* files list */
 	int cd_on_quit;
 	int classify;
 	int clear_screen;
@@ -812,6 +1056,8 @@ struct config_t {
 	int purge_jumpdb;
 	int print_selfiles;
 	int private_ws_settings;
+	int quoting_style;
+	int readonly;
 	int relative_time;
 	int restore_last_path;
 	int rm_force;
@@ -828,12 +1074,12 @@ struct config_t {
 #ifndef _NO_TRASH
 	int tr_as_rm;
 #else
-	int pad2; /* Keep the struct alignment */
+	int pad1; /* Keep the struct alignment */
 #endif /* !_NO_TRASH */
 	int unicode;
 	int warning_prompt;
 	int welcome_message;
-	int pad3;
+	int pad2;
 };
 
 extern struct config_t conf;
@@ -844,7 +1090,7 @@ struct filter_t {
 	int rev;
 	int type;
 	int env;
-	int pad;
+	int pad0;
 };
 
 extern struct filter_t filter;
@@ -860,7 +1106,7 @@ struct user_t {
 	gid_t gid;     /* Primary user group */
 	gid_t *groups; /* Secondary groups ID's */
 	int ngroups;   /* Number of secondary groups */
-	int pad;
+	int pad0;
 };
 
 extern struct user_t user;
@@ -886,7 +1132,7 @@ struct ws_t {
 	char *path;
 	char *name;
 	int num;
-	int pad;
+	int pad0;
 };
 
 extern struct ws_t *workspaces;
@@ -904,6 +1150,9 @@ struct jump_t {
 	char *path;
 	size_t len;
 	size_t visits;
+#ifdef __arm__
+	char *pad0;
+#endif /* __arm__ */
 	time_t first_visit;
 	time_t last_visit;
 	int keep;
@@ -936,15 +1185,20 @@ struct fileinfo {
 	char *icon;
 	char *icon_color;
 	char *name;
-	size_t len;
+	filesn_t filesn;
+	size_t len;   /* File name len (columns needed to display file name) */
+	size_t bytes; /* Bytes consumed by file name */
+#if defined(__arm__) && !defined(__ANDROID__)
+	size_t pad0;
+#endif /* __arm__ && !__ANDROID__ */
 	time_t ltime; /* For long view mode */
 	time_t time;
 	ino_t inode;
 	off_t size;
-	dev_t rdev; /* 4 bytes on OpenBSD (used to calculate major and minor devs in long view) */
-	nlink_t linkn; /* 4 bytes on Solaris/OpenBSD */
+	dev_t rdev; /* 4 bytes in OpenBSD (used to calculate major and minor devs in long view) */
+	nlink_t linkn; /* 4 bytes in Solaris/OpenBSD */
 #if !defined(__sun)
-	int pad;
+	int pad1;
 #endif /* __sun */
 	uid_t uid;
 	gid_t gid;
@@ -953,15 +1207,16 @@ struct fileinfo {
 	int dir;
 	int eln_n;
 	int exec;
-	int filesn; /* Number of files in subdir. Is a signed integer enough? */
 	int ruser;  /* User read permission for dir */
 	int symlink;
 	int sel;
 	int xattr;
 	int du_status; /* Exit status of du(1) for dir full sizes */
-#if defined(__FreeBSD__) || defined(__NetBSD__)
+	int utf8; /* Name contains at least one UTF-8 character */
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__DragonFly__) \
+|| defined(__arm__)
 	int pad2;
-#endif /* __FreeBSD__ || __NetBSD__ */
+#endif /* __FreeBSD__ || __NetBSD__ || __DragonFly__ || __arm__ */
 };
 
 extern struct fileinfo *file_info;
@@ -969,23 +1224,23 @@ extern struct fileinfo *file_info;
 /* Struct to store the length of the longest values in the current list of
  * files (needed to properly construct columns in long view). */
 struct maxes_t {
-	size_t ids;
-	size_t inode;
-	size_t files_counter;
-	size_t name;
-	size_t size;
+	int ids;
+	int inode;
+	int files_counter;
+	int name;
+	int size;
 };
 
 struct devino_t {
 	ino_t ino;
-	dev_t dev; /* 4 bytes on OpenBSD */
+	dev_t dev; /* 4 bytes on OpenBSD, DragonFly, and Haiku */
 	char mark;
+	char pad0;
 	char pad1;
 	char pad2;
-	char pad3;
-#if !defined(__OpenBSD__)
-	int pad4;
-#endif /* !__OpenBSD__ */
+#if !defined(__OpenBSD__) && !defined(__DragonFly__) && !defined(__HAIKU__)
+	int pad3;
+#endif /* !__OpenBSD__ && !__DragonFly__ && !__HAIKU__ */
 };
 
 extern struct devino_t *sel_devino;
@@ -1022,7 +1277,7 @@ struct opts_t {
 	int show_hidden;
 	int sort;
 	int sort_reverse;
-	int pad;
+	int pad0;
 };
 
 extern struct opts_t opts;
@@ -1098,6 +1353,7 @@ struct param_t {
 	int path;
 	int preview;
 	int printsel;
+	int readonly;
 	int refresh_on_empty_line;
 	int refresh_on_resize;
 	int restore_last_path;
@@ -1111,6 +1367,7 @@ struct param_t {
 	int sort;
 	int sort_reverse;
 	int splash;
+	int stat;
 	int stealth_mode;
 #ifndef _NO_SUGGESTIONS
 	int suggestions;
@@ -1138,7 +1395,7 @@ struct remote_t {
 	int auto_mount;
 	int auto_unmount;
 	int mounted;
-	int padding;
+	int pad0;
 };
 
 extern struct remote_t *remotes;
@@ -1159,6 +1416,9 @@ extern struct suggestions_t suggestion;
 /* Hold information about selected files */
 struct sel_t {
 	char *name;
+#ifdef __arm__
+	char *pad0;
+#endif /* __arm__ */
 	off_t size;
 };
 
@@ -1187,7 +1447,15 @@ struct stats_t {
 	size_t unstat; /* Non-statable file */
 #ifdef __sun
 	size_t door;
+	size_t port;
 #endif /* __sun */
+#ifdef S_ARCH1
+	size_t arch1;
+	size_t arch2;
+#endif /* S_ARCH1 */
+#ifdef S_IFWHT
+	size_t whiteout;
+#endif /* S_IFWHT */
 };
 
 extern struct stats_t stats;
@@ -1195,10 +1463,10 @@ extern struct stats_t stats;
 struct sort_t {
 	const char *name;
 	int num;
-	int pad;
+	int pad0;
 };
 
-extern const struct sort_t _sorts[];
+extern const struct sort_t sort_methods[];
 
 /* Prompts and prompt settings */
 struct prompts_t {
@@ -1225,10 +1493,10 @@ struct termcaps_t {
 	int suggestions;
 	int pager;
 	int hide_cursor;
-	int home;  // Move cursor to line 1, column 1
-	int clear; // ED (erase display)
-	int del_scrollback; // E3
-	int req_cur_pos; // CPR (cursor position request)
+	int home;  /* Move cursor to line 1, column 1 */
+	int clear; /* ED (erase display) */
+	int del_scrollback; /* E3 */
+	int req_cur_pos; /* CPR (cursor position request) */
 };
 extern struct termcaps_t term_caps;
 
@@ -1280,6 +1548,9 @@ extern struct shades_t size_shades;
 
 struct paths_t {
 	char *path;
+#ifdef __arm__
+	char *pad0;
+#endif /* __arm__ */
 	time_t mtime;
 };
 extern struct paths_t *paths;
@@ -1287,11 +1558,25 @@ extern struct paths_t *paths;
 struct ext_t {
 	char  *name;
 	char  *value;
-	size_t len;
+	size_t len; /* Name length */
+	size_t value_len;
 //	size_t hash;
 };
 extern struct ext_t *ext_colors;
 
+#ifdef LINUX_FSINFO
+#define EXT2_FSTYPE 0x002
+#define EXT3_FSTYPE 0x003
+#define EXT4_FSTYPE 0x004
+struct ext_mnt_t {
+	char *mnt_point;
+	int  type; /* One of EXTNFSTYPE macros */
+	int  pad0;
+};
+
+extern struct ext_mnt_t *ext_mnt;
+#endif /* LINUX_FSINFO */
+
 enum tab_mode {
 	STD_TAB =   0,
 	FZF_TAB =   1,
@@ -1405,6 +1690,7 @@ extern int
 	kbind_busy,
 	max_files,
 	mime_match,
+	nesting_level, /* Is this a nested instance? */
 	no_log,
 	open_in_foreground, /* Overrides mimelist file value: used by mime_open */
 	prev_ws,
@@ -1442,7 +1728,7 @@ extern size_t
 	current_hist_n,
 	curhistindex,
 	ext_colors_n,
-	files, /* Amount of files in the current dir. Is size_t enough? */
+//	files, // Amount of files in the current dir. Is size_t enough?
 	jump_n,
 	kbinds_n,
 	longest,
@@ -1479,7 +1765,7 @@ extern char
 	finder_in_file[PATH_MAX + 1],
 	finder_out_file[PATH_MAX + 1],
 #endif /* _NO_FZF */
-	_fmatch[PATH_MAX + 1], /* First regular match if fuzzy matching is enabled */
+	fz_match[PATH_MAX + 1], /* First regular match if fuzzy matching is enabled */
 	prop_fields_str[PROP_FIELDS_SIZE + 1],
 	invalid_time_str[MAX_TIME_STR],
 
@@ -1523,6 +1809,7 @@ extern char
 /*	*right_prompt, */
 	*sel_file,
 	*smenutab_options_env,
+	*stat_filename,
 	*stdin_tmp_dir,
 	*sudo_cmd,
 #ifndef _NO_SUGGESTIONS
@@ -1548,7 +1835,9 @@ extern char
 	**prompt_cmds,
 	**tags;
 
-extern regex_t regex_exp;
+extern regex_t regex_exp;     /* Files list */
+extern regex_t regex_hist;    /* Commands history */
+extern regex_t regex_dirhist; /* Directory history */
 extern char **environ;
 
 /* To store all the 39 color variables we use, with 46 bytes each, we need
@@ -1560,6 +1849,7 @@ extern char **environ;
 extern char
 	/* File types */
 	bd_c[MAX_COLOR], /* Block device */
+	bk_c[MAX_COLOR], /* Backup files (ending with tilde) */
 	ca_c[MAX_COLOR], /* Cap file */
 	cd_c[MAX_COLOR], /* Char device */
 	ed_c[MAX_COLOR], /* Empty dir */
@@ -1574,7 +1864,7 @@ extern char
 	nf_c[MAX_COLOR], /* No read file */
 	no_c[MAX_COLOR], /* Unknown */
 #ifdef __sun
-	oo_c[MAX_COLOR], /* Solaris door */
+	oo_c[MAX_COLOR], /* Solaris door/port */
 #endif /* __sun */
 	or_c[MAX_COLOR], /* Broken symlink */
 	ow_c[MAX_COLOR], /* Other writable */
@@ -1624,6 +1914,7 @@ extern char
 	hr_c[MAX_COLOR], /* Redirection > */
 	hs_c[MAX_COLOR], /* Process separators | & ; */
 	hv_c[MAX_COLOR], /* Variables $ */
+	hw_c[MAX_COLOR], /* Backslash (aka whack) */
 
 	/* File properties */
 	dr_c[MAX_COLOR],  /* Read */
@@ -1655,6 +1946,7 @@ extern char
 	nm_c[MAX_COLOR + 2], /* Notice msg */
 	ti_c[MAX_COLOR + 2], /* Trash indicator */
 	tx_c[MAX_COLOR + 2], /* Text */
+	ro_c[MAX_COLOR + 2], /* Read-only indicator */
 	si_c[MAX_COLOR + 2], /* Stealth indicator */
 	wm_c[MAX_COLOR + 2], /* Warning msg */
 	xs_c[MAX_COLOR + 2], /* Exit code: success */
