@@ -128,7 +128,7 @@ print_logs(const int flag)
 
 	FILE *log_fp = fopen(file, "r");
 	if (!log_fp) {
-		_err(0, NOPRINT_PROMPT, "log: %s: %s\n", file, strerror(errno));
+		err(0, NOPRINT_PROMPT, "log: %s: %s\n", file, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
@@ -222,7 +222,7 @@ log_cmd(void)
 
 	log_fp = fopen(cmds_log_file, "a");
 	if (!log_fp) {
-		_err('e', PRINT_PROMPT, "log: %s: %s\n", cmds_log_file, strerror(errno));
+		err('e', PRINT_PROMPT, "log: %s: %s\n", cmds_log_file, strerror(errno));
 		free(full_log);
 		return EXIT_FAILURE;
 	}
@@ -243,7 +243,7 @@ write_msg_into_logfile(const char *_msg)
 		/* Do not log this error: We might enter into an infinite loop
 		 * trying to access a file that cannot be accessed. Just warn the user
 		 * and print the error to STDERR */
-		xerror("%s: %s: %s\n", PROGRAM_NAME, msgs_log_file, strerror(errno));
+		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, msgs_log_file, strerror(errno));
 		fputs("Press any key to continue... ", stdout);
 		xgetchar();
 		putchar('\n');
@@ -462,9 +462,9 @@ edit_history(char **args)
 {
 	struct stat attr;
 	if (stat(hist_file, &attr) == -1) {
-		int err = errno;
+		int tmp_err = errno;
 		xerror("history: %s: %s\n", hist_file, strerror(errno));
-		return err;
+		return tmp_err;
 	}
 	time_t mtime_bfr = (time_t)attr.st_mtime;
 
@@ -497,12 +497,12 @@ edit_history(char **args)
 }
 
 static int
-_clear_history(char **args)
+clear_history_func(char **args)
 {
 	/* Let's overwrite whatever was there. */
 	FILE *hist_fp = fopen(hist_file, "w+");
 	if (!hist_fp) {
-		_err(0, NOPRINT_PROMPT, "history: %s: %s\n", hist_file, strerror(errno));
+		err(0, NOPRINT_PROMPT, "history: %s: %s\n", hist_file, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
@@ -596,7 +596,7 @@ history_function(char **args)
 		return edit_history(args);
 
 	if (*args[1] == 'c' && strcmp(args[1], "clear") == 0)
-		return _clear_history(args);
+		return clear_history_func(args);
 
 	/* If "history -n [show-time]", print the last -n elements */
 	if (*args[1] == '-' && is_number(args[1] + 1))
@@ -617,157 +617,6 @@ history_function(char **args)
 	return EXIT_SUCCESS;
 }
 
-static int
-exec_hist_cmd(char **cmd)
-{
-	int i, exit_status = EXIT_SUCCESS;
-
-	char **alias_cmd = check_for_alias(cmd);
-	if (alias_cmd) {
-		/* If an alias is found, check_for_alias frees CMD and returns
-		 * alias_cmd in its place to be executed by exec_cmd() */
-		if (exec_cmd(alias_cmd) != 0)
-			exit_status = EXIT_FAILURE;
-
-		for (i = 0; alias_cmd[i]; i++)
-			free(alias_cmd[i]);
-		free(alias_cmd);
-		alias_cmd = (char **)NULL;
-	} else {
-		if ((flags & FAILED_ALIAS) || exec_cmd(cmd) != 0)
-			exit_status = EXIT_FAILURE;
-
-		flags &= ~FAILED_ALIAS;
-		for (i = 0; cmd[i]; i++)
-			free(cmd[i]);
-		free(cmd);
-	}
-
-	return exit_status;
-}
-
-/* Run history command number CMD */
-static int
-run_hist_num(const char *cmd)
-{
-	size_t old_args = args_n;
-	int num = atoi(cmd);
-
-	if (num <= 0 || num > (int)current_hist_n) {
-		xerror(_("history: !%s: event not found\n"), cmd);
-		return EXIT_FAILURE;
-	}
-
-	if (record_cmd(history[num - 1].cmd))
-		add_to_cmdhist(history[num - 1].cmd);
-
-	char **cmd_hist = parse_input_str(history[num - 1].cmd);
-	if (!cmd_hist) {
-		xerror("%s\n", _("history: Error parsing history command"));
-		return EXIT_FAILURE;
-	}
-
-	int exit_status = exec_hist_cmd(cmd_hist);
-	args_n = old_args;
-	return exit_status;
-}
-
-/* Run the last command in history */
-static int
-run_last_hist_cmd(void)
-{
-	size_t old_args = args_n;
-	if (record_cmd(history[current_hist_n - 1].cmd))
-		add_to_cmdhist(history[current_hist_n - 1].cmd);
-
-	char **cmd_hist = parse_input_str(history[current_hist_n - 1].cmd);
-	if (!cmd_hist) {
-		xerror("%s\n", _("history: Error parsing history command"));
-		return EXIT_FAILURE;
-	}
-
-	int exit_status = exec_hist_cmd(cmd_hist);
-	args_n = old_args;
-	return exit_status;
-}
-
-/* Run history command number "last - N" */
-static int
-run_last_lessn_hist_cmd(const char *cmd)
-{
-	size_t old_args = args_n;
-	int acmd = atoi(cmd + 1);
-
-	if (!is_number(cmd + 1) || acmd <= 0 || acmd > (int)current_hist_n - 1) {
-		xerror(_("history: !%s: Event not found\n"), cmd);
-		return EXIT_FAILURE;
-	}
-
-	size_t n = current_hist_n - (size_t)acmd - 1;
-
-	if (record_cmd(history[n].cmd))
-		add_to_cmdhist(history[n].cmd);
-
-	char **cmd_hist = parse_input_str(history[n].cmd);
-	if (cmd_hist) {
-		int exit_status = exec_hist_cmd(cmd_hist);
-		args_n = old_args;
-		return exit_status;
-	}
-
-	xerror("%s\n", _("history: Error parsing history command"));
-	return EXIT_FAILURE;
-}
-
-/* Run history command matching CMD */
-static int
-run_hist_string(const char *cmd)
-{
-	size_t i, len = strlen(cmd), old_args = args_n;
-
-	for (i = 0; history[i].cmd; i++) {
-		if (*cmd != *history[i].cmd || strncmp(cmd, history[i].cmd, len) != 0)
-			continue;
-
-		char **cmd_hist = parse_input_str(history[i].cmd);
-		if (!cmd_hist)
-			continue;
-
-		int exit_status = exec_hist_cmd(cmd_hist);
-		args_n = old_args;
-		return exit_status;
-	}
-
-	xerror(_("history: !%s: Event not found\n"), cmd);
-	return EXIT_FAILURE;
-}
-
-/* Takes as argument the history cmd less the first exclamation mark.
- * Example: if exec_cmd() gets "!-10" it pass to this function "-10",
- * that is, comm + 1 */
-int
-run_history_cmd(const char *cmd)
-{
-	/* If "!n" */
-	if (is_number(cmd))
-		return run_hist_num(cmd);
-
-	/* If "!!", execute the last command */
-	if (*cmd == '!' && !cmd[1])
-		return run_last_hist_cmd();
-
-	/* If "!-n" */
-	if (*cmd == '-' && *(cmd + 1) && is_number(cmd + 1))
-		return run_last_lessn_hist_cmd(cmd);
-
-	/* If !STRING */
-	if ((*cmd >= 'a' && *cmd <= 'z') || (*cmd >= 'A' && *cmd <= 'Z'))
-		return run_hist_string(cmd);
-
-	puts(_(HISTEXEC_USAGE));
-	return EXIT_SUCCESS;
-}
-
 int
 get_history(void)
 {
@@ -785,7 +634,7 @@ get_history(void)
 
 	FILE *hist_fp = fopen(hist_file, "r");
 	if (!hist_fp) {
-		_err('e', PRINT_PROMPT, "history: %s: %s\n", hist_file, strerror(errno));
+		err('e', PRINT_PROMPT, "history: %s: %s\n", hist_file, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
@@ -799,11 +648,10 @@ get_history(void)
 		if (!*line_buff)
 			continue;
 
-		/* Store the command timestamp and continue: the next line is the cmd itself */
+		/* Store the command timestamp and continue: the next line is
+		 * the cmd itself. */
 		if (*line_buff == history_comment_char && *(line_buff + 1)
 		&& is_number(line_buff + 1)) {
-//			if (conf.hist_timestamp == 0)
-//				continue;
 			int d = atoi(line_buff + 1);
 			tdate = d == INT_MIN ? -1 : (time_t)d;
 			continue;
@@ -863,94 +711,22 @@ add_to_cmdhist(char *cmd)
 	history[current_hist_n].date = -1;
 }
 
-/* Returns 1 if INPUT should be stored in history and 0 if not */
+/* Returns 1 if INPUT should be saved on history or 0 if not. */
 int
 record_cmd(char *input)
 {
-	/* NULL input, self or parent (. ..), commands starting with space,
-	 * and input generating a BAEJ suggestion */
-	if (!input || !*input || SELFORPARENT(input) || *input == ' ')
+	if (!input || !*input)
 		return 0;
 
-	/* Blank lines */
-	size_t blank = 1;
-	char *p = input;
-
-	while (*p) {
-		if (*p > ' ') {
-			blank = 0;
-			break;
-		}
-		p++;
-	}
-
-	if (blank == 1)
-		return 0;
-
-	/* Rewind the pointer to the beginning of the input line */
-	p = input;
-
-	size_t len = strlen(p), amp_rm = 0;
-	if (len > 0 && p[len - 1] == '&') {
-		p[len - 1] = '\0';
-		amp_rm = 1;
-	}
-
-	/* Do not record single ELN's */
-	if (*p > '0' && *p <= '9' && is_number(p)) {
-		if (amp_rm == 1)
-			p[len - 1] = '&';
-		return 0;
-	}
-
-	if (amp_rm == 1)
-		p[len - 1] = '&';
-
-	switch (*p) {
-	case '.': /* Self, parent, and fastback */
-		if (!*(p + 1) || (*(p + 1) == '.' && (!*(p + 2) || *(p + 2) == '.') ) )
-			return 0;
-		break;
-
-	/* Do not record the history command itself */
-	case 'h':
-		if (*(p + 1) == 'i' && strcmp(p + 2, "story") == 0)
-			return 0;
-		break;
-
-	case 'r': /* rf command */
-		if (*(p + 1) == 'f' && !*(p + 2))
-			return 0;
-		break;
-
-	/* Do not record exit commands */
-	case 'q':
-		if (!*(p + 1) || strcmp(p + 1, "uit") == 0)
-			return 0;
-		break;
-
-	case 'Q':
-		if (!*(p + 1))
-			return 0;
-		break;
-
-	case 'e':
-		if (*(p + 1) == 'x' && strcmp(p + 2, "it") == 0)
-			return 0;
-		break;
-
-	default: break;
-	}
-
-	/* History */
-	if (*p == '!' && (IS_DIGIT(*(p + 1)) || (*(p + 1) == '-'
-	&& IS_DIGIT(*(p + 2))) || ((*(p + 1) == '!') && *(p + 2) == '\0')))
+	/* Ignore entries matching HistIgnore */
+	if (conf.histignore_regex && *conf.histignore_regex
+	&& regexec(&regex_hist, input, 0, NULL, 0) == EXIT_SUCCESS)
 		return 0;
 
 	/* Consequtively equal commands in history */
 	if (history && current_hist_n > 0 && history[current_hist_n - 1].cmd
-	&& *p == *history[current_hist_n - 1].cmd
-	&& strcmp(p, history[current_hist_n - 1].cmd) == 0) {
+	&& *input == *history[current_hist_n - 1].cmd
+	&& strcmp(input, history[current_hist_n - 1].cmd) == 0) {
 		/* Update timestamp */
 		history[current_hist_n - 1].date = time(NULL);
 		return 0;
