@@ -77,6 +77,7 @@ char
     ICON_DART[] = NERD_DART,
     ICON_DATABASE[] = NERD_DATABASE,
     ICON_DIFF[] = NERD_DIFF,
+    ICON_DJVU[] = NERD_DJVU,
     ICON_DOCKER[] = NERD_DOCKER,
     ICON_ELECTRON[] = NERD_ELECTRON,
     ICON_ELF[] = NERD_BINARY,
@@ -190,6 +191,7 @@ char
     ICON_DART[] = DEV_DART,
     ICON_DATABASE[] = FA_DATABASE,
     ICON_DIFF[] = FILE_DIFF,
+    ICON_DJVU[] = FA_PAPERCLIP,
     ICON_DOCKER[] = MFIZZ_DOCKER,
     ICON_ELECTRON[] = FILE_ELECTRON,
     ICON_ELF[] = OCT_FILE_BINARY,
@@ -303,6 +305,7 @@ char
     ICON_DART[] = EMOJI_DART,
     ICON_DATABASE[] = EMOJI_DATABASE,
     ICON_DIFF[] = EMOJI_DIFF,
+    ICON_DJVU[] = EMOJI_DJVU,
     ICON_DOCKER[] = EMOJI_DOCKER,
     ICON_ELECTRON[] = EMOJI_ELECTRON,
     ICON_ELF[] = EMOJI_BINARY,
@@ -480,7 +483,7 @@ struct icons_t icon_ext[] = {
     {"db", ICON_DATABASE, WHITE},
     {"deb", ICON_DEBIAN, RED},
     {"diff", ICON_DIFF, WHITE},
-    {"djvu", ICON_PDF, B_RED},
+    {"djvu", ICON_DJVU, BLUE},
     {"dll", ICON_SHARE, BLUE},
     {"doc", ICON_WORD, BLUE},
     {"docx", ICON_WORD, BLUE},
