@@ -26,7 +26,6 @@
 
 #include <limits.h> /* INT_MIN */
 #include <errno.h>
-#include <fcntl.h>
 
 #if defined(__linux__) || defined(__HAIKU__) || defined(__sun) \
 || defined(__CYGWIN__)
@@ -52,7 +51,13 @@
 # include <ctype.h>
 #endif /* __NetBSD__ */
 
-#include <paths.h> /* _PATH_STDPATH */
+#ifndef _BE_POSIX
+# include <paths.h> /* _PATH_STDPATH */
+#endif /* _BE_POSIX */
+
+#ifdef LINUX_FSINFO
+# include <mntent.h> /* xxxmntent functions, used by get_ext_mountpoints() */
+#endif /* LINUX_FSINFO */
 
 #include "aux.h"
 #include "checks.h" /* truncate_file(), is_number() */
@@ -69,12 +74,62 @@
 /* We need this for get_user_groups() */
 #if !defined(NGROUPS_MAX)
 # if defined(__linux__)
-#  define NGROUPS_MAX 65536
+#  if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 4)
+#   define NGROUPS_MAX 65536
+#  else
+#   define NGROUPS_MAX 32
+#  endif /* linux >= 2.6.4 */
 # else
 #  define NGROUPS_MAX 1024
 # endif /* __linux__ */
 #endif /* !NGROUPS_MAX */
 
+#ifdef LINUX_FSINFO
+void
+get_ext_mountpoints(void)
+{
+	if (ext_mnt) {
+		int i;
+		for (i = 0; ext_mnt[i].mnt_point; i++)
+			free(ext_mnt[i].mnt_point);
+		free(ext_mnt);
+		ext_mnt = (struct ext_mnt_t *)NULL;
+	}
+
+	FILE *fp = setmntent(_PATH_MOUNTED, "r");
+	if (!fp)
+		return;
+
+	size_t n = 0;
+	struct mntent *ent;
+	while ((ent = getmntent(fp)) != NULL) {
+		char *t = ent->mnt_type;
+		if (*t != 'e' || t[1] != 'x' || t[2] != 't' || !t[3] || t[4])
+			continue;
+
+		ext_mnt = (struct ext_mnt_t *)xrealloc(ext_mnt,
+			(n + 2) * sizeof(struct ext_mnt_t));
+		ext_mnt[n].mnt_point = savestring(ent->mnt_dir, strlen(ent->mnt_dir));
+
+		switch (t[3]) {
+		case '2': ext_mnt[n].type = EXT2_FSTYPE; break;
+		case '3': ext_mnt[n].type = EXT3_FSTYPE; break;
+		case '4': ext_mnt[n].type = EXT4_FSTYPE; break;
+		default: ext_mnt[n].type = -1; break;
+		}
+
+		n++;
+	}
+
+	if (n > 0) {
+		ext_mnt[n].mnt_point = (char *)NULL;
+		ext_mnt[n].type = -1;
+	}
+
+	endmntent(fp);
+}
+#endif /* LINUX_FSINFO */
+
 void
 init_workspaces_opts(void)
 {
@@ -172,6 +227,7 @@ init_conf_struct(void)
 	conf.max_hist = UNSET;
 	conf.max_log = UNSET;
 	conf.max_jump_total_rank = UNSET;
+
 	conf.max_name_len = DEF_MAX_NAME_LEN;
 
 	conf.max_name_len_bk = 0;
@@ -186,6 +242,10 @@ init_conf_struct(void)
 	conf.pager = UNSET;
 	conf.private_ws_settings = UNSET;
 	conf.purge_jumpdb = UNSET;
+
+	conf.quoting_style = DEF_QUOTING_STYLE;
+
+	conf.readonly = DEF_READONLY;
 	conf.relative_time = UNSET;
 	conf.restore_last_path = UNSET;
 	conf.rm_force = UNSET;
@@ -210,12 +270,14 @@ init_conf_struct(void)
 
 	conf.encoded_prompt = (char *)NULL;
 	conf.fzftab_options = (char *)NULL;
+	conf.histignore_regex = (char *)NULL;
 	conf.opener = (char *)NULL;
 #ifndef _NO_SUGGESTIONS
 	conf.suggestion_strategy = (char *)NULL;
 #endif /* !_NO_SUGGESTIONS */
 	conf.term = (char *)NULL;
 	conf.time_str = (char *)NULL;
+	conf.ptime_str = (char *)NULL;
 	conf.usr_cscheme = (char *)NULL;
 	conf.wprompt_str = (char *)NULL;
 	conf.welcome_message_str = (char *)NULL;
@@ -251,9 +313,9 @@ set_prop_fields(const char *line)
 		case 'm': prop_fields.time = PROP_TIME_MOD; break;
 		case 's': prop_fields.size = PROP_SIZE_HUMAN; break;
 		case 'S': prop_fields.size = PROP_SIZE_BYTES; break;
-#if defined(_LINUX_XATTR)
+#if defined(LINUX_FILE_XATTRS)
 		case 'x': prop_fields.xattr = 1; break;
-#endif /* _LINUX_XATTR */
+#endif /* LINUX_FILE_XATTRS */
 		default: break;
 		}
 	}
@@ -361,10 +423,10 @@ get_home(void)
 	if (!user.home || access(user.home, W_OK) == -1) {
 		/* If no user's home, or if it's not writable, there won't be
 		 * any config directory. These flags are used to prevent functions
-		 * from trying to access any of these directories */
+		 * from trying to access any of these directories. */
 		home_ok = config_ok = 0;
 
-		_err('e', PRINT_PROMPT, _("%s: Cannot access the home directory. "
+		err('e', PRINT_PROMPT, _("%s: Cannot access the home directory. "
 			"Bookmarks, commands logs, and commands history are "
 			"disabled. Program messages, selected files, and the jump database "
 			"won't be persistent. Using default options\n"), PROGRAM_NAME);
@@ -391,7 +453,7 @@ init_history(void)
 	history_write_timestamps = 1;
 
 	struct stat attr;
-	if (stat(hist_file, &attr) == 0 && FILE_SIZE != 0) {
+	if (stat(hist_file, &attr) == 0 && FILE_SIZE(attr) != 0) {
 		/* If the size condition is not included, and in case of a zero
 		 * size file, read_history() produces malloc errors */
 		/* Recover history from the history file */
@@ -403,7 +465,7 @@ init_history(void)
 		int fd = 0;
 		FILE *hist_fp = open_fwrite(hist_file, &fd);
 		if (!hist_fp) {
-			_err('w', PRINT_PROMPT, "%s: fopen: '%s': %s\n",
+			err('w', PRINT_PROMPT, "%s: fopen: '%s': %s\n",
 			    PROGRAM_NAME, hist_file, strerror(errno));
 		} else {
 			/* To avoid malloc errors in read_history(), do not
@@ -466,7 +528,11 @@ is_secure_env(void)
 	size_t i;
 	for (i = 0; argv_bk[i]; i++) {
 		if (*argv_bk[i] == '-'
+#ifndef _BE_POSIX
 		&& (strncmp(argv_bk[i], "--secure-", 9) == 0))
+#else
+		&& strcspn(argv_bk[i], "xXY") != strlen(argv_bk[i]))
+#endif /* !_BE_POSIX */
 			return 1;
 	}
 
@@ -474,27 +540,28 @@ is_secure_env(void)
 }
 
 /* Retrieve user groups.
- * Return an array with the ID's of groups to which the user belongs
+ * Return an array with the ID's of groups to which the user belongs.
  * NGROUPS is set to the number of groups.
- * NOTE: getgroups(3) does not include the user's main group.
- * We use getgroups(3) on TERMUX because getgrouplist(3) always returns
+ * NOTE: getgroups(2) does not include the user's main group.
+ * We use getgroups(2) on TERMUX because getgrouplist(3) always returns
  * zero groups. */
 static gid_t *
 get_user_groups(const char *name, const gid_t gid, int *ngroups)
 {
 	int n = *ngroups;
 
-#if defined(__TERMUX__)
+#if defined(__TERMUX__) || defined(_BE_POSIX)
 	UNUSED(name); UNUSED(gid);
 	gid_t *g = (gid_t *)xnmalloc(NGROUPS_MAX, sizeof(g));
 	if ((n = getgroups(NGROUPS_MAX, g)) == -1) {
-		_err('e', PRINT_PROMPT, "%s: getgroups: %s\n",
+		err('e', PRINT_PROMPT, "%s: getgroups: %s\n",
 			PROGRAM_NAME, strerror(errno));
 		free(g);
 		return (gid_t *)NULL;
 	}
 	if (NGROUPS_MAX > n) /* Reduce array to actual amount of groups (N) */
 		g = (gid_t *)xrealloc(g, (size_t)n * sizeof(g));
+
 #elif defined(__linux__)
 	n = 0;
 	getgrouplist(name, gid, NULL, &n);
@@ -538,12 +605,12 @@ xgetenv(const char *s, const int alloc)
  * '/etc/shells'.
  * Return EXIT_SUCCESS if found or EXIT_FAILURE if not. */
 static int
-check_etc_shells(const char *file, int *_errno)
+check_etc_shells(const char *file, char *shells_file, int *tmp_errno)
 {
 	int fd;
-	FILE *fp = open_fread("/etc/shells", &fd);
+	FILE *fp = open_fread(shells_file, &fd);
 	if (!fp) {
-		*_errno = errno;
+		*tmp_errno = errno;
 		return EXIT_FAILURE;
 	}
 
@@ -571,25 +638,39 @@ check_etc_shells(const char *file, int *_errno)
 static void
 validate_custom_shell(char **file)
 {
-	int _errno = 0;
+	int tmp_errno = 0;
 	errno = 0;
 
-	if (*file && check_etc_shells(*file, &_errno) == EXIT_SUCCESS)
+#ifdef _PATH_SHELLS
+	char *shells_file = _PATH_SHELLS;
+#else
+	char *shells_file = "/etc/shells";
+#endif /* _PATH_SHELLS */
+
+#ifdef _PATH_BSHELL
+	char *def_shell = _PATH_BSHELL;
+#else
+	char *def_shell = "/bin/sh";
+#endif
+
+	if (*file && check_etc_shells(*file, shells_file,
+	&tmp_errno) == EXIT_SUCCESS)
 		return;
 
 	/* check_etc_shells() sets errno to a positive value only if /etc/shells
 	 * couldn't be found/accessed. */
-	if (_errno == 0) {
-		_err('w', PRINT_PROMPT, _("%s: %s: Invalid shell. Falling back to "
-			"'/bin/sh'.\nCheck '/etc/shells' for a list of valid shells.\n"),
-			PROGRAM_NAME, *file ? *file : "NULL");
+	if (tmp_errno == 0) {
+		err('w', PRINT_PROMPT, _("%s: %s: Invalid shell. Falling back to "
+			"'%s'.\nCheck '%s' for a list of valid shells.\n"),
+			PROGRAM_NAME, *file ? *file : "NULL", def_shell, shells_file);
 	} else {
-		_err('w', PRINT_PROMPT, _("%s: /etc/shells: %s.\nCannot validate shell. "
-			"Falling back to '/bin/sh'.\n"), PROGRAM_NAME, strerror(_errno));
+		err('w', PRINT_PROMPT, _("%s: %s: %s.\nCannot validate shell. "
+			"Falling back to '%s'.\n"), PROGRAM_NAME, shells_file,
+			strerror(tmp_errno), def_shell);
 	}
 
 	free(*file);
-	*file = savestring("/bin/sh", 7);
+	*file = savestring(def_shell, strlen(def_shell));
 }
 
 /* Get user data from environment variables. Used only in case getpwuid() failed */
@@ -597,7 +678,7 @@ static struct user_t
 get_user_data_env(void)
 {
 	struct user_t tmp_user = {0};
-//	tmp_user.home = (char *)NULL;
+
 	/* If secure-env, do not fallback to environment variables */
 	int sec_env = is_secure_env();
 	char *t = sec_env == 0 ? xgetenv("HOME", 0) : (char *)NULL;
@@ -633,7 +714,7 @@ get_user_data_env(void)
 	tmp_user.shell = t ? savestring(t, strlen(t)) : (char *)NULL;
 
 	tmp_user.shell_basename = (char *)NULL;
-	if (p && t == p) // CLIFM_SHELL
+	if (p && t == p) /* CLIFM_SHELL */
 		validate_custom_shell(&tmp_user.shell);
 
 	return tmp_user;
@@ -650,7 +731,7 @@ get_user_data(void)
 	tmp_user.uid = geteuid();
 	pw = getpwuid(tmp_user.uid);
 	if (!pw) { /* Fallback to environment variables (if not secure-env) */
-		_err('e', PRINT_PROMPT, "%s: getpwuid: %s\n",
+		err('e', PRINT_PROMPT, "%s: getpwuid: %s\n",
 			PROGRAM_NAME, strerror(errno));
 		return get_user_data_env();
 	}
@@ -679,7 +760,7 @@ get_user_data(void)
 		homedir = p ? p : pw->pw_dir;
 
 		if (homedir == p && p && (stat(p, &a) == -1 || !S_ISDIR(a.st_mode))) {
-			_err('e', PRINT_PROMPT, _("%s: %s: Home directory not found\n"
+			err('e', PRINT_PROMPT, _("%s: %s: Home directory not found\n"
 				"Falling back to %s\n"), PROGRAM_NAME, p, pw->pw_dir);
 			homedir = pw->pw_dir;
 		}
@@ -1253,6 +1334,7 @@ load_remotes(void)
 			remotes[n].desc = (char *)xrealloc(remotes[n].desc,
 				(ret_len + 1) * sizeof(char));
 			xstrsncpy(remotes[n].desc, ret, ret_len + 1);
+
 		} else if (strncmp(line, "Mountpoint=", 11) == 0) {
 			char *tmp = (char *)NULL;
 			if (*ret == '~')
@@ -1264,6 +1346,7 @@ load_remotes(void)
 			free(tmp);
 			if (count_dir(remotes[n].mountpoint, CPOP) > 2)
 				remotes[n].mounted = 1;
+
 		} else if (strncmp(line, "MountCmd=", 9) == 0) {
 			int replaced = 0;
 			if (remotes[n].mountpoint) {
@@ -1283,6 +1366,7 @@ load_remotes(void)
 					(ret_len + 1) * sizeof(char));
 				xstrsncpy(remotes[n].mount_cmd, ret, ret_len + 1);
 			}
+
 		} else if (strncmp(line, "UnmountCmd=", 11) == 0) {
 			int replaced = 0;
 			if (remotes[n].mountpoint) {
@@ -1302,9 +1386,11 @@ load_remotes(void)
 					(ret_len + 1) * sizeof(char));
 				xstrsncpy(remotes[n].unmount_cmd, ret, ret_len + 1);
 			}
+
 		} else if (strncmp(line, "AutoUnmount=", 12) == 0) {
 			if (strcmp(ret, "true") == 0)
 				remotes[n].auto_unmount = 1;
+
 		} else {
 			if (strncmp(line, "AutoMount=", 10) == 0) {
 				if (strcmp(ret, "true") == 0)
@@ -1358,7 +1444,7 @@ set_prompts_file(void)
 	if (stat(t, &a) == -1 || !S_ISREG(a.st_mode))
 		goto ERROR;
 
-	char *cmd[] = {"cp", t, f, NULL};
+	char *cmd[] = {"cp", "--", t, f, NULL};
 	int ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
 	if (ret == EXIT_SUCCESS)
 		return f;
@@ -1479,8 +1565,10 @@ load_prompts(void)
 	}
 
 	prompts_n = n;
+
 	if (conf.encoded_prompt)
 		expand_prompt_name(conf.encoded_prompt);
+
 	return EXIT_SUCCESS;
 }
 
@@ -1505,7 +1593,6 @@ unset_xargs(void)
 	xargs.colorize = UNSET;
 	xargs.columns = UNSET;
 	xargs.config = UNSET;
-//	xargs.control_d_exits = UNSET;
 	xargs.cwd_in_title = UNSET;
 	xargs.desktop_notifications = UNSET;
 	xargs.dirmap = UNSET;
@@ -1552,6 +1639,7 @@ unset_xargs(void)
 	xargs.pager = UNSET;
 	xargs.path = UNSET;
 	xargs.printsel = UNSET;
+	xargs.readonly = UNSET;
 	xargs.refresh_on_empty_line = UNSET;
 	xargs.refresh_on_resize = UNSET;
 	xargs.restore_last_path = UNSET;
@@ -1565,6 +1653,9 @@ unset_xargs(void)
 	xargs.sort = UNSET;
 	xargs.sort_reverse = UNSET;
 	xargs.splash = UNSET;
+
+	xargs.stat = 0;
+
 	xargs.stealth_mode = UNSET;
 
 #ifndef _NO_SUGGESTIONS
@@ -1582,6 +1673,30 @@ unset_xargs(void)
 	xargs.warning_prompt = UNSET;
 }
 
+static int
+check_nest_level(void)
+{
+	/* If running on a fully sanitized environment, no variable is imported
+	 * at all, but CLIFMLVL is nevertheless consulted (by xsecure_env()) to
+	 * know whether we are running a nested instance, in which case
+	 * NESTING_LEVEL is set to 2. */
+	if (xargs.secure_env_full == 1 && nesting_level == 2)
+		return 2;
+
+	char *level = getenv("CLIFMLVL");
+	if (!level)
+		goto FALLBACK;
+
+	int a = atoi(level);
+	if (a < 1 || a > MAX_SHELL_LEVEL)
+		goto FALLBACK;
+
+	return a + 1;
+
+FALLBACK:
+	return (getenv("CLIFM") ? 2 : 1);
+}
+
 /* Keep track of attributes of the shell. Make sure the shell is running
  * interactively as the foreground job before proceeding.
  * Based on https://www.gnu.org/software/libc/manual/html_node/Initializing-the-Shell.html#Initializing-the-Shell
@@ -1594,7 +1709,7 @@ init_shell(void)
 		return;
 	}
 
-	if (getenv("CLIFM_OWN_CHILD")) {
+	if ((nesting_level = check_nest_level()) > 1) {
 		set_signals_to_ignore();
 		own_pid = get_own_pid();
 		tcgetattr(STDIN_FILENO, &shell_tmodes);
@@ -1618,7 +1733,7 @@ init_shell(void)
 	setpgid(shell_pgid, shell_pgid);
 /*	if (setpgid(shell_pgid, shell_pgid) < 0) {
 		// This fails with EPERM when running as 'term -e clifm'
-		_err(0, NOPRINT_PROMPT, "%s: setpgid: %s\n", PROGRAM_NAME, strerror(errno));
+		err(0, NOPRINT_PROMPT, "%s: setpgid: %s\n", PROGRAM_NAME, strerror(errno));
 		exit(errno);
 	} */
 
@@ -1629,7 +1744,7 @@ init_shell(void)
 	tcgetattr(STDIN_FILENO, &shell_tmodes);
 }
 
-/* Get current entries in the Selection Box, if any */
+/* Get current entries in the Selection Box, if any. */
 int
 get_sel_files(void)
 {
@@ -1637,7 +1752,7 @@ get_sel_files(void)
 		return EXIT_FAILURE;
 
 	size_t selnbk = sel_n;
-	/* First, clear the sel array, in case it was already used */
+	/* First, clear the sel array, in case it was already used. */
 	if (sel_n > 0) {
 		int i = (int)sel_n;
 		while (--i >= 0)
@@ -1645,7 +1760,7 @@ get_sel_files(void)
 	}
 	sel_n = 0;
 
-	/* Open the tmp sel file and load its contents into the sel array */
+	/* Open the tmp sel file and load its contents into the sel array. */
 	int fd;
 	FILE *fp = open_fread(sel_file, &fd);
 	/*  sel_elements = xcalloc(1, sizeof(char *)); */
@@ -1653,7 +1768,7 @@ get_sel_files(void)
 		return EXIT_FAILURE;
 
 	struct stat a;
-	/* Since this file contains only paths, PATH_MAX should be enough */
+	/* Since this file contains only paths, PATH_MAX should be enough. */
 	char line[PATH_MAX];
 	while (fgets(line, (int)sizeof(line), fp) != NULL) {
 		size_t len = strlen(line);
@@ -1665,7 +1780,7 @@ get_sel_files(void)
 		}
 
 		/* Remove the ending slash: fstatat() won't take a symlink to dir as
-		 * a symlink (but as a dir), if the file name ends with a slash */
+		 * a symlink (but as a dir), if the file name ends with a slash. */
 		if (len > 1 && line[len - 1] == '/') {
 			len--;
 			line[len] = '\0';
@@ -1682,7 +1797,7 @@ get_sel_files(void)
 		sel_elements[sel_n].name = savestring(line, len);
 		sel_elements[sel_n].size = (off_t)UNSET;
 		/* Store device and inode number to identify later selected files
-		 * and mark them in the files list */
+		 * and mark them in the files list. */
 		sel_devino = (struct devino_t *)xrealloc(sel_devino,
 				(sel_n + 1) * sizeof(struct devino_t));
 		sel_devino[sel_n].ino = a.st_ino;
@@ -1695,7 +1810,7 @@ get_sel_files(void)
 	fclose(fp);
 	/* If previous and current amount of sel files don't match (mostly
 	 * because some selected files were removed), recreate the selections
-	 * file to reflect current state */
+	 * file to reflect current state. */
 	if (selnbk != sel_n)
 		save_sel();
 
@@ -2001,8 +2116,43 @@ cygwin_exclude_file(char *name)
 }
 #endif /* __CYGWIN__ */
 
-/* Get the list of files in PATH, plus CliFM internal commands, and send
- * them into an array to be read by my_rl_completion(). */
+/* Check whether the path NAME is a symbolic link to any other path specified
+ * in PATH. Returns 1 if true or 0 otherwise.
+ * Used to avoid scanning paths which are symlinks to another path, for example,
+ * /bin and /sbin, which are usually symlinks to /usr/bin and /usr/sbin
+ * respectively. */
+static int
+skip_this_path(char *name)
+{
+	if (!name || !*name)
+		return 1;
+
+	struct stat a;
+	if (lstat(name, &a) == -1)
+		return 1;
+
+	if (!S_ISLNK(a.st_mode))
+		return 0;
+
+	char *rpath = realpath(name, NULL);
+	if (!rpath)
+		return 1;
+
+	size_t i;
+	for (i = 0; paths[i].path; i++) {
+		if (strcmp(paths[i].path, rpath) == 0) {
+			free(rpath);
+			return 1;
+		}
+	}
+
+	free(rpath);
+	return 0;
+}
+
+/* Get the list of files in PATH, plus CliFM internal commands, aliases, and
+ * action names, and store them into an array (bin_commands) to be read by
+ * my_rl_completion(). */
 void
 get_path_programs(void)
 {
@@ -2021,6 +2171,7 @@ get_path_programs(void)
 		i = (int)path_n;
 		while (--i >= 0) {
 			if (!paths[i].path || !*paths[i].path
+			|| skip_this_path(paths[i].path) == 1
 			|| xchdir(paths[i].path, NO_TITLE) == -1) {
 				cmd_n[i] = 0;
 				continue;
@@ -2030,15 +2181,16 @@ get_path_programs(void)
 #if defined(__CYGWIN__)
 					NULL, xalphasort);
 #else
-					conf.light_mode ? NULL : skip_nonexec, xalphasort);
+					conf.light_mode == 1 ? NULL : skip_nonexec, xalphasort);
 #endif /* __CYGWIN__ */
 			/* If paths[i] directory does not exist, scandir returns -1.
-			 * Fedora, for example, adds $HOME/bin and $HOME/.local/bin to
+			 * Fedora, for example, adds HOME/bin and HOME/.local/bin to
 			 * PATH disregarding if they exist or not. If paths[i] dir is
-			 * empty do not use it either */
+			 * empty do not use it either. */
 			if (cmd_n[i] > 0)
 				total_cmd += cmd_n[i];
 		}
+
 		xchdir(cwd, NO_TITLE);
 	}
 
@@ -2046,8 +2198,8 @@ get_path_programs(void)
 	for (internal_cmds_n = 0; internal_cmds[internal_cmds_n].name;
 		internal_cmds_n++);
 
-	bin_commands = (char **)xnmalloc((size_t)total_cmd + internal_cmds_n +
-			     aliases_n + actions_n + 2, sizeof(char *));
+	bin_commands = (char **)xnmalloc((size_t)total_cmd
+		+ internal_cmds_n + aliases_n + actions_n + 2, sizeof(char *));
 
 	i = (int)internal_cmds_n;
 	while (--i >= 0) {
@@ -2175,7 +2327,7 @@ get_aliases(void)
 	int fd;
 	FILE *fp = open_fread(config_file, &fd);
 	if (!fp) {
-		_err('e', PRINT_PROMPT, "%s: alias: '%s': %s\n",
+		err('e', PRINT_PROMPT, "%s: alias: '%s': %s\n",
 		    PROGRAM_NAME, config_file, strerror(errno));
 		return;
 	}
@@ -2286,7 +2438,7 @@ get_prompt_cmds(void)
 	int fd;
 	FILE *fp = open_fread(config_file, &fd);
 	if (!fp) {
-		_err('e', PRINT_PROMPT, "%s: prompt: '%s': %s\n",
+		err('e', PRINT_PROMPT, "%s: prompt: '%s': %s\n",
 		    PROGRAM_NAME, config_file, strerror(errno));
 		return;
 	}
@@ -2316,9 +2468,9 @@ get_prompt_cmds(void)
 	fclose(fp);
 }
 
-/* Get the length of the current time format
+/* Get the length of the current time format.
  * We need this to construct the time string in case of invalid timestamp (0),
- * and to calculate the space left to print file names in long view */
+ * and to calculate the space left to print file names in long view. */
 
 /* GCC (not clang though) complains about tfmt being not a string literal.
  * Let's silence this warning until we find a better approach.
@@ -2333,13 +2485,13 @@ check_time_str(void)
 		return;
 
 	if (conf.relative_time == 1) {
-	/* +1 = extra space to avoid hitting the screen right edge in long view */
+	/* +1 = extra space to avoid hitting the screen right edge in long view. */
 		prop_fields.len += (7 + 1);
 		xstrsncpy(invalid_time_str, " -     ", sizeof(invalid_time_str));
 		return;
 	}
 
-	/* Get length of the current time format */
+	/* Get length of the current time format. */
 	struct tm tm;
 	time_t t = time(NULL);
 	char tim[MAX_TIME_STR];
@@ -2347,7 +2499,7 @@ check_time_str(void)
 	size_t l = localtime_r(&t, &tm) ? strftime(tim, sizeof(tim), tfmt, &tm) : 0;
 
 	/* Construct the invalid time format string (used when we get an
-	 * invalid file timestamp) */
+	 * invalid file timestamp). */
 	if (l > MAX_TIME_STR)
 		l = MAX_TIME_STR;
 
@@ -2358,7 +2510,7 @@ check_time_str(void)
 	invalid_time_str[i] = '\0';
 
 	/* Append the time string length to the properties total length, so that
-	 * we can better calculate how much space left we have to print file names */
+	 * we can better calculate how much space left we have to print file names. */
 	prop_fields.len += (int)(l + 1);
 }
 #pragma GCC diagnostic pop
@@ -2383,7 +2535,7 @@ set_sudo_cmd(void)
 		return;
 	}
 
-	_err('w', PRINT_PROMPT, _("%s: %s: %s\nInvalid authentication program "
+	err('w', PRINT_PROMPT, _("%s: %s: %s\nInvalid authentication program "
 		"(falling back to '%s')\n"), PROGRAM_NAME, sudo_cmd,
 		strerror(errno), DEF_SUDO_CMD);
 	sudo_cmd = DEF_SUDO_CMD;
@@ -2395,6 +2547,12 @@ check_options(void)
 {
 	set_sudo_cmd();
 
+	if (!conf.histignore_regex
+	&& regcomp(&regex_hist, DEF_HISTIGNORE, REG_NOSUB | REG_EXTENDED) == 0) {
+		conf.histignore_regex =
+			savestring(DEF_HISTIGNORE, sizeof(DEF_HISTIGNORE) - 1);
+	}
+
 	if (conf.trim_names == UNSET) {
 		if (xargs.trim_names == UNSET)
 			conf.trim_names = DEF_TRIM_NAMES;
@@ -2477,13 +2635,13 @@ check_options(void)
 			hist_status = xargs.history;
 	}
 
-	if (!conf.wprompt_str) {
+/*	if (!conf.wprompt_str) {
 		if (conf.colorize == 1)
 			conf.wprompt_str = savestring(DEF_WPROMPT_STR, sizeof(DEF_WPROMPT_STR) - 1);
 		else
 			conf.wprompt_str = savestring(DEF_WPROMPT_STR_NO_COLOR,
 				sizeof(DEF_WPROMPT_STR_NO_COLOR) - 1);
-	}
+	} */
 
 	/* Do no override command line options */
 	if (xargs.cwd_in_title == UNSET)
@@ -2616,7 +2774,7 @@ check_options(void)
 
 	if (smenutab_options_env
 	&& sanitize_cmd(smenutab_options_env, SNT_BLACKLIST) != 0) {
-		_err('w', PRINT_PROMPT, "%s: CLIFM_SMENU_OPTIONS contains unsafe "
+		err('w', PRINT_PROMPT, "%s: CLIFM_SMENU_OPTIONS contains unsafe "
 			"characters (<>|;&$`). Falling back to default values.\n",
 			PROGRAM_NAME);
 		smenutab_options_env = (char *)NULL;
@@ -2626,6 +2784,7 @@ check_options(void)
 	tabmode = STD_TAB;
 #endif /* _NO_FZF */
 
+#ifndef _NO_LIRA
 	if (xargs.stealth_mode == 1) {
 		xargs.fzf_preview = conf.fzf_preview = 0;
 	} else if (conf.fzf_preview == UNSET) {
@@ -2634,6 +2793,10 @@ check_options(void)
 		else
 			conf.fzf_preview = xargs.fzf_preview;
 	}
+#else
+	if (conf.fzf_preview == UNSET)
+		xargs.fzf_preview = conf.fzf_preview = 0;
+#endif /* !_NO_LIRA */
 
 #ifndef _NO_ICONS
 	if (conf.icons == UNSET) {
@@ -2923,6 +3086,9 @@ check_options(void)
 	if (!conf.term)
 		conf.term = savestring(DEF_TERM_CMD, sizeof(DEF_TERM_CMD) - 1);
 
+	if (conf.colorize == 0)
+		expand_prompt_name(DEF_PROMPT_NO_COLOR_NAME);
+
 	if (!conf.encoded_prompt || !*conf.encoded_prompt) {
 		free(conf.encoded_prompt);
 		if (conf.colorize == 1) {
@@ -2933,8 +3099,14 @@ check_options(void)
 				savestring(DEFAULT_PROMPT_NO_COLOR,
 				sizeof(DEFAULT_PROMPT_NO_COLOR) - 1);
 		}
-/*		char *t = conf.colorize == 1 ? DEFAULT_PROMPT : DEFAULT_PROMPT_NO_COLOR;
-		conf.encoded_prompt = savestring(t, strlen(t)); */
+	}
+
+	if (!conf.wprompt_str) {
+		if (conf.colorize == 1)
+			conf.wprompt_str = savestring(DEF_WPROMPT_STR, sizeof(DEF_WPROMPT_STR) - 1);
+		else
+			conf.wprompt_str = savestring(DEF_WPROMPT_STR_NO_COLOR,
+				sizeof(DEF_WPROMPT_STR_NO_COLOR) - 1);
 	}
 
 	if ((xargs.stealth_mode == 1 || home_ok == 0 ||
