@@ -65,6 +65,9 @@ struct jump_entry_t {
 	char *match;
 	char *needle;
 	size_t visits;
+#ifdef __arm__
+	char *pad0;
+#endif /* __arm__ */
 	time_t first;
 	time_t last;
 };
@@ -390,7 +393,7 @@ save_jump_suggestion(char *str)
 }
 
 static char *
-_get_dir_color(const char *filename, const struct stat a)
+get_directory_color(const char *filename, const struct stat a)
 {
 	if (check_file_access(a.st_mode, a.st_uid, a.st_gid) == 0)
 		return nd_c;
@@ -518,7 +521,7 @@ print_jump_table(const int reduce, const time_t now)
 		if (lstat(tmp_jump[i].path, &a) == -1)
 			color = uf_c;
 
-		char *dir_color = color == uf_c ? uf_c : _get_dir_color(tmp_jump[i].path, a);
+		char *dir_color = color == uf_c ? uf_c : get_directory_color(tmp_jump[i].path, a);
 
 		printf(" %s%*zu\t%*zu\t%*d\t%*d\t%s%*d%s%s%c%s\t%c%s%s%s\n",
 			color != uf_c ? color : df_c, max_order, i + 1,
@@ -706,10 +709,10 @@ get_needle(const char *needle, const char *match, const char *query,
 {
 	char *ret = conf.case_sens_dirjump == 1 ? strstr(needle, query)
 #ifdef _BE_POSIX
-		/* strcasestr() calls xstrcasestr(), which does not take consts. */
-		: strcasestr((char *)needle, (char *)query);
+		/* xstrcasestr() calls x_strcasestr(), which does not take consts. */
+		: xstrcasestr((char *)needle, (char *)query);
 #else
-		: strcasestr(needle, query);
+		: xstrcasestr(needle, query);
 #endif /* _BE_POSIX */
 
 	if (!ret || ((segment & LAST_SEGMENT) && strchr(ret, '/')))
@@ -867,6 +870,7 @@ dirjump(char **args, const int mode)
 					if (workspaces[cur_ws].path
 					&& !strstr(jump_db[j].path, workspaces[cur_ws].path))
 						exclude = 1;
+					break;
 
 				case NONE: /* fallthrough */
 				default: break;
