@@ -29,10 +29,8 @@
 #ifdef __OpenBSD__
 typedef char *rl_cpvfunc_t;
 # include <ereadline/readline/readline.h>
-//# include <ereadline/readline/history.h> // history_arg_extract
 #else
 # include <readline/readline.h>
-//# include <readline/history.h> // history_arg_extract
 #endif /* __OpenBSD__ */
 
 #ifdef __TINYC__
@@ -47,9 +45,7 @@ typedef char *rl_cpvfunc_t;
 # include <string.h>
 #endif /* __NetBSD__ */
 
-#include <dirent.h>
 #include <errno.h>
-#include <fcntl.h>
 
 #include "aux.h"
 #include "config.h"
@@ -71,6 +67,8 @@ typedef char *rl_cpvfunc_t;
 # include "highlight.h"
 #endif /* !_NO_HIGHLIGHT */
 
+#include "strings.h" /* quote_str() */
+
 /* Let's use these word delimiters to print first suggested word
  * and to delete last typed word */
 #define WORD_DELIMITERS " /.-_=,:;@+*&$#<>%~|({[]})¿?¡!"
@@ -79,6 +77,15 @@ typedef char *rl_cpvfunc_t;
 static int accept_first_word = 0;
 #endif /* !_NO_SUGGESTIONS */
 
+/* This is just an ugly workaround. You've been warned!
+ * Long story short: prompt commands are executed after SOME keybindings, but
+ * not after others. I'm not sure why, and this irregularity is what is wrong
+ * in the first place.
+ * In the second case, when prompt commands are not executed, we sometimes do
+ * want to run them (mostly when changing directories): in this case, we set
+ * exec_prompt_cmds to 1. */
+static int exec_prompt_cmds = 0;
+
 static void
 xrl_reset_line_state(void)
 {
@@ -103,7 +110,7 @@ kbinds_reset(void)
 	}
 
 	if (exit_status == EXIT_SUCCESS)
-		_err('n', PRINT_PROMPT, _("%s: kb: Restart the program for changes "
+		err('n', PRINT_PROMPT, _("kb: Restart %s for changes "
 			"to take effect\n"), PROGRAM_NAME);
 
 	return exit_status;
@@ -145,7 +152,7 @@ kbinds_edit(char *app)
 	if (mtime_bfr == (time_t)attr.st_mtime)
 		return EXIT_SUCCESS;
 
-	_err('n', PRINT_PROMPT, _("%s: kb: Restart the program for changes to "
+	err('n', PRINT_PROMPT, _("kb: Restart %s for changes to "
 			"take effect\n"), PROGRAM_NAME);
 	return EXIT_SUCCESS;
 }
@@ -246,18 +253,36 @@ load_keybinds(void)
 	return EXIT_SUCCESS;
 }
 
-/* This call to prompt() just updates the prompt in case it was modified,
- * for example, in case of chdir, files selection, and so on */
+/* This call to prompt() just updates the prompt in case it was modified by
+ * a keybinding, for example, chdir, files selection, trash, and so on. */
 static void
 rl_update_prompt(void)
+{
+	if (rl_line_buffer) {
+		memset(rl_line_buffer, '\0', (size_t)rl_end);
+		rl_point = rl_end = 0;
+	}
+
+	/* Set this flag to prevent prompt() from refreshing the screen. */
+	rl_pending_input = 1;
+	/* In UPDATE mode, prompt() always returns NULL. */
+	prompt(exec_prompt_cmds ? PROMPT_UPDATE_RUN_CMDS : PROMPT_UPDATE);
+	exec_prompt_cmds = 0;
+	UNHIDE_CURSOR;
+}
+
+/* Old version of rl_update_prompt(). Used only by rl_profile_previous() and
+ * rl_profile_next(): if any of these functions use rl_update_prompt() instead,
+ * no prompt is printed (not sure why). FIX. */
+static void
+rl_update_prompt_old(void)
 {
 	HIDE_CURSOR;
 	int b = xargs.refresh_on_empty_line;
 	xargs.refresh_on_empty_line = 0;
-	char *input = prompt();
+	char *input = prompt(PROMPT_SHOW);
 	free(input);
 	xargs.refresh_on_empty_line = b;
-	UNHIDE_CURSOR;
 }
 
 /* Runs any command recognized by CliFM via a keybind. Example:
@@ -283,7 +308,7 @@ keybind_exec_cmd(char *str)
 		/* While in the bookmarks or mountpoints screen, the kbind_busy
 		 * flag will be set to 1 and no keybinding will work. Once the
 		 * corresponding function exited, set the kbind_busy flag to zero,
-		 * so that keybindings work again */
+		 * so that keybindings work again. */
 		if (kbind_busy == 1)
 			kbind_busy = 0;
 
@@ -292,7 +317,11 @@ keybind_exec_cmd(char *str)
 			free(cmd[i]);
 		free(cmd);
 
+#ifdef __HAIKU__
+		rl_update_prompt_old();
+#else
 		rl_update_prompt();
+#endif /* __HAIKU__ */
 	}
 
 	args_n = old_args;
@@ -308,10 +337,8 @@ run_kb_cmd(char *cmd)
 	if (kbind_busy == 1)
 		return EXIT_SUCCESS;
 
-	if (conf.colorize == 1 && wrong_cmd == 1) {
-		fputs(tx_c, stdout);
-		fflush(stdout);
-	}
+	if (conf.colorize == 1 && wrong_cmd == 1)
+		recover_from_wrong_cmd();
 
 	keybind_exec_cmd(cmd);
 	rl_reset_line_state();
@@ -336,7 +363,8 @@ find_key(char *function)
 	return (char *)NULL;
 }
 
-int rl_toggle_max_filename_len(int count, int key)
+int
+rl_toggle_max_filename_len(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
@@ -440,6 +468,16 @@ rl_prepend_sudo(int count, int key)
 		rl_point = r;
 	}
 #endif /* !_NO_SUGGESTIONS */
+
+#ifndef _NO_HIGHLIGHT
+	if (conf.highlight == 1) {
+		int r = rl_point;
+		rl_point = 0;
+		recolorize_line();
+		rl_point = r;
+	}
+#endif /* !_NO_HIGHLIGHT */
+
 	return EXIT_SUCCESS;
 }
 
@@ -447,8 +485,7 @@ static int
 rl_create_file(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "n";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("n");
 }
 
 #ifndef _NO_SUGGESTIONS
@@ -586,15 +623,15 @@ rl_accept_suggestion(int count, int key)
 	if (!conf.suggestions || rl_point != rl_end || !suggestion_buf
 	|| suggestion.type == CMD_DESC_SUG) {
 		if (rl_point < rl_end) {
-			/* Just move the cursor forward one column */
-			int mlen = mblen(rl_line_buffer + rl_point, __MB_LEN_MAX);
+			/* Just move the cursor forward one character */
+			int mlen = mblen(rl_line_buffer + rl_point, MB_CUR_MAX);
 			rl_point += mlen;
 		}
 		return EXIT_SUCCESS;
 	}
 
 	/* If accepting the first suggested word, accept only up to next
-	 * word delimiter */
+	 * word delimiter. */
 	char *s = (char *)NULL, _s = 0;
 	int trimmed = 0, accept_first_word_last = 0;
 	if (accept_first_word == 1) {
@@ -666,8 +703,13 @@ rl_accept_suggestion(int count, int key)
 			}
 		}
 
-		if (isquote == 1 && backslash == 0)
-			tmp = escape_str(suggestion_buf);
+		if (isquote == 1 && backslash == 0) {
+			if (suggestion.type == ELN_SUG && suggestion.filetype == DT_REG
+			&& conf.quoting_style != QUOTING_STYLE_BACKSLASH)
+				tmp = quote_str(suggestion_buf);
+			else
+				tmp = escape_str(suggestion_buf);
+		}
 
 		my_insert_text(tmp ? tmp : suggestion_buf, NULL, 0);
 		free(tmp);
@@ -803,15 +845,8 @@ static int
 rl_refresh(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-
-	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
-
-	char cmd[] = "rf";
-	keybind_exec_cmd(cmd);
-	rl_reset_line_state();
-
-	return EXIT_SUCCESS;
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("rf");
 }
 
 static int
@@ -822,8 +857,8 @@ rl_dir_parent(int count, int key)
 	if (*workspaces[cur_ws].path == '/' && !workspaces[cur_ws].path[1])
 		return EXIT_SUCCESS;
 
-	char cmd[] = "cd ..";
-	return run_kb_cmd(cmd);
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("cd ..");
 }
 
 static int
@@ -834,8 +869,7 @@ rl_dir_root(int count, int key)
 	if (*workspaces[cur_ws].path == '/' && !workspaces[cur_ws].path[1])
 		return EXIT_SUCCESS;
 
-	char cmd[] = "cd /";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("cd /");
 }
 
 static int
@@ -847,8 +881,8 @@ rl_dir_home(int count, int key)
 	&& strcmp(workspaces[cur_ws].path, user.home) == 0)
 		return EXIT_SUCCESS;
 
-	char cmd[] = "cd";
-	return run_kb_cmd(cmd);
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("cd");
 }
 
 static int
@@ -859,8 +893,8 @@ rl_dir_previous(int count, int key)
 	if (dirhist_cur_index == 0)
 		return EXIT_SUCCESS;
 
-	char cmd[] = "b";
-	return run_kb_cmd(cmd);
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("b");
 }
 
 static int
@@ -872,38 +906,8 @@ rl_dir_next(int count, int key)
 	if (dirhist_cur_index + 1 == dirhist_total_index)
 		return EXIT_SUCCESS;
 
-	char cmd[] = "f";
-	return run_kb_cmd(cmd);
-}
-
-static int
-rl_dir_first(int count, int key)
-{
-	UNUSED(count); UNUSED(key);
-	/* If already at the beginning of dir hist, do nothing */
-	if (dirhist_cur_index == 0)
-		return EXIT_SUCCESS;
-
-	char cmd[] = "b !1";
-	return run_kb_cmd(cmd);
-}
-
-static int
-rl_dir_last(int count, int key)
-{
-	UNUSED(count); UNUSED(key);
-	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
-
-	/* If already at the end of dir hist, do nothing */
-	if (dirhist_cur_index + 1 == dirhist_total_index)
-		return EXIT_SUCCESS;
-
-	char cmd[32 + 4]; /* 32 should be more than enough to store a signed int */
-	snprintf(cmd, sizeof(cmd), "b !%d", dirhist_total_index);
-	keybind_exec_cmd(cmd);
-	rl_reset_line_state();
-	return EXIT_SUCCESS;
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("f");
 }
 
 int
@@ -923,7 +927,7 @@ rl_toggle_long_view(int count, int key)
 	if (conf.autols == 1) {
 		if (conf.clear_screen == 0)
 		/* Without this putchar(), the first entries of the directories
-		 * list are printed in the prompt line */
+		 * list are printed in the prompt line. */
 			putchar('\n');
 		reload_dirlist();
 	}
@@ -1023,40 +1027,35 @@ static int
 rl_open_config(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "config";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("config");
 }
 
 static int
 rl_open_keybinds(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "kb edit";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("kb edit");
 }
 
 static int
 rl_open_cscheme(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "cs edit";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("cs edit");
 }
 
 static int
 rl_open_bm_file(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "bm edit";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("bm edit");
 }
 
 static int
 rl_open_jump_db(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "je";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("je");
 }
 
 static int
@@ -1079,61 +1078,44 @@ static int
 rl_open_mime(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "mm edit";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("mm edit");
 }
 
 static int
 rl_mountpoints(int count, int key)
 {
-	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
-
 	UNUSED(count); UNUSED(key);
-	/* Call the function only if it's not already running */
-	kbind_busy = 1;
-	char cmd[] = "mp";
-	keybind_exec_cmd(cmd);
-	rl_reset_line_state();
-	return EXIT_SUCCESS;
+	return run_kb_cmd("mp");
 }
 
 static int
 rl_select_all(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "s ^";
-	return run_kb_cmd(cmd);
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("s ^");
 }
 
 static int
 rl_deselect_all(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "ds *";
-	return run_kb_cmd(cmd);
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("ds *");
 }
 
 static int
 rl_bookmarks(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
-
-	kbind_busy = 1;
-	char cmd[] = "bm";
-	keybind_exec_cmd(cmd);
-	rl_reset_line_state();
-	return EXIT_SUCCESS;
+	return run_kb_cmd("bm");
 }
 
 static int
 rl_selbox(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "ds";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("ds");
 }
 
 static int
@@ -1259,6 +1241,41 @@ rl_lock(int count, int key)
 	return EXIT_SUCCESS;
 }
 
+static int
+handle_no_sel(const char *func)
+{
+	if (conf.colorize == 1 && wrong_cmd == 1)
+		recover_from_wrong_cmd();
+
+#ifndef _NO_SUGGESTIONS
+	if (suggestion.printed == 1 && suggestion_buf)
+		clear_suggestion(CS_FREEBUF);
+#endif /* !_NO_SUGGESTIONS */
+
+	if (rl_end > 0) {
+		rl_delete_text(0, rl_end);
+		rl_point = rl_end = 0;
+	}
+
+	printf(_("\n%s: sel: %s\n"), func, strerror(ENOENT));
+	rl_reset_line_state();
+
+	return EXIT_SUCCESS;
+}
+
+static int
+rl_archive_sel(int count, int key)
+{
+	UNUSED(count); UNUSED(key);
+	if (kbind_busy == 1)
+		return EXIT_SUCCESS;
+
+	if (sel_n == 0)
+		return handle_no_sel("ac");
+
+	return run_kb_cmd("ac sel");
+}
+
 static int
 rl_remove_sel(int count, int key)
 {
@@ -1266,6 +1283,9 @@ rl_remove_sel(int count, int key)
 	if (kbind_busy == 1)
 		return EXIT_SUCCESS;
 
+	if (sel_n == 0)
+		return handle_no_sel("r");
+
 	rl_deprep_terminal();
 	char cmd[] = "r sel";
 	keybind_exec_cmd(cmd);
@@ -1281,10 +1301,10 @@ rl_export_sel(int count, int key)
 	if (kbind_busy == 1)
 		return EXIT_SUCCESS;
 
-	char cmd[] = "exp sel";
-	keybind_exec_cmd(cmd);
-	rl_reset_line_state();
-	return EXIT_SUCCESS;
+	if (sel_n == 0)
+		return handle_no_sel("exp");
+
+	return run_kb_cmd("exp sel");
 }
 
 static int
@@ -1294,10 +1314,11 @@ rl_move_sel(int count, int key)
 	if (kbind_busy == 1)
 		return EXIT_SUCCESS;
 
-	char cmd[] = "m sel";
-	keybind_exec_cmd(cmd);
-	rl_reset_line_state();
-	return EXIT_SUCCESS;
+	if (sel_n == 0)
+		return handle_no_sel("m");
+
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("m sel");
 }
 
 static int
@@ -1307,10 +1328,11 @@ rl_rename_sel(int count, int key)
 	if (kbind_busy == 1)
 		return EXIT_SUCCESS;
 
-	char cmd[] = "br sel";
-	keybind_exec_cmd(cmd);
-	rl_reset_line_state();
-	return EXIT_SUCCESS;
+	if (sel_n == 0)
+		return handle_no_sel("br");
+
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("br sel");
 }
 
 static int
@@ -1320,6 +1342,10 @@ rl_paste_sel(int count, int key)
 	if (kbind_busy == 1)
 		return EXIT_SUCCESS;
 
+	if (sel_n == 0)
+		return handle_no_sel("c");
+
+	exec_prompt_cmds = 1;
 	rl_deprep_terminal();
 	char cmd[] = "c sel";
 	keybind_exec_cmd(cmd);
@@ -1372,10 +1398,12 @@ rl_profile_previous(int count, int key)
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
 		return EXIT_SUCCESS;
+
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
 		free_suggestion();
 #endif /* !_NO_SUGGESTIONS */
+
 	int prev_prof, cur_prof = -1, total_profs = 0;
 	get_cur_prof(&cur_prof, &total_profs);
 
@@ -1396,7 +1424,7 @@ rl_profile_previous(int count, int key)
 	}
 
 	if (profile_set(profile_names[prev_prof]) == EXIT_SUCCESS)
-		rl_update_prompt();
+		rl_update_prompt_old();
 
 	return EXIT_SUCCESS;
 }
@@ -1407,10 +1435,12 @@ rl_profile_next(int count, int key)
 	UNUSED(count); UNUSED(key);
 	if (kbind_busy == 1)
 		return EXIT_SUCCESS;
+
 #ifndef _NO_SUGGESTIONS
 	if (suggestion.printed && suggestion_buf)
 		free_suggestion();
 #endif /* !_NO_SUGGESTIONS */
+
 	int next_prof, cur_prof = -1, total_profs = 0;
 	get_cur_prof(&cur_prof, &total_profs);
 
@@ -1431,7 +1461,7 @@ rl_profile_next(int count, int key)
 	}
 
 	if (profile_set(profile_names[next_prof]) == EXIT_SUCCESS)
-		rl_update_prompt();
+		rl_update_prompt_old();
 
 	return EXIT_SUCCESS;
 }
@@ -1441,46 +1471,38 @@ static int
 rl_dirhist(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "bh";
-	return run_kb_cmd(cmd);
-}
-
-static int
-rl_archive_sel(int count, int key)
-{
-	UNUSED(count); UNUSED(key);
-	char cmd[] = "ac sel";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("bh");
 }
 
 static int
 rl_new_instance(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "x .";
-	return run_kb_cmd(cmd);
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("x .");
 }
 
 static int
 rl_clear_msgs(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "msg clear";
-	return run_kb_cmd(cmd);
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("msg clear");
 }
 
 static int
 rl_trash_sel(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "t sel";
-	return run_kb_cmd(cmd);
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("t sel");
 }
 
 static int
 rl_untrash_all(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
+	exec_prompt_cmds = 1;
 	return run_kb_cmd("u *");
 }
 
@@ -1495,25 +1517,7 @@ rl_open_sel(int count, int key)
 	snprintf(cmd, sizeof(cmd), "o %s", (sel_n && sel_elements[sel_n - 1].name)
 		? sel_elements[sel_n - 1].name : "sel");
 
-	keybind_exec_cmd(cmd);
-	rl_reset_line_state();
-	return EXIT_SUCCESS;
-}
-
-static int
-rl_bm_sel(int count, int key)
-{
-	UNUSED(count); UNUSED(key);
-	if (kbind_busy == 1)
-		return EXIT_SUCCESS;
-
-	char cmd[PATH_MAX + 6];
-	snprintf(cmd, sizeof(cmd), "bm a %s", (sel_n && sel_elements[sel_n - 1].name)
-		? sel_elements[sel_n - 1].name : "sel");
-
-	keybind_exec_cmd(cmd);
-	rl_reset_line_state();
-	return EXIT_SUCCESS;
+	return run_kb_cmd(cmd);
 }
 
 static int
@@ -1604,13 +1608,12 @@ rl_dir_pinned(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
 	if (!pinned_dir) {
-		printf(_("%s: No pinned file\n"), PROGRAM_NAME);
+		printf(_("\n%s: No pinned file\n"), PROGRAM_NAME);
 		rl_reset_line_state();
 		return EXIT_SUCCESS;
 	}
 
-	char cmd[] = ",";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd(",");
 }
 
 static int
@@ -1620,6 +1623,7 @@ rl_ws1(int count, int key)
 	if (rl_line_buffer && *rl_line_buffer)
 		rl_delete_text(0, rl_end);
 
+	exec_prompt_cmds = 1;
 	if (cur_ws == 0) {
 		/* If the user attempts to switch to the same workspace they're
 		 * currently in, switch rather to the previous workspace. */
@@ -1631,8 +1635,7 @@ rl_ws1(int count, int key)
 		return EXIT_SUCCESS;
 	}
 
-	char cmd[] = "ws 1";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("ws 1");
 }
 
 static int
@@ -1642,6 +1645,7 @@ rl_ws2(int count, int key)
 	if (rl_line_buffer && *rl_line_buffer)
 		rl_delete_text(0, rl_end);
 
+	exec_prompt_cmds = 1;
 	if (cur_ws == 1) {
 		if (prev_ws != cur_ws) {
 			char t[16];
@@ -1651,8 +1655,7 @@ rl_ws2(int count, int key)
 		return EXIT_SUCCESS;
 	}
 
-	char cmd[] = "ws 2";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("ws 2");
 }
 
 static int
@@ -1662,6 +1665,7 @@ rl_ws3(int count, int key)
 	if (rl_line_buffer && *rl_line_buffer)
 		rl_delete_text(0, rl_end);
 
+	exec_prompt_cmds = 1;
 	if (cur_ws == 2) {
 		if (prev_ws != cur_ws) {
 			char t[16];
@@ -1671,8 +1675,7 @@ rl_ws3(int count, int key)
 		return EXIT_SUCCESS;
 	}
 
-	char cmd[] = "ws 3";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("ws 3");
 }
 
 static int
@@ -1682,6 +1685,7 @@ rl_ws4(int count, int key)
 	if (rl_line_buffer && *rl_line_buffer)
 		rl_delete_text(0, rl_end);
 
+	exec_prompt_cmds = 1;
 	if (cur_ws == 3) {
 		if (prev_ws != cur_ws) {
 			char t[16];
@@ -1691,8 +1695,7 @@ rl_ws4(int count, int key)
 		return EXIT_SUCCESS;
 	}
 
-	char cmd[] = "ws 4";
-	return run_kb_cmd(cmd);
+	return run_kb_cmd("ws 4");
 }
 
 static int
@@ -1742,8 +1745,8 @@ static int
 rl_launch_view(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	char cmd[] = "view";
-	return run_kb_cmd(cmd);
+	exec_prompt_cmds = 1;
+	return run_kb_cmd("view");
 }
 
 static int
@@ -1812,6 +1815,11 @@ print_highlight_string(char *s, const int insert_point)
 static int
 print_cmdhist_line(int n, int beg_line)
 {
+#ifndef _NO_SUGGESTIONS
+	if (wrong_cmd == 1)
+		recover_from_wrong_cmd();
+#endif /* !_NO_SUGGESTIONS */
+
 	curhistindex = (size_t)n;
 
 	HIDE_CURSOR;
@@ -2039,7 +2047,7 @@ rl_toggle_virtualdir_full_paths(int count, int key)
 	xchmod(stdin_tmp_dir, "0700", 1);
 	xargs.virtual_dir_full_paths = xargs.virtual_dir_full_paths == 1 ? 0 : 1;
 
-	int i = (int)files;
+	filesn_t i = files;
 	while (--i >= 0) {
 		char *rp = realpath(file_info[i].name, NULL);
 		if (!rp) continue;
@@ -2055,7 +2063,7 @@ rl_toggle_virtualdir_full_paths(int count, int key)
 		if (!p || !*p) continue;
 
 		if (renameat(XAT_FDCWD, file_info[i].name, XAT_FDCWD, p) == -1)
-			_err('w', PRINT_PROMPT, "renameat: %s: %s\n",
+			err('w', PRINT_PROMPT, "renameat: %s: %s\n",
 				file_info[i].name, strerror(errno));
 
 		if (xargs.virtual_dir_full_paths == 1) free(p);
@@ -2074,6 +2082,14 @@ rl_toggle_virtualdir_full_paths(int count, int key)
 	return EXIT_SUCCESS;
 }
 
+/* Used to disable keybindings. */
+static int
+do_nothing(int count, int key)
+{
+	UNUSED(count); UNUSED(key);
+	return EXIT_SUCCESS;
+}
+
 static void
 set_keybinds_from_file(void)
 {
@@ -2107,8 +2123,6 @@ set_keybinds_from_file(void)
 	rl_bind_keyseq(find_key("root-dir"), rl_dir_root);
 	rl_bind_keyseq(find_key("root-dir2"), rl_dir_root);
 	rl_bind_keyseq(find_key("root-dir3"), rl_dir_root);
-	rl_bind_keyseq(find_key("first-dir"), rl_dir_first);
-	rl_bind_keyseq(find_key("last-dir"), rl_dir_last);
 	rl_bind_keyseq(find_key("pinned-dir"), rl_dir_pinned);
 	rl_bind_keyseq(find_key("workspace1"), rl_ws1);
 	rl_bind_keyseq(find_key("workspace2"), rl_ws2);
@@ -2117,7 +2131,6 @@ set_keybinds_from_file(void)
 
 	/* Operations on files */
 	rl_bind_keyseq(find_key("create-file"), rl_create_file);
-	rl_bind_keyseq(find_key("bookmark-sel"), rl_bm_sel);
 	rl_bind_keyseq(find_key("archive-sel"), rl_archive_sel);
 	rl_bind_keyseq(find_key("open-sel"), rl_open_sel);
 	rl_bind_keyseq(find_key("export-sel"), rl_export_sel);
@@ -2208,8 +2221,6 @@ set_default_keybinds(void)
 	rl_bind_keyseq("\\e[H", rl_dir_home);
 	rl_bind_keyseq("\\M-r", rl_dir_root);
 	rl_bind_keyseq("\\e/", rl_dir_root);
-	rl_bind_keyseq("\\C-\\M-j", rl_dir_first);
-	rl_bind_keyseq("\\C-\\M-k", rl_dir_last);
 	rl_bind_keyseq("\\M-p", rl_dir_pinned);
 	rl_bind_keyseq("\\M-1", rl_ws1);
 	rl_bind_keyseq("\\M-2", rl_ws2);
@@ -2218,7 +2229,6 @@ set_default_keybinds(void)
 
 	/* Operations on files */
 	rl_bind_keyseq("\\M-n", rl_create_file);
-	rl_bind_keyseq("\\C-\\M-b", rl_bm_sel);
 	rl_bind_keyseq("\\C-\\M-a", rl_archive_sel);
 	rl_bind_keyseq("\\C-\\M-g", rl_open_sel);
 	rl_bind_keyseq("\\C-\\M-e", rl_export_sel);
@@ -2303,6 +2313,12 @@ set_hardcoded_keybinds(void)
 void
 readline_kbinds(void)
 {
+	/* Disable "Esc + Enter". Otherwise, it switches to vi mode, which is not
+	 * intended (for instance, in a secondary prompt).
+	 * Disable it here so that the user can rebind it using the config file
+	 * (readline.clifm or keybindings.clifm). */
+	rl_bind_keyseq("\x1b\xd", do_nothing);
+
 	if (kbinds_file)
 		set_keybinds_from_file();
 	else
