@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,11 +25,19 @@
 #ifndef KEYBINDS_H
 #define KEYBINDS_H
 
+__BEGIN_DECLS
+
 /*void add_func_to_rl(void); */
+int  keybind_exec_cmd(char *);
+int  kbinds_function(char **);
+int  load_keybinds(void);
 void readline_kbinds(void);
-int kbinds_function(char **args);
-int load_keybinds(void);
-int rl_quit(int count, int key);
-int keybind_exec_cmd(char *str);
+int  rl_quit(int, int);
+int  rl_toggle_dirs_first(int, int);
+int  rl_toggle_long_view(int, int);
+int  rl_toggle_hidden_files(int, int);
+int  rl_toggle_light_mode(int, int);
+
+__END_DECLS
 
 #endif /* KEYBINDS_H */
