@@ -52,8 +52,12 @@
 # include "profiles.h"
 #endif /* !_NO_PROFILES */
 #include "prompt.h"
+#include "properties.h" /* do_stat_and_exit() */
 #include "readline.h"
 #include "remotes.h"
+#ifdef SECURITY_PARANOID
+# include "sanitize.h"
+#endif /* SECURITY_PARANOID */
 
 /* Globals */
 
@@ -88,8 +92,11 @@ struct shades_t date_shades = {0};
 struct shades_t size_shades = {0};
 struct paths_t *paths = (struct paths_t *)NULL;
 struct ext_t *ext_colors = (struct ext_t *)NULL;
+#ifdef LINUX_FSINFO
+struct ext_mnt_t *ext_mnt = (struct ext_mnt_t *)NULL;
+#endif /* LINUX_FSINFO */
 
-const struct sort_t _sorts[] = {
+const struct sort_t sort_methods[] = {
     {"none", 0, 0},
     {"name", 1, 0},
     {"size", 2, 0},
@@ -131,6 +138,8 @@ unsigned short
 	term_lines = 0;
 
 regex_t regex_exp;
+regex_t regex_hist;
+regex_t regex_dirhist;
 
 /* Internal status flags */
 int
@@ -166,6 +175,7 @@ int
 	list_dirs_first = UNSET,
 	max_files = UNSET,
 	mime_match = 0,
+	nesting_level = 0,
 	no_log = 0,
 	open_in_foreground = 0,
 	prev_ws = UNSET,
@@ -190,6 +200,8 @@ int
 	wrong_cmd = 0,
 	xrename = 0;
 
+filesn_t files = 0;
+
 size_t
 	actions_n = 0,
 	aliases_n = 0,
@@ -202,7 +214,6 @@ size_t
 	current_hist_n = 0,
 	curhistindex = 0,
 	ext_colors_n = 0,
-	files = 0,
 	jump_n = 0,
 	kbinds_n = 0,
 	longest = 0,
@@ -235,7 +246,7 @@ char
 	finder_in_file[PATH_MAX + 1],
 	finder_out_file[PATH_MAX + 1],
 #endif /* _NO_FZF */
-	_fmatch[PATH_MAX + 1],
+	fz_match[PATH_MAX + 1],
 	prop_fields_str[PROP_FIELDS_SIZE + 1] = "",
 	invalid_time_str[MAX_TIME_STR] = "",
 
@@ -279,6 +290,7 @@ char
 /*	*right_prompt = (char *)NULL, */
 	*sel_file = (char *)NULL,
 	*smenutab_options_env = (char *)NULL,
+	*stat_filename = (char *)NULL,
 	*stdin_tmp_dir = (char *)NULL,
 	*sudo_cmd = (char *)NULL,
 #ifndef _NO_SUGGESTIONS
@@ -308,6 +320,7 @@ char
 char
 	/* File types */
 	bd_c[MAX_COLOR], /* Block device */
+	bk_c[MAX_COLOR], /* Backup files (ending with tilde) */
 	ca_c[MAX_COLOR], /* Cap file */
 	cd_c[MAX_COLOR], /* Char device */
 	di_c[MAX_COLOR], /* Directory */
@@ -321,9 +334,9 @@ char
 	nd_c[MAX_COLOR], /* No read directory */
 	nf_c[MAX_COLOR], /* No read file */
 	no_c[MAX_COLOR], /* Unknown */
-#ifdef __sun
-	oo_c[MAX_COLOR], /* Solaris door */
-#endif /* __sun */
+#ifdef SOLARIS_DOORS
+	oo_c[MAX_COLOR], /* Solaris door/port */
+#endif /* SOLARIS_DOORS */
 	or_c[MAX_COLOR], /* Broken symlink */
 	ow_c[MAX_COLOR], /* Other writable */
 	pi_c[MAX_COLOR], /* FIFO, pipe */
@@ -372,6 +385,7 @@ char
 	hr_c[MAX_COLOR], /* Redirection: > */
 	hs_c[MAX_COLOR], /* Process separators: | & ; */
 	hv_c[MAX_COLOR], /* Variables: $ */
+	hw_c[MAX_COLOR], /* Backslash (aka whack) */
 
 	dr_c[MAX_COLOR],  /* Read */
 	dw_c[MAX_COLOR],  /* Write */
@@ -401,7 +415,8 @@ char
 	li_cb[MAX_COLOR], /* Sel indicator color (for the files list) */
 	nm_c[MAX_COLOR + 2], /* Notice msg color */
 	wm_c[MAX_COLOR + 2], /* Warning msg color */
-	si_c[MAX_COLOR + 2], /* stealth indicator color */
+	ro_c[MAX_COLOR + 2], /* read-only indicator color */
+	si_c[MAX_COLOR + 2], /* stealth-mode indicator color */
 	ti_c[MAX_COLOR + 2], /* Trash indicator color */
 	tx_c[MAX_COLOR + 2], /* Text color */
 	xs_c[MAX_COLOR + 2], /* Exit code: success */
@@ -854,7 +869,8 @@ run_and_exit(void)
 	 4 - Grab user input again
 	 See https://brennan.io/2015/01/16/write-a-shell-in-c/
 */
-static inline void
+__attribute__ ((noreturn))
+static void
 run_main_loop(void)
 {
 #ifdef RUN_CMD
@@ -866,7 +882,7 @@ run_main_loop(void)
 	/* 1) Infinite loop to keep the program running */
 	while (1) {
 		/* 2) Grab input string from the prompt */
-		char *input = prompt();
+		char *input = prompt(PROMPT_SHOW);
 		if (!input)
 			continue;
 
@@ -906,14 +922,14 @@ static inline void
 set_root_indicator(void)
 {
 	if (user.uid == 0) {
-		_err(ERR_NO_LOG, PRINT_PROMPT, _("%s->%s Running as root%s\n"),
-			conf.colorize == 1 ? mi_c : "", conf.colorize == 1 ? _RED : "",
+		err(ERR_NO_LOG, PRINT_PROMPT, _("%s->%s Running as root%s\n"),
+			conf.colorize == 1 ? mi_c : "", conf.colorize == 1 ? BOLD_RED : "",
 			conf.colorize == 1 ? df_c : "");
 	}
 }
 
 static inline void
-_list(void)
+list_files(void)
 {
 #ifdef RUN_CMD
 	if (cmd_line_cmd)
@@ -925,13 +941,13 @@ _list(void)
 		/* Initialize inotify */
 		inotify_fd = inotify_init1(IN_NONBLOCK);
 		if (inotify_fd < 0) {
-			_err('w', PRINT_PROMPT, "%s: inotify: %s\n",
+			err('w', PRINT_PROMPT, "%s: inotify: %s\n",
 				PROGRAM_NAME, strerror(errno));
 		}
 #elif defined(BSD_KQUEUE)
 		kq = kqueue();
 		if (kq < 0) {
-			_err('w', PRINT_PROMPT, "%s: kqueue: %s\n",
+			err('w', PRINT_PROMPT, "%s: kqueue: %s\n",
 				PROGRAM_NAME, strerror(errno));
 		}
 #endif /* LINUX_INOTIFY */
@@ -944,7 +960,7 @@ _list(void)
 }
 
 static inline void
-_splash(void)
+print_splash_screen(void)
 {
 	if (conf.splash_screen) {
 		splash();
@@ -955,7 +971,7 @@ _splash(void)
 
 /* Set terminal window title */
 static inline void
-_set_term_title(void)
+set_term_win_title(void)
 {
 	if (!(flags & GUI) || xargs.list_and_quit == 1 || xargs.vt100 == 1)
 		return;
@@ -973,7 +989,7 @@ check_working_directory(void)
 {
 	if (workspaces == (struct ws_t *)NULL || !workspaces[cur_ws].path
 	|| !*workspaces[cur_ws].path) {
-		_err(0, NOPRINT_PROMPT, _("%s: Fatal error! Failure "
+		err(0, NOPRINT_PROMPT, _("%s: Fatal error! Failure "
 			"retrieving current working directory\n"), PROGRAM_NAME);
 		exit(EXIT_FAILURE);
 	}
@@ -985,7 +1001,7 @@ static inline void
 check_working_shell(void)
 {
 	if (access(user.shell, X_OK) == -1) {
-		_err('w', PRINT_PROMPT, _("%s: %s: System shell not found. "
+		err('w', PRINT_PROMPT, _("%s: %s: System shell not found. "
 			"Please edit the configuration file to specify a working "
 			"shell.\n"), PROGRAM_NAME, user.shell);
 	}
@@ -1009,7 +1025,7 @@ get_hostname(void)
 	if (gethostname(hostname, sizeof(hostname)) == -1) {
 		hostname[0] = '?';
 		hostname[1] = '\0';
-		_err('e', PRINT_PROMPT, _("%s: Error getting hostname\n"),
+		err('e', PRINT_PROMPT, _("%s: Error getting hostname\n"),
 			PROGRAM_NAME);
 	}
 }
@@ -1037,7 +1053,7 @@ set_locale(void)
 	/* Use the locale specified by the environment */
 	setlocale(LC_ALL, "");
 	if (strcmp(nl_langinfo(CODESET), "UTF-8") != 0) {
-		_err('w', PRINT_PROMPT, _("%s: Locale is not UTF-8. To avoid "
+		err('w', PRINT_PROMPT, _("%s: Locale is not UTF-8. To avoid "
 			"encoding issues you might want to set an UTF-8 locale. Ex: "
 			"export LANG=es_AR.UTF-8\n"), PROGRAM_NAME);
 	}
@@ -1055,6 +1071,30 @@ check_gui(void)
 	}
 }
 
+#ifdef SECURITY_PARANOID
+static void
+set_security_paranoid_mode(void)
+{
+# if SECURITY_PARANOID == 0
+	return;
+# elif SECURITY_PARANOID == 1
+	if (xargs.secure_env != 1 && xargs.secure_env_full != 1) {
+		xsecure_env(SECURE_ENV_IMPORT);
+		xargs.secure_env = 1;
+	}
+	xargs.secure_cmds = 1;
+# else
+	if (xargs.secure_env_full != 1)
+		xsecure_env(SECURE_ENV_FULL);
+	xargs.secure_cmds = xargs.secure_env_full = 1;
+	xargs.secure_env = UNSET;
+#  if SECURITY_PARANOID > 2
+	xargs.stealth_mode = 1;
+#  endif /* SECURITY_PARANOID > 2 */
+# endif /* SECURITY_PARANOID == 0 */
+}
+#endif /* SECURITY_PARANOID */
+
 /*
 static void
 init_file_flags(void)
@@ -1079,13 +1119,13 @@ main(int argc, char *argv[])
 	/* Quite unlikely to happen, but one never knows. See
 	 * https://lwn.net/SubscriberLink/882799/cb8f313c57c6d8a6/
 	 * and
-	 * https://stackoverflow.com/questions/49817316/can-argc-be-zero-on-a-posix-system*/
+	 * https://stackoverflow.com/questions/49817316/can-argc-be-zero-on-a-posix-system */
 	if (argc == 0) {
 		fprintf(stderr, "%s: %s\n", PROGRAM_NAME, strerror(EINVAL));
 		exit(EINVAL);
 	}
 
-	check_term(); /* Let's check terminal capabilities */
+	check_term(); /* Let's check terminal capabilities. */
 
 	/* # 1. INITIALIZE EVERYTHING WE NEED # */
 
@@ -1102,29 +1142,32 @@ main(int argc, char *argv[])
 	/* Store external arguments to be able to rerun parse_cmdline_args()
 	 * in case the user edits the config file, in which case the program
 	 * must rerun init_config(), get_aliases(), get_prompt_cmds(), and
-	 * then parse_cmdline_args() */
+	 * then parse_cmdline_args(). */
 	backup_argv(argc, argv);
 
-	atexit(free_stuff); /* free_stuff does some cleaning */
+	atexit(free_stuff); /* free_stuff does some cleaning. */
 
 	user = get_user_data();
 	get_home();
 
 	check_gui();
 
-	P_tmpdir_len = strlen(P_tmpdir);
+	P_tmpdir_len = sizeof(P_tmpdir) - 1;
 	init_workspaces();
 
-	/* Set all external arguments flags to uninitialized state */
+	/* Set all external arguments flags to uninitialized state. */
 	unset_xargs();
 
-	/* Manage external arguments
-	 * Command line arguments will override initialization values (init_config) */
+	/* Manage external arguments.
+	 * Command line arguments will override initialization values (init_config). */
 	if (argc > 1)
 		parse_cmdline_args(argc, argv);
 	/* parse_cmdline_args is executed before init_config() because, if
 	 * specified (-P option), it sets the value of alt_profile, which
 	 * is then checked by init_config(). */
+#ifdef SECURITY_PARANOID
+	set_security_paranoid_mode();
+#endif /* SECURITY_PARANOID */
 
 	/* Get paths from PATH environment variable. These paths will be
 	 * used later by get_path_programs (for the autocomplete function)
@@ -1139,31 +1182,36 @@ main(int argc, char *argv[])
 	/* Initialize program paths and files, set options from the config
 	 * file, if they were not already set via external arguments, and
 	 * load sel elements, if any. All these configurations are made
-	 * per user basis */
+	 * per user basis. */
 	init_config();
 	check_options();
 
+	if (xargs.stat > 0) /* Running with --stat(-full). Print and exit. */
+		do_stat_and_exit(xargs.stat == FULL_STAT ? 1 : 0);
+
+	if (xargs.list_and_quit == 1) {
+		list_files();
+		exit(EXIT_SUCCESS); /* Never reached */
+	}
+
 	set_sel_file();
 	create_tmp_files();
-#ifndef _NO_FZF
-	set_finder_paths();
-#endif /* _NO_FZF */
 	load_actions();
 	get_aliases();
 
 	/* Get the list of available programs in PATH to be used by the
-	 * custom TAB-completion function (tab_complete(), in tabcomp.c) */
+	 * custom TAB-completion function (tab_complete(), in tabcomp.c). */
 	if (!(flags & PATH_PROGRAMS_ALREADY_LOADED))
 		get_path_programs();
 
 	/* Check third-party programs availability: finders (fzf, fnf, smenu),
-	 * udevil, and udisks2 */
+	 * udevil, and udisks2. */
 	check_third_party_cmds();
 #ifndef _NO_FZF
 	check_completion_mode();
 #endif /* _NO_FZF */
 
-	/* Initialize gettext() for translations */
+	/* Initialize gettext() for translations. */
 #ifndef _NO_GETTEXT
 	init_gettext();
 #endif /* !_NO_GETTEXT */
@@ -1172,23 +1220,23 @@ main(int argc, char *argv[])
 	fflush(stdout);
 
 #ifndef __HAIKU__
-	/* No need for this warning on Haiku: it runs as root by default */
+	/* No need for this warning on Haiku: it runs as root by default. */
 	set_root_indicator();
 #endif /* !__HAIKU__ */
 
 	load_remotes();
 	automount_remotes();
-	_splash();
+	print_splash_screen();
 	set_start_path();
 	check_working_directory();
-	_set_term_title();
+	set_term_win_title();
 	exec_profile();
 	load_dirhist();
 	add_to_dirhist(workspaces[cur_ws].path);
 	get_sel_files();
 
-	/* Start listing as soon as possible to speed up startup time */
-	_list();
+	/* Start listing as soon as possible to speed up startup time. */
+	list_files();
 
 	shell = get_sys_shell();
 	create_kbinds_file();
@@ -1208,12 +1256,12 @@ main(int argc, char *argv[])
 
 	get_hostname();
 	init_shell();
-	set_env();
+	set_env(0);
 
 	if (config_ok == 1)
 		init_history();
 
-	/* Store history into an array to be able to manipulate it */
+	/* Store history into an array to be able to manipulate it. */
 	get_history();
 
 #ifndef _NO_PROFILES
@@ -1225,6 +1273,4 @@ main(int argc, char *argv[])
 
 	/* # 2. MAIN PROGRAM LOOP # */
 	run_main_loop();
-
-	return exit_code; /* Never reached */
 }
