@@ -22,6 +22,8 @@
  * MA 02110-1301, USA.
 */
 
+#ifndef NO_MEDIA_FUNC
+
 #include "helpers.h"
 
 #include <stdio.h>
@@ -51,8 +53,6 @@
 # endif /* MNTTAB */
 #endif /* __linux__ || __CYGWIN__ */
 
-#include <dirent.h>
-
 #include "aux.h"
 #include "readline.h"
 #include "navigation.h"
@@ -692,3 +692,7 @@ EXIT:
 	free_media();
 	return exit_status;
 }
+
+#else
+void *_skip_me_media;
+#endif /* !NO_MEDIA_FUNC */
