@@ -26,8 +26,7 @@
 #define MESSAGES_H
 
 /* Usage messages */
-#define GRAL_USAGE "[-aAeEfFgGhHiIlLmoOprsStuUvwWxy] [-b FILE] [-c FILE] [-D DIR] \
-[-k FILE] [-P PROFILE] [-z METHOD] [PATH]"
+# define GRAL_USAGE "[OPTIONS] [DIR]"
 
 #define ACTIONS_USAGE "List or edit actions/plugins\n\n\
 \x1b[1mUSAGE\x1b[0m\n\
@@ -521,7 +520,9 @@ You can also access the commands history via the exclamation mark (!)\n\
 - Execute the command number 'n' in the history list\n\
     !n\n\
 - Execute the 'last - n' command in the history list\n\
-    !-n"
+    !-n\n\n\
+Tip: Use HistIgnore in the configuration file to exclude command lines\n\
+from the history list"
 
 #define ICONS_USAGE "Set icons on/off\n\n\
 \x1b[1mUSAGE\x1b[0m\n\
@@ -855,11 +856,10 @@ whatever prompt you like."
     p sel\n\
 - Print the properties of the directory 'dir' (including total size)\n\
     pp dir\n\n\
-Note that, in case of symbolic links to directories, this command displays\n\
+Note that, in case of symbolic links to directories, the 'p' command displays\n\
 information about the link target if the provided file name ends with a slash.\n\
-Otherwise, information about the link itself is displayed.\n\n\
-Secondly, and unlike 'p', which always provides information about links\n\
-themselves, 'pp' always follows symlinks to their target file."
+Otherwise, information about the link itself is displayed.\n\
+Unlike 'p', however, 'pp' always follows symlinks to their target file."
 
 #define PWD_DESC "Print the name of the current working directory\n\n\
 \x1b[1mUSAGE\x1b[0m\n\
@@ -1110,6 +1110,66 @@ Note: Use the 'trash' command to trash files. Try 'trash --help'"
     view edit vi\n\n\
 Enter 'help file-previews' for more information"
 
+#define WRAPPERS_USAGE "c, m, md, and r commands are wrappers \
+for cp(1), mv(1),\nmkdir(1), and rm(1) shell commands respectively.\n\n\
+\x1b[1mUSAGE\x1b[0m\n\
+c  -> cp -iRp\n\
+m  -> mv -i\n\
+md -> mkdir -p\n\
+r  -> rm -f ('-rf' for directories) (1)\n\n\
+(1) The user is asked for confirmation if the list of files contains:\n\
+a. At least one directory\n\
+b. Three or more files\n\
+c. At least one non-explicitly-expanded ELN (ex: 'r 12')\n\n\
+The 'paste' command is equivalent to 'c' and exists only for semantic\n\
+reasons. For example, if you want to copy selected files into the current\n\
+directory, it makes sense to write 'paste sel'.\n\n\
+By default, both the 'c' and 'm' commands run cp(1)/mv(1) interactively\n\
+(-i), i.e. prompting before overwriting a file. To run non-interactivelly\n\
+instead, use the -f,--force parameter (see the examples below). You can\n\
+also permanently run in non-interactive mode using the cpCmd/mvCmd options\n\
+in the configuration file ('config' or F10).\n\n\
+Just as 'c' and 'm', the 'r' command accepts -f,--force as paramater to\n\
+prevent 'r' from prompting before removals. Set rmForce to true in the\n\
+configuration file to make this option permanent.\n\n\
+To use different parameters, run the corresponding utility, as usual.\n\
+Example: cp -abf ...\n\n\
+\x1b[1mEXAMPLES\x1b[0m\n\
+- Create a copy of file1 named file2\n\
+    c file1 file2\n\
+- Create a copy of file1 in the directory dir1 named file2\n\
+    c file1 dir1/file2\n\
+- Copy all selected files into the current directory\n\
+    c sel\n\
+  Note: If destiny directory is omitted, the current directory is assumed\n\
+- Copy all selected files into the current directory (non-interactively):\n\
+    c -f sel\n\
+- Move all selected files into the directory named testdir\n\
+    m sel testdir\n\
+- Rename 'file1' as 'file_test'\n\
+    m file1 file_test\n\
+- Interactively rename 'file1'\n\
+    m file1\n\
+  Note: The user is prompted to enter a new name using the old name as\n\
+  template\n\
+- Move all selected files into the current directory (non-interactively)\n\
+    m -f sel\n\
+- Remove all selected files\n\
+    r sel\n\
+- Remove all selected files (non-interactively)\n\
+    r -f sel\n\
+  Note: Use the 't' command to send files to the trash can. Try 't --help'.\n\
+- Create a directory named 'mydir'\n\
+    md mydir\n\
+  Note: To create files and directories you can use the 'n' command as\n\
+  well. Try 'n --help'.\n\n\
+Use the 'vv' command to copy files into a directory and bulk rename them\n\
+at once. Try 'vv --help'.\n\n\
+Use the 'cr' plugin to send a file to a remote location:\n\
+    cr FILE (run 'cr --edit' before to set up your remotes)\n\n\
+Use the 'l' command to create symbolic links, and 'le' to edit them."
+
+/*
 #define WRAPPERS_USAGE "c, m, md, and r commands are wrappers \
 for cp(1), mv(1),\nmkdir(1), and rm(1) shell commands respectively.\n\n\
 \x1b[1mUSAGE\x1b[0m\n\
@@ -1170,7 +1230,7 @@ Use the 'vv' command to copy files into a directory and bulk rename them\n\
 at once. Try 'vv --help'.\n\n\
 Use the 'cr' plugin to send a file to a remote location:\n\
     cr FILE (run 'cr --edit' before to set up your remotes)\n\n\
-Use the 'l' command to create symbolic links, and 'le' to edit them."
+Use the 'l' command to create symbolic links, and 'le' to edit them." */
 
 #define WS_USAGE "Switch workspaces\n\n\
 \x1b[1mUSAGE\x1b[0m\n\
@@ -1218,6 +1278,58 @@ Page Down: Advance one page\nq: Stop pagging\n"
 # define HELP_MESSAGE "Enter '?' or press F5-F7 for instructions"
 #endif /* !__HAIKU__ */
 
+#ifdef _BE_POSIX
+#define OPTIONS_LIST "\
+\n -a       List hidden files\
+\n -A       Do not list hidden files\
+\n -b FILE  Set an alternative bookmarks file\
+\n -B NUM   Set TAB-completion mode (NUM=[0-3])\
+\n -c FILE  Set an alternative main configuration file\
+\n -C       Do not clear the screen when changing directories\
+\n -d       Print disk usage (free/total)\
+\n -D       List directories only\
+\n -e       Force the use of the 'o/open' command to open files\
+\n -E       Force the use of 'cd' to change directories\
+\n -f       Print full directories size (long view mode only)\
+\n -F       Disable the files counter (directories)\
+\n -g       Print file sizes in powers of 1000 instead of 1024\
+\n -G       Print file sizes as used blocks instead of used bytes\
+\n -h       Print this help and exit\
+\n -H       Disable syntax-highlighting\
+\n -i       Enable icons\
+\n -j FILE  Run the 'p' command on FILE and exit\
+\n -J FILE  Run the 'pp' command on FILE and exit\
+\n -k FILE  Set an alternative keybindings file\
+\n -l       List files in long view mode\
+\n -L       Do not follow symbolic links\
+\n -m       Enable fuzzy matching\
+\n -M       Disable colors\
+\n -n       Disable commands history\
+\n -N       Disable bold colors\
+\n -o PATH  Set a custom resource opener (instead of the built-in Lira)\
+\n -O FILE  Open FILE (via Lira) and exit\
+\n -p NAME  Set/create the profile NAME\
+\n -P FILE  Generate a preview of FILE and exit\
+\n -q       List files and quit\
+\n -Q       Enable cd-on-quit (consult the manpage)\
+\n -r       Files removed via the 'r' command are sent to the trash can\
+\n -R       Do not append file type indicators\
+\n -s       Run in stealth mode (incognito)\
+\n -S       Disable suggestions\
+\n -t NAME  Use the color scheme NAME\
+\n -T       Do not trim file names\
+\n -u       Run in disk usage analyzer mode\
+\n -v       Print version information and exit\
+\n -V DIR   Set a custom virtual directory\
+\n -w NUM   Start in workspace NUM (1-8)\
+\n -W       Keep the list of selected files in sight\
+\n -x       Run in secure-environment mode\
+\n -X       Run in secure-environment mode (paranoid)\
+\n -y       Run in light mode\
+\n -Y       Run in secure-commands mode\
+\n -z NUM   Set a files sorting method (NUM=[1-11])\
+\n -Z NUM   List only up to NUM files"
+#else
 #define SHORT_OPTIONS "\
 \n  -a, --no-hidden\t\t Do not show hidden files (default)\
 \n  -A, --show-hidden\t\t Show hidden files\
@@ -1311,6 +1423,7 @@ and paths\
 CliFM's built-in opener\
 \n      --preview=FILE\t\t Display a preview of FILE (via Shotgun) and exit\
 \n      --print-sel\t\t Keep the list of selected files in sight\
+\n      --readonly\t\t Disable internal commands able to modify the file system\
 \n      --rl-vi-mode\t\t Set readline to vi editing mode (defaults to emacs mode)\
 \n      --secure-cmds\t\t Filter commands to prevent command injection\
 \n      --secure-env\t\t Run in a sanitized environment (regular mode)\
@@ -1323,12 +1436,15 @@ CliFM's built-in opener\
       --si\t\t\t Print sizes in powers of 1000 instead of 1024\
 \n      --smenutab\t\t Use smenu to display completion matches\
 \n      --sort-reverse\t\t Sort in reverse order, e.g. z-a instead of a-z\
+\n      --stat=FILE\t\t Run the 'p' command on FILE and exit\
+\n      --stat-full=FILE\t\t Run the 'pp' command on FILE and exit\
 \n      --stdtab\t\t\t Force the use of the standard TAB completion mode (readline)\
 \n      --trash-as-rm\t\t The 'r' command executes 'trash' instead of \
 rm(1) to prevent accidental deletions\
 \n      --virtual-dir-full-paths\t Files in virtual directories are listed as full paths instead of target base names\
 \n      --virtual-dir=PATH\t Absolute path to a directory to be used as virtual directory\
 \n      --vt100\t\t\t Run in vt100 compatibility mode\n"
+#endif /* _BE_POSIX */
 
 #define CLIFM_COMMANDS_HEADER "\
 For a complete description of all the below \
@@ -1434,6 +1550,7 @@ For more information about a specific command run 'CMD -h' or 'CMD --help'.\n"
  M-l           Toggle long/detail view mode on/off\n\
  M-.           Toggle hidden files on/off\n\
  M-,           Toggle list only directories on/off\n\
+ M--           Preview files in the current directory (requires fzf)\n\
  M-m           List mountpoints\n\
  M-h           Show directory history\n\
  M-t           Clear messages\n\
@@ -1451,8 +1568,6 @@ For more information about a specific command run 'CMD -h' or 'CMD --help'.\n"
  M-p           Change to pinned directory\n\
  M-v           Toggle prepend sudo\n\
  M-[1-4]       Switch to workspace 1-4\n\
- C-M-j         Change to first visited directory\n\
- C-M-k         Change to last visited directory\n\
  C-M-o         Switch to previous profile\n\
  C-M-p         Switch to next profile\n\
  C-M-a         Archive selected files\n\
@@ -1461,7 +1576,6 @@ For more information about a specific command run 'CMD -h' or 'CMD --help'.\n"
  C-M-d         Remove selected files\n\
  C-M-t         Trash selected files\n\
  C-M-u         Restore trashed files\n\
- C-M-b         Bookmark last selected file or directory\n\
  C-M-g         Open/change-into last selected file/directory\n\
  C-M-n         Move selected files into the current working directory\n\
  C-M-v         Copy selected files into the current working directory\n\
