@@ -154,8 +154,8 @@ skip_line(char *line, char **pattern, char **cmds)
 	return 0;
 }
 
-/* Test PATTERN against either FILENAME or the mime-type MIME
- * Returns zero in case of a match, and 1 otherwise */
+/* Test PATTERN against either FILENAME or the mime-type MIME.
+ * Returns zero in case of a match, and 1 otherwise. */
 static int
 test_pattern(const char *pattern, const char *filename, const char *mime)
 {
@@ -164,7 +164,8 @@ test_pattern(const char *pattern, const char *filename, const char *mime)
 
 	if (filename && (*pattern == 'N' || *pattern == 'E')
 	&& *(pattern + 1) == ':') {
-		if (regcomp(&regex, pattern + 2, REG_NOSUB | REG_EXTENDED) == 0
+		if (regcomp(&regex, pattern + 2,
+		REG_NOSUB | REG_EXTENDED | REG_ICASE) == 0
 		&& regexec(&regex, filename, 0, NULL, 0) == 0)
 			ret = EXIT_SUCCESS;
 	} else {
@@ -179,7 +180,7 @@ test_pattern(const char *pattern, const char *filename, const char *mime)
 	return ret;
 }
 
-/* Return 1 if APP is a valid and existent application. Zero otherwise */
+/* Return 1 if APP is a valid and existent application. Zero otherwise. */
 static int
 check_app_existence(char **app, char **arg)
 {
@@ -287,7 +288,7 @@ retrieve_app(char *line)
 
 /* Get application associated to a given MIME type or file name.
  * Returns the first matching line in the MIME file or NULL if none is
- * found */
+ * found. */
 static char *
 get_app(const char *mime, const char *filename)
 {
@@ -334,19 +335,14 @@ xmagic(const char *file, const int query_mime)
 		return (char *)NULL;
 
 	magic_t cookie = magic_open(query_mime ? (MAGIC_MIME_TYPE | MAGIC_ERROR)
-					: MAGIC_ERROR);
-	if (!cookie) {
-//		fprintf(stderr, "%s: xmagic: %s\n", PROGRAM_NAME, strerror(errno));
+		: MAGIC_ERROR);
+	if (!cookie)
 		return (char *)NULL;
-	}
 
 	magic_load(cookie, NULL);
 
 	const char *mime = magic_file(cookie, file);
 	if (!mime) {
-/*		const char *err_str = magic_error(cookie);
-		if (err_str)
-			fprintf(stderr, "%s: xmagic: %s\n", PROGRAM_NAME, err_str); */
 		magic_close(cookie);
 		return (char *)NULL;
 	}
@@ -365,53 +361,53 @@ get_mime(char *file)
 		return (char *)NULL;
 	}
 
-	char *rand_ext = gen_rand_str(6);
+	char *mime_type = (char *)NULL;
+	char *rand_ext = gen_rand_str(10);
 
-	char mime_tmp_file[PATH_MAX];
-	snprintf(mime_tmp_file, sizeof(mime_tmp_file), "%s/mime.%s", tmp_dir,
-		rand_ext ? rand_ext : "Hu?+65");
+	char tmp_file[PATH_MAX + 1];
+	snprintf(tmp_file, sizeof(tmp_file), "%s/mime.%s", tmp_dir,
+		rand_ext ? rand_ext : "Hu?+6545jk");
 	free(rand_ext);
 
-	if (access(mime_tmp_file, F_OK) == 0)
-		unlink(mime_tmp_file);
-
 	int fd = 0;
-	FILE *file_fp = open_fwrite(mime_tmp_file, &fd);
-	if (!file_fp) {
-		xerror("%s: fopen: %s: %s\n", err_name, mime_tmp_file, strerror(errno));
+	FILE *fp_out = open_fwrite(tmp_file, &fd);
+	if (!fp_out) {
+		xerror("%s: fopen: %s: %s\n", err_name, tmp_file, strerror(errno));
 		return (char *)NULL;
 	}
 
-	FILE *file_fp_err = fopen("/dev/null", "w");
-	if (!file_fp_err) {
+	FILE *fp_err = fopen("/dev/null", "w");
+	if (!fp_err) {
 		xerror("%s: /dev/null: %s\n", err_name, strerror(errno));
-		fclose(file_fp);
-		return (char *)NULL;
+		goto END;
 	}
 
 	int stdout_bk = dup(STDOUT_FILENO); /* Store original stdout */
 	int stderr_bk = dup(STDERR_FILENO); /* Store original stderr */
 
+	if (stdout_bk == -1 || stderr_bk == -1)
+		goto ERROR;
+
 	/* Redirect stdout to the desired file */
-	if (dup2(fileno(file_fp), STDOUT_FILENO) == -1) {
-		xerror("%s: %s\n", err_name, strerror(errno));
-		fclose(file_fp);
-		fclose(file_fp_err);
-		return (char *)NULL;
-	}
+	if (dup2(fileno(fp_out), STDOUT_FILENO) == -1)
+		goto ERROR;
 
 	/* Redirect stderr to /dev/null */
-	if (dup2(fileno(file_fp_err), STDERR_FILENO) == -1) {
-		xerror("%s: %s\n", err_name, strerror(errno));
-		fclose(file_fp);
-		fclose(file_fp_err);
-		return (char *)NULL;
-	}
+	if (dup2(fileno(fp_err), STDERR_FILENO) == -1)
+		goto ERROR;
 
-	fclose(file_fp);
-	fclose(file_fp_err);
+	fclose(fp_out);
+	fclose(fp_err);
 
-	char *cmd[] = {"file", "--mime-type", file, NULL};
+/* --mime-type is only available since file 4.24 (Mar, 2008), while the -i
+ * flag (-I in MacOS) is supported since 3.30 (Apr, 2000).
+ * NOTE: the -i flag in the POSIX file(1) specification is a completely
+ * different thing. */
+#ifdef __APPLE__
+	char *cmd[] = {"file", "-bI", file, NULL};
+#else
+	char *cmd[] = {"file", "-bi", file, NULL};
+#endif /* __APPLE__ */
 	int ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
 
 	dup2(stdout_bk, STDOUT_FILENO); /* Restore original stdout */
@@ -419,41 +415,47 @@ get_mime(char *file)
 	close(stdout_bk);
 	close(stderr_bk);
 
-	if (ret != EXIT_SUCCESS)
-		return (char *)NULL;
-
-	if (access(mime_tmp_file, F_OK) != 0)
-		return (char *)NULL;
-
-	file_fp = fopen(mime_tmp_file, "r");
-	if (!file_fp) {
-		unlink(mime_tmp_file);
+	if (ret != EXIT_SUCCESS) {
+		unlink(tmp_file);
 		return (char *)NULL;
 	}
 
-	char *mime_type = (char *)NULL;
-
-	char line[255] = "";
-	if (fgets(line, (int)sizeof(line), file_fp) == NULL) {
-		fclose(file_fp);
-		unlink(mime_tmp_file);
+	fp_out = fopen(tmp_file, "r");
+	if (!fp_out) {
+		unlink(tmp_file);
 		return (char *)NULL;
 	}
 
-	char *tmp = strrchr(line, ' ');
-	if (tmp) {
-		size_t len = strlen(tmp);
-		if (len > 0 && tmp[len - 1] == '\n') {
-			tmp[len - 1] = '\0';
-			len--;
-		}
-		mime_type = len > 0 ? savestring(tmp + 1, len - 1) : (char *)NULL;
+	char line[NAME_MAX] = "";
+	if (fgets(line, (int)sizeof(line), fp_out) == NULL)
+		goto END;
+
+	char *s = strrchr(line, ';');
+	if (s)
+		*s = '\0';
+
+	size_t len = strlen(line);
+	if (len > 0 && line[len - 1] == '\n') {
+		line[len - 1] = '\0';
+		len--;
 	}
 
-	fclose(file_fp);
-	unlink(mime_tmp_file);
+	mime_type = len > 0 ? savestring(line, len) : (char *)NULL;
+
+END:
+	fclose(fp_out);
+	unlink(tmp_file);
 
 	return mime_type;
+
+ERROR:
+	xerror("%s: %s\n", err_name, strerror(errno));
+	fclose(fp_out);
+	fclose(fp_err);
+	unlink(tmp_file);
+	close(stdout_bk);
+	close(stderr_bk);
+	return (char *)NULL;
 }
 #endif /* !_NO_MAGIC */
 
@@ -516,7 +518,8 @@ mime_import(char *file)
 	const char *const mime_paths[] = {config_path, local_path,
 	    "/usr/local/share/applications/mimeapps.list",
 	    "/usr/share/applications/mimeapps.list",
-	    "/etc/xdg/mimeapps.list", NULL};
+	    "/etc/xdg/mimeapps.list",
+	    NULL};
 
 	/* Check each mimeapps.list file and store its associations into FILE */
 	size_t i;
@@ -748,7 +751,6 @@ run_mime_app(char **app, char **fpath)
 	if (!cmd) {
 		free(*app);
 #ifndef __CYGWIN__
-	// fpath is an argument passed to mime_open(), which should not be freed
 		free(*fpath);
 #endif /* !__CYGWIN__ */
 		return EXIT_FAILURE;
@@ -776,13 +778,10 @@ run_mime_app(char **app, char **fpath)
 
 	free(*app);
 #ifndef __CYGWIN__
-	// fpath is an argument passed to mime_open(), which should not be freed
 	free(*fpath);
 #endif /* !__CYGWIN */
 
-	if (ret == EXIT_SUCCESS)
-		return EXIT_SUCCESS;
-	return EXIT_FAILURE;
+	return (ret != EXIT_SUCCESS) ? EXIT_FAILURE : ret;
 }
 
 static int
@@ -946,6 +945,11 @@ mime_open_with_tab(char *filename, const char *prefix, const int only_names)
 		if (!ename)
 			return (char **)NULL;
 		name = ename;
+	} else {
+		if (*filename == '\'' || *filename == '"') {
+			char *tmp_name = savestring(filename + 1, strlen(filename + 1));
+			name = remove_quotes(tmp_name);
+		}
 	}
 
 	if (strchr(name ? name : filename, '\\')) {
@@ -1543,9 +1547,9 @@ mime_open_url(char *url)
 static int
 import_mime(void)
 {
-	char *suffix = gen_rand_str(6);
+	char *suffix = gen_rand_str(10);
 	char new[PATH_MAX];
-	snprintf(new, sizeof(new), "%s.%s", mime_file, suffix ? suffix : "5i0TM#");
+	snprintf(new, sizeof(new), "%s.%s", mime_file, suffix ? suffix : "5i0TM#r3j&");
 	free(suffix);
 
 	int mime_defs = mime_import(new);
