@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,11 +25,14 @@
 #ifndef MIME_H
 #define MIME_H
 
-/* mime.c */
-int mime_open(char **args);
-char *xmagic(const char *file, const int mime);
-int mime_open_with(char *filename, char **args);
-char **mime_open_with_tab(char *filename, const char *prefix);
-int mime_open_url(char *url);
+__BEGIN_DECLS
+
+int  mime_open(char **);
+char *xmagic(const char *, const int);
+int  mime_open_with(char *filename, char **);
+char **mime_open_with_tab(char *, const char *, const int);
+int  mime_open_url(char *);
+
+__END_DECLS
 
 #endif /* MIME_H */
