@@ -22,7 +22,7 @@
  * MA 02110-1301, USA.
 */
 
-/* The _err function is based on littlstar's asprintf implementation
+/* The err function is based on littlstar's asprintf implementation
  * (https://github.com/littlstar/asprintf.c/blob/master/asprintf.c),
  * licensed under MIT.
  * All changes are licensed under GPL-2.0-or-later. */
@@ -30,13 +30,16 @@
 #include "helpers.h"
 
 #include <errno.h>
-#include <fcntl.h> /* AT_FDCWD macro */
 #include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/ioctl.h>
 
+#ifdef __sun
+# include <sys/termios.h> /* TIOCGWINSZ */
+#endif /* __sun */
+
 #if defined(__NetBSD__) || defined(__FreeBSD__)
 # include <sys/param.h>
 # include <sys/sysctl.h>
@@ -69,6 +72,22 @@ typedef char *rl_cpvfunc_t;
 #include "messages.h"
 #include "file_operations.h"
 
+char *
+gen_diff_str(const int diff)
+{
+	if (diff == 1)
+		return "\x1b[1C";
+	if (diff == 2)
+		return "\x1b[2C";
+	if (diff == 3)
+		return "\x1b[3C";
+
+	static char diff_str[14];
+	snprintf(diff_str, sizeof(diff_str), "\x1b[%dC", diff);
+
+	return diff_str;
+}
+
 int
 is_blank_name(const char *s)
 {
@@ -179,7 +198,7 @@ __attribute__((__format__(__printf__, 3, 0)))
 /* We use __attribute__ here to silence clang warning: "format string is
  * not a string literal" */
 int
-_err(const int msg_type, const int prompt_flag, const char *format, ...)
+err(const int msg_type, const int prompt_flag, const char *format, ...)
 {
 	va_list arglist, tmp_list;
 
@@ -276,7 +295,7 @@ reset_inotify(void)
 		close(inotify_fd);
 	inotify_fd = inotify_init1(IN_NONBLOCK);
 	if (inotify_fd < 0) {
-		_err('w', PRINT_PROMPT, "%s: inotify: %s\n",
+		err('w', PRINT_PROMPT, "%s: inotify: %s\n",
 			PROGRAM_NAME, strerror(errno));
 		return;
 	}
@@ -290,7 +309,7 @@ reset_inotify(void)
 	if (inotify_wd > 0)
 		watch = 1;
 	else
-		_err('w', PRINT_PROMPT, "%s: inotify: %s: %s\n",
+		err('w', PRINT_PROMPT, "%s: inotify: %s: %s\n",
 			PROGRAM_NAME, rpath, strerror(errno));
 }
 
@@ -345,7 +364,7 @@ read_inotify(void)
 
 		/* A file was renamed */
 		if (event->mask & IN_MOVED_TO) {
-			int j = (int)files;
+			filesn_t j = files;
 			while (--j >= 0) {
 				if (*file_info[j].name == *event->name
 				&& strcmp(file_info[j].name, event->name) == 0)
@@ -486,7 +505,7 @@ validate_file_type_filter(void)
 	|| !*(filter.str + 1) || *(filter.str + 2))
 		return EXIT_FAILURE;
 
-	char c = *(filter.str + 1);
+	const char c = *(filter.str + 1);
 	if (c == 'b' || c == 'c' || c == 'd' || c == 'f'
 	|| c == 'l' || c == 'p' || c == 's')
 		return EXIT_SUCCESS;
@@ -505,8 +524,9 @@ static int
 compile_filter(void)
 {
 	if (filter.type == FILTER_FILE_NAME) {
-		if (regcomp(&regex_exp, filter.str, REG_NOSUB | REG_EXTENDED) != EXIT_SUCCESS) {
-			xerror("%s\n", _("ft: Invalid regular expression"));
+		int ret = regcomp(&regex_exp, filter.str, REG_NOSUB | REG_EXTENDED);
+		if (ret != EXIT_SUCCESS) {
+			xregerror("ft", filter.str, ret, regex_exp, 0);
 			regfree(&regex_exp);
 			goto ERR;
 		}
@@ -577,10 +597,39 @@ void
 print_tips(const int all)
 {
 	const char *TIPS[] = {
+#ifndef _BE_POSIX
+		"Add a new entry to the mimelist file: 'mm edit' or F6",
+		"Get mime information for a file. Ex: 'mm info file'",
+		"Manage default associated applications with the 'mime' command",
+		"Customize previewing applications: 'view edit' or F7",
+		"List mountpoints: 'mp' or Alt-m",
+		"Compress and decompress files using the 'ac' and 'ad' "
+		"commands respectively. Ex: 'ac sel' or 'ad file.zip'",
+		"Disallow the use of shell commands with the -x option: 'clifm -x'",
+		"Don't like ELN's? Disable them using the -e command line switch",
+		"Disable file previews for TAB completion (fzf mode only) via --no-fzfpreview",
+# ifdef __linux__
+		"Manage removable devices via the 'media' command",
+# endif /* __linux__ */
+#endif /* _BE_POSIX */
+#ifndef _BE_POSIX
+		"Create a new profile: 'pf add PROFILE' or 'clifm -P PROFILE'",
+		"Run in incognito/private mode (-S, --stealth-mode)",
+		"Run in read-only mode via --readonly",
+		"Running in an untrusted environment? Try the --secure-env and "
+		"--secure-cmds command line switches",
+		"Run in disk usage analyzer mode using the -t command line switch",
+		"Fuzzy suggestions are supported. Ex: 'dwn > Downloads'. Enable them via --fuzzy-matching, or FuzzyMatching in the configuration file",
+#else
+		"Create a new profile: 'pf add PROFILE' or 'clifm -p PROFILE'",
+		"Run in incognito/private mode via the -s flag",
+		"Running in an untrusted environment? Try the -x, -X, and -Y flags",
+		"Run in disk usage analyzer mode using the -u flag",
+		"Fuzzy suggestions are supported. Ex: 'dwn > Downloads'. Enable them via the -m flag (or FuzzyMatching in the configuration file)",
+#endif /* _BE_POSIX */
 		"Clear the screen: 'rf', '.', Enter (on empty line), or Ctrl-l",
 		"Try the autocd and auto-open functions: run 'FILE' instead "
 		"of 'cd FILE' or 'open FILE'",
-		"Add a new entry to the mimelist file: 'mm edit' or F6",
 		"Do not forget to take a look at the manpage",
 		"Need more speed? Try the light mode ('lm' or Alt-y)",
 		"The Selection Box is shared among different instances of CliFM",
@@ -594,7 +643,6 @@ print_tips(const int all)
 		"Press TAB to automatically expand an ELN. Ex: 's 2<TAB>' -> 's FILENAME'",
 		"Use ranges (ELN-ELN) to easily move multiple files. Ex: 'm 3-12 dir/'",
 		"Trash files with a simple 't ELN/FILE'",
-		"Get mime information for a file. Ex: 'mm info file'",
 		"If too many files are listed, try enabling the pager: 'pg on'",
 		"Once in the pager, go backwards by pressing the keyboard shortcut "
 		"provided by your terminal emulator",
@@ -605,13 +653,13 @@ print_tips(const int all)
 		"Add a new bookmark: 'bm add FILENAME BM_NAME'",
 		"Use c, l, m, md, and r instead of cp, ln, mv, mkdir, and rm",
 		"Access a remote file system using the 'net' command",
-		"Manage default associated applications with the 'mime' command",
 		"Go back and forth in the directory history with Alt-j and Alt-k "
 		"(also Shift-Left and Shift-Right)",
 		"Run a new instance of CliFM: 'x ELN/DIR'",
 		"Send a command directly to the system shell. Ex: ';ls -l *'",
 		"Run the last executed command: '!!'",
 		"Access the commands history list: '!<TAB>'",
+		"Exclude commands from history using the HistIgnore option in the configuration file (F10)",
 		"Access the directory history list: 'dh <TAB>'",
 		"List previously used search patterns: '/*<TAB>'",
 		"Import aliases from file: 'alias import FILE'",
@@ -621,8 +669,6 @@ print_tips(const int all)
 		"Get a brief description for each CliFM command: 'cmd<TAB>'",
 		"Print the currently used color codes: 'colors'",
 		"Toggle hidden files on/off: 'hh' or Alt-.",
-		"List mountpoints: 'mp' or Alt-m",
-		"Disallow the use of shell commands with the -x option: 'clifm -x'",
 		"Go to the root directory: Alt-r",
 		"Go to the home directory: Alt-e (or just 'cd')",
 		"Open and edit the current color scheme file: F8 (or 'cs edit')",
@@ -642,7 +688,6 @@ print_tips(const int all)
 		"Change to a bookmark: 'bm NAME' or 'b:NAME'",
 		"Chain commands using ';' and '&&'. Ex: 's 2 7-10; r sel'",
 		"Add emojis to your prompt by copying them to the prompt line ('prompt edit')",
-		"Create a new profile: 'pf add PROFILE' or 'clifm -P PROFILE'",
 		"Switch profiles: 'pf set PROFILE'",
 		"Delete a profile: 'pf del PROFILE'",
 		"Rename a profile: 'pf rename PROFILE'",
@@ -659,21 +704,15 @@ print_tips(const int all)
 		"Launch the default system shell in CWD using ':' or ';'",
 		"Switch files sorting order: Alt-z and Alt-x",
 		"Reverse files sorting order: 'st rev'",
-		"Compress and decompress files using the 'ac' and 'ad' "
-		"commands respectively. Ex: 'ac sel' or 'ad file.zip'",
 		"Rename multiple files at once. Ex: 'br *.txt'",
 		"Need no more tips? Disable this feature in the configuration file",
 		"Need root privileges? Launch a new instance of CliFM as root "
 		"running the 'X' command (note the uppercase)",
-#ifdef __linux__
-		"Manage removable devices via the 'media' command",
-#endif /* __linux__ */
 		"Create a fresh configuration file: 'config reset'",
 		"Use 'ln edit' (or 'le') to edit symbolic links",
 		"Change default keyboard shortcuts: F9 (or 'kb edit')",
 		"Keep in sight previous and next visited directories enabling the "
 		"DirhistMap option in the configuration file (F10)",
-		"Run in incognito/private mode (-S, --stealth-mode)",
 		"Pin a file via the 'pin' command and then use it with the "
 		"period keyword (,). Ex: 'pin DIR' and then 'cd ,'",
 		"Switch color schemes using the 'cs' command",
@@ -685,7 +724,6 @@ print_tips(const int all)
 		"When searching or selecting files, use the exclamation mark "
 		"to negate a pattern. Ex: 's !*.pdf'",
 		"Enable the TrashAsRm option to always send removed files to the trash can",
-		"Don't like ELN's? Disable them using the -e command line switch",
 		"Use the 'n' command to create multiple files/directories. Ex: 'n file dir/'",
 		"Add prompt commands via the 'promptcmd' keyword: 'config' (F10)",
 		"Need git integration? Consult the manpage",
@@ -702,10 +740,7 @@ print_tips(const int all)
 		"Use the 'm' command to interactively rename a file. Ex: 'm 12'",
 		"Set options on a per directory basis via the autocommands function. Try 'help autocommands'",
 		"Clean up non-ASCII file names using the 'bleach' command",
-		"Running in an untrusted environment? Try the --secure-env and "
-		"--secure-cmds command line switches",
 		"Get help for any internal command via the -h or --help parameters: 'p -h'",
-		"Run in disk usage analyzer mode using the -t command line switch",
 		"Enable icons with 'icons on'",
 		"Quickly change to any parent directory using the 'bd' command",
 		"Use 'stats' to print statistics on files in the current directory",
@@ -717,16 +752,13 @@ print_tips(const int all)
 		"Quickly switch prompts via 'prompt NAME' (or 'prompt set <TAB>')",
 		"Press Alt-TAB to toggle the disk usage analyzer mode",
 		"Press Ctrl-Alt-l to toggle max file name length on/off",
-		"Fuzzy suggestions are supported. Ex: 'dwn > Downloads'. Enable them via --fuzzy-matching, or FuzzyMatching in the configuration file",
 		"Wildcards can be expanded via TAB. Ex: 's *.c<TAB>'",
 		"Try the help topics: 'help <TAB>'",
 		"List clifm commands, together with a brief description: 'cmd<TAB>'",
-		"List symlinks in the current directory: '=l<TAB>'. Enter 'help file-filters' for more information",
+		"List symlinks in the current directory: '=l<TAB>'. Try 'help file-filters' for more information",
 		"Use PropFields in the configuration file to customize fields in long view mode",
-		"Disable file previews for TAB completion (fzf mode only) via --no-fzfpreview",
 		"Preview files in the current directory using the 'view' command (requires fzf)",
 		"Press Alt+- to launch the files previewer (requires fzf)",
-		"Customize previewing applications: 'view edit' or F7",
 		"Interactively select files (requires fzf, fnf, or smenu). Ex: 's /dir/*<TAB>'",
 		"Change files permissions/ownership using the 'pc' and 'oc' commands respectively",
 		"Set a custom shell to run external commands. Ex: 'CLIFM_SHELL=/bin/dash clifm'",
@@ -746,10 +778,16 @@ print_tips(const int all)
 		return;
 	}
 
-	srand((unsigned int)time(NULL));
+#ifndef HAVE_ARC4RANDOM
+	srandom((unsigned int)time(NULL));
+	long tip_num = random() % (int)tipsn;
+#else
+	uint32_t tip_num = arc4random_uniform((uint32_t)tipsn);
+#endif /* !HAVE_ARC4RANDOM */
+
 	printf("%s%sTIP%s: %s\n", conf.colorize == 1 ? df_c : "",
-		conf.colorize == 1 ? BOLD : "", conf.colorize == 1 ? df_c : "",
-		TIPS[rand() % (int)tipsn]);
+		conf.colorize == 1 ? BOLD : "", conf.colorize == 1
+		? df_c : "", TIPS[tip_num]);
 }
 
 /* Check whether the conditions to run the new_instance function are
@@ -889,7 +927,7 @@ launch_new_instance_cmd(char ***cmd, char **self, char **_sudo,
 	sudo = 0;
 #endif /* __HAIKU__ */
 
-	setenv("CLIFM_OWN_CHILD", "1", 1);
+//	setenv("CLIFM_OWN_CHILD", "1", 1);
 
 	if (*cmd) {
 		ret = (sudo == 0 || confirm_sudo_cmd(*cmd) == 1)
@@ -915,7 +953,7 @@ launch_new_instance_cmd(char ***cmd, char **self, char **_sudo,
 	free(*self);
 	free(*dir);
 
-	unsetenv("CLIFM_OWN_CHILD");
+//	unsetenv("CLIFM_OWN_CHILD");
 
 	return ret;
 }
@@ -956,7 +994,7 @@ new_instance(char *dir, const int sudo)
 
 	int ret = check_dir(&deq_dir);
 	if (ret != EXIT_SUCCESS) {
-		free(deq_dir); free(self), free(_sudo);
+		free(deq_dir); free(self); free(_sudo);
 		return ret;
 	}
 
@@ -1249,6 +1287,8 @@ free_remotes(const int exit)
 	return EXIT_SUCCESS;
 }
 
+/* Load both regular and warning prompt, if enabled, from the prompt name NAME.
+ * Return EXIT_SUCCESS if found or EXIT_FAILURE if not. */
 int
 expand_prompt_name(char *name)
 {
@@ -1256,7 +1296,7 @@ expand_prompt_name(char *name)
 		return EXIT_FAILURE;
 
 	char *p = remove_quotes(name);
-	if (!p || !*p || strchr(p, '\\'))
+	if (!p || !*p || strchr(p, '\\')) /* Exclude prompt codes. */
 		return EXIT_FAILURE;
 
 	int i = (int)prompts_n;
@@ -1406,7 +1446,7 @@ save_last_path(char *last_path_tmp)
 static void
 handle_last_path(void)
 {
-	if (!config_dir_gral)
+	if (!config_dir_gral) /* NULL if running with --open or --preview */
 		return;
 
 	/* The cd_on_quit.sh function changes the shell current directory to the
@@ -1415,7 +1455,7 @@ handle_last_path(void)
 	 * Remove the file to prevent the function from changing the directory
 	 * if cd-on-quit is disabled (e.g., not exiting via 'Q'). If necessary,
 	 * it will be recreated by save_last_path() below. */
-	size_t len = strlen(config_dir_gral) + 7;
+	const size_t len = strlen(config_dir_gral) + 7;
 	char *last_path_tmp = (char *)xnmalloc(len, sizeof(char));
 	snprintf(last_path_tmp, len, "%s/.last", config_dir_gral);
 
@@ -1437,6 +1477,14 @@ free_stuff(void)
 {
 	int i = 0;
 
+#ifdef LINUX_FSINFO
+	if (ext_mnt) {
+		for (i = 0; ext_mnt[i].mnt_point; i++)
+			free(ext_mnt[i].mnt_point);
+		free(ext_mnt);
+	}
+#endif /* LINUX_FSINFO */
+
 #ifdef RUN_CMD
 	free(cmd_line_cmd);
 #endif /* RUN_CMD */
@@ -1448,6 +1496,7 @@ free_stuff(void)
 #endif /* !_NO_ICONS */
 
 	free(conf.time_str);
+	free(conf.ptime_str);
 
 #ifdef LINUX_INOTIFY
 	/* Shutdown inotify */
@@ -1509,6 +1558,16 @@ free_stuff(void)
 		free(filter.str);
 	}
 
+	if (conf.histignore_regex) {
+		regfree(&regex_hist);
+		free(conf.histignore_regex);
+	}
+
+	if (conf.dirhistignore_regex) {
+		regfree(&regex_dirhist);
+		free(conf.dirhistignore_regex);
+	}
+
 	free_workspaces_filters();
 
 	if (profile_names) {
@@ -1692,6 +1751,7 @@ get_term_size(void)
 	term_lines = w.ws_row;
 }
 
+#ifndef _BE_POSIX
 /* Get new window size and update/refresh the screen accordingly */
 static void
 sigwinch_handler(int sig)
@@ -1703,6 +1763,7 @@ sigwinch_handler(int sig)
 	get_term_size();
 	flags |= DELAYED_REFRESH;
 }
+#endif /* _BE_POSIX */
 
 void
 set_signals_to_ignore(void)
@@ -1710,7 +1771,9 @@ set_signals_to_ignore(void)
 	signal(SIGINT, SIG_IGN);  /* C-c */
 	signal(SIGQUIT, SIG_IGN); /* C-\ */
 	signal(SIGTSTP, SIG_IGN); /* C-z */
+#ifndef _BE_POSIX
 	signal(SIGWINCH, sigwinch_handler);
+#endif /* _BE_POSIX */
 /*	signal(SIGUSR1, sigusr_handler);
 	signal(SIGUSR2, sigusr_handler); */
 }
@@ -1720,10 +1783,10 @@ create_virtual_dir(const int user_provided)
 {
 	if (!stdin_tmp_dir || !*stdin_tmp_dir) {
 		if (user_provided == 1) {
-			_err('e', PRINT_PROMPT, "%s: Empty buffer for virtual "
+			err('e', PRINT_PROMPT, "%s: Empty buffer for virtual "
 				"directory name. Trying with default value\n", PROGRAM_NAME);
 		} else {
-			_err('e', PRINT_PROMPT, "%s: Empty buffer for virtual "
+			err('e', PRINT_PROMPT, "%s: Empty buffer for virtual "
 				"directory name\n", PROGRAM_NAME);
 		}
 		return EXIT_FAILURE;
@@ -1733,10 +1796,10 @@ create_virtual_dir(const int user_provided)
 	int ret = 0;
 	if ((ret = launch_execv(cmd, FOREGROUND, E_MUTE)) != EXIT_SUCCESS) {
 		if (user_provided == 1) {
-			_err('e', PRINT_PROMPT, "%s: mkdir: %s: %s. Trying with "
+			err('e', PRINT_PROMPT, "%s: mkdir: %s: %s. Trying with "
 				"default value\n", PROGRAM_NAME, stdin_tmp_dir, strerror(ret));
 		} else {
-			_err('e', PRINT_PROMPT, "%s: mkdir: %s: %s\n",
+			err('e', PRINT_PROMPT, "%s: mkdir: %s: %s\n",
 				PROGRAM_NAME, stdin_tmp_dir, strerror(ret));
 		}
 		return ret;
@@ -1800,12 +1863,12 @@ handle_stdin(void)
 	if (!stdin_tmp_dir || (exit_status = create_virtual_dir(1)) != EXIT_SUCCESS) {
 		free(stdin_tmp_dir);
 
-		suffix = gen_rand_str(6);
+		suffix = gen_rand_str(10);
 		char *temp = tmp_dir ? tmp_dir : P_tmpdir;
 		size_t tmp_len = strlen(temp) + 13;
 		stdin_tmp_dir = (char *)xnmalloc(tmp_len, sizeof(char));
 		snprintf(stdin_tmp_dir, tmp_len, "%s/vdir.%s", temp,
-			suffix ? suffix : "nTmp0B");
+			suffix ? suffix : "nTmp0B9&54");
 		free(suffix);
 
 		if ((exit_status = create_virtual_dir(0)) != EXIT_SUCCESS)
@@ -1838,7 +1901,7 @@ handle_stdin(void)
 
 			struct stat attr;
 			if (lstat(q, &attr) == -1) {
-				_err('w', PRINT_PROMPT, "%s: %s: %s\n",
+				err('w', PRINT_PROMPT, "%s: %s: %s\n",
 					PROGRAM_NAME, q, strerror(errno));
 				goto END;
 			}
@@ -1861,7 +1924,7 @@ handle_stdin(void)
 			} else {
 				tmp_file = replace_slashes(q, ':');
 				if (!tmp_file) {
-					_err('w', PRINT_PROMPT, "%s: %s: Error formatting "
+					err('w', PRINT_PROMPT, "%s: %s: Error formatting "
 						"file name\n", PROGRAM_NAME, q);
 					goto END;
 				}
@@ -1880,19 +1943,19 @@ handle_stdin(void)
 			if (symlink(source, dest) == -1) {
 				if (errno == EEXIST && xargs.virtual_dir_full_paths != 1) {
 					/* File already exists: append a random six digits suffix */
-					suffix = gen_rand_str(6);
-					char tmp[PATH_MAX + 8];
+					suffix = gen_rand_str(10);
+					char tmp[PATH_MAX + 12];
 					snprintf(tmp, sizeof(tmp), "%s.%s",
-						dest, suffix ? suffix : "#dn7R4");
+						dest, suffix ? suffix : "#dn7R4.d6?");
 					if (symlink(source, tmp) == -1)
-						_err('w', PRINT_PROMPT, "symlink: %s: %s\n",
+						err('w', PRINT_PROMPT, "symlink: %s: %s\n",
 							q, strerror(errno));
 					else
-						_err('w', PRINT_PROMPT, "symlink: %s: Destiny exists. "
+						err('w', PRINT_PROMPT, "symlink: %s: Destiny exists. "
 							"Created as %s\n", q, tmp);
 					free(suffix);
 				} else {
-					_err('w', PRINT_PROMPT, "symlink: %s: %s\n",
+					err('w', PRINT_PROMPT, "symlink: %s: %s\n",
 						q, strerror(errno));
 				}
 			} else {
@@ -1911,7 +1974,7 @@ END:
 
 	if (links_counter == 0) { /* No symlink was created. Exit */
 		dup2(STDOUT_FILENO, STDIN_FILENO);
-		_err(0, NOPRINT_PROMPT, "%s: Empty file names buffer. "
+		err(0, NOPRINT_PROMPT, "%s: Empty file names buffer. "
 			"Nothing to do\n", PROGRAM_NAME);
 		if (getenv("CLIFM_VT_RUNNING")) {
 			fprintf(stderr, "Press any key to continue... ");
@@ -2071,7 +2134,7 @@ list_commands(void)
 	return EXIT_SUCCESS;
 }
 
-#if !defined(__HAIKU__) && !defined(__sun)
+#ifdef HAVE_DPRINTF
 /* Retrieve pager path, first from PAGER, then try less(1), and finally
  * more(1). If none is found returns NULL. */
 static char *
@@ -2102,7 +2165,7 @@ get_pager(void)
 
 	return (char *)NULL;
 }
-#endif /* !__HAIKU__ && !__sun */
+#endif /* HAVE_DPRINTF */
 
 /* Help topics */
 static void
@@ -2262,7 +2325,7 @@ print_trash_topic(void)
 }
 
 static int
-run_help_topic(char *topic)
+run_help_topic(const char *topic)
 {
 	if (*topic == '-' && IS_HELP(topic)) {
 		puts(HELP_USAGE);
@@ -2317,12 +2380,12 @@ run_help_topic(char *topic)
 }
 
 int
-quick_help(char *topic)
+quick_help(const char *topic)
 {
 	if (topic && *topic)
 		return run_help_topic(topic);
 
-#if defined(__HAIKU__) || defined(__sun)
+#ifndef HAVE_DPRINTF
 	printf("%s                                %s\n\n%s\n\n%s",
 		ASCII_LOGO, PROGRAM_NAME_UPPERCASE, QUICK_HELP_HEADER,
 		QUICK_HELP_NAVIGATION);
@@ -2330,7 +2393,7 @@ quick_help(char *topic)
 # ifdef __HAIKU__
 	puts(_("\nNOTE: Some keybindings on Haiku might differ. Take a look "
 		"at your current keybindings via the 'kb' command"));
-# endif /* __HAIKU__ */
+# endif /* !HAVE_DPRINTF */
 	return EXIT_SUCCESS;
 #else
 	char *_pager = (char *)NULL;
@@ -2354,11 +2417,12 @@ quick_help(char *topic)
 		return EXIT_FAILURE;
 	}
 
-	FILE *fp;
-	fp = open_fwrite(tmp_file, &fd);
+	FILE *fp = open_fwrite(tmp_file, &fd);
 	if (!fp) {
 		xerror("%s: fopen: %s: %s\n", PROGRAM_NAME, tmp_file, strerror(errno));
 		free(_pager);
+		if (unlink(tmp_file) == -1)
+			err('w', PRINT_PROMPT, "help: %s: %s\n", tmp_file, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
@@ -2377,8 +2441,9 @@ quick_help(char *topic)
 		char *cmd[] = {_pager, tmp_file, NULL};
 		ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
 	}
-	unlink(tmp_file);
 
+	if (unlink(tmp_file) == -1)
+		err('w', PRINT_PROMPT, "help: %s: %s\n", tmp_file, strerror(errno));
 	fclose(fp);
 	free(_pager);
 
@@ -2391,20 +2456,34 @@ quick_help(char *topic)
 #endif /* __HAIKU || __sun */
 }
 
+__attribute__ ((noreturn))
 void
 help_function(void)
 {
+#ifdef _BE_POSIX
+	const char *posix = "-POSIX";
+#else
+	const char *posix = "";
+#endif /* _BE_POSIX */
+
 	fputs(NC, stdout);
 	printf("%s\n", ASCII_LOGO);
-	printf(_("%s %s (%s), by %s\n"), PROGRAM_NAME, VERSION, DATE, AUTHOR);
-	printf("\nUSAGE: %s %s\n%s%s%s", PROGRAM_NAME, GRAL_USAGE, _(SHORT_OPTIONS),
-		_(LONG_OPTIONS_A), _(LONG_OPTIONS_B));
+	printf(_("%s %s%s (%s), by %s\n"), PROGRAM_NAME, VERSION,
+		posix, DATE, AUTHOR);
+#ifdef _BE_POSIX
+	printf("\nUSAGE: %s %s\n%s\n", PROGRAM_NAME, GRAL_USAGE,
+		_(OPTIONS_LIST));
+#else
+	printf("\nUSAGE: %s %s\n%s%s%s", PROGRAM_NAME, GRAL_USAGE,
+		_(SHORT_OPTIONS), _(LONG_OPTIONS_A), _(LONG_OPTIONS_B));
+#endif /* _BE_POSIX */
 
 	puts("\nBUILT-IN COMMANDS:\n");
 	puts(_(CLIFM_COMMANDS_HEADER));
 	puts(_(CLIFM_COMMANDS));
 	puts(_(CLIFM_KEYBOARD_SHORTCUTS));
 	puts(_(HELP_END_NOTE));
+	exit(EXIT_SUCCESS);
 }
 
 void
@@ -2416,15 +2495,20 @@ free_software(void)
 void
 version_function(void)
 {
-	printf(_("%s %s (%s)\n%s\nLicense %s\nWritten by %s\n"), PROGRAM_NAME,
-		VERSION, DATE, CONTACT, LICENSE, AUTHOR);
+#ifndef _BE_POSIX
+	const char *posix = "";
+#else
+	const char *posix = "-POSIX";
+#endif /* _BE_POSIX */
+	printf(_("%s %s%s (%s)\n%s\nLicense %s\nWritten by %s\n"), PROGRAM_NAME,
+		VERSION, posix, DATE, CONTACT, LICENSE, AUTHOR);
 }
 
 void
 splash(void)
 {
 	printf("\n%s%s\n\n%s%s\t\t       %s%s\n           %s\n",
-		conf.colorize ? D_CYAN : "", ASCII_LOGO_BIG, df_c,
+		conf.colorize ? REG_CYAN : "", ASCII_LOGO_BIG, df_c,
 		BOLD, df_c, PROGRAM_NAME_UPPERCASE, _(PROGRAM_DESC));
 
 	if (conf.splash_screen) {
@@ -2487,6 +2571,10 @@ bonus_function(void)
 		NULL};
 
 	size_t num = (sizeof(phrases) / sizeof(phrases[0])) - 1;
-	srand((unsigned int)time(NULL));
-	puts(phrases[rand() % (int)num]);
+#ifndef HAVE_ARC4RANDOM
+	srandom((unsigned int)time(NULL));
+	puts(phrases[random() % (int)num]);
+#else
+	puts(phrases[arc4random_uniform((uint32_t)num)]);
+#endif /* !HAVE_ARC4RANDOM */
 }
