@@ -36,14 +36,13 @@
 #include <string.h>
 #include <time.h>
 #include <errno.h>
-#include <fcntl.h>
 
 #include "aux.h"
 #include "file_operations.h"
 #include "history.h"
-#ifdef NO_FS_EVENTS_MONITOR
+#ifdef GENERIC_FS_MONITOR
 # include "listing.h" /* reload_dirlist() */
-#endif /* NO_FS_EVENTS_MONITOR */
+#endif /* GENERIC_FS_MONITOR */
 #include "messages.h"
 #include "misc.h"
 #include "cleaner_table.h"
@@ -54,9 +53,6 @@
 #define DEFAULT_TRANSLATION  '_'
 #define BRACKETS_TRANSLATION '-'
 
-#define UNMOD_NAMES 0
-#define MOD_NAMES   1
-
 #define UTF_8_ENCODED_MASK  0xC0
 #define UTF_8_ENCODED_START 0xC0
 #define UTF_8_ENCODED_CONT  0x80
@@ -258,6 +254,9 @@ clean_file_name(char *restrict name)
 
 		/* Extended ASCII and Unicode chars */
 		int dec_value = get_uft8_dec_value(&i, name);
+		if (!name[i])
+			break;
+
 		if (dec_value == -1)
 			continue;
 		if (dec_value == -2) {
@@ -346,43 +345,43 @@ edit_replacements(struct bleach_t *bfiles, size_t *n, int *edited_names)
 	*edited_names = 1;
 
 	char f[PATH_MAX];
-	if (xargs.stealth_mode == 1)
-		snprintf(f, sizeof(f), "%s/%s", P_tmpdir, TMP_FILENAME);
-	else
-		snprintf(f, sizeof(f), "%s/%s", tmp_dir, TMP_FILENAME);
+	snprintf(f, sizeof(f), "%s/%s", (xargs.stealth_mode == 1)
+		? P_tmpdir : tmp_dir, TMP_FILENAME);
 
 	int fd = mkstemp(f);
 	if (fd == -1) {
-		_err('e', PRINT_PROMPT, "bleach: mkstemp: %s: %s\n", f, strerror(errno));
+		err('e', PRINT_PROMPT, "bleach: mkstemp: %s: %s\n", f, strerror(errno));
 		return (struct bleach_t *)NULL;
 	}
 
 	FILE *fp = (FILE *)NULL;
 
-#if defined(__HAIKU__) || defined(__sun)
+#ifndef HAVE_DPRINTF
 	fp = fopen(f, "w");
 	if (!fp) {
-		_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		if (unlink(f) == -1)
+			err('w', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
 		return (struct bleach_t *)NULL;
 	}
-#endif /* __HAIKU__ || __sun */
+#endif /* !HAVE_DPRINT */
 
-#if !defined(__HAIKU__) && !defined(__sun)
+#ifdef HAVE_DPRINTF
 	dprintf(fd, BLEACH_TMP_HEADER);
 #else
 	fprintf(fp, BLEACH_TMP_HEADER);
-#endif /* !__HAIKU__ && !__sun */
+#endif /* HAVE_DPRINTF */
 
 	size_t i;
 	/* Copy all files to be renamed to the temp file */
 	for (i = 0; i < *n; i++) {
-#if !defined(__HAIKU__) && !defined(__sun)
+#ifdef HAVE_DPRINTF
 		dprintf(fd, "original: %s\nreplacement: %s\n\n",
 			bfiles[i].original, bfiles[i].replacement);
 #else
 		fprintf(fp, "original: %s\nreplacement: %s\n\n",
 			bfiles[i].original, bfiles[i].replacement);
-#endif /* !__HAIKU__ && !__sun */
+#endif /* HAVE_DPRINTF */
 	}
 	size_t total_files = i;
 
@@ -393,7 +392,9 @@ edit_replacements(struct bleach_t *bfiles, size_t *n, int *edited_names)
 
 	fp = open_fread(f, &fd);
 	if (!fp) {
-		_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		if (unlink(f) == -1)
+			err('w', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
 		return (struct bleach_t *)NULL;
 	}
 
@@ -407,8 +408,8 @@ edit_replacements(struct bleach_t *bfiles, size_t *n, int *edited_names)
 	open_in_foreground = 0;
 	if (exit_status != EXIT_SUCCESS) {
 		xerror("bleach: %s: %s\n", f, strerror(errno));
-		if (unlinkat(fd, f, 0) == -1)
-			_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		if (unlink(f) == -1)
+			xerror("bleach: %s: %s\n", f, strerror(errno));
 		fclose(fp);
 		return (struct bleach_t *)NULL;
 	}
@@ -416,16 +417,18 @@ edit_replacements(struct bleach_t *bfiles, size_t *n, int *edited_names)
 	fclose(fp);
 	fp = open_fread(f, &fd);
 	if (!fp) {
-		_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		if (unlink(f) == -1)
+			err('w', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
 		return (struct bleach_t *)NULL;
 	}
 
 	/* Compare the new modification time to the stored one: if they
-	 * match, nothing was modified */
+	 * match, nothing has been modified. */
 	fstat(fd, &attr);
 	if (mtime_bfr == (time_t)attr.st_mtime) {
 		if (unlinkat(fd, f, 0) == -1)
-			_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+			err('w', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
 		fclose(fp);
 		*edited_names = 0;
 		return bfiles; /* Return the original list of files */
@@ -504,7 +507,7 @@ edit_replacements(struct bleach_t *bfiles, size_t *n, int *edited_names)
 	free(line);
 
 	if (unlinkat(fd, f, 0) == -1)
-		_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		err('w', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
 	fclose(fp);
 
 	return bfiles;
@@ -617,7 +620,7 @@ CONFIRM:
 
 	if (f == 0) {
 		/* Just in case either the original or the replacement file name
-		 * was removed from the list by the user, leaving only one of the two */
+		 * was removed from the list by the user, leaving only one of the two. */
 		free(bfiles[0].original);
 		free(bfiles[0].replacement);
 		free(bfiles);
@@ -627,7 +630,7 @@ CONFIRM:
 
 	/* The user entered 'e' to edit the file, but nothing was modified
 	 * Ask for confirmation in case the user just wanted to see what would
-	 * be done */
+	 * be done. */
 	if (_edit == 1) {
 		printf(_("%zu %s will be bleached\n"), f, f > 1 ? _("files") : _("file"));
 		if (rl_get_y_or_n(_("Continue? [y/n] ")) != 1) {
@@ -658,7 +661,7 @@ CONFIRM:
 			while (lstat(r, &a) == 0) {
 				char tmp[PATH_MAX];
 				xstrsncpy(tmp, r, sizeof(tmp));
-				r = (char *)xrealloc(r,	(PATH_MAX + 32) * sizeof(char));
+				r = (char *)xrealloc(r, (PATH_MAX + 32) * sizeof(char));
 				snprintf(r, PATH_MAX + 31, "%s-%zu", tmp, rep_suffix);
 				rep_suffix++;
 			}
@@ -679,8 +682,15 @@ CONFIRM:
 	if (exit_status == EXIT_FAILURE || total_rename == 0) {
 		printf(_("%s: %d file(s) bleached\n"), FUNC_NAME, total_rename);
 	} else {
-		_err(ERR_NO_LOG, PRINT_PROMPT, _("%s: %d file(s) bleached\n"),
+#ifdef GENERIC_FS_MONITOR
+		if (conf.autols == 1)
+			reload_dirlist();
+		print_reload_msg(_("%s: %d file(s) bleached\n"),
 			FUNC_NAME, total_rename);
+#else
+		err(ERR_NO_LOG, PRINT_PROMPT, _("%s->%s %s: %d file(s) bleached\n"),
+			mi_c, df_c, FUNC_NAME, total_rename);
+#endif /* GENERIC_FS_MONITOR */
 	}
 
 	return exit_status;
