@@ -24,7 +24,6 @@
 
 #include "helpers.h"
 
-#include <dirent.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
@@ -481,7 +480,7 @@ get_bd_matches(const char *str, int *n, const int mode)
 		char *p = (char *)NULL;
 		if (str && *str) { /* Non-empty query string */
 			p = conf.case_sens_path_comp
-				? strstr(cwd, str) : strcasestr(cwd, (char *)str);
+				? strstr(cwd, str) : xstrcasestr(cwd, (char *)str);
 			if (!p)
 				break;
 		}
@@ -785,10 +784,10 @@ go_home(const int cd_flag)
 	}
 
 	if (xchdir(user.home, SET_TITLE) != EXIT_SUCCESS) {
-		int err = errno;
+		int tmp_err = errno;
 		if (cd_flag == CD_PRINT_ERROR)
-			xerror("cd: %s: %s\n", user.home, strerror(err));
-		return err;
+			xerror("cd: %s: %s\n", user.home, strerror(tmp_err));
+		return tmp_err;
 	}
 
 	if (workspaces) {
@@ -830,9 +829,9 @@ change_to_path(char *new_path, const int cd_flag)
 	free(p);
 
 	if (xchdir(q, SET_TITLE) != EXIT_SUCCESS) {
-		int err = errno;
+		int tmp_err = errno;
 		if (cd_flag == CD_PRINT_ERROR)
-			xerror("cd: %s: %s\n", new_path, strerror(err));
+			xerror("cd: %s: %s\n", new_path, strerror(tmp_err));
 
 		free(q);
 
@@ -840,10 +839,10 @@ change_to_path(char *new_path, const int cd_flag)
 		 * a general error code, is not quite informative. Why not to return the
 		 * actual error code returned by chdir(3)? Note that POSIX only requires
 		 * for cd to return >0 in case of error (see cd(1p)). */
-		if (err == EACCES || err == ENOENT)
+		if (tmp_err == EACCES || tmp_err == ENOENT)
 			return EXIT_FAILURE;
 
-		return err;
+		return tmp_err;
 	}
 
 	if (workspaces) {
@@ -856,6 +855,13 @@ change_to_path(char *new_path, const int cd_flag)
 	return EXIT_SUCCESS;
 }
 
+static int
+skip_directory(const char *dir)
+{
+	return (conf.dirhistignore_regex && *conf.dirhistignore_regex
+		&& regexec(&regex_dirhist, dir, 0, NULL, 0) == EXIT_SUCCESS);
+}
+
 /* Change current directory to NEW_PATH, or to HOME if new_path is NULL.
  * Errors are printed only if CD_FLAG is set to CD_PRINT_ERROR (1) */
 int
@@ -882,7 +888,10 @@ cd_function(char *new_path, const int cd_flag)
 			return ret;
 	}
 
-	add_to_dirhist(workspaces[cur_ws].path);
+	const int skip = skip_directory(workspaces[cur_ws].path);
+
+	if (skip == 0)
+		add_to_dirhist(workspaces[cur_ws].path);
 
 	dir_changed = 1;
 	if (conf.autols == 1) {
@@ -891,7 +900,9 @@ cd_function(char *new_path, const int cd_flag)
 			ret = EXIT_FAILURE;
 	}
 
-	add_to_jumpdb(workspaces[cur_ws].path);
+	if (skip == 0)
+		add_to_jumpdb(workspaces[cur_ws].path);
+
 	return ret;
 }
 
@@ -972,7 +983,7 @@ print_dirhist(char *query)
 		? FUZZY_FILES_UTF8 : FUZZY_FILES_ASCII;
 
 	for (i = 0; i < dirhist_total_index; i++) {
-		if (!old_pwd[i] || *old_pwd[i] == _ESC)
+		if (!old_pwd[i] || *old_pwd[i] == KEY_ESC)
 			continue;
 
 		if (query && (conf.fuzzy_match == 1
@@ -1015,7 +1026,7 @@ change_to_dirhist_num(int n)
 	}
 
 	n--;
-	if (!old_pwd[n] || *old_pwd[n] == _ESC) {
+	if (!old_pwd[n] || *old_pwd[n] == KEY_ESC) {
 		xerror("%s\n", _("history: Invalid history entry"));
 		return EXIT_FAILURE;
 	}
@@ -1098,7 +1109,7 @@ back_function(char **args)
 	/* Find the previous non-consecutive equal and valid entry */
 	int i = dirhist_cur_index;
 	while (--i >= 0) {
-		if (old_pwd[i] && *old_pwd[i] != _ESC && (!workspaces[cur_ws].path
+		if (old_pwd[i] && *old_pwd[i] != KEY_ESC && (!workspaces[cur_ws].path
 		|| strcmp(workspaces[cur_ws].path, old_pwd[i]) != 0))
 			break;
 	}
@@ -1108,14 +1119,13 @@ back_function(char **args)
 
 	dirhist_cur_index = i;
 
-
 	if (xchdir(old_pwd[dirhist_cur_index], SET_TITLE) == EXIT_SUCCESS)
 		return set_path(old_pwd[dirhist_cur_index]);
 
 	xerror("cd: %s: %s\n", old_pwd[dirhist_cur_index], strerror(errno));
 
 	/* Invalidate this entry */
-	*old_pwd[dirhist_cur_index] = _ESC;
+	*old_pwd[dirhist_cur_index] = KEY_ESC;
 	if (dirhist_cur_index > 0)
 		dirhist_cur_index--;
 
@@ -1139,7 +1149,7 @@ forth_function(char **args)
 	/* Find the next valid entry */
 	int i = dirhist_cur_index;
 	while (++i <= dirhist_total_index) {
-		if (old_pwd[i] && (*old_pwd[i] != _ESC && (!workspaces[cur_ws].path
+		if (old_pwd[i] && (*old_pwd[i] != KEY_ESC && (!workspaces[cur_ws].path
 		|| strcmp(workspaces[cur_ws].path, old_pwd[i]) != 0)))
 			break;
 	}
@@ -1156,7 +1166,7 @@ forth_function(char **args)
 	xerror("cd: %s: %s\n", old_pwd[dirhist_cur_index], strerror(errno));
 
 	/* Invalidate this entry */
-	*old_pwd[dirhist_cur_index] = _ESC;
+	*old_pwd[dirhist_cur_index] = KEY_ESC;
 	if (dirhist_cur_index < dirhist_total_index
 	&& old_pwd[dirhist_cur_index + 1])
 		dirhist_cur_index++;
