@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,8 +25,13 @@
 #ifndef PROFILES_H
 #define PROFILES_H
 
+__BEGIN_DECLS
+
 int get_profile_names(void);
-int profile_function(char **comm);
-int profile_set(char *prof);
+int profile_function(char **);
+int profile_set(const char *);
+int validate_profile_name(const char *name);
+
+__END_DECLS
 
 #endif /* PROFILES_H */
