@@ -33,6 +33,7 @@
 # include <wordexp.h>
 #endif /* !__HAIKU__ && !__OpenBSD__ && !__ANDROID__ */
 #include <readline/readline.h>
+#include <readline/history.h> /* history_expand() */
 #include <errno.h>
 
 #include "aux.h"
@@ -58,13 +59,14 @@
 #define ROOT_IND "\001\x1b[1;31m\002R\001\x1b[0m\002"
 #define ROOT_IND_NO_COLOR "R"
 #define ROOT_IND_SIZE 17
+#define RDONLY_IND "RO\001\x1b[0m\002"
+#define RDONLY_IND_SIZE (MAX_COLOR + 8 + 1)
 #define STEALTH_IND "S\001\x1b[0m\002"
-#define STEALTH_IND_SIZE MAX_COLOR + 7 + 1
+#define STEALTH_IND_SIZE (MAX_COLOR + 7 + 1)
 
 #define EMERGENCY_PROMPT_MSG "Error decoding prompt line. Using an \
 emergency prompt"
 #define EMERGENCY_PROMPT "\001\x1b[0m\002> "
-#define EMERGENCY_PROMPT_LEN 8
 
 /* Flag macros to generate files statistic string for the prompt */
 #define STATS_DIR      0
@@ -86,9 +88,10 @@ emergency prompt"
 #define STATS_EXTENDED 16 /* Extended attributes (acl) */
 #define STATS_UNKNOWN  17
 #define STATS_UNSTAT   18
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 # define STATS_DOOR    19
-#endif /* __sun */
+# define STATS_PORT    20
+#endif /* SOLARIS_DOORS */
 
 #define NOTIF_SEL     0
 #define NOTIF_TRASH   1
@@ -473,7 +476,7 @@ gen_emergency_prompt(void)
 		f = 1;
 		xerror("%s: %s\n", PROGRAM_NAME, EMERGENCY_PROMPT_MSG);
 	}
-	char *_prompt = savestring(EMERGENCY_PROMPT, EMERGENCY_PROMPT_LEN);
+	char *_prompt = savestring(EMERGENCY_PROMPT, sizeof(EMERGENCY_PROMPT) - 1);
 	return _prompt;
 }
 
@@ -488,9 +491,10 @@ gen_stats_str(const int flag)
 	case STATS_CAP: val = stats.caps; break;
 	case STATS_CHR: val = stats.char_dev; break;
 	case STATS_DIR: val = stats.dir; break;
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 	case STATS_DOOR: val = stats.door; break;
-#endif /* __sun */
+	case STATS_PORT: val = stats.port; break;
+#endif /* SOLARIS_DOORS */
 	case STATS_EXE: val = stats.exec; break;
 	case STATS_EXTENDED: val = stats.extended; break;
 	case STATS_FIFO: val = stats.fifo; break;
@@ -527,13 +531,7 @@ gen_notification(const int flag)
 	char *p;
 	size_t len = 32;
 
-/*	if (user.uid == 0) {
-		p = (char *)xnmalloc(2, sizeof(char));
-		len = 2;
-	} else { */
-//	len = DIGINUM(UINT_MAX) + 2;
 	p = (char *)xnmalloc(len, sizeof(char));
-//	}
 	*p = '\0';
 
 	switch (flag) {
@@ -567,6 +565,31 @@ gen_notification(const int flag)
 	return p;
 }
 
+static inline char *
+gen_nesting_level(const int mode)
+{
+	char *p = (char *)NULL;
+
+	if (mode == 'i') {
+		p = (char *)xnmalloc(32, sizeof(char));
+		snprintf(p, 32, "%d", nesting_level);
+		return p;
+	}
+
+	/* I == full mode (nothing if first level) */
+	if (nesting_level <= 1) {
+		p = xnmalloc(1, sizeof(char));
+		*p = '\0';
+		return p;
+	}
+
+	size_t len = (MAX_COLOR * 2) + 32;
+	p = (char *)xnmalloc(len, sizeof(char));
+	snprintf(p, len, "(%d)", nesting_level);
+
+	return p;
+}
+
 /* Decode the prompt string (encoded_prompt global variable) taken from
  * the configuration file. */
 char *
@@ -605,9 +628,10 @@ decode_prompt(char *line)
 			case '"': temp = gen_stats_str(STATS_STICKY); goto ADD_STRING;
 			case '?': temp = gen_stats_str(STATS_UNKNOWN); goto ADD_STRING;
 			case '!': temp = gen_stats_str(STATS_UNSTAT); goto ADD_STRING;
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 			case '>': temp = gen_stats_str(STATS_DOOR); goto ADD_STRING;
-#endif /* __sun */
+			case '<': temp = gen_stats_str(STATS_PORT); goto ADD_STRING;
+#endif /* SOLARIS_DOORS */
 
 			case '*': temp = gen_notification(NOTIF_SEL); goto ADD_STRING;
 			case '%': temp = gen_notification(NOTIF_TRASH); goto ADD_STRING;
@@ -653,6 +677,10 @@ decode_prompt(char *line)
 			case 'H': /* Full hostname */
 				temp = gen_hostname(c); goto ADD_STRING;
 
+			case 'i': /* fallthrough */ /* Nest level (number only) */
+			case 'I': /* Nest level (full format) */
+				temp = gen_nesting_level(c); goto ADD_STRING;
+
 			case 's': /* Shell name (after last slash)*/
 				if (!user.shell) { line++; break; }
 				temp = gen_shell_name(); goto ADD_STRING;
@@ -775,7 +803,7 @@ print_welcome_msg(void)
 }
 
 static inline void
-_print_tips(void)
+print_tips_func(void)
 {
 	if (conf.tips == 0)
 		return;
@@ -805,16 +833,28 @@ run_prompt_cmds(void)
 }
 
 #ifndef _NO_TRASH
-static inline void
+static void
 update_trash_indicator(void)
 {
-	if (trash_ok == 1) {
-		int n = count_dir(trash_files_dir, NO_CPOP);
-		if (n <= 2)
-			trash_n = 0;
-		else
-			trash_n = (size_t)n;
-	}
+	static time_t trash_files_dir_mtime = 0;
+
+	if (trash_ok == 0)
+		return;
+
+	struct stat a;
+	if (stat(trash_files_dir, &a) == -1)
+		return;
+
+	if (trash_files_dir_mtime == a.st_mtime)
+		return;
+
+	trash_files_dir_mtime = a.st_mtime;
+
+	filesn_t n = count_dir(trash_files_dir, NO_CPOP);
+	if (n <= 2)
+		trash_n = 0;
+	else
+		trash_n = (size_t)n;
 }
 #endif /* !_NO_TRASH */
 
@@ -826,7 +866,7 @@ setenv_prompt(void)
 
 	/* Set environment variables with CliFM state information
 	 * (sel files, trash, stealth mode, messages) to be handled by
-	 * the prompt itself */
+	 * the prompt itself. */
 	char t[32];
 	snprintf(t, sizeof(t), "%d", (int)sel_n);
 	setenv("CLIFM_STAT_SEL", t, 1);
@@ -858,6 +898,7 @@ set_prompt_length(const size_t decoded_prompt_len)
 		len = (size_t)(decoded_prompt_len
 		+ (xargs.stealth_mode == 1 ? STEALTH_IND_SIZE : 0)
 		+ (user.uid == 0 ? ROOT_IND_SIZE : 0)
+		+ (conf.readonly == 1 ? RDONLY_IND_SIZE : 0)
 		+ ((sel_n > 0) ? N_IND : 0)
 		+ ((trash_n > 0) ? N_IND : 0)
 		+ ((msgs.error > 0) ? N_IND : 0)
@@ -898,8 +939,11 @@ construct_prompt(const char *decoded_prompt)
 
 	if (prompt_notif == 1) {
 		snprintf(the_prompt, prompt_len,
-			"%s%s%s%s%s%s%s%s%s%s\001%s\002",
-			(user.uid == 0) ? (conf.colorize == 1 ? ROOT_IND : ROOT_IND_NO_COLOR) : "",
+			"%s%s%s%s%s%s%s%s%s%s%s%s\001%s\002",
+			(user.uid == 0) ? (conf.colorize == 1
+				? ROOT_IND : ROOT_IND_NO_COLOR) : "",
+			(conf.readonly == 1) ? ro_c : "",
+			(conf.readonly == 1) ? RDONLY_IND : "",
 			(msgs.error > 0) ? err_ind : "",
 			(msgs.warning > 0) ? warn_ind : "",
 			(msgs.notice > 0) ? notice_ind : "",
@@ -916,12 +960,12 @@ construct_prompt(const char *decoded_prompt)
 }
 
 static inline void
-initialize_prompt_data(void)
+initialize_prompt_data(const int prompt_flag)
 {
 	check_cwd();
 	trim_final_slashes();
 	print_welcome_msg();
-	_print_tips();
+	print_tips_func();
 
 	/* If autols is disabled, and since terminal dimensions are gathered
 	 * in list_dir() via get_term_size(), let's get terminal dimensions
@@ -933,7 +977,10 @@ initialize_prompt_data(void)
 	fputs(df_c, stdout);
 	fflush(stdout);
 
-	run_prompt_cmds();
+	if (prompt_flag != PROMPT_UPDATE)
+	/* We're just updating the prompt string: no need to run prompt commands. */
+		run_prompt_cmds();
+
 #ifndef _NO_TRASH
 	update_trash_indicator();
 #endif /* !_NO_TRASH */
@@ -981,24 +1028,102 @@ print_right_prompt(void)
 	free(p);
 } */
 
+/* Some commands take '!' as parameter modifier: quick search, 'filter',
+ * and 'sel', in which case history expansion must not be performed.
+ * Return 1 if we have one of these commands or 0 otherwise. */
+static int
+exclude_from_history(const char *s)
+{
+	if (*s == '/'
+	|| (*s == 's' && (s[1] == ' ' || strncmp(s + 1, "el ", 3) == 0))
+	|| (*s == 'f' && ((s[1] == 't' && s[2] == ' ')
+	|| strncmp(s + 1, "ilter ", 6) == 0))
+	|| (*s == 'd' && s[1] == 'h' && s[2] == ' ')
+	|| (*s == 'b' && s[1] == ' '))
+		return 1;
+
+	return 0;
+}
+
+/* Replace history expressions ("!*") in the string INPUT by the corresponding
+ * history entry. */
+static int
+expand_history(char **input)
+{
+	/* history_expansion_char defaults to '!' */
+	char *hist_c = strchr(*input, history_expansion_char);
+	if (!hist_c || (hist_c != *input && *(hist_c - 1) != ' ')
+	|| exclude_from_history(*input) == 1)
+		return EXIT_SUCCESS;
+
+	char *exp_input = (char *)NULL;
+	int ret = history_expand(*input, &exp_input);
+
+	if (ret == -1) { /* Error in expansion */
+		xerror("%s: %s\n", PROGRAM_NAME, exp_input);
+		free(*input);
+		free(exp_input);
+		return EXIT_FAILURE;
+	}
+
+	if (ret == 0) { /* No expansion took place */
+		free(exp_input);
+		return EXIT_SUCCESS;
+	}
+
+	printf("%s\n", exp_input);
+	free(*input);
+
+	if (ret == 2) { /* Display but do not execute the expanded command (:p) */
+		free(exp_input);
+		return (-1);
+	}
+
+	/* (ret == 1) Display and execute */
+	*input = exp_input;
+
+	return EXIT_SUCCESS;
+}
+
+static char *
+handle_empty_line(void)
+{
+	if (conf.autols == 1 && ((flags & DELAYED_REFRESH)
+	|| xargs.refresh_on_empty_line == 1) && rl_pending_input == 0) {
+		flags &= ~DELAYED_REFRESH;
+		refresh_screen();
+	} else {
+		flags &= ~DELAYED_REFRESH;
+	}
+
+	rl_pending_input = 0;
+	return (char *)NULL;
+}
+
 /* Print the prompt and return the string entered by the user, to be
  * parsed later by parse_input_str() */
 char *
-prompt(void)
+prompt(const int prompt_flag)
 {
-	initialize_prompt_data();
+	initialize_prompt_data(prompt_flag);
 
 	/* Generate the prompt string using the prompt line in the config
-	 * file (stored in encoded_prompt at startup) */
+	 * file (stored in encoded_prompt at startup). */
 	char *decoded_prompt = decode_prompt(conf.encoded_prompt);
 	char *the_prompt = construct_prompt(decoded_prompt
 		? decoded_prompt : EMERGENCY_PROMPT);
 	free(decoded_prompt);
 
+	if (prompt_flag == PROMPT_UPDATE || prompt_flag == PROMPT_UPDATE_RUN_CMDS) {
+		rl_set_prompt(the_prompt);
+		free(the_prompt);
+		return (char *)NULL;
+	}
+
 	/* Tell my_rl_getc() (readline.c) to recalculate the length
 	 * of the last prompt line, needed to calculate the finder's offset
 	 * and the current cursor column. This length might vary if the
-	 * prompt contains dynamic values */
+	 * prompt contains dynamic values. */
 	prompt_offset = UNSET;
 
 /*	if (right_prompt && *right_prompt)
@@ -1007,23 +1132,19 @@ prompt(void)
 	UNHIDE_CURSOR;
 
 	/* Print the prompt and get user input */
-	char *input = (char *)NULL;
-	input = readline(the_prompt);
+	char *input = readline(the_prompt);
 	free(the_prompt);
 
 	if (!input || !*input || rl_end == 0) {
 		free(input);
-		if (conf.autols == 1 && ((flags & DELAYED_REFRESH)
-		|| xargs.refresh_on_empty_line == 1)) {
-			flags &= ~DELAYED_REFRESH;
-			refresh_screen();
-		} else {
-			flags &= ~DELAYED_REFRESH;
-		}
-		return (char *)NULL;
+		return handle_empty_line();
 	}
+
 	flags &= ~DELAYED_REFRESH;
 
+	if (expand_history(&input) != EXIT_SUCCESS)
+		return (char *)NULL;
+
 	log_and_record(input);
 
 	return input;
@@ -1041,6 +1162,7 @@ list_prompts(void)
 	for (i = 0; i < prompts_n; i++) {
 		if (!prompts[i].name)
 			continue;
+
 		if (*cur_prompt_name == *prompts[i].name
 		&& strcmp(cur_prompt_name, prompts[i].name) == 0)
 			printf("%s>%s %s\n", mi_c, df_c, prompts[i].name);
@@ -1095,7 +1217,7 @@ set_prompt(char *name)
 	}
 
 	int i = (int)prompts_n;
-	while(--i >= 0) {
+	while (--i >= 0) {
 		if (*p != *prompts[i].name || strcmp(p, prompts[i].name) != 0)
 			continue;
 		free(p);
@@ -1112,9 +1234,9 @@ static int
 set_default_prompt(void)
 {
 	free(conf.encoded_prompt);
-	conf.encoded_prompt = savestring(DEFAULT_PROMPT, strlen(DEFAULT_PROMPT));
+	conf.encoded_prompt = savestring(DEFAULT_PROMPT, sizeof(DEFAULT_PROMPT) - 1);
 	free(conf.wprompt_str);
-	conf.wprompt_str = savestring(DEF_WPROMPT_STR, strlen(DEF_WPROMPT_STR));
+	conf.wprompt_str = savestring(DEF_WPROMPT_STR, sizeof(DEF_WPROMPT_STR) - 1);
 	*cur_prompt_name = '\0';
 	prompt_notif = DEF_PROMPT_NOTIF;
 	return EXIT_SUCCESS;
