@@ -25,9 +25,14 @@
 #ifndef PROMPT_H
 #define PROMPT_H
 
+/* Mode macros for the prompt function */
+#define PROMPT_UPDATE 0
+#define PROMPT_SHOW   1
+#define PROMPT_UPDATE_RUN_CMDS 2
+
 __BEGIN_DECLS
 
-char *prompt(void);
+char *prompt(const int);
 char *decode_prompt(char *);
 int  prompt_function(char **);
 
