@@ -25,24 +25,33 @@
 #include "helpers.h"
 
 #include <string.h>
-#include <paths.h> /* _PATH_STDPATH */
+#ifndef _BE_POSIX
+# include <paths.h> /* _PATH_STDPATH */
+#endif /* _BE_POSIX */
 #include <stdio.h>
 #include <errno.h>
+#include <sys/resource.h> /* getrlimit(3), setrlimit(3) */
+#include <unistd.h> /* close(2), sysconf(3) */
+#include <limits.h> /* OPEN_MAX */
 
 #include "sanitize.h"
 #include "aux.h"
+#include "checks.h" /* is_number() */
 #include "misc.h"
 
 #define UNSAFE_CMD "Unsafe command. Consult the manpage for more information"
 /* If PATH cannot be retrieved from any other source, let's use this value */
-#define MINIMAL_PATH "/usr/local/bin:/bin:/usr/bin:/sbin:/usr/sbin"
+#if !defined(_PATH_STDPATH) && !defined(_CS_PATH)
+# define MINIMAL_PATH "/usr/local/bin:/bin:/usr/bin:/sbin:/usr/sbin"
+#endif
 
 /* Unset environ: little implementation of clearenv(3), not available
  * on some systems (not POSIX) */
 static void
 xclearenv(void)
 {
-#if !defined(__NetBSD__) && !defined(__HAIKU__) && !defined(__APPLE__)
+#if !defined(__NetBSD__) && !defined(__HAIKU__) && !defined(__APPLE__) \
+&& !defined(__TERMUX__)
 	/* This seems to be enough (it is it according to the Linux/FreeBSD
 	 * manpage for clearenv(3)). */
 	environ = NULL;
@@ -68,7 +77,7 @@ xclearenv(void)
 			exit(EXIT_FAILURE);
 		}
 	}
-#endif /* !__NetBSD__ && !__HAIKU__ && !__APPLE__ */
+#endif /* !__NetBSD__ && !__HAIKU__ && !__APPLE__ && !__TERMUX__ */
 }
 
 static void
@@ -102,17 +111,86 @@ xsetenv(const char *name, const char *value)
 		xerror("%s: setenv: %s: %s\n", PROGRAM_NAME, name, strerror(errno));
 }
 
+/* See https://www.oreilly.com/library/view/secure-programming-cookbook/0596003943/ch01s09.html */
+static void
+disable_coredumps(void)
+{
+#if defined(ALLOW_COREDUMPS)
+	return;
+#endif /* ALLOW_COREDUMPS */
+
+	struct rlimit rlim;
+	rlim.rlim_cur = rlim.rlim_max = 0;
+	if (setrlimit(RLIMIT_CORE, &rlim) == -1)
+		xerror("setrlimit: Cannot set RLIMIT_CORE: %s\n", strerror(errno));
+}
+
+/* Return the maximum number of files a process can have open. */
+static int
+get_open_max(void)
+{
+#ifdef _SC_OPEN_MAX
+	return (int)sysconf(_SC_OPEN_MAX);
+#else
+	/* This is what getdtablesize(3) does */
+	struct rlimit rlim;
+	if (getrlimit(RLIMIT_NOFILE, &rlim) != -1)
+		return (int)rlim.rlim_cur;
+#endif /* _SC_OPEN_MAX */
+
+#ifdef OPEN_MAX /* Not defined in Linux */
+	return OPEN_MAX;
+#elif defined(__linux__) && defined(NR_OPEN)
+	return NR_OPEN;
+#else
+	return 256; /* Let's fallback to a sane default */
+#endif /* OPEN_MAX */
+}
+
+/* Close all non-standard file descriptors (> 2) to avoid FD exhaustion. */
+static void
+sanitize_file_descriptors(void)
+{
+	int fd = 0, fds = get_open_max();
+
+	for (fd = 3; fd < fds; fd++)
+		close(fd);
+}
+
+static int
+sanitize_shell_level(char *str)
+{
+	if (!str || !*str || !is_number(str))
+		return EXIT_FAILURE;
+
+	int a = atoi(str);
+	if (a < 1 || a > MAX_SHELL_LEVEL)
+		return EXIT_FAILURE;
+
+	return EXIT_SUCCESS;
+}
+
 /* Sanitize the environment: set environ to NULL and then set a few
- * environment variables to get a minimally working environment. */
+ * environment variables to get a minimally working environment.
+ * Non-standard file descriptors are closed.
+ * Core dumps are disabled.
+ * Umask is set to the most restrictive value: 0077. */
 int
 xsecure_env(const int mode)
 {
+	sanitize_file_descriptors();
+	disable_coredumps();
+	umask(0077);
+
 	char *display = (char *)NULL,
 		 *wayland_display = (char *)NULL,
 		 *_term = (char *)NULL,
 		 *tz = (char *)NULL,
 		 *lang = (char *)NULL,
-		 *fzfopts = (char *)NULL;
+		 *fzfopts = (char *)NULL,
+		 *clifm_level = (char *)NULL;
+
+	clifm_level = getenv("CLIFMLVL");
 
 	if (mode != SECURE_ENV_FULL) {
 		/* Let's keep these values from the current environment */
@@ -124,6 +202,9 @@ xsecure_env(const int mode)
 		lang = getenv("LANG");
 		if (fzftab)
 			fzfopts = getenv("FZF_DEFAULT_OPTS");
+	} else {
+		if (clifm_level)
+			nesting_level = 2; /* This is a nested instance */
 	}
 
 	xclearenv();
@@ -147,6 +228,9 @@ xsecure_env(const int mode)
 			xsetenv("WAYLAND_DISPLAY", wayland_display);
 	}
 
+	if (clifm_level && sanitize_shell_level(clifm_level) == EXIT_SUCCESS)
+		xsetenv("CLIFMLVL", clifm_level);
+
 	if (_term && sanitize_cmd(_term, SNT_MISC) == EXIT_SUCCESS)
 		xsetenv("TERM", _term);
 
@@ -226,8 +310,8 @@ sanitize_gral(const char *cmd)
 	return EXIT_SUCCESS;
 }
 
-/* Returns 1 if at least one character in CMD is a non-escaped
- * blacklisted char (<>|;&$). Oterwise, returns 0 */
+/* Return EXIT_SUCCESS if at least one character in CMD is a non-escaped
+ * blacklisted char (<>|;&$). Oterwise, return EXIT_FAILURE. */
 static int
 sanitize_blacklist(const char *cmd)
 {
@@ -256,7 +340,7 @@ sanitize_blacklist(const char *cmd)
 
 /* Check if command name in STR contains slashes. Return 1 if found,
  * zero otherwise. This means: do not allow custom scripts or binaries,
- * but only whatever could be found in the sanitized PATH variable */
+ * but only whatever could be found in the sanitized PATH variable. */
 static int
 clean_cmd(const char *str)
 {
@@ -272,7 +356,7 @@ clean_cmd(const char *str)
 		*p = ' ';
 
 	if (q) {
-		_err('w', PRINT_PROMPT, _("%s: %s: Only command base names "
+		err('w', PRINT_PROMPT, _("%s: %s: Only command base names "
 			"are allowed. Ex: 'nano' instead of '/usr/bin/nano'\n"),
 			PROGRAM_NAME, str);
 		return EXIT_FAILURE;
@@ -281,8 +365,8 @@ clean_cmd(const char *str)
 	return EXIT_SUCCESS;
 }
 
-/* Sanitize CMD according to TYPE. Returns zero if command is safe or
- * one if not. */
+/* Sanitize CMD according to TYPE. Returns EXIT_SUCCESS if command is safe or
+ * EXIT_FAILURE if not. */
 int
 sanitize_cmd(const char *str, const int type)
 {
@@ -315,7 +399,7 @@ sanitize_cmd(const char *str, const int type)
 	}
 
 	if (exit_status == EXIT_FAILURE) {
-		_err('w', PRINT_PROMPT, "%s: %s: %s\n",
+		err('w', PRINT_PROMPT, "%s: %s: %s\n",
 			PROGRAM_NAME, str, _(UNSAFE_CMD));
 		return EXIT_FAILURE;
 	}
