@@ -24,13 +24,15 @@
 
 #include "helpers.h"
 
-#include <dirent.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
-#include <glob.h>
+
+#ifdef __sun
+# include <sys/termios.h> // TIOCGWINSZ
+#endif // __sun
 
 /* We need rl_line_buffer in case of no matches and no metacharacter */
 #include <readline/readline.h>
@@ -46,7 +48,7 @@
 
 #define ERR_SKIP_REGEX 2
 
-#if defined(__OpenBSD__) || defined(__sun)
+#if (defined(__OpenBSD__) || defined(__sun)) && !defined(_BE_POSIX)
 /* OpenBSD/Solaris find(1) has neither -regex nor -iregex. We'll try to use
  * gfind(1) instead.*/
 # define FIND_HAS_NO_REGEX
@@ -178,9 +180,10 @@ set_file_type_and_search_path(char **args, mode_t *file_type,
 	case 'b': *file_type = invert == 1 ? DT_BLK : S_IFBLK; break;
 	case 'c': *file_type = invert == 1 ? DT_CHR : S_IFCHR; break;
 	case 'd': *file_type = invert == 1 ? DT_DIR : S_IFDIR; break;
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 	case 'D': *file_type = invert == 1 ? DT_DOOR : S_IFDOOR; break;
-#endif /* __sun */
+	case 'P': *file_type = invert == 1 ? DT_PORT : S_IFPORT; break;
+#endif /* SOLARIS_DOORS */
 	case 'f': *file_type = invert == 1 ? DT_REG : S_IFREG; break;
 	case 'l': *file_type = invert == 1 ? DT_LNK : S_IFLNK; break;
 	case 'p': *file_type = invert == 1 ? DT_FIFO : S_IFIFO; break;
@@ -387,9 +390,9 @@ get_glob_matches_invert(char **gfiles, const char *search_path,
 	if (search_path)
 		return get_non_matches_from_search_path(search_path, gfiles, file_type);
 
-	int i, j, n = 0;
+	filesn_t i, j, n = 0;
 	struct search_t *matches =
-		(struct search_t *)xnmalloc(files + 1, sizeof(struct search_t));
+		(struct search_t *)xnmalloc((size_t)files + 1, sizeof(struct search_t));
 
 	for (i = 0; file_info[i].name; i++) {
 		int f = 0;
@@ -720,9 +723,10 @@ check_regex_file_type(struct dirent **reg_dirlist, const int index,
 		case S_IFBLK: type = DT_BLK; break;
 		case S_IFCHR: type = DT_CHR; break;
 		case S_IFDIR: type = DT_DIR; break;
-# ifdef __sun
+# ifdef SOLARIS_DOORS
 		case S_IFDOOR: type = DT_DOOR; break;
-# endif /* __sun */
+		case S_IFPORT: type = DT_PORT; break;
+# endif /* SOLARIS_DOORS */
 		case S_IFIFO: type = DT_FIFO; break;
 		case S_IFLNK: type = DT_LNK; break;
 		case S_IFREG: type = DT_REG; break;
@@ -885,7 +889,7 @@ print_regex_matches(const mode_t file_type, struct dirent **reg_dirlist,
 		print_regex_entry(list[i], name_pad, eln_pad, newline);
 	}
 
-	print_reload_msg("Matches found: %zu\n", count);
+	print_reload_msg(_("Matches found: %zu\n"), count);
 
 END:
 	free(list);
@@ -950,8 +954,8 @@ search_regex(char **args)
 
 	size_t found = 0;
 	int *regex_index = (int *)xnmalloc((search_path ? (size_t)tmp_files
-		: files) + 2, sizeof(int));
-	size_t max = (search_path && *search_path) ? (size_t)tmp_files : files;
+		: (size_t)files) + 2, sizeof(int));
+	size_t max = (search_path && *search_path) ? (size_t)tmp_files : (size_t)files;
 
 	for (i = 0; i < max; i++) {
 		char *name = (search_path && *search_path) ? reg_dirlist[i]->d_name
