@@ -24,9 +24,7 @@
 
 #include "helpers.h"
 
-#include <dirent.h>
 #include <errno.h>
-#include <glob.h>
 #include <stdio.h>
 #include <readline/tilde.h>
 #include <string.h>
@@ -41,6 +39,10 @@
 #endif /* __linux__ || __HAIKU__ || __APPLE__ || __sun || __CYGWIN__ */
 #include <limits.h>
 
+#ifdef __sun
+# include <sys/termios.h> /* TIOCGWINSZ */
+#endif /* __sun */
+
 #include "aux.h"
 #include "checks.h"
 #include "colors.h"
@@ -131,14 +133,14 @@ select_file(char *file)
 static char **
 load_matches_invert_cwd(glob_t gbuf, const mode_t filetype, int *matches)
 {
-	char **list = (char **)xnmalloc(files + 2, sizeof(char *));
+	char **list = (char **)xnmalloc((size_t)files + 2, sizeof(char *));
 
-	int i = (int)files;
+	filesn_t i = files;
 	while (--i >= 0) {
 		if (filetype != 0 && file_info[i].type != filetype)
 			continue;
 
-		int j = (int)gbuf.gl_pathc;
+		filesn_t j = (filesn_t)gbuf.gl_pathc;
 		while (--j >= 0) {
 			if (*file_info[i].name == *gbuf.gl_pathv[j]
 			&& strcmp(file_info[i].name, gbuf.gl_pathv[j]) == 0)
@@ -346,7 +348,7 @@ static int
 sel_regex_cwd(regex_t regex, const mode_t filetype, const int invert)
 {
 	int new_sel = 0;
-	int i = (int)files;
+	filesn_t i = files;
 
 	while (--i >= 0) {
 		if (filetype != 0 && file_info[i].type != filetype)
@@ -597,7 +599,7 @@ get_sel_file_size(const size_t i, int *status)
 		sel_elements[i].size = (off_t)(dir_size(sel_elements[i].name,
 			0, status) * base);
 	} else {
-		sel_elements[i].size = (off_t)FILE_SIZE;
+		sel_elements[i].size = (off_t)FILE_SIZE(attr);
 	}
 
 	return sel_elements[i].size;
@@ -606,14 +608,13 @@ get_sel_file_size(const size_t i, int *status)
 static void
 print_total_size(const off_t total, const int status)
 {
-	char *human_size = get_size_unit(total);
+	char *human_size = construct_human_size(total);
 
 	char err[sizeof(xf_c) + 6]; *err = '\0';
 	if (status != 0)
 		snprintf(err, sizeof(err), "%s%c%s", xf_c, DU_ERR_CHAR, NC);
 
 	printf(_("%s%sTotal size%s: %s%s\n"), df_c, BOLD, df_c, err, human_size);
-	free(human_size);
 }
 
 static void
@@ -655,11 +656,11 @@ print_selected_files(void)
 
 static int
 print_sel_results(const int new_sel, const char *sel_path,
-	const char *pattern, const int err)
+	const char *pattern, const int error)
 {
 	if (new_sel > 0 && xargs.stealth_mode != 1 && sel_file
 	&& save_sel() != EXIT_SUCCESS) {
-		_err('e', PRINT_PROMPT, _("sel: Error writing selected files "
+		err('e', PRINT_PROMPT, _("sel: Error writing selected files "
 			"into the selections file\n"));
 		return EXIT_FAILURE;
 	}
@@ -670,14 +671,14 @@ print_sel_results(const int new_sel, const char *sel_path,
 	}
 
 	if (new_sel <= 0) {
-		if (pattern && err == 0)
+		if (pattern && error == 0)
 			fputs(_("sel: No matches found\n"), stderr);
 		return EXIT_FAILURE;
 	}
 
 	get_sel_files();
 
-	if (conf.autols == 1 && err == 0)
+	if (conf.autols == 1 && error == 0)
 		reload_dirlist();
 
 	print_reload_msg(_("%d file(s) selected\n"), new_sel);
@@ -717,7 +718,7 @@ construct_sel_filename(const char *dir, const char *name)
 }
 
 static int
-select_filename(char *arg, char *dir, int *err)
+select_filename(char *arg, char *dir, int *errors)
 {
 	int new_sel = 0;
 
@@ -738,12 +739,12 @@ select_filename(char *arg, char *dir, int *err)
 		struct stat attr;
 		if (lstat(tmp, &attr) == -1) {
 			xerror("sel: %s: %s\n", arg, strerror(errno));
-			(*err)++;
+			(*errors)++;
 		} else {
 			int r = select_file(tmp);
 			new_sel += r;
 			if (r == 0)
-				(*err)++;
+				(*errors)++;
 		}
 		free(tmp);
 		return new_sel;
@@ -752,25 +753,39 @@ select_filename(char *arg, char *dir, int *err)
 	struct stat a;
 	if (lstat(arg, &a) == -1) {
 		xerror("sel: %s: %s\n", name, strerror(errno));
-		(*err)++;
+		(*errors)++;
 	} else {
 		int r = select_file(name);
 		new_sel += r;
 		if (r == 0)
-			(*err)++;
+			(*errors)++;
 	}
 
 	return new_sel;
 }
 
+static int
+not_just_star(const char *s)
+{
+	while (*s) {
+		if (*s == '?' || *s == '[' || *s == '{' || *s == '^'
+		|| *s == '.' || *s == '|' || *s == '+' || *s == '$')
+			return 1;
+		s++;
+	}
+
+	return 0;
+}
+
 static int
 select_pattern(char *arg, const char *dir, const mode_t filetype, int *err)
 {
 	int ret = sel_glob(arg, dir, filetype);
 
-	/* Glob failed. Try REGEX. */
+	/* Glob failed. Try REGEX (if at least one regex metacharacter not being
+	 * asterisk if found). */
 	if (ret == -1)
-		ret = sel_regex(arg, dir, filetype);
+		ret = not_just_star(arg) == 1 ? sel_regex(arg, dir, filetype) : 0;
 
 	if (ret == -1)
 		(*err)++;
@@ -870,14 +885,13 @@ show_sel_files(void)
 			case 99: /* fallthrough */  /* 'c' */
 			case 112: /* fallthrough */ /* 'p' */
 			case 113:
-				conf.pager = 0, reset_pager = 1; /* 'q' */
+				conf.pager = 0; reset_pager = 1; /* 'q' */
 				break;
 			/* If another key is pressed, go back one position.
 			 * Otherwise, some file names won't be listed.*/
 			default:
 				i--;
 				continue;
-				break;
 			}
 		}
 
@@ -898,13 +912,12 @@ show_sel_files(void)
 	flags &= ~IN_SELBOX_SCREEN;
 	tab_offset = t;
 
-	char *human_size = get_size_unit(total);
+	char *human_size = construct_human_size(total);
 	char err[sizeof(xf_c) + 6]; *err = '\0';
 	if (status != 0)
 		snprintf(err, sizeof(err), "%s%c%s", xf_c, DU_ERR_CHAR, NC);
 
 	printf(_("\n%s%sTotal size%s: %s%s\n"), df_c, BOLD, df_c, err, human_size);
-	free(human_size);
 
 	if (reset_pager == 1)
 		conf.pager = 1;
@@ -947,7 +960,7 @@ ERROR:
 }
 
 static int
-deselect_entries(char **desel_path, const size_t desel_n, int *err,
+deselect_entries(char **desel_path, const size_t desel_n, int *error,
 	const int desel_screen)
 {
 	int dn = (int)desel_n;
@@ -971,7 +984,7 @@ deselect_entries(char **desel_path, const size_t desel_n, int *err,
 
 		if (desel_index == -1) {
 			dn--;
-			*err = 1;
+			*error = 1;
 			if (desel_screen == 0) {
 				xerror(_("%s: %s: No such selected file\n"),
 					PROGRAM_NAME, desel_path[i]);
