@@ -24,13 +24,10 @@
 
 #include "helpers.h"
 
-#include <dirent.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#if defined(__OpenBSD__)
-# include <strings.h>
-#endif /* __OpenBSD__ */
+#include <strings.h> /* str(n)casecmp() */
 
 #include "checks.h"
 #include "aux.h" /* xatoi */
@@ -40,29 +37,14 @@
 int
 skip_nonexec(const struct dirent *ent)
 {
+#ifdef _DIRENT_HAVE_D_TYPE
+	if (ent->d_type == DT_REG && access(ent->d_name, X_OK) == -1)
+#else
 	if (access(ent->d_name, X_OK) == -1)
+#endif /* _DIRENT_HAVE_D_TYPE */
 		return 0;
+
 	return 1;
-
-/*	int f = 0; // Hold file ownership flags
-
-	struct stat a;
-	if (stat(ent->d_name, &a) == -1)
-		return 0;
-
-	mode_t val = (a.st_mode & (mode_t)~S_IFMT);
-	if (val & S_IXUSR) f |= X_USR;
-	if (val & S_IXGRP) f |= X_GRP;
-	if (val & S_IXOTH) f |= X_OTH;
-
-	if ((f & X_USR) && a.st_uid == user.uid)
-		return 1;
-	if ((f & X_GRP) && a.st_gid == user.gid)
-		return 1;
-	if (f & X_OTH)
-		return 1;
-
-	return 0; */
 }
 
 int
@@ -176,13 +158,13 @@ static inline int
 sort_by_size(struct fileinfo *pa, struct fileinfo *pb)
 {
 	off_t as = pa->size, bs = pb->size;
-	if (conf.long_view == 1 && conf.full_dir_size == 1) {
-		int base = xargs.si == 1 ? 1000 : 1024;
-		if (pa->dir == 1)
-			as = pa->size * base;
-		if (pb->dir == 1)
-			bs = pb->size * base;
-	}
+//	if (conf.long_view == 1 && conf.full_dir_size == 1) {
+//		const int base = xargs.si == 1 ? 1000 : 1024;
+//		if (pa->dir == 1)
+//			as = pa->size * base;
+//		if (pb->dir == 1)
+//			bs = pb->size * base;
+//	}
 
 	if (as > bs)
 		return 1;
@@ -398,12 +380,12 @@ print_sort_method(void)
 	case SATIME:
 		printf(_("atime %s\n"), (conf.sort_reverse) ? "[rev]" : "");	break;
 	case SBTIME:
-#if defined(HAVE_ST_BIRTHTIME) || defined(__BSD_VISIBLE) || defined(_STATX)
+#if defined(HAVE_ST_BIRTHTIME) || defined(__BSD_VISIBLE) || defined(LINUX_STATX)
 		printf(_("btime %s\n"), (conf.sort_reverse) ? "[rev]" : ""); break;
 #else
 		printf(_("btime (not available: using 'ctime') %s\n"),
 		    (conf.sort_reverse) ? "[rev]" : ""); break;
-#endif /* HAVE_ST_BIRTHTIME || __BSD_VISIBLE || _STATX */
+#endif /* HAVE_ST_BIRTHTIME || __BSD_VISIBLE || LINUX_STATX */
 	case SCTIME:
 		printf(_("ctime %s\n"), (conf.sort_reverse) ? "[rev]" : ""); break;
 	case SMTIME:
@@ -450,13 +432,14 @@ re_sort_files_list(void)
  * Return zero if ARG corresponds to a valid sorting method or one
  * otherwise. */
 static inline int
-_set_sort_by_name(char **arg)
+set_sort_by_name(char **arg)
 {
 	size_t i;
 	for (i = 0; i <= SORT_TYPES; i++) {
-		if (*(*arg) == *_sorts[i].name && strcmp(*arg, _sorts[i].name) == 0) {
+		if (*(*arg) == *sort_methods[i].name
+		&& strcmp(*arg, sort_methods[i].name) == 0) {
 			*arg = (char *)xrealloc(*arg, 32 * sizeof(char));
-			snprintf(*arg, 32, "%d", _sorts[i].num);
+			snprintf(*arg, 32, "%d", sort_methods[i].num);
 			return EXIT_SUCCESS;
 		}
 	}
@@ -482,7 +465,7 @@ sort_function(char **arg)
 			return re_sort_files_list();
 		}
 
-		if (_set_sort_by_name(&arg[1]) == EXIT_FAILURE)
+		if (set_sort_by_name(&arg[1]) == EXIT_FAILURE)
 			return EXIT_FAILURE;
 	}
 
