@@ -35,17 +35,17 @@
 #if defined(__HAIKU__)
 # include <stdint.h>
 #endif /* __HAIKU__ */
-#include <glob.h>
 #include <stdio.h>
 #include <string.h>
-#include <time.h>
+#ifndef HAVE_ARC4RANDOM
+# include <time.h> /* time(2) */
+#endif /* !HAVE_ARC4RANDOM */
 #include <wchar.h>
 #if !defined(__HAIKU__) && !defined(__OpenBSD__) && !defined(__ANDROID__)
 # define HAVE_WORDEXP
 # include <wordexp.h>
 #endif /* !__HAIKU__ && !__OpenBSD && !__ANDROID */
 #include <limits.h>
-#include <dirent.h>
 #include <errno.h>
 
 #if defined(__OpenBSD__)
@@ -86,31 +86,47 @@ typedef char *rl_cpvfunc_t;
 #define MAX_STR_LEN 4096
 #define INT_ARRAY_MAX 32
 
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 # define IS_FILE_TYPE_FILTER(x) ((x) == 'b' || (x) == 'c' || (x) == 'C' \
 || (x) == 'd' || (x) == 'f' || (x) == 'g' || (x) == 'h' || (x) == 'l' \
 || (x) == 'o' || (x) == 'p' || (x) == 's' || (x) == 't' || (x) == 'u' \
-|| (x) == 'x' || (x) == 'D')
+|| (x) == 'x' || (x) == 'D' || (x) == 'P')
 #else
 # define IS_FILE_TYPE_FILTER(x) ((x) == 'b' || (x) == 'c' || (x) == 'C' \
 || (x) == 'd' || (x) == 'f' || (x) == 'g' || (x) == 'h' || (x) == 'l' \
 || (x) == 'o' || (x) == 'p' || (x) == 's' || (x) == 't' || (x) == 'u' \
 || (x) == 'x')
-#endif /* __sun */
+#endif /* SOLARIS_DOORS */
 
 #define IS_GLOB(x, y) (((x) == '*' || (x) == '?' || (x) == '{' ) && (y) != ' ')
 
 #define IS_WORD(x, y) (((x) == '$' && ((y) == '(' || (y) == '{')) \
 || ((x) == '`' && (y) != ' ') || (x) == '~' || (x) == '$')
 
-static char len_buf[ARG_MAX * sizeof(wchar_t)] __attribute__((aligned));
-
 /* QUOTED_WORDS stores indices of words quoted in the command line so that we
  * can keep track of them and prevent expanding them when spliting the
  * input string (in parse_input_str()). */
 static int quoted_words[INT_ARRAY_MAX];
 #define QWORDS_ARRAY_LEN (sizeof(quoted_words) / sizeof(int))
 
+/* Quote the string STR according to conf.quoting_style, that is, using either
+ * single or double quotes. */
+char *
+quote_str(const char *str)
+{
+	if (!str || !*str || conf.quoting_style == QUOTING_STYLE_BACKSLASH)
+		return (char *)NULL;
+
+	const size_t len = strlen(str) + 3;
+	char *p = (char *)xnmalloc(len, sizeof(char));
+	char quote_char =
+		conf.quoting_style == QUOTING_STYLE_DOUBLE_QUOTES ? '"' : '\'';
+
+	snprintf(p, len, "%c%s%c", quote_char, str, quote_char);
+
+	return p;
+}
+
 /* Get the last occurrence of the (non-escaped) character C in STR (whose
  * length is LEN). Return a pointer to it if found or NULL if not. */
 char *
@@ -160,7 +176,7 @@ xstrcasechr(char *s, char c)
 	if (!s || !*s)
 		return (char *)NULL;
 
-	char uc = TOUPPER(c);
+	const char uc = TOUPPER(c);
 	while (*s) {
 		if (TOUPPER(*s) != uc) {
 			s++;
@@ -193,39 +209,39 @@ xstrrpbrk(char *s, const char *accept)
 	return (char *)NULL;
 }
 
-#if (defined(__linux__) || defined(__CYGWIN__)) && defined(_BE_POSIX)
-/* strcasestr(3) is a GNU extension on Linux. If GNU extensions are not
- * available, let's use this as replacement. */
+#ifdef _BE_POSIX
+/* strcasestr(3) is a not POSIX: let's use this as replacement.
+ * Find the first occurrence of the string B in the string A, ignoring case. */
 char *
-xstrcasestr(char *a, char *b)
+x_strcasestr(char *a, char *b)
 {
 	if (!a || !b)
 		return (char *)NULL;
 
 	size_t f = 0;
 	char *p = (char *)NULL, *bb = b;
-	while (*a) {
+	while (*a && *b) {
 		if (TOUPPER(*a) != TOUPPER(*b)) {
-			if (f == 1)
+			if (f == 1) {
 				b = bb;
-			++a;
+				f = 0;
+			} else {
+				++a;
+			}
+
 			continue;
 		}
 
-		p = a;
+		if (f == 0)
+			p = a;
 		f = 1;
 		++a;
-
-		if (!*(++b))
-			break;
+		++b;
 	}
 
-	if (!*b && f == 1)
-		return p;
-
-	return (char *)NULL;
+	return (!*b && f == 1) ? p : (char *)NULL;
 }
-#endif /* (__linux__ || __CYGWIN__) && _BE_POSIX */
+#endif /* _BE_POSIX */
 
 /* Just a strlen that sets a read limit in case of non-null terminated string. */
 size_t
@@ -288,7 +304,7 @@ xstrverscmp(const char *s1, const char *s2)
 	/* Jump to first alphanumeric character in both strings */
 	while (*p1) {
 		if (!IS_DIGIT(*p1) && !IS_ALPHA(*p1) && (*p1 < 'A' || *p1 > 'Z')) {
-			p1++;
+			++p1;
 			continue;
 		}
 		break;
@@ -299,7 +315,7 @@ xstrverscmp(const char *s1, const char *s2)
 
 	while (*p2) {
 		if (!IS_DIGIT(*p2) && !IS_ALPHA(*p2) && (*p2 < 'A' || *p2 > 'Z')) {
-			p2++;
+			++p2;
 			continue;
 		}
 		break;
@@ -340,8 +356,8 @@ xstrverscmp(const char *s1, const char *s2)
 	} else {
 		c1 = *p1;
 		c2 = *p2;
-		p1++;
-		p2++;
+		++p1;
+		++p2;
 	}
 
 	/* Hint: '0' is a digit too */
@@ -360,8 +376,8 @@ xstrverscmp(const char *s1, const char *s2)
 		} else {
 			c1 = *p1;
 			c2 = *p2;
-			p1++;
-			p2++;
+			++p1;
+			++p2;
 		}
 		state += (c1 == '0') + (IS_DIGIT(c1) != 0);
 	}
@@ -372,10 +388,10 @@ xstrverscmp(const char *s1, const char *s2)
 	case VCMP: return diff;
 	case VLEN:
 		while (*p1 && IS_DIGIT(*p1)) {
-			p1++;
+			++p1;
 			if (!*p2 || !IS_DIGIT(*p2))
 				return 1;
-			p2++;
+			++p2;
 		}
 		return (*p2 && IS_DIGIT(*p2)) ? -1 : diff;
 
@@ -383,21 +399,30 @@ xstrverscmp(const char *s1, const char *s2)
 	}
 }
 
-/* A strlen implementation able to handle wide chars. */
+/* A strlen implementation able to handle wide chars.
+ * Returns the number of columns needed to print the string STR (instead
+ * of the number of bytes needed to store STR). */
 size_t
 wc_xstrlen(const char *restrict str)
 {
-	size_t len;
-	wchar_t *const wbuf = (wchar_t *)len_buf;
-
 	/* Convert multi-byte to wide char */
-	len = mbstowcs(wbuf, str, ARG_MAX);
+	/* Most of the time we use this function to get the number of characters
+	 * in names: in this case a buffer of NAME_MAX + 1 is enough. However, we
+	 * sometimes use this function to get the number of characters in the
+	 * current command line (rl_line_buffer), which has no size limit beyond
+	 * what an int can hold (INT_MAX), which is huge. ARG_MAX (128k or more),
+	 * though much smaller than INT_MAX, should be enough most of the time.
+	 * Even PATH_MAX, usually 4096, is still too narrow to hold a complete
+	 * command line: a history entry involving several paths might easily be
+	 * longer than PATH_MAX. */
+	static wchar_t wbuf[ARG_MAX];
+	const size_t len = mbstowcs(wbuf, str, ARG_MAX);
 	if (len == (size_t)-1) /* Invalid multi-byte sequence found */
 		return 0;
 
-	int w = wcswidth(wbuf, len);
-	if (w != -1)
-		return (size_t)w;
+	int width = wcswidth(wbuf, len);
+	if (width != -1)
+		return (size_t)width;
 
 	/* A non-printable wide char was found */
 	return 0;
@@ -416,9 +441,9 @@ int
 u8truncstr(char *restrict str, const size_t max)
 {
 	int len = 0;
-	wchar_t buf[PATH_MAX];
+	static wchar_t buf[NAME_MAX + 1];
 	*buf = L'\0';
-	if (mbstowcs(buf, str, PATH_MAX) == (size_t)-1)
+	if (mbstowcs(buf, str, NAME_MAX + 1) == (size_t)-1)
 		return 0;
 
 	int i, bmax = (int)max;
@@ -434,30 +459,62 @@ u8truncstr(char *restrict str, const size_t max)
 		len += l;
 	}
 
-	wcsncpy((wchar_t *)str, buf, (size_t)bmax * sizeof(wchar_t));
-//	wcscpy((wchar_t *)str, buf);
+	wcsncpy((wchar_t *)str, buf, (size_t)bmax * sizeof(wchar_t)); /* flawfinder: ignore */
+
 	return bmax - len;
 }
 
-/* Replace control characters in NAME by '^'. */
+/* Replace invalid characters in NAME by INVALID_CHAR ('^').
+ * This function is called only if wc_xstrlen() returns zero, in which case
+ * we have either a non-printable char or an invalid multi-byte sequence. */
 char *
-replace_ctrl_chars(const char *name)
+replace_invalid_chars(const char *name)
 {
-	int i;
-	char *n = (char *)xnmalloc(NAME_MAX, sizeof(char));
+	size_t len = strlen(name);
+	char *n = (char *)xnmalloc(len + 1, sizeof(char));
 	char *p = n;
 
-	for (i = 0; name[i]; i++) {
-		if (i == NAME_MAX)
-			break;
-		if ((signed char)name[i] >= 0 && (signed char)name[i] < ' ')
-			*n = '^';
-		else
-			*n = name[i];
-		n++;
+	mbstate_t mbstate = {0};
+	char *q = (char *)name;
+	char *qlimit = q + len;
+
+	while (*q) {
+		if (*q >= ' ' && *q < 127) { /* Printable ASCII char */
+			*n = *q;
+			n++; q++;
+			continue;
+		}
+
+#if defined(CHAR_MIN) && CHAR_MIN < 0
+		if (*q >= '\0' && *q < ' ') { /* Control char */
+#else
+		if (*q < ' ') {
+#endif /* CHAR_MIN < 0 */
+			*n = INVALID_CHR;
+			n++; q++;
+			continue;
+		}
+
+		do {
+			wchar_t wc;
+			size_t bytes = mbrtowc(&wc, q, (size_t)(qlimit - q), &mbstate);
+			if (bytes == (size_t)-1 || bytes == (size_t)-2) {
+				*n = INVALID_CHR; /* Invalid UTF-8 */
+				n++; q++;
+			} else { /* Valid UTF-8 */
+				for (; bytes > 0; --bytes) {
+					*n = *q;
+					n++; q++;
+				}
+			}
+
+			if (!*q)
+				break;
+		} while (mbsinit(&mbstate) == 0);
 	}
 
 	*n = '\0';
+
 	return p;
 }
 
@@ -524,7 +581,7 @@ strbfrlst(char *str, const char c)
 
 	*q = '\0';
 
-	size_t buf_len = (size_t)(q - str);
+	const size_t buf_len = (size_t)(q - str);
 	char *buf = (char *)malloc(buf_len + 1);
 	if (!buf) {
 		*q = c;
@@ -562,7 +619,7 @@ strbtw(char *str, const char a, const char b)
 
 	*pb = '\0';
 
-	size_t buf_len = (size_t)(pb - pa);
+	const size_t buf_len = (size_t)(pb - pa);
 	char *buf = (char *)malloc(buf_len + 1);
 
 	if (!buf) {
@@ -592,7 +649,7 @@ replace_substr(const char *haystack, const char *needle, char *rep)
 	size_t new_str_len = 0;
 
 	if (*needle_end) {
-		size_t rem_len = strlen(needle_end);
+		const size_t rem_len = strlen(needle_end);
 		char *rem = (char *)xnmalloc(rem_len + 1, sizeof(char));
 		xstrsncpy(rem, needle_end, rem_len + 1);
 
@@ -614,18 +671,24 @@ replace_substr(const char *haystack, const char *needle, char *rep)
 char *
 gen_rand_str(const size_t len)
 {
-	const char charset[] = "0123456789#%-_"
-			 "abcdefghijklmnopqrstuvwxyz"
-			 "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+	static const char charset[] = "0123456789#%-_"
+		"abcdefghijklmnopqrstuvwxyz"
+		"ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
-	srand((unsigned int)time(NULL));
+#ifndef HAVE_ARC4RANDOM
+	srandom((unsigned int)time(NULL));
+#endif /* !HAVE_ARC4RANDOM */
 
 	char *p = (char *)xnmalloc(len + 1, sizeof(char));
 	char *str = p;
 
 	int x = (int)len;
 	while (x--) {
-		int i = rand() % (int)(sizeof(charset) - 1);
+#ifndef HAVE_ARC4RANDOM
+		const long i = random() % (int)(sizeof(charset) - 1);
+#else
+		const uint32_t i = arc4random_uniform(sizeof(charset) - 1);
+#endif /* !HAVE_ARC4RANDOM */
 		*p = charset[i];
 		p++;
 	}
@@ -724,11 +787,26 @@ is_quoted_word(const size_t index)
 /* Some commands need quotes to be preserved (they'll handle quotes themselves
  * later). Return 1 if true or 0 otherwise. */
 int
-cmd_keeps_quotes(const char *str)
+cmd_keeps_quotes(char *str)
 {
+	if (rl_dispatching == 1)
+		return 0;
+
 	if (flags & IN_BOOKMARKS_SCREEN)
 		return 0;
 
+	if (*str == '\'' || *str == '"') /* Quoted first word */
+		return 0;
+
+	char *p = strchr(str, ' ');
+	if (p) {
+		*p = '\0';
+		int ret = is_internal_c(str);
+		*p = ' ';
+		if (ret == 0)
+			return 1;
+	}
+
 	return (strncmp(str, "ft ", 3) == 0 || strncmp(str, "filter ", 7) == 0);
 }
 
@@ -742,7 +820,7 @@ cmd_keeps_quotes(const char *str)
  * str is NULL or if no substring was found, i.e., if str contains
  * only spaces. */
 char **
-split_str(const char *str, const int update_args)
+split_str(char *str, const int update_args)
 {
 	if (!str)
 		return (char **)NULL;
@@ -957,9 +1035,9 @@ split_str(const char *str, const int update_args)
 
 /* Return 1 if STR contains only numbers or a range of numbers, or 0 if not. */
 static int
-check_fused_param(const char *str)
+check_fused_param(char *str)
 {
-	char *p = (char *)str;
+	char *p = str;
 	size_t c = 0, i = 0;
 	int ok = 1;
 
@@ -989,8 +1067,8 @@ split_fused_param(char *str)
 	if (!str || !*str || *str == ';' || *str == ':' || *str == '\\')
 		return (char *)NULL;
 
-	char *space = strchr(str, ' ');
-	char *slash = strchr(str, '/');
+	const char *space = strchr(str, ' ');
+	const char *slash = strchr(str, '/');
 
 	if (!space && slash) /* If "/some/path/" */
 		return (char *)NULL;
@@ -1042,7 +1120,7 @@ split_fused_param(char *str)
 	*b = '\0';
 
 	/* Readjust the buffer size */
-	size_t len = strlen(buf);
+	const size_t len = strlen(buf);
 	buf = (char *)xrealloc(buf, (len + 1) * sizeof(char));
 	return buf;
 }
@@ -1054,8 +1132,8 @@ check_shell_functions(const char *str)
 		return 0;
 
 	if (int_vars == 0) { /* Take assignements as shell functions */
-		char *s = strchr(str, ' ');
-		char *e = strchr(str, '=');
+		const char *s = strchr(str, ' ');
+		const char *e = strchr(str, '=');
 		if (!s && e)
 			return 1;
 		if (s && e && e < s)
@@ -1143,7 +1221,7 @@ is_fused_param(char *str)
  * ARGS is an array with the current input substrings, and TAG_INDEX
  * is the index of the tag expresion (t:) in this array.
  * The expansion is performed in ARGS array itself.
- * Returns the number of files tagged as ARGS[TAG_INDEX] or zero on error */
+ * Returns the number of files tagged as ARGS[TAG_INDEX] or zero on error. */
 static size_t
 expand_tag(char ***args, const int tag_index)
 {
@@ -1284,9 +1362,9 @@ expand_mime_type_filter(const char *pattern)
 	if (!pattern || !*pattern)
 		return (char **)NULL;
 
-	char **t = (char **)xnmalloc(files + 1, sizeof(char *));
+	char **t = (char **)xnmalloc((size_t)files + 1, sizeof(char *));
 
-	size_t i, n = 0;
+	filesn_t i, n = 0;
 	for (i = 0; i < files; i++) {
 		char *m = xmagic(file_info[i].name, MIME_TYPE);
 		if (!m) continue;
@@ -1305,7 +1383,7 @@ expand_mime_type_filter(const char *pattern)
 	if (n == 0)
 		{ free(t); return (char **)NULL; }
 
-	t = (char **)xrealloc(t, (n + 1) * sizeof(char *));
+	t = (char **)xrealloc(t, ((size_t)n + 1) * sizeof(char *));
 	return t;
 }
 #endif /* !_NO_MAGIC */
@@ -1316,10 +1394,10 @@ expand_file_type_filter(const char t)
 	if (files == 0)
 		return (char **)NULL;
 
-	size_t i = 0, n = 0;
+	filesn_t i = 0, n = 0;
 
 	char *name = (char *)NULL;
-	char **f = (char **)xnmalloc(files + 1, sizeof(char *));
+	char **f = (char **)xnmalloc((size_t)files + 1, sizeof(char *));
 
 	while (i < files && (name = file_info[i].name)) {
 		switch (t) {
@@ -1339,12 +1417,16 @@ expand_file_type_filter(const char t)
 			if (file_info[i].dir == 1)
 				f[n++] = strdup(name);
 			break;
-#ifdef __sun
+#ifdef SOLARIS_DOORS
 		case 'D':
 			if (file_info[i].type == DT_DOOR)
 				f[n++] = strdup(name);
 			break;
-#endif /* __sun */
+		case 'P':
+			if (file_info[i].type == DT_PORT)
+				f[n++] = strdup(name);
+			break;
+#endif /* SOLARIS_DOORS */
 		case 'f':
 			if (file_info[i].type == DT_REG)
 				f[n++] = strdup(name);
@@ -1399,7 +1481,7 @@ expand_file_type_filter(const char t)
 	}
 
 	f[n] = (char *)NULL;
-	f = (char **)xrealloc(f, (n + 1) * sizeof(char *));
+	f = (char **)xrealloc(f, ((size_t)n + 1) * sizeof(char *));
 
 	return f;
 }
@@ -1476,17 +1558,38 @@ insert_fields(char ***dst, char ***src, const size_t i, size_t *num)
 	return d;
 }
 
-/* Expand the ELN at SUBSTR[I] into the corresponding file name. */
+/* Expand the ELN at SUBSTR[I] into the corresponding file name, which is
+ * quoted, if necessary, according to the value of conf.quoting_style. */
 static void
 eln_expand(char ***substr, const size_t i)
 {
-	int num = atoi((*substr)[i]);
-	/* Because of _expand_eln(), which is called immediately before this
+	const filesn_t num = xatof((*substr)[i]);
+	if (num == -1)
+		return;
+
+	/* Because of should_expand_eln(), which is called immediately before this
 	 * function, it is guaranteed that NUM won't over/under-flow:
 	 * NUM is > 0 and <= the amount of listed files (and this latter is
-	 * never greater than INT_MAX). */
-	int j = num - 1;
-	char *esc_str = escape_str(file_info[j].name);
+	 * never bigger than FILESN_MAX). */
+	filesn_t j = num - 1;
+
+	/* If file name starts with a dash, and the command is external,
+	 * use the absolute path to the file name, to prevent the command from
+	 * taking the file name as a command option. */
+	char *abs_path = (char *)NULL;
+	if (file_info[j].name && *file_info[j].name == '-'
+	&& !is_internal_c((*substr)[0]))
+		abs_path = realpath(file_info[j].name, NULL);
+
+	char *esc_str = (char *)NULL;
+	if (conf.quoting_style == QUOTING_STYLE_BACKSLASH
+	|| is_internal_c((*substr)[0]) || is_number((*substr)[0]))
+		esc_str = escape_str(abs_path ? abs_path : file_info[j].name);
+	else
+		esc_str = quote_str(abs_path ? abs_path : file_info[j].name);
+
+	free(abs_path);
+
 	if (!esc_str)
 		return;
 
@@ -1504,6 +1607,9 @@ eln_expand(char ***substr, const size_t i)
 		free((*substr)[i]);
 		(*substr)[i] = esc_str;
 	}
+
+	if ((*substr)[0][0] == 'r' && !(*substr)[0][1])
+		flags |= REMOVE_ELN;
 }
 
 static void
@@ -1577,6 +1683,16 @@ expand_sel_keyword(char ***substr)
 			expand_sel(substr);
 		}
 	}
+
+	/* If "sel" is last, and there are selected elements, and the command
+	 * is either "cp" or "mv", emulate "c" or "m" respectivelly: later,
+	 * and final "." will be added to the command, so that we avoid
+	 * overwriting files. */
+	if (sel_n == 0 || is_sel == 0 || sel_is_last == 0)
+		return;
+
+	if (strcmp((*substr)[0], "cp") == 0 || strcmp((*substr)[0], "mv") == 0)
+		(*substr)[0][1] = '\0';
 }
 
 /* Expand the bookmark NAME into the corresponding bookmark path.
@@ -1877,9 +1993,8 @@ expand_word(char ***substr, const int *word_array, const size_t word_n)
 	size_t old_pathc = 0;
 	size_t w = 0, i = 0;
 
-	int is_sel_cmd = 0;
-	if (strcmp((*substr)[0], "s") == 0 || strcmp((*substr)[0], "sel") == 0)
-		is_sel_cmd = 1;
+	const int is_sel_cmd =
+		(strcmp((*substr)[0], "s") == 0 || strcmp((*substr)[0], "sel") == 0);
 
 	for (w = 0; w < word_n; w++) {
 		if (is_sel_cmd == 1) {
@@ -1967,15 +2082,15 @@ check_ranges(char ***substr, int **range_array)
 		if (!(*substr)[i] || is_quoted_word(i) || lstat((*substr)[i], &a) != -1)
 			continue;
 
-		size_t len = strlen((*substr)[i]);
+		const size_t len = strlen((*substr)[i]);
 
 		for (j = 0; (*substr)[i][j]; j++) {
 			/* If some alphabetic char, besides '-', is found in the
-			 * string, we have no range */
+			 * string, we have no range. */
 			if ((*substr)[i][j] != '-' && !IS_DIGIT((*substr)[i][j]))
 				break;
 
-			/* If a range is found, store its index */
+			/* If a range is found, store its index. */
 			if (j > 0 && j < len && (*substr)[i][j] == '-'
 			&& IS_DIGIT((*substr)[i][j - 1])
 			&& IS_DIGIT((*substr)[i][j + 1])) {
@@ -1990,13 +2105,71 @@ check_ranges(char ***substr, int **range_array)
 	return n;
 }
 
+/* Expand a range of numbers given by STR. It will expand the range
+ * provided that both extremes are numbers, bigger than zero, equal or
+ * smaller than the amount of files currently listed on the screen, and
+ * the second (right) extreme is bigger than the first (left). Returns
+ * an array of int's with the expanded range or NULL if one of the
+ * above conditions is not met. */
+static filesn_t *
+expand_range(char *str, int listdir)
+{
+	if (!str || !*str)
+		return (filesn_t *)NULL;
+
+	struct stat a;
+	if (lstat(str, &a) != -1)
+		return (filesn_t *)NULL;
+
+	char *p = strchr(str, '-');
+	if (!p || p == str || *(p - 1) < '0' || *(p - 1) > '9'
+	|| !*(p + 1) || *(p + 1) < '0' || *(p + 1) > '9')
+		return (filesn_t *)NULL;
+
+	*p = '\0';
+	int ret = is_number(str);
+	*p = '-';
+	if (!ret)
+		return (filesn_t *)NULL;
+
+	const filesn_t afirst = xatof(str);
+
+	++p;
+	if (!is_number(p))
+		return (filesn_t *)NULL;
+
+	const filesn_t asecond = xatof(p);
+	if (afirst == -1 || asecond == -1)
+		return (filesn_t *)NULL;
+
+	if (listdir) {
+		if (afirst <= 0 || afirst > files || asecond <= 0
+		|| asecond > files || afirst >= asecond)
+			return (filesn_t *)NULL;
+	} else {
+		if (afirst >= asecond) 
+			return (filesn_t *)NULL;
+	}
+
+	filesn_t *buf =
+		(filesn_t *)xcalloc((size_t)(asecond - afirst) + 2, sizeof(filesn_t));
+
+	filesn_t i, j = 0;
+	for (i = afirst; i <= asecond; i++) {
+		buf[j] = i;
+		j++;
+	}
+
+	return buf;
+}
+
 static void
 expand_ranges(char ***substr)
 {
 	size_t i = 0, j = 0;
 
 	int *range_array = (int *)xnmalloc(INT_ARRAY_MAX, sizeof(int));
-	size_t ranges_ok = check_ranges(substr, &range_array);
+	const size_t ranges_ok = check_ranges(substr, &range_array);
 
 	if (ranges_ok == 0) {
 		free(range_array);
@@ -2008,7 +2181,7 @@ expand_ranges(char ***substr)
 
 	for (r = 0; r < ranges_ok; r++) {
 		size_t ranges_n = 0;
-		int *ranges = expand_range((*substr)[range_array[r] +
+		filesn_t *ranges = expand_range((*substr)[range_array[r] +
 			(int)old_ranges_n], 1);
 
 		if (ranges) {
@@ -2030,9 +2203,9 @@ expand_ranges(char ***substr)
 			}
 
 			for (i = 0; i < ranges_n; i++) {
-				size_t len = (size_t)DIGINUM(ranges[i]) + 1;
+				const size_t len = (size_t)DIGINUM(ranges[i]) + 1;
 				ranges_cmd[j] = (char *)xnmalloc(len, sizeof(int));
-				snprintf(ranges_cmd[j], len, "%d", ranges[i]);
+				snprintf(ranges_cmd[j], len, "%zd", ranges[i]);
 				j++;
 			}
 
@@ -2066,15 +2239,18 @@ expand_regex(char ***substr)
 {
 	/* Let's store all strings currently in substr plus REGEX expanded
 	 * files, if any, in a temporary array. */
-	char **tmp = (char **)xnmalloc(files + args_n + 2, sizeof(char *));
-	size_t i, j, n = 0;
+	char **tmp = (char **)xnmalloc((size_t)files + args_n + 2, sizeof(char *));
+	filesn_t i, j;
+	size_t n = 0;
 	regex_t regex;
 
-	int reg_flags = conf.case_sens_list == 1 ? (REG_NOSUB | REG_EXTENDED)
-			: (REG_NOSUB | REG_EXTENDED | REG_ICASE);
+/*	int reg_flags = conf.case_sens_list == 1 ? (REG_NOSUB | REG_EXTENDED)
+			: (REG_NOSUB | REG_EXTENDED | REG_ICASE); */
+
+	int reg_flags = (REG_NOSUB | REG_EXTENDED);
 
 	for (i = 0; (*substr)[i]; i++) {
-		if (n > (files + args_n))
+		if (n > ((size_t)files + args_n))
 			break;
 
 		/* Ignore the first string of the search function: it will be
@@ -2092,20 +2268,30 @@ expand_regex(char ***substr)
 		if (p)
 			dstr = dequote_str((*substr)[i], 0);
 
-		char *rstr = dstr ? dstr : (*substr)[i];
+		char *t = dstr ? dstr : (*substr)[i];
+
+		/* Prepend an initial '^' and append and ending '$' to prevent
+		 * accidental file expansions. For example, a file named file.txt
+		 * must not be expanded given the pattern "ile.t". In other words,
+		 * we force the use of ".*PATTERN.*" instead of just "PATTERN". */
+		size_t l = strlen(t) + 3;
+		char *rstr = (char *)xnmalloc(l, sizeof(char));
+		snprintf(rstr, l, "^%s$", t);
+		free(dstr);
+
 		int ret = check_regex(rstr);
 
 		if (ret != EXIT_SUCCESS
 		|| regcomp(&regex, rstr, reg_flags) != EXIT_SUCCESS) {
 			if (ret == EXIT_SUCCESS)
 				regfree(&regex);
-			free(dstr);
+			free(rstr);
 			tmp[n] = (*substr)[i];
 			n++;
 			continue;
 		}
 
-		free(dstr);
+		free(rstr);
 		int reg_found = 0;
 
 		for (j = 0; j < files; j++) {
@@ -2113,7 +2299,7 @@ expand_regex(char ***substr)
 				continue;
 
 			/* Make sure the matching file name is not already in the tmp array */
-			int m = (int)n, found = 0;
+			filesn_t m = (filesn_t)n, found = 0;
 			while (--m >= 0) {
 				if (*file_info[j].name == *tmp[m]
 				&& strcmp(file_info[j].name, tmp[m]) == 0)
@@ -2212,7 +2398,7 @@ glob_expand(char **cmd)
 
 /* Return 0 if CMD should be regex expanded, or 1 otherwise. */
 static int
-regex_expand(char *cmd)
+regex_expand(const char *cmd)
 {
 	if (!cmd || !*cmd)
 		return 0;
@@ -2357,6 +2543,8 @@ parse_input_str(char *str)
 	int fusedcmd_ok = 0;
 
 	flags &= ~FIRST_WORD_IS_ELN;
+	flags &= ~REMOVE_ELN;
+	flags &= ~IS_USRVAR_DEF;
 
 	/* If internal command plus fused parameter, split it */
 	if (is_fused_param(str) == EXIT_SUCCESS) {
@@ -2480,7 +2668,7 @@ parse_input_str(char *str)
 	 * even with external commands, they must be expanded here, before
 	 * sending the input string, in case the command is external, to
 	 * the system shell. */
-	is_sel = 0, sel_is_last = 0;
+	is_sel = 0; sel_is_last = 0;
 
 	int stdin_dir_ok = 0;
 	if (stdin_tmp_dir && strcmp(workspaces[cur_ws].path, stdin_tmp_dir) == 0)
@@ -2500,8 +2688,17 @@ parse_input_str(char *str)
 				 * #   2.1) ELN EXPANSION   #
 				 * ########################## */
 
-		if (_expand_eln(substr[i]) == 1)
+		/* should_expand_eln() will check rl_line_buffer looking for the
+		 * command name. Now, if the command name has a fused ELN plus space
+		 * and at least a second parameter (ex: CMD1 2), should_expand_eln()
+		 * will fail. Let's redirect rl_line_buffer to the buffered command
+		 * name (CMD) so that the check will be properly performed. */
+		char *lb_tmp = rl_line_buffer;
+		if (rl_dispatching == 0 && fusedcmd_ok == 1)
+			rl_line_buffer = substr[0];
+		if (should_expand_eln(substr[i]) == 1)
 			eln_expand(&substr, i);
+		rl_line_buffer = lb_tmp;
 
 				/* ################################
 				 * #  2.2) ENVIRONEMNT VARIABLES  #
@@ -2510,9 +2707,9 @@ parse_input_str(char *str)
 		if (*substr[i] == '$') {
 			char *p = getenv(substr[i] + 1);
 			if (p) {
-				size_t plen = strlen(p);
-				substr[i] = (char *)xrealloc(substr[i], (plen + 1) * sizeof(char));
-				xstrsncpy(substr[i], p, plen + 1);
+				size_t plen = strlen(p) + 1;
+				substr[i] = (char *)xrealloc(substr[i], plen * sizeof(char));
+				xstrsncpy(substr[i], p, plen);
 			}
 		}
 
@@ -2666,8 +2863,8 @@ parse_input_str(char *str)
 	 * made here (in the lobby [got it?]) in order to be able to understand
 	 * these expansions at all. */
 
-	/* Let's first mark substrings containing special expansion made by either
-	 * glob(3) and wordexp(3) */
+	/* Let's first mark substrings containing special expansions made by either
+	 * glob(3) and wordexp(3). */
 
 	int *glob_array = (int *)xnmalloc(INT_ARRAY_MAX, sizeof(int));
 	size_t glob_n = 0;
@@ -2684,14 +2881,14 @@ parse_input_str(char *str)
 		 * elements: they are full path file names that, as such, do not
 		 * need any expansion. */
 		if (is_sel > 0) { /* is_sel is true only for the current input and if
-			there was some "sel" keyword in it */
+			there was some "sel" keyword in it. */
 			/* Strings between is_sel and sel_n are selected file names.
 			 * Skip them. */
 			if (i >= (size_t)is_sel && i <= sel_n)
 				continue;
 		}
 
-		/* Ignore the first string of the search function: it will be
+		/* Ignore the first word of the search function: it will be
 		 * expanded by the search function itself. */
 		if (substr[0][0] == '/' && i == 0)
 			continue;
@@ -2752,7 +2949,6 @@ parse_input_str(char *str)
 	free(word_array);
 #endif /* HAVE_WORDEXP */
 
-
 			/* #######################################
 			 * #       3.3) REGEX EXPANSION          #
 			 * ####################################### */
@@ -2805,64 +3001,6 @@ home_tilde(char *new_path, int *free_buf)
 	return new_path;
 }
 
-/* Expand a range of numbers given by STR. It will expand the range
- * provided that both extremes are numbers, bigger than zero, equal or
- * smaller than the amount of files currently listed on the screen, and
- * the second (right) extreme is bigger than the first (left). Returns
- * an array of int's with the expanded range or NULL if one of the
- * above conditions is not met. */
-int *
-expand_range(char *str, int listdir)
-{
-	if (!str || !*str)
-		return (int *)NULL;
-
-	struct stat a;
-	if (lstat(str, &a) != -1)
-		return (int *)NULL;
-
-	char *p = strchr(str, '-');
-	if (!p || p == str || *(p - 1) < '0' || *(p - 1) > '9'
-	|| !*(p + 1) || *(p + 1) < '0' || *(p + 1) > '9')
-		return (int *)NULL;
-
-	*p = '\0';
-	int ret = is_number(str);
-	*p = '-';
-	if (!ret)
-		return (int *)NULL;
-
-	int afirst = atoi(str);
-
-	++p;
-	if (!is_number(p))
-		return (int *)NULL;
-
-	int asecond = atoi(p);
-	if (afirst == INT_MIN || asecond == INT_MIN)
-		return (int *)NULL;
-
-	if (listdir) {
-		if (afirst <= 0 || afirst > (int)files || asecond <= 0
-		|| asecond > (int)files || afirst >= asecond)
-			return (int *)NULL;
-	} else {
-		if (afirst >= asecond) 
-			return (int *)NULL;
-	}
-
-	int *buf = (int *)NULL;
-	buf = (int *)xcalloc((size_t)(asecond - afirst) + 2, sizeof(int));
-
-	size_t i, j = 0;
-	for (i = (size_t)afirst; i <= (size_t)asecond; i++) {
-		buf[j] = (int)i;
-		j++;
-	}
-
-	return buf;
-}
-
 /* Returns a pointer to a copy of the string STR, malloc'ed with size SIZE, or
  * NULL on error. */
 char *
@@ -2881,9 +3019,9 @@ savestring(const char *restrict str, size_t size)
 	return ptr;
 }
 
-/* Take a string and returns the same string escaped. If nothing to be
- * escaped, the original string is returned. In either cases, the returned
- * string must be free'd by the caller. */
+/* Take a string and returns the same string escaped.
+ * If there is nothing to be escaped, the original string is returned. In
+ * either cases, the returned string must be free'd by the caller. */
 char *
 escape_str(const char *str)
 {
@@ -2923,7 +3061,7 @@ get_substr(char *str, const char ifs)
 
 	char **substr = (char **)NULL;
 	void *p = (char *)NULL;
-	size_t str_len = strlen(str);
+	const size_t str_len = strlen(str);
 	size_t length = 0, substr_n = 0;
 	char *buf = (char *)xnmalloc(str_len + 1, sizeof(char));
 
@@ -3017,7 +3155,7 @@ get_substr(char *str, const char ifs)
 
 				/* Make sure we have a valid range */
 				if (is_number(first) && is_number(second)) {
-					afirst = atoi(first), asecond = atoi(second);
+					afirst = atoi(first); asecond = atoi(second);
 					if (afirst == INT_MIN || asecond == INT_MIN) {
 						free(first);
 						free(second);
