@@ -33,36 +33,37 @@ __BEGIN_DECLS
 
 char *dequote_str(char *, int);
 char *escape_str(const char *);
-int  *expand_range(char *, int);
+//int  *expand_range(char *, int);
 char *gen_rand_str(const size_t);
 char *get_last_chr(char *, const char, const int);
 char **get_substr(char *, const char);
 char *home_tilde(char *, int *);
 char **parse_input_str(char *);
+char *quote_str(const char *);
 char *remove_quotes(char *);
 char *replace_slashes(char *, const char);
 char *replace_substr(const char *, const char *, char *);
 char *savestring(const char *restrict, size_t);
-char **split_str(const char *, const int);
+char **split_str(char *, const int);
 char *strbfrlst(char *, const char);
 char *strbtw(char *, const char, const char);
 int  strcntchr(const char *, const char);
 int  strcntchrlst(const char *, const char);
-char *replace_ctrl_chars(const char *);
+char *replace_invalid_chars(const char *);
 int  u8truncstr(char *restrict, size_t);
 size_t wc_xstrlen(const char *restrict);
 char *xstrrpbrk(char *, const char *);
 
-#if (defined(__linux__) || defined(__CYGWIN__)) && defined(_BE_POSIX)
-char *xstrcasestr(char *, char *);
-#endif /* (__linux__ || __CYGWIN__) && _BE_POSIX */
+#if defined(_BE_POSIX)
+char *x_strcasestr(char *, char *);
+#endif /* _BE_POSIX */
 
 char * xstrcasechr(char *, char);
 size_t xstrnlen(const char *restrict);
 size_t xstrsncpy(char *restrict, const char *restrict, size_t);
 char * xstrncat(char *restrict dst, const size_t dst_end,
 	const char *restrict src, const size_t dst_len);
-int xstrverscmp(const char *, const char *);
+int  xstrverscmp(const char *, const char *);
 
 __END_DECLS
 
