@@ -25,14 +25,10 @@
 
 #include "helpers.h"
 
-#include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
-#if defined(__OpenBSD__)
-# include <strings.h>
-#endif /* __OpenBSD__ */
+#include <strings.h> /* str(n)casecmp() */
 #include <unistd.h>
-#include <dirent.h>
 #include <pwd.h>
 
 #if defined(__linux__)
@@ -493,7 +489,7 @@ calculate_suggestion_lines(int *baej, const size_t suggestion_len)
 
 	if (cucs > term_cols) {
 		slines = cucs / (size_t)term_cols;
-		int cucs_rem = (int)cucs % term_cols;
+		const int cucs_rem = (int)cucs % term_cols;
 		if (cucs_rem > 0)
 			slines++;
 	}
@@ -508,9 +504,9 @@ truncate_name(const char *str)
 
 	if (suggestion.type == ELN_SUG || suggestion.type == COMP_SUG
 	|| suggestion.type == FILE_SUG) {
-		size_t wlen = wc_xstrlen(str);
+		const size_t wlen = wc_xstrlen(str);
 		if (wlen == 0)
-			wname = replace_ctrl_chars(str);
+			wname = replace_invalid_chars(str);
 	}
 
 	return wname;
@@ -547,7 +543,7 @@ check_conditions(const size_t offset, const size_t wlen, int *baej,
 	 * it equals ARG_MAX, in which case we most probably have a truncated
 	 * suggestion (mbstowcs will convert only up to ARG_MAX chars), exit */
 //	size_t suggestion_len = wc_xstrlen(str + offset);
-	size_t suggestion_len = wlen - offset;
+	const size_t suggestion_len = wlen - offset;
 	if (suggestion_len == 0 || suggestion_len == ARG_MAX
 	|| (int)suggestion_len > (term_cols * term_lines) - curcol)
 		return EXIT_FAILURE;
@@ -561,7 +557,7 @@ check_conditions(const size_t offset, const size_t wlen, int *baej,
 }
 
 static inline void
-_print_suggestion(const char *str, const size_t offset, const char *color)
+make_suggestion(const char *str, const size_t offset, const char *color)
 {
 	if (suggestion.type == FUZZY_FILENAME
 	|| (suggestion.type == COMP_SUG && (flags & BAEJ_SUGGESTION)))
@@ -631,7 +627,8 @@ print_suggestion(const char *str, size_t offset, char *color)
 			curcol -= term_cols;
 	}
 
-	size_t str_len = wc_xstrlen(str), slines = 0;
+	const size_t str_len = wc_xstrlen(str);
+	size_t slines = 0;
 	if (check_conditions(offset, str_len, &baej, &slines) == EXIT_FAILURE) {
 		UNHIDE_CURSOR;
 		return;
@@ -651,7 +648,7 @@ print_suggestion(const char *str, size_t offset, char *color)
 	}
 
 	set_cursor_position(baej);
-	_print_suggestion(str, offset, color);
+	make_suggestion(str, offset, color);
 
 	restore_cursor_position(slines);
 
@@ -674,41 +671,41 @@ get_reg_file_color(const char *filename, const struct stat *attr,
 	if (attr->st_mode & S_ISUID) return su_c;
 	if (attr->st_mode & S_ISGID) return sg_c;
 
-#ifdef _LINUX_CAP
-	cap_t cap = cap_get_file(filename);
+#ifdef LINUX_FILE_CAPS
+	const cap_t cap = cap_get_file(filename);
 	if (cap) {
 		cap_free(cap);
 		return ca_c;
 	}
-#endif /* _LINUX_CAP */
+#endif /* LINUX_FILE_CAPS */
 	if (attr->st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
-		return (FILE_SIZE_PTR == 0) ? ee_c : ex_c;
+		return (FILE_SIZE_PTR(attr) == 0) ? ee_c : ex_c;
 
-	if (FILE_SIZE_PTR == 0)	return ef_c;
+	if (FILE_SIZE_PTR(attr) == 0) return ef_c;
 	if (attr->st_nlink > 1)	return mh_c;
 
-	char *ext = check_ext == 1 ? strrchr(filename, '.') : (char *)NULL;
+	const char *ext = check_ext == 1 ? strrchr(filename, '.') : (char *)NULL;
 	if (!ext || ext == filename)
 		return fi_c;
 
-	char *extcolor = get_ext_color(ext);
+	size_t color_len = 0;
+	char *extcolor = get_ext_color(ext, &color_len);
 	if (!extcolor)
 		return fi_c;
 
-	size_t len = strlen(extcolor) + 4;
-	char *ext_color = (char *)xnmalloc(len, sizeof(char));
-	snprintf(ext_color, len, "\x1b[%sm", extcolor);
+	color_len += 4;
+	char *ext_color = (char *)xnmalloc(color_len, sizeof(char));
+	snprintf(ext_color, color_len, "\x1b[%sm", extcolor);
 
 	*free_color = 1;
 	return ext_color;
 }
 
 /* Used by the check_completions function to get file names color
- * according to file type */
+ * according to file type. */
 static char *
 get_comp_color(const char *filename, const struct stat *attr, int *free_color)
 {
-//	char *color = no_c;
 	char *color = (char *)NULL;
 
 	switch (attr->st_mode & S_IFMT) {
@@ -732,7 +729,6 @@ get_comp_color(const char *filename, const struct stat *attr, int *free_color)
 		}
 		return or_c;
 		}
-		break;
 
 	case S_IFSOCK: return so_c;
 	case S_IFBLK: return bd_c;
@@ -906,53 +902,53 @@ check_completions(char *str, size_t len, const int print)
 		return NO_MATCH;
 	}
 
-	*_fmatch = '\0';
+	*fz_match = '\0';
 	flags |= STATE_SUGGESTING;
-	char *_match = my_rl_path_completion(str, 0);
+	char *match = my_rl_path_completion(str, 0);
 	flags &= ~STATE_SUGGESTING;
-	if (!_match && !*_fmatch)
+	if (!match && !*fz_match)
 		return NO_MATCH;
 
-	if (print == 0 && _match) {
-		int ret = get_print_status(str, _match, len);
-		free(_match);
+	if (print == 0 && match) {
+		int ret = get_print_status(str, match, len);
+		free(match);
 		cur_comp_type = TCMP_PATH;
 		return ret;
 	}
 
 	cur_comp_type = TCMP_PATH; /* Required by print_match() */
-	printed = print_match(_match ? _match : _fmatch, len);
-	*_fmatch = '\0';
+	printed = print_match(match ? match : fz_match, len);
+	*fz_match = '\0';
 
 	cur_comp_type = printed == NO_MATCH ? TCMP_NONE : TCMP_PATH;
-	free(_match);
+	free(match);
 
 	return printed;
 }
 
 static inline void
-print_directory_suggestion(const size_t i, const size_t len, char *color)
+print_directory_suggestion(const filesn_t i, const size_t len, char *color)
 {
 	if (conf.suggest_filetype_color == 1)
 		color = file_info[i].color;
 
 	suggestion.filetype = DT_DIR;
 
-	char tmp[NAME_MAX + 2];
-	snprintf(tmp, NAME_MAX + 2, "%s/", file_info[i].name);
+	char name[NAME_MAX + 2];
+	snprintf(name, sizeof(name), "%s/", file_info[i].name);
 
-	char *_tmp = escape_str(tmp);
-	if (_tmp) {
-		print_suggestion(_tmp, len, color);
-		free(_tmp);
+	char *tmp = escape_str(name);
+	if (tmp) {
+		print_suggestion(tmp, len, color);
+		free(tmp);
 		return;
 	}
 
-	print_suggestion(tmp, len, color);
+	print_suggestion(name, len, color);
 }
 
 static inline void
-print_reg_file_suggestion(char *str, const size_t i, size_t len,
+print_reg_file_suggestion(char *str, const filesn_t i, size_t len,
 	char *color, const int dot_slash)
 {
 	if (conf.suggest_filetype_color)
@@ -971,7 +967,7 @@ print_reg_file_suggestion(char *str, const size_t i, size_t len,
 
 		if (dot_slash) { /* Reinsert './', removed to check file name */
 			char t[NAME_MAX + 2];
-			snprintf(t, NAME_MAX + 1, "./%s", tmp);
+			snprintf(t, sizeof(t), "./%s", tmp);
 			print_suggestion(t, len + 2, color);
 		} else {
 			print_suggestion(tmp, len, color);
@@ -983,7 +979,7 @@ print_reg_file_suggestion(char *str, const size_t i, size_t len,
 
 	if (dot_slash) {
 		char t[NAME_MAX + 2];
-		snprintf(t, NAME_MAX + 1, "./%s", file_info[i].name);
+		snprintf(t, sizeof(t), "./%s", file_info[i].name);
 		print_suggestion(t, len + 2, color);
 		return;
 	}
@@ -998,15 +994,16 @@ check_filenames(char *str, size_t len, const int first_word,
 	char *color = (conf.suggest_filetype_color == 1) ? no_c : sf_c;
 
 	skip_leading_backslashes(&str, &len);
-	int dot_slash = skip_leading_dot_slash(&str, &len), fuzzy_index = -1;
+	int dot_slash = skip_leading_dot_slash(&str, &len);
 	skip_trailing_spaces(&str, &len);
 	int removed_slash = remove_trailing_slash(&str, &len);
 
+	filesn_t fuzzy_index = -1;
 	int fuzzy_str_type = (conf.fuzzy_match == 1 && contains_utf8(str) == 1)
 		? FUZZY_FILES_UTF8 : FUZZY_FILES_ASCII;
 	int best_fz_score = 0;
 
-	size_t i;
+	filesn_t i;
 
 	for (i = 0; i < files; i++) {
 		if (!file_info[i].name)	continue;
@@ -1055,7 +1052,7 @@ check_filenames(char *str, size_t len, const int first_word,
 		else {
 			int s = fuzzy_match(str, file_info[i].name, len, fuzzy_str_type);
 			if (s > best_fz_score) {
-				fuzzy_index = (int)i;
+				fuzzy_index = i;
 				if (s == TARGET_BEGINNING_BONUS)
 					break;
 				best_fz_score = s;
@@ -1070,9 +1067,9 @@ check_filenames(char *str, size_t len, const int first_word,
 		suggestion.type = i < files ? FILE_SUG : FUZZY_FILENAME;
 
 		if (file_info[fuzzy_index].dir)
-			print_directory_suggestion((size_t)fuzzy_index, len, color);
+			print_directory_suggestion(fuzzy_index, len, color);
 		else
-			print_reg_file_suggestion(str, (size_t)fuzzy_index, len, color,
+			print_reg_file_suggestion(str, fuzzy_index, len, color,
 				dot_slash);
 
 		return PARTIAL_MATCH;
@@ -1142,7 +1139,7 @@ check_builtins(const char *str, const size_t len, const int print)
 		if (strncmp(b[i], str, len) != 0)
 			continue;
 
-		size_t blen = strlen(b[i]);
+		const size_t blen = strlen(b[i]);
 		if (blen > len) {
 			suggestion.type = CMD_SUG;
 			print_suggestion(b[i], len, sb_c);
@@ -1309,8 +1306,8 @@ check_eln(const char *str, const int print)
 	if (!str || !*str)
 		return NO_MATCH;
 
-	int n = atoi(str);
-	if ( n < 1 || n > (int)files || !file_info[n - 1].name
+	filesn_t n = xatof(str);
+	if ( n < 1 || n > files || !file_info[n - 1].name
 	|| ( words_num == 1 && ( (file_info[n - 1].dir == 1 && conf.autocd == 0)
 	|| (file_info[n - 1].dir == 0 && conf.auto_open == 0) ) ) )
 		return NO_MATCH;
@@ -1643,7 +1640,7 @@ check_sort_methods(const char *str, const size_t len)
 	}
 
 	suggestion.type = SORT_SUG;
-	print_suggestion(_sorts[a].name, 0, sf_c);
+	print_suggestion(sort_methods[a].name, 0, sf_c);
 	return PARTIAL_MATCH;
 }
 
@@ -1707,7 +1704,7 @@ check_workspaces(char *word, size_t wlen)
 		return NO_MATCH;
 
 	if (*word >= '1' && *word <= MAX_WS + '0' && !*(word + 1)) {
-		int a = atoi(word);
+		const int a = atoi(word);
 		if (a > 0 && workspaces[a - 1].name) {
 			suggestion.type = WS_NUM_SUG;
 			print_suggestion(workspaces[a - 1].name, 0, sf_c);
@@ -1943,7 +1940,7 @@ check_backdir(void)
 
 	/* Find the query string in the list of parent directories */
 	char *p = conf.case_sens_path_comp == 1 ? strstr(bk_cwd, ds)
-	: strcasestr(bk_cwd, ds);
+		: xstrcasestr(bk_cwd, ds);
 	if (p) {
 		char *pp = strchr(p, '/');
 		if (pp)
@@ -1971,7 +1968,7 @@ check_dirhist(char *word, const size_t len)
 
 	int i = dirhist_total_index;
 	while (--i >= 0) {
-		if (!old_pwd[i] || !*old_pwd[i] || *old_pwd[i] == _ESC)
+		if (!old_pwd[i] || !*old_pwd[i] || *old_pwd[i] == KEY_ESC)
 			continue;
 
 		if (conf.fuzzy_match == 0 || rl_point < rl_end) {
@@ -2193,8 +2190,6 @@ rl_suggestions(const unsigned char c)
 		else if (lb[1] == 'd' && lb[2] == ' ' && lb[3]) {
 				if (*(lb + 3) == '/' && !*(lb + 4)) {
 					/* The query string is a single slash: do nothing */
-//					if (suggestion.printed)
-//						clear_suggestion(CS_FREEBUF);
 					goto FAIL;
 				}
 				if ((printed = check_backdir()) != NO_MATCH)
@@ -2207,13 +2202,10 @@ rl_suggestions(const unsigned char c)
 			if (lb[1] == 'h' && lb[2] == ' ' && (lb[3] == '-'
 			|| strncmp(lb + 3, "--help", strlen(lb + 3)) == 0))
 				break;
-			if ((printed = check_dirhist(word, wlen)) != NO_MATCH) {
+			if ((printed = check_dirhist(word, wlen)) != NO_MATCH)
 				goto SUCCESS;
-			} else {
-//				if (suggestion.printed)
-//					clear_suggestion(CS_FREEBUF);
+			else
 				goto FAIL;
-			}
 		}
 		break;
 
@@ -2232,13 +2224,10 @@ rl_suggestions(const unsigned char c)
 			break;
 		if (words_num == 2 && old_pwd && dirhist_total_index > 0 && wlen > 0
 		&& lb[1] == 'h' && lb[2] == ' ' && !strchr(word, '/')) {
-			if ((printed = check_dirhist(word, wlen)) != NO_MATCH) {
+			if ((printed = check_dirhist(word, wlen)) != NO_MATCH)
 				goto SUCCESS;
-			} else {
-//				if (suggestion.printed)
-//					clear_suggestion(CS_FREEBUF);
+			else
 				goto FAIL;
-			}
 		}
 		break;
 
@@ -2269,13 +2258,10 @@ rl_suggestions(const unsigned char c)
 		if (profile_names && words_num == 3 && lb[1] == 'f' && lb[2] == ' '
 		&& (strncmp(lb + 3, "set ", 4) == 0 || strncmp(lb + 3, "del ", 4) == 0
 		|| strncmp(lb + 3, "rename ", 7) == 0)) {
-			if ((printed = check_profiles(word, wlen)) != NO_MATCH) {
+			if ((printed = check_profiles(word, wlen)) != NO_MATCH)
 				goto SUCCESS;
-			} else {
-//				if (suggestion.printed)
-//					clear_suggestion(CS_FREEBUF);
+			else
 				goto FAIL;
-			}
 		}
 #endif /* !_NO_PROFILES */
 
@@ -2324,7 +2310,7 @@ rl_suggestions(const unsigned char c)
 	default: break;
 	}
 
-	/* 3.d.1) Variable names, both environment and internal */
+	/* 3.d.1) Variable names, both environment and internal. */
 	if (*word == '$') {
 		if ((printed = check_variables(word + 1, wlen - 1)) != NO_MATCH)
 			goto SUCCESS;
@@ -2352,11 +2338,11 @@ rl_suggestions(const unsigned char c)
 #endif /* _NO_TAGS */
 
 	/* 3.e) Execute the following checks in the order specified by
-	 * suggestion_strategy (the value is taken from the configuration file) */
+	 * suggestion_strategy (the value is taken from the configuration file). */
 	size_t st;
 	int flag = 0;
 
-	/* Let's find out whether the last entered character is escaped */
+	/* Let's find out whether the last entered character is escaped. */
 	int escaped = (wlen > 1 && word[wlen - 2] == '\\') ? 1 : 0;
 
 	for (st = 0; st < SUG_STRATS; st++) {
@@ -2393,10 +2379,8 @@ rl_suggestions(const unsigned char c)
 				wlen = strlen(word);
 			}
 
-			if (wlen > 0 && word[wlen - 1] == ' ' && escaped == 0) {
+			if (wlen > 0 && word[wlen - 1] == ' ' && escaped == 0)
 				word[wlen - 1] = '\0';
-//				wlen--;
-			}
 
 			flag = (c == ' ' && escaped == 0) ? CHECK_MATCH : PRINT_MATCH;
 
@@ -2446,7 +2430,7 @@ rl_suggestions(const unsigned char c)
 				clear_suggestion(CS_FREEBUF);
 
 			if (*lb != ';' && *lb != ':' && *word >= '1' && *word <= '9') {
-				if (_expand_eln(word) == 1
+				if (should_expand_eln(word) == 1
 				&& (printed = check_eln(word, flag)) > 0)
 					goto SUCCESS;
 			}
@@ -2488,10 +2472,8 @@ rl_suggestions(const unsigned char c)
 				flags &= ~STATE_COMPLETING;
 			}
 
-			if (wlen > 0 && word[wlen - 1] == ' ' && escaped == 0) {
+			if (wlen > 0 && word[wlen - 1] == ' ' && escaped == 0)
 				word[wlen - 1] = '\0';
-//				wlen--;
-			}
 
 			if (c == ' ' && escaped == 0 && suggestion.printed)
 				clear_suggestion(CS_FREEBUF);
@@ -2522,10 +2504,8 @@ rl_suggestions(const unsigned char c)
 				wlen = strlen(word);
 			}
 
-			if (wlen > 0 && word[wlen - 1] == ' ' && escaped == 0) {
+			if (wlen > 0 && word[wlen - 1] == ' ' && escaped == 0)
 				word[wlen - 1] = '\0';
-//				wlen--;
-			}
 
 			flag = (c == ' ' || full_word) ? CHECK_MATCH : PRINT_MATCH;
 
@@ -2542,7 +2522,7 @@ rl_suggestions(const unsigned char c)
 		}
 	}
 
-	/* 3.f) Cmds in PATH and CliFM internals cmds, but only for the first word */
+	/* 3.f) Cmds in PATH and CliFM internals cmds, but only for the first word. */
 	if (words_num > 1)
 		goto NO_SUGGESTION;
 
@@ -2550,7 +2530,7 @@ CHECK_FIRST_WORD:
 
 	word = first_word ? first_word : last_word;
 
-	/* Skip 'b:' (bookmarks), 's:' (sel files) and 't:' (tags) constructs */
+	/* Skip 'b:' (bookmarks), 's:' (sel files) and 't:' (tags) constructs. */
 	if ((*word == 'b' || *word == 's' || *word == 't') && *(word + 1) == ':')
 		goto NO_SUGGESTION;
 
@@ -2558,7 +2538,6 @@ CHECK_FIRST_WORD:
 
 	|| (c == ' ' && (*word == '\''
 	|| *word == '"' || *word == '$' || *word == '#'))
-
 	|| *word == '<' || *word == '>' || *word == '!' || *word == '{'
 	|| *word == '[' || *word == '(' || strchr(word, '=')
 	|| *rl_line_buffer == ' ' || *word == '|' || *word == ';'
@@ -2575,15 +2554,15 @@ CHECK_FIRST_WORD:
 
 	} else if (point_is_first_word && rl_point < rl_end
 	&& *word >= '1' && *word <= '9' && is_number(word)) {
-		int a = atoi(word);
-		if (a > 0 && a <= (int)files)
+		const filesn_t a = xatof(word);
+		if (a > 0 && a <= files)
 			printed = PARTIAL_MATCH;
 
 	} else if (point_is_first_word && rl_point < rl_end
 	&& (printed = check_completions(word, wlen, CHECK_MATCH)) != NO_MATCH) {
 		if (c == ' ' && printed != FULL_MATCH)
 			/* We have a partial match for a file name. If not a command
-			 * name, let's return NO_MATCH */
+			 * name, let's return NO_MATCH. */
 			printed = check_cmds(word, wlen, CHECK_MATCH);
 
 	} else {
@@ -2602,7 +2581,7 @@ CHECK_FIRST_WORD:
 		}
 		goto SUCCESS;
 
-	/* Let's suppose that two slashes do not constitue a search expression */
+	/* Let's assume that two slashes do not constitue a search expression */
 	} else {
 	/* There's no suggestion nor any command name matching the first entered
 	 * word. So, we assume we have an invalid command name. Switch to the
@@ -2616,8 +2595,7 @@ NO_SUGGESTION:
 	 * in the current input sequence. This is mainly to avoid erasing
 	 * the suggestion if moving thought the text via the arrow keys. */
 	if (suggestion.printed) {
-		if (!strchr(word, _ESC)) {
-//			clear_suggestion(CS_FREEBUF);
+		if (!strchr(word, KEY_ESC)) {
 			goto FAIL;
 		} else {
 			/* Go to SUCCESS to avoid removing the suggestion buffer. */
@@ -2668,17 +2646,17 @@ SUCCESS:
 	free(first_word);
 	free(last_word);
 	last_word = (char *)NULL;
+
 	return EXIT_SUCCESS;
 
 FAIL:
 	if (suggestion.printed == 1)
 		clear_suggestion(CS_FREEBUF);
-//	suggestion.printed = 0;
+
 	free(first_word);
 	free(last_word);
 	last_word = (char *)NULL;
-//	free(suggestion_buf);
-//	suggestion_buf = (char *)NULL;
+
 	return EXIT_FAILURE;
 }
 #else
