@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,12 +25,15 @@
 #ifndef SUGGESTIONS_H
 #define SUGGESTIONS_H
 
-void clear_suggestion(const int free_sug);
-int rl_suggestions(const unsigned char c);
+__BEGIN_DECLS
+
+void clear_suggestion(const int);
 void free_suggestion(void);
+void print_suggestion(char *, size_t, char *);
 void remove_suggestion_not_end(void);
-int check_cmds(const char *str, const size_t len, const int print);
-void print_suggestion(char *str, size_t offset, char *color);
-int recover_from_wrong_cmd(void);
+int  recover_from_wrong_cmd(void);
+int  rl_suggestions(const unsigned char);
+
+__END_DECLS
 
 #endif /* SUGGESTIONS_H */
