@@ -52,7 +52,6 @@ typedef char *rl_cpvfunc_t;
 #endif /* __OpenBSD__ */
 
 #include <errno.h>
-#include <fcntl.h>
 
 #include <termios.h> /* Get cursor position functions */
 #include <limits.h> /* INT_MIN */
@@ -75,6 +74,8 @@ typedef char *rl_cpvfunc_t;
 # include "suggestions.h"
 #endif /* !_NO_SUGGESTIONS */
 
+#include "strings.h" /* quote_str() */
+
 #define CPR     "\x1b[6n" /* Cursor position report */
 #define CPR_LEN (sizeof(CPR) - 1)
 
@@ -179,7 +180,7 @@ get_cursor_position(int *c, int *l)
 		return EXIT_FAILURE;
 
 	/* 3. Parse the response */
-	if (*buf != _ESC || *(buf + 1) != '[' || !*(buf + 2))
+	if (*buf != KEY_ESC || *(buf + 1) != '[' || !*(buf + 2))
 		return EXIT_FAILURE;
 
 	char *p = strchr(buf + 2, ';');
@@ -205,7 +206,7 @@ stat_char(const char *filename)
 	struct stat attr;
 	int r;
 
-#if defined(S_ISLNK)
+#ifdef S_ISLNK
 	r = lstat(filename, &attr);
 #else
 	r = stat(filename, &attr);
@@ -216,22 +217,34 @@ stat_char(const char *filename)
 
 	int c = 0;
 	if (S_ISDIR(attr.st_mode)) {
-		c = '/';
-#if defined(S_ISLNK)
+		c = DIR_CHR;
+#ifdef S_ISLNK
 	} else if (S_ISLNK(attr.st_mode)) {
-		c = '@';
+		c = LINK_CHR;
 #endif /* S_ISLNK */
-#if defined(S_ISSOCK)
+#ifdef S_ISSOCK
 	} else if (S_ISSOCK(attr.st_mode)) {
-		c = '=';
+		c = SOCK_CHR;
 #endif /* S_ISSOCK */
 	} else if (S_ISREG(attr.st_mode)) {
 		if (access(filename, X_OK) == 0)
-			c = '*';
-#if defined(S_ISFIFO)
+			c = EXEC_CHR;
+	} else if (S_ISBLK(attr.st_mode)) {
+		c = BLK_CHR;
+	} else if (S_ISCHR(attr.st_mode)) {
+		c = CHR_CHR;
+#ifdef SOLARIS_DOORS
+	} else if (S_ISDOOR(attr.st_mode)) {
+		c = DOOR_CHR;
+#endif /* SOLARIS_DOORS */
+#ifdef S_ISWHT
+	} else if (S_ISWHT(attr.st_mode)) {
+		c = WHT_CHR;
+#endif /* S_IFWHT */
+#ifdef S_ISFIFO
 	} else {
 		if (S_ISFIFO(attr.st_mode))
-			c = '|';
+			c = FIFO_CHR;
 #endif /* S_ISFIFO */
 	}
 
@@ -291,7 +304,7 @@ print_filename(char *to_print, char *full_pathname)
 
 			s = tilde_expand(full_pathname);
 			if (rl_directory_completion_hook)
-				(*rl_directory_completion_hook) (&s);
+				(*rl_directory_completion_hook)(&s);
 
 			size_t slen = strlen(s);
 			size_t tlen = strlen(to_print);
@@ -401,7 +414,7 @@ fzftab_color(char *filename, const struct stat *attr)
 		char *ext_cl = (char *)NULL;
 		char *ext = strrchr(filename, '.');
 		if (ext && ext != filename)
-			ext_cl = get_ext_color(ext);
+			ext_cl = get_ext_color(ext, NULL);
 
 		if (te) *te = '.';
 
@@ -620,7 +633,7 @@ write_completion(char *buf, const size_t offset, const int multi)
 
 	if (t == TCMP_ENVIRON || t == TCMP_USERS) {
 		/* Skip the leading dollar sign (env vars) and tilde (users) */
-		buf++;
+		rl_insert_text(buf + 1 + offset);
 
 	} else if (t == TCMP_PATH && multi == 0) {
 		char *esc_buf = escape_str(buf);
@@ -757,8 +770,11 @@ static int
 run_finder(const size_t height, const int offset, const char *lw,
 	const int multi)
 {
-	int prev = (conf.fzf_preview > 0 && SHOW_PREVIEWS(cur_comp_type) == 1) ? 1 : 0;
+	int prev = (conf.fzf_preview > 0 && SHOW_PREVIEWS(cur_comp_type) == 1)
+		? FZF_INTERNAL_PREVIEWER : 0;
 	int prev_hidden = conf.fzf_preview == 2 ? 1 : 0;
+	if (conf.fzf_preview == FZF_EXTERNAL_PREVIEWER)
+		prev = FZF_EXTERNAL_PREVIEWER;
 
 	/* Some shells, like xonsh (the only one to my knowledge), have problems
 	 * parsing the command constructed below. Let's force launch_execl() to
@@ -798,7 +814,7 @@ run_finder(const size_t height, const int offset, const char *lw,
 		*prev_opts = '\0';
 		char prev_str[] = "--preview \"clifm --preview {}\"";
 
-		if (prev == 1) {
+		if (prev > 0) { /* Either internal of external previewer */
 			set_fzf_env_vars((int)height);
 			size_t s = get_preview_win_width(offset);
 			if (s != (size_t)-1)
@@ -816,8 +832,8 @@ run_finder(const size_t height, const int offset, const char *lw,
 			lw ? lw : "", conf.colorize == 0 ? "--color=bw" : "",
 			multi == 1 ? "--multi --bind tab:toggle+down,ctrl-s:select-all,\
 ctrl-d:deselect-all,ctrl-t:toggle-all" : "",
-			prev == 1 ? prev_str : "",
-			(prev == 1 && prev_hidden == 1)
+			prev == FZF_INTERNAL_PREVIEWER ? prev_str : "",
+			(prev == FZF_INTERNAL_PREVIEWER && prev_hidden == 1)
 				? "--preview-window=hidden --bind alt-p:toggle-preview" : "",
 			*prev_opts ? prev_opts : "",
 			finder_in_file, finder_out_file);
@@ -902,7 +918,7 @@ get_tagged_file_target(char *filename)
 static char *
 print_no_finder_file(void)
 {
-	_err('e', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME,
+	err('e', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME,
 		finder_out_file, strerror(errno));
 	return (char *)NULL;
 }
@@ -931,14 +947,17 @@ get_glob_file_target(char *str, const char *initial_path)
 	return p;
 }
 
-/* Recover finder (fzf/fnf/smenu) output from FINDER_OUT_FILE file
- * Return this output (reformated if needed) or NULL in case of error */
+/* Recover finder (fzf/fnf/smenu) output from FINDER_OUT_FILE file.
+ * Return this output (reformated if needed) or NULL in case of error.
+ * FINDER_OUT_FILE is removed immediately after use.  */
 static char *
 get_finder_output(const int multi, char *base)
 {
 	FILE *fp = fopen(finder_out_file, "r");
-	if (!fp)
+	if (!fp) {
+		unlink(finder_out_file);
 		return print_no_finder_file();
+	}
 
 	char *buf = (char *)xnmalloc(1, sizeof(char)), *line = (char *)NULL;
 	*buf = '\0';
@@ -1027,7 +1046,7 @@ write_comp_to_file(char *entry, const char *color, FILE *fp)
 	char end_char = tabmode == SMENU_TAB ? '\n' : '\0';
 
 	if (wc_xstrlen(entry) == 0) {
-		char *wname = replace_ctrl_chars(entry);
+		char *wname = replace_invalid_chars(entry);
 		fprintf(fp, "%s%c", wname ? wname : entry, end_char);
 		free(wname);
 		return;
@@ -1064,7 +1083,7 @@ store_completions(char **matches)
 	int fd = 0;
 	FILE *fp = open_fwrite(finder_in_file, &fd);
 	if (!fp) {
-		_err('e', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME,
+		err('e', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME,
 			finder_in_file, strerror(errno));
 		return (size_t)-1;
 	}
@@ -1133,13 +1152,13 @@ store_completions(char **matches)
 			/* If color does not start with escape, then we have a color
 			 * for a file extension. In this case, we need to properly
 			 * construct the color code. */
-			if (cl && *cl != _ESC)
+			if (cl && *cl != KEY_ESC)
 				snprintf(tmp_color, sizeof(tmp_color), "\x1b[%sm", cl);
 
 			color = *tmp_color ? tmp_color : (cl ? cl : "");
 
 			if (ct != TCMP_SEL && ct != TCMP_DESEL && ct != TCMP_BM_PATHS
-			&& ct != TCMP_DIRHIST && ct != TCMP_OPENWITH && ct != TCMP_BACKDIR
+			&& ct != TCMP_DIRHIST && ct != TCMP_OPENWITH
 			&& ct != TCMP_JUMP && ct != TCMP_TAGS_F) {
 				_path = strrchr(entry, '/');
 				entry = (_path && *(++_path)) ? _path : entry;
@@ -1206,8 +1225,13 @@ get_query_str(char *lw)
 		query = (!tmp || !*(tmp++)) ? (char *)NULL : tmp;
 		break;
 
-	case TCMP_HIST:	/* Skip the leading ! char of the input string. */
-		query = lb + 1;
+	case TCMP_HIST:
+		if (lb && *lb == '/' && lb[1] == '*') { /* Search history */
+			query = lb + 1;
+		} else { /* Commands history */
+			tmp = lb ? strrchr(lb, '!') : (char *)NULL;
+			query = (!tmp || !*(tmp++)) ? (char *)NULL : tmp;
+		}
 		break;
 
 	case TCMP_JUMP:
@@ -1246,16 +1270,18 @@ calculate_prefix_len(const char *str, const char *query, const char *lw)
 
 	if (ct == TCMP_FILE_TYPES_OPTS || ct == TCMP_SEL || ct == TCMP_RANGES
 	|| ct == TCMP_TAGS_T || ct == TCMP_BM_PREFIX || ct == TCMP_HIST
-	|| ct == TCMP_JUMP || ct == TCMP_BM_PREFIX || ct == TCMP_BM_PATHS
-	|| ct == TCMP_TAGS_F || ct == TCMP_GLOB || ct == TCMP_DIRHIST
+	|| ct == TCMP_JUMP || ct == TCMP_BM_PATHS || ct == TCMP_TAGS_F
+	|| ct == TCMP_GLOB || ct == TCMP_DIRHIST
 	|| ct == TCMP_FILE_TYPES_FILES || ct == TCMP_CMD_DESC)
 		/* None of these completions produces a partial match (prefix) */
 		return 0;
 
 	size_t prefix_len = 0, len = strlen(str);
 
-	if (len == 0 || str[len - 1] == '/')
-		return 0;
+	if (len == 0 || str[len - 1] == '/') {
+		if (ct != TCMP_DESEL)
+			return 0;
+	}
 
 	if (ct == TCMP_DESEL)
 		return strlen(query ? query : (lw ? lw : ""));
@@ -1341,11 +1367,11 @@ is_multi_sel(void)
 	}
 }
 
-/* Clean the input buffer in case the user cancelled the completion pressing ESC. */
-/* If all possible completions share a common prefix, this prefix is
- * automatically appended to the query string. However, the user
- * cancelled here the completion (pressing ESC), so that we need to
- * remove this prefix by reinserting the original query string. */
+/* Clean the input buffer in case the user cancelled the completion pressing ESC.
+ * If all possible completions share a common prefix, this prefix is
+ * automatically appended to the query string. However, the user cancelled
+ * here the completion (pressing ESC), so that we need to remove this prefix
+ * by reinserting the original query string. */
 static int
 clean_rl_buffer(const char *text)
 {
@@ -1447,10 +1473,10 @@ get_finder_offset(const char *query, const char *text, char **matches,
 	}
 
 	if (ct == TCMP_OWNERSHIP && query) {
-		if (query == lb)
-			finder_offset = lb ? (int)wc_xstrlen(lb) - 3 : 0;
+		if (lb && query == lb)
+			finder_offset = (int)wc_xstrlen(lb) - 3;
 		else
-			finder_offset = (int)(query - lb);
+			finder_offset = lb ? (int)(query - lb) : 0;
 	}
 
 	else if (ct == TCMP_DESEL && query) {
@@ -1458,7 +1484,13 @@ get_finder_offset(const char *query, const char *text, char **matches,
 	}
 
 	else if (ct == TCMP_HIST) {
-		finder_offset = 1 + prompt_offset - ((query && *query) ? 3 : 4);
+		if (lb && *lb == '/' && lb[1] == '*') { /* Search history */
+			finder_offset = 1 + prompt_offset - ((query && *query) ? 3 : 4);
+		} else { /* Commands history */
+			char *sp = lb ? get_last_chr(lb, '!', rl_point) : (char *)NULL;
+			finder_offset = prompt_offset + (sp ? (int)(sp - lb)
+				- ((query && *query) ? 2 : 3) : -1);
+		}
 	}
 
 	else if (ct == TCMP_JUMP) {
@@ -1539,10 +1571,6 @@ get_finder_offset(const char *query, const char *text, char **matches,
 		}
 	}
 
-	else if (ct == TCMP_HIST && *rl_line_buffer == '/') { /* Search patterns */
-		finder_offset--;
-	}
-
 	if ((!query || !*query) && ct != TCMP_RANGES && ct != TCMP_SEL
 	&& ct != TCMP_BM_PATHS && ct != TCMP_BM_PREFIX && ct != TCMP_GLOB
 	&& ct != TCMP_CMD_DESC && ct != TCMP_FILE_TYPES_OPTS
@@ -1577,6 +1605,10 @@ do_some_cleanup(char **buf, char **matches, const char *query,
 {
 	enum comp_type ct = cur_comp_type;
 
+	/* TCMP_HIST might be either search or command history */
+	int cmd_hist = (ct == TCMP_HIST && rl_line_buffer
+		&& (*rl_line_buffer != '/' || rl_line_buffer[1] != '*'));
+
 	if (rl_point < rl_end && ct != TCMP_PATH && ct != TCMP_CMD) {
 		char *s = rl_line_buffer
 			? get_last_chr(rl_line_buffer, ' ', rl_point) : (char *)NULL;
@@ -1594,12 +1626,12 @@ do_some_cleanup(char **buf, char **matches, const char *query,
 		*buf = tmp;
 	}
 
-	else if (ct == TCMP_HIST || ct == TCMP_JUMP) {
+	else if ((ct == TCMP_HIST && cmd_hist == 0) || ct == TCMP_JUMP) {
 		rl_delete_text(0, rl_end);
 		rl_point = rl_end = 0;
 	}
 
-	else if (ct == TCMP_RANGES || ct == TCMP_SEL
+	else if (cmd_hist == 1 || ct == TCMP_RANGES || ct == TCMP_SEL
 	|| ct == TCMP_TAGS_F || ct == TCMP_GLOB
 	|| ct == TCMP_BM_PATHS || ct == TCMP_BM_PREFIX
 	|| ct == TCMP_TAGS_T || ct == TCMP_DIRHIST) {
@@ -1610,7 +1642,7 @@ do_some_cleanup(char **buf, char **matches, const char *query,
 			rl_point = (int)(s - rl_line_buffer + 1);
 			rl_delete_text(rl_point, rl_end);
 			rl_end = rl_point;
-		} else if (ct == TCMP_BM_PATHS || ct == TCMP_TAGS_F
+		} else if (cmd_hist == 1 || ct == TCMP_BM_PATHS || ct == TCMP_TAGS_F
 		|| ct == TCMP_BM_PREFIX || ct == TCMP_TAGS_T
 		|| ct == TCMP_SEL || ct == TCMP_DIRHIST || ct == TCMP_GLOB) {
 			rl_delete_text(0, rl_end);
@@ -1694,6 +1726,38 @@ move_cursor_up(const int total_line_len)
 	MOVE_CURSOR_UP(lines);
 }
 
+/* Determine input and output files to be used by the fuzzy finder (either
+ * fzf or fnf).
+ * Let's do this even if fzftab is not enabled at startup, because this feature
+ * can be enabled in place editing the config file.
+ *
+ * NOTE: Why do the random file extensions have different lengths? If
+ * compiled in POSIX mode, gen_rand_ext() uses srandom(3) and random(3) to
+ * generate the string, but since both extensions are generated at the same
+ * time, they happen to be the same, resulting in both file names being
+ * identical. As a workaround, we use different lengths for both extensions.
+ *
+ * These files are created immediately after this function returns by
+ * store_completions() with permissions 600 (in a directory to which only the
+ * current has read/write access). These files are then immediately read
+ * by the finder application, and deleted as soon as this latter returns. */
+static void
+set_finder_paths(void)
+{
+	const int sm = (xargs.stealth_mode == 1);
+	const char *p = sm ? P_tmpdir : tmp_dir;
+
+	char *rand_ext = gen_rand_str(sm ? 16 : 10);
+	snprintf(finder_in_file, sizeof(finder_in_file), "%s/.temp%s",
+		p, rand_ext ? rand_ext : "a3_2yu!d43");
+	free(rand_ext);
+
+	rand_ext = gen_rand_str(sm ? 20 : 14);
+	snprintf(finder_out_file, sizeof(finder_out_file), "%s/.temp%s",
+		p, rand_ext ? rand_ext : "0rNkds7++@");
+	free(rand_ext);
+}
+
 /* Display possible completions using the corresponding finder. If one of these
  * possible completions is selected, insert it into the current line buffer.
  *
@@ -1714,6 +1778,9 @@ move_cursor_up(const int total_line_len)
 static int
 finder_tabcomp(char **matches, const char *text, char *original_query)
 {
+	/* Set random file names for both FINDER_IN_FILE and FINDER_OUT_FILE. */
+	set_finder_paths();
+
 	/* Store possible completions in FINDER_IN_FILE to pass them to the finder. */
 	size_t num_matches = store_completions(matches);
 	if (num_matches == (size_t)-1)
@@ -1776,7 +1843,8 @@ finder_tabcomp(char **matches, const char *text, char *original_query)
 		return EXIT_FAILURE;
 
 	/* Calculate the length of the matching prefix to insert into the
-	 * line buffer only the non-matched part of the string returned by FZF. */
+	 * line buffer only the non-matched part of the string returned by the
+	 * finder. */
 	size_t prefix_len = calculate_prefix_len(matches[0], query, lw);
 
 	do_some_cleanup(&buf, matches, query, &prefix_len);
@@ -1802,6 +1870,18 @@ finder_tabcomp(char **matches, const char *text, char *original_query)
 }
 #endif /* !_NO_FZF */
 
+static char *
+gen_quoted_str(const char *str)
+{
+	struct stat a;
+	if (conf.quoting_style == QUOTING_STYLE_BACKSLASH
+	|| cur_comp_type != TCMP_ELN
+	|| (stat(str, &a) != -1 && S_ISDIR(a.st_mode)))
+		return escape_str(str);
+
+	return quote_str(str);
+}
+
 /* Complete the word at or before point.
    WHAT_TO_DO says what to do with the completion.
    `?' means list the possible completions.
@@ -1826,7 +1906,7 @@ tab_complete(const int what_to_do)
 
 	rl_compentry_func_t *our_func = rl_completion_entry_function;
 
-	/* Only the completion entry function can change these */
+	/* Only the completion entry function can change these. */
 	rl_filename_completion_desired = 0;
 	rl_filename_quoting_desired = 1;
 	rl_sort_completion_matches = 1;
@@ -1937,10 +2017,9 @@ AFTER_USUAL_COMPLETION:
 #ifndef _NO_FZF
 	/* Common prefix for multiple matches is appended to the input query.
 	 * Let's rise a flag to know if we should reinsert the original query
-	 * in case the user cancels the completion (pressing ESC) */
-	int common_prefix_added = 0;
-	if (fzftab == 1 && matches[1] && strcmp(matches[0], text) != 0)
-		common_prefix_added = 1;
+	 * in case the user cancels the completion (pressing ESC). */
+	int common_prefix_added =
+		(fzftab == 1 && matches[1] && strcmp(matches[0], text) != 0);
 #endif /* _NO_FZF */
 
 	size_t i;
@@ -1958,13 +2037,13 @@ AFTER_USUAL_COMPLETION:
 
 		if (cur_comp_type == TCMP_HIST) {// || cur_comp_type == TCMP_EXT_OPTS) {
 			/* Sort the array without matches[0]: we need it to stay in
-			 * place no matter what */
+			 * place no matter what. */
 			for (i = 0; matches[i]; i++);
 			if (i > 0)
 				qsort(matches + 1, i - 1, sizeof(char *), (QSFUNC *)compare_strings);
 		}
 
-		/* Remember the lowest common denominator, for it may be unique */
+		/* Remember the lowest common denominator: it may be unique. */
 		lowest_common = savestring(matches[0], strlen(matches[0]));
 
 		for (i = 0; matches[i + 1]; i++) {
@@ -1977,7 +2056,7 @@ AFTER_USUAL_COMPLETION:
 		}
 
 		/* We have marked all the dead slots with (char *)&dead_slot
-		 * Copy all the non-dead entries into a new array */
+		 * Copy all the non-dead entries into a new array. */
 		temp_array = (char **)xnmalloc(3 + newlen, sizeof (char *));
 		for (i = j = 1; matches[i]; i++) {
 			if (matches[i] != (char *)&dead_slot) {
@@ -1997,13 +2076,15 @@ AFTER_USUAL_COMPLETION:
 		matches[0] = lowest_common;
 
 		/* If there is one string left, and it is identical to the lowest
-		 * common denominator (LCD), then the LCD is the string to insert */
+		 * common denominator (LCD), then the LCD is the string to insert. */
 		if (j == 2 && strcmp(matches[0], matches[1]) == 0) {
 			free(matches[1]);
 			matches[1] = (char *)NULL;
 		}
 	}
 
+	int len = 0, count = 0, limit = 0, max = 0;
+
 	switch (what_to_do) {
 	case '!':
 		/* If we are matching filenames, then here is our chance to
@@ -2035,7 +2116,7 @@ AFTER_USUAL_COMPLETION:
 		rl_filename_completion_desired && rl_filename_quoting_desired;
 
 		if (should_quote)
-			should_quote = should_quote && !quote_char;
+			should_quote = (should_quote && !quote_char);
 
 		if (should_quote) {
 			int do_replace;
@@ -2046,7 +2127,7 @@ AFTER_USUAL_COMPLETION:
 			This also checks whether the common prefix of several
 			matches needs to be quoted.  If the common prefix should
 			not be checked, add !matches[1] to the if clause. */
-			should_quote = rl_strpbrk(matches[0], quote_chars) != 0;
+			should_quote = (rl_strpbrk(matches[0], quote_chars) != 0);
 
 			if (should_quote)
 				do_replace = matches[1] ? MULT_MATCH : SINGLE_MATCH;
@@ -2055,7 +2136,8 @@ AFTER_USUAL_COMPLETION:
 				/* Found an embedded word break character in a potential
 				 match, so we need to prepend a quote character if we
 				 are replacing the completion string. */
-				replacement = escape_str(matches[0]);
+//				replacement = escape_str(matches[0]);
+				replacement = gen_quoted_str(matches[0]);
 			}
 		}
 
@@ -2081,8 +2163,7 @@ AFTER_USUAL_COMPLETION:
 			|| c == TCMP_TAGS_C || c == TCMP_TAGS_S || c == TCMP_TAGS_T
 			|| c == TCMP_TAGS_U || c == TCMP_BOOKMARK || c == TCMP_GLOB
 			|| c == TCMP_PROMPTS || c == TCMP_CSCHEME || c == TCMP_WORKSPACES
-			|| c == TCMP_PROF || c == TCMP_BM_PREFIX)
-			&& !strchr(replacement, '\\')) {
+			|| c == TCMP_BM_PREFIX) && !strchr(replacement, '\\')) {
 				char *r = escape_str(replacement);
 				if (!r) {
 					if (replacement != matches[0])
@@ -2094,7 +2175,7 @@ AFTER_USUAL_COMPLETION:
 				replacement = r;
 			}
 
-			/* Let's keep the backslash, used to bypass alias names */
+			/* Let's keep the backslash, used to bypass alias names. */
 			if (c == TCMP_CMD && text && *text == '\\' && *(text + 1))
 				start++;
 
@@ -2127,13 +2208,12 @@ AFTER_USUAL_COMPLETION:
 					t[1] = '\0';
 					rl_insert_text(t);
 
-/////////////////////////////////
 					/* WORKAROUND: If we are not at the end of the line,
 					 * redisplay only up to the cursor position, to prevent
 					 * whatever is after it from being printed using the
 					 * last printed color.
 					 * Drawback: there will be no color after the cursor
-					 * position (no color however is better than a wrong color) */
+					 * position (no color however is better than a wrong color). */
 					if (!replacement[k + 1] && rl_point < rl_end && cur_color != tx_c) {
 						int _end = rl_end;
 						rl_end = rl_point;
@@ -2142,7 +2222,6 @@ AFTER_USUAL_COMPLETION:
 						fputs(tx_c, stdout);
 						fflush(stdout);
 					}
-////////////////////////////////
 
 					rl_redisplay();
 
@@ -2180,7 +2259,7 @@ AFTER_USUAL_COMPLETION:
 		 * are at the end of the line, then add a space. */
 		if (matches[1]) {
 			if (what_to_do == '!') {
-				goto DISPLAY_MATCHES;		/* XXX */
+				goto DISPLAY_MATCHES;
 			} else {
 				if (rl_editing_mode != 0) /* vi_mode */
 					rl_ding();	/* There are other matches remaining. */
@@ -2193,7 +2272,7 @@ AFTER_USUAL_COMPLETION:
 			|| cur_comp_type == TCMP_PROF || cur_comp_type == TCMP_BM_PREFIX)
 				break;
 
-			/* Let's append an ending character to the inserted match */
+			/* Let's append an ending character to the inserted match. */
 			if (cur_comp_type == TCMP_OWNERSHIP) {
 				char *sc = rl_line_buffer
 					? strchr(rl_line_buffer, ':') : (char *)NULL;
@@ -2240,13 +2319,12 @@ AFTER_USUAL_COMPLETION:
 							fputs(hd_c, stdout);
 							rl_insert_text("/");
 
-/////////////////////////////////
 					/* WORKAROUND: If we are not at the end of the line,
 					 * redisplay only up to the cursor position, to prevent
 					 * whatever is after it from being printed using the
 					 * last printed color.
 					 * Drawback: there will be no color after the cursor
-					 * position (no color however is better than a wrong color) */
+					 * position (no color however is better than a wrong color). */
 							if (rl_point < rl_end) {
 								int _end = rl_end;
 								rl_end = rl_point;
@@ -2257,10 +2335,7 @@ AFTER_USUAL_COMPLETION:
 							} else {
 								rl_redisplay();
 							}
-/////////////////////////////
 
-//							rl_redisplay();
-//							fputs(cc ? cc : "", stdout);
 							fputs(rl_point < rl_end ? tx_c
 								: (cc ? cc : ""), stdout);
 						} else {
@@ -2283,13 +2358,12 @@ AFTER_USUAL_COMPLETION:
 	break;
 
 	case '?': {
-		int len = 0, count = 0, limit = 0, max = 0;
 		int j = 0, k = 0, l = 0;
 
 		if (flags & PREVIEWER)
 			goto CALC_OFFSET;
 
-		/* Handle simple case first. Just one match */
+		/* Handle simple case first. Just one match. */
 		if (!matches[1]) {
 			char *temp;
 			temp = printable_part(matches[0]);
@@ -2300,7 +2374,7 @@ AFTER_USUAL_COMPLETION:
 		}
 
 		/* There is more than one match. Find out how many there are, and
-		 * find out what the maximum printed length of a single entry is */
+		 * find out what the maximum printed length of a single entry is. */
 
 DISPLAY_MATCHES:
 #ifndef _NO_FZF
@@ -2321,8 +2395,8 @@ DISPLAY_MATCHES:
 
 			len = (int)i - 1;
 
-			/* If there are multiple items, ask the user if she really
-			 * wants to see them all */
+			/* If there are multiple items, ask the user if they really
+			 * wants to see them all. */
 			if (len >= rl_completion_query_items) {
 				putchar('\n');
 #ifndef _NO_HIGHLIGHT
@@ -2421,7 +2495,7 @@ DISPLAY_MATCHES:
 
 CALC_OFFSET:
 #ifndef _NO_FZF
-		/* Alternative TAB completion: fzf, fnf, smenu */
+		/* Alternative TAB completion: fzf, fnf, smenu. */
 		if (fzftab == 1) {
 			char *t = text ? text : (char *)NULL;
 			if (finder_tabcomp(matches, common_prefix_added == 1 ? t : NULL,
@@ -2470,7 +2544,7 @@ CALC_OFFSET:
 		|| cur_comp_type == TCMP_DIRHIST
 		|| (tabmode == STD_TAB && (cur_comp_type == TCMP_JUMP
 		|| cur_comp_type == TCMP_TAGS_F) ) )
-			/* We don't want to highlight the matching part */
+			/* We don't want to highlight the matching part. */
 			tab_offset = 0;
 
 		if (cur_comp_type == TCMP_HIST && ptr && *ptr == '!' && tab_offset > 0) {
@@ -2482,7 +2556,7 @@ CALC_OFFSET:
 
 #ifndef _NO_TRASH
 		/* If printing trashed files, let's change to the trash dir
-		 * to allow files colorization */
+		 * to allow files colorization. */
 		if ((cur_comp_type == TCMP_UNTRASH || cur_comp_type == TCMP_TRASHDEL)
 		&& conf.colorize == 1 && trash_files_dir) {
 			did_chdir = 1;
@@ -2497,7 +2571,7 @@ CALC_OFFSET:
 				/* A little pager */
 				fputs("\x1b[7;97m--Mas--\x1b[0;49m", stdout);
 				int c = 0;
-				while ((c = xgetchar()) == _ESC);
+				while ((c = xgetchar()) == KEY_ESC);
 				if (c == 'q') {
 					/* Delete the --Mas-- label */
 					fputs("\x1b[7D\x1b[7X\x1b[1A\n", stdout);
@@ -2512,14 +2586,14 @@ CALC_OFFSET:
 					break;
 				} else {
 					if (tab_offset) {
-						/* Print the matching part of the match */
+						/* Print the matching part of the match. */
 						printf("\x1b[0m%s%s\x1b[0m%s",
 							ts_c, qq ? qq : matches[0],
 							(cur_comp_type == TCMP_CMD) ? (conf.colorize
 							? ex_c : "") : fc_c);
 					}
 
-					/* Now print the non-matching part of the match */
+					/* Now print the non-matching part of the match. */
 					char *temp = printable_part(matches[l]);
 					int printed_length = (int)wc_xstrlen(temp);
 					printed_length += print_filename(temp, matches[l]);
@@ -2592,7 +2666,6 @@ RESTART:
 		xerror("\r\nreadline: %c: Bad value for what_to_do "
 			"in tab_complete\n", what_to_do);
 		exit(EXIT_FAILURE);
-		break;
 	}
 
 	for (i = 0; matches[i]; i++)
