@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,7 +25,11 @@
 #ifndef TABCOMP_H
 #define TABCOMP_H
 
-int tab_complete(int what_to_do);
-void reinsert_slashes(char *str);
+__BEGIN_DECLS
+
+int  tab_complete(const int);
+void reinsert_slashes(char *);
+
+__END_DECLS
 
 #endif /* TABCOMP_H */
