@@ -29,9 +29,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <dirent.h>
 #include <unistd.h>
-#include <fcntl.h>
 #include <readline/tilde.h>
 
 #include "aux.h"
@@ -155,7 +153,8 @@ list_files_in_tag(char *name)
 	snprintf(tmp, sizeof(tmp), "%s/%s", tags_dir, name);
 
 	struct dirent **t = (struct dirent **)NULL;
-	int n = scandir(tmp, &t, NULL, conf.case_sens_list ? xalphasort : alphasort_insensitive);
+	int n = scandir(tmp, &t, NULL, conf.case_sens_list
+		? xalphasort : alphasort_insensitive);
 	if (n == -1)
 		return errno;
 
@@ -302,9 +301,10 @@ list_tags(char **args)
 		for (i = 0; tags[i]; i++) {
 			char p[PATH_MAX];
 			snprintf(p, sizeof(p), "%s/%s", tags_dir, tags[i]);
-			int n = count_dir(p, NO_CPOP);
+			filesn_t n = count_dir(p, NO_CPOP);
 			if (n > 2)
-				printf("%-*s [%s%d%s]\n", pad, tags[i], mi_c, n - 2, df_c);
+				printf("%-*s [%s%jd%s]\n", pad, tags[i], mi_c,
+					(intmax_t)n - 2, df_c);
 			else
 				printf("%-*s  -\n", pad, tags[i]);
 		}
@@ -690,7 +690,7 @@ recursive_mv_tags(const char *src, const char *dst)
 		}
 		char src_file[PATH_MAX + NAME_MAX + 2];
 		snprintf(src_file, sizeof(src_file), "%s/%s", src_dir, a[i]->d_name);
-		char *cmd[] = {"mv", src_file, dst_dir, NULL};
+		char *cmd[] = {"mv", "--", src_file, dst_dir, NULL};
 		if ((ret = launch_execv(cmd, FOREGROUND, E_NOFLAG)) != EXIT_SUCCESS)
 			exit_status = ret;
 		free(a[i]);
