@@ -455,6 +455,8 @@ const struct terms_t TERM_INFO[] = {
 	{"foot", 4, 256, 1, 1, 1, 1, 1, 1, 1},
 	{"foot+base", 9, -1, 1, 1, 1, 1, 1, 1, 1},
 	{"foot-direct", 11, 16777216, 1, 1, 1, 1, 1, 1, 1},
+	{"foot-extra", 10, 256, 1, 1, 1, 1, 1, 1, 1},
+	{"foot-extra-direct", 17, 16777216, 1, 1, 1, 1, 1, 1, 1},
 	{"gigi", 4, -1, 1, 1, 1, 0, 0, 0, 0},
 	{"gnome", 5, 8, 1, 1, 1, 1, 0, 1, 1},
 	{"gnome-2007", 10, 8, 1, 1, 1, 1, 0, 1, 1},
@@ -891,6 +893,8 @@ const struct terms_t TERM_INFO[] = {
 	{"qvt203-w", 8, -1, 1, 1, 1, 0, 0, 1, 0},
 	{"qvt203-w-am", 11, -1, 1, 1, 1, 0, 0, 1, 0},
 	{"rcons-color", 11, 8, 0, 1, 1, 0, 0, 0, 0},
+	{"rio", 3, 256, 1, 1, 1, 1, 1, 1, 1},
+	{"rio+base", 8, 8, 1, 1, 1, 1, 1, 1, 1},
 	{"rt6221", 6, -1, 1, 1, 1, 1, 0, 1, 0},
 	{"rt6221-w", 8, -1, 1, 1, 1, 1, 0, 1, 0},
 	{"rxvt", 4, 8, 1, 1, 1, 1, 0, 1, 1},
